<?php
require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/audit_logs.php');
require_once(__DIR__ . '/error_handling.php');

//function createAccount($pdo, string $code, string $name, string $type, string $description = '', string $parent = null, bool $isSystem = false, int $userId): bool
function createAccount($pdo,string $code,string $name,string $type,int $userId,string $description = '',string $parent = null,bool $isSystem = false): bool
{
    try {
        $stmt = $pdo->prepare("INSERT INTO accounts (code, name, description, account_type, parent, is_system) VALUES (?, ?, ?, ?, ?, ?);");
        $stmt->execute([$code, $name, $description, $type, $parent, $isSystem ? 1 : 0]);

        // Audit log
        createAuditLog($pdo, $userId, 'create', 'accounts', 0, "Created Account: $name");

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Create Account Error: ' . $e->getMessage());
        return false;
    }
}

function getAccount($pdo, string $code): array
{
    try {
        $stmt = $pdo->prepare("SELECT * FROM accounts WHERE code = ?;");
        $stmt->execute([$code]);
        return $stmt->fetch(PDO::FETCH_ASSOC) ?: [];
    } catch (PDOException $e) {
        logError($pdo, 'Get Account Error: ' . $e->getMessage());
        return [];
    }
}

//function listAccounts($pdo, array $filters = []): array
//{
//    try {
//        $query = "SELECT * FROM accounts WHERE 1=1";
//        $params = [];
//        if (isset($filters['account_type'])) {
//            $query .= " AND account_type = ?";
//            $params[] = $filters['account_type'];
//        }
//        if (isset($filters['is_system'])) {
//            $query .= " AND is_system = ?";
//            $params[] = $filters['is_system'] ? 1 : 0;
//        }
//        $stmt = $pdo->prepare($query);
//        $stmt->execute($params);
//        return $stmt->fetchAll(PDO::FETCH_ASSOC);
//    } catch (PDOException $e) {
//        logError($pdo, 'List Accounts Error: ' . $e->getMessage());
//        return [];
//    }
//}
function listAccounts($pdo, array $filters = []): array
{
    try {
        $query = "SELECT * FROM accounts WHERE 1=1";
        $params = [];

        if (isset($filters['account_type'])) {
            $query .= " AND account_type = ?";
            $params[] = $filters['account_type'];
        }

        if (isset($filters['is_system'])) {
            $query .= " AND is_system = ?";
            $params[] = $filters['is_system'] ? 1 : 0;
        }

        // Add parent filter support if specified
        if (isset($filters['parent'])) {
            $query .= " AND parent = ?";
            $params[] = $filters['parent'];
        }

        $stmt = $pdo->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'List Accounts Error: ' . $e->getMessage());
        return [];
    }
}

function updateAccount($pdo, string $code, string $name, string $description, int $userId): bool
{
    try {
        $stmt = $pdo->prepare("UPDATE accounts SET name = ?, description = ? WHERE code = ?;");
        $stmt->execute([$name, $description, $code]);

        // Audit log
        createAuditLog($pdo, $userId, 'update', 'accounts', 0, "Updated Account: $name");

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Update Account Error: ' . $e->getMessage());
        return false;
    }
}

function deleteAccount($pdo, string $code, int $userId): bool
{
    try {
        $balance = calculateAccountBalance($pdo, $code);
        if ($balance !== 0.0) {
            return false;
        }
        $stmt = $pdo->prepare("DELETE FROM accounts WHERE code = ?;");
        $stmt->execute([$code]);

        // Audit log
        createAuditLog($pdo, $userId, 'delete', 'accounts', 0, "Deleted Account Code: $code");

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Delete Account Error: ' . $e->getMessage());
        return false;
    }
}

function calculateAccountBalance($pdo, string $code): float
{
    try {
        $stmt = $pdo->prepare("SELECT SUM(amount) as total_debit FROM transactions WHERE debit_account = ? AND is_deleted = 0;");
        $stmt->execute([$code]);
        $debits = (float) $stmt->fetchColumn();
        $stmt = $pdo->prepare("SELECT SUM(amount) as total_credit FROM transactions WHERE credit_account = ? AND is_deleted = 0;");
        $stmt->execute([$code]);
        $credits = (float) $stmt->fetchColumn();
        $account = getAccount($pdo, $code);
        if ($account) {
            switch (strtolower($account['account_type'])) {
                case 'asset':
                case 'expense':
                    return $debits - $credits;
                case 'liability':
                case 'equity':
                case 'income':
                    return $credits - $debits;
            }
        }
        return 0.0;
    } catch (PDOException $e) {
        logError($pdo, 'Calculate Account Balance Error: ' . $e->getMessage());
        return 0.0;
    }
}

try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    logError($pdo, 'Database Connection Error: ' . $e->getMessage());
}
?>
