<?php
require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/error_handling.php'); // Include error handling

/**
 * Create a new audit log entry.
 *
 * @param PDO $pdo The PDO database connection.
 * @param int $userId The ID of the user performing the action.
 * @param string $action The action performed.
 * @param string $entity The entity affected.
 * @param int $entityId The ID of the entity affected.
 * @param string $additionalInfo Additional information about the action.
 * @return int The ID of the newly created audit log entry, or 0 on failure.
 */
function createAuditLog($pdo, int $userId, string $action, string $entity, int $entityId, string $additionalInfo = '', int $isSystem = 1): int
{
    // Input validation
    $errors = [];

    if ($userId <= 0) {
        $errors[] = 'Invalid user ID.';
    }
    if (empty($action)) {
        $errors[] = 'Action is required.';
    }
    if (empty($entity)) {
        $errors[] = 'Entity is required.';
    }
    if ($entityId <= 0) {
        $errors[] = 'Invalid entity ID.';
    }
    if (strlen($action) > 100) {
        $errors[] = 'Action must be less than 100 characters.';
    }
    if (strlen($entity) > 100) {
        $errors[] = 'Entity must be less than 100 characters.';
    }
    if (strlen($additionalInfo) > 255) {
        $errors[] = 'Additional info must be less than 255 characters.';
    }
    if ($isSystem !== 0 && $isSystem !== 1) {
        $errors[] = 'Invalid value for is_system.';
    }

    if (!empty($errors)) {
        logError($pdo, 'Create Audit Log Validation Errors: ' . implode('; ', $errors));
        return 0;
    }

    try {
        $stmt = $pdo->prepare("INSERT INTO audit_logs (user_id, action, entity, entity_id, additional_info, is_system) VALUES (?, ?, ?, ?, ?, ?)");
        $stmt->execute([$userId, $action, $entity, $entityId, $additionalInfo, $isSystem]);
        return (int)$pdo->lastInsertId();
    } catch (PDOException $e) {
        logError($pdo, 'Create Audit Log Error: ' . $e->getMessage());
        return 0;
    }
}
/**
 * Retrieve a single audit log entry by ID.
 *
 * @param PDO $pdo The PDO database connection.
 * @param int $auditLogId The ID of the audit log entry.
 * @return array The audit log entry data, or an empty array on failure.
 */
function getAuditLog($pdo, int $auditLogId): array
{
    try {
        $stmt = $pdo->prepare("SELECT audit_logs.*, users.username FROM audit_logs LEFT JOIN users ON audit_logs.user_id = users.id WHERE audit_logs.id = ?");
        $stmt->execute([$auditLogId]);
        $auditLog = $stmt->fetch(PDO::FETCH_ASSOC);
        return $auditLog ? $auditLog : [];
    } catch (PDOException $e) {
        logError($pdo, 'Get Audit Log Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * List audit logs with optional filters and pagination.
 *
 * @param PDO $pdo The PDO database connection.
 * @param array $filters An associative array of filters.
 * @param int $page The page number for pagination.
 * @param int $pageSize The number of records per page.
 * @return array An array containing audit logs and pagination data.
 */
function listAuditLogs($pdo, array $filters = [], int $page = 1, int $pageSize = 20): array
{
    try {
        $offset = ($page - 1) * $pageSize;
        $query = "SELECT audit_logs.*, users.username FROM audit_logs LEFT JOIN users ON audit_logs.user_id = users.id";
        $params = [];
        $conditions = [];

        // Filters
        if (isset($filters['user_id']) && $filters['user_id'] !== '') {
            $conditions[] = "audit_logs.user_id = :user_id";
            $params[':user_id'] = $filters['user_id'];
        }
        if (isset($filters['action']) && $filters['action'] !== '') {
            $conditions[] = "audit_logs.action LIKE :action";
            $params[':action'] = '%' . $filters['action'] . '%';
        }
        if (isset($filters['entity']) && $filters['entity'] !== '') {
            $conditions[] = "audit_logs.entity LIKE :entity";
            $params[':entity'] = '%' . $filters['entity'] . '%';
        }
        if (isset($filters['entity_id']) && $filters['entity_id'] !== '') {
            $conditions[] = "audit_logs.entity_id = :entity_id";
            $params[':entity_id'] = $filters['entity_id'];
        }
        if (isset($filters['additional_info']) && $filters['additional_info'] !== '') {
            $conditions[] = "audit_logs.additional_info LIKE :additional_info";
            $params[':additional_info'] = '%' . $filters['additional_info'] . '%';
        }
        if (isset($filters['start_date']) && $filters['start_date'] !== '') {
            $conditions[] = "audit_logs.created_at >= :start_date";
            $params[':start_date'] = $filters['start_date'];
        }
        if (isset($filters['end_date']) && $filters['end_date'] !== '') {
            $conditions[] = "audit_logs.created_at <= :end_date";
            $params[':end_date'] = $filters['end_date'];
        }

        if ($conditions) {
            $query .= " WHERE " . implode(' AND ', $conditions);
        }

        // Total count for pagination
        $countQuery = "SELECT COUNT(*) FROM audit_logs LEFT JOIN users ON audit_logs.user_id = users.id";
        if ($conditions) {
            $countQuery .= " WHERE " . implode(' AND ', $conditions);
        }
        $countStmt = $pdo->prepare($countQuery);
        $countStmt->execute($params);
        $totalRecords = (int)$countStmt->fetchColumn();

        // Pagination calculations
        $totalPages = ceil($totalRecords / $pageSize);

        // Sorting
        $sortBy = $filters['sort_by'] ?? 'audit_logs.created_at';
        $sortOrder = strtoupper($filters['sort_order'] ?? 'DESC');
        $allowedSortColumns = ['audit_logs.id', 'audit_logs.user_id', 'audit_logs.action', 'audit_logs.entity', 'audit_logs.entity_id', 'audit_logs.created_at'];
        if (!in_array($sortBy, $allowedSortColumns)) {
            $sortBy = 'audit_logs.created_at';
        }
        $sortOrder = ($sortOrder === 'ASC') ? 'ASC' : 'DESC';

        // Limit and Offset
        $query .= " ORDER BY $sortBy $sortOrder LIMIT :limit OFFSET :offset";
        $stmt = $pdo->prepare($query);

        // Bind parameters
        foreach ($params as $key => &$value) {
            $stmt->bindParam($key, $value);
        }
        $stmt->bindValue(':limit', $pageSize, PDO::PARAM_INT);
        $stmt->bindValue(':offset', $offset, PDO::PARAM_INT);

        $stmt->execute();
        $auditLogs = $stmt->fetchAll(PDO::FETCH_ASSOC);

        return [
            'audit_logs' => $auditLogs ? $auditLogs : [],
            'total_records' => $totalRecords,
            'current_page' => $page,
            'page_size' => $pageSize,
            'total_pages' => $totalPages
        ];
    } catch (PDOException $e) {
        logError($pdo, 'List Audit Logs Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * Delete an audit log entry by ID.
 *
 * @param PDO $pdo The PDO database connection.
 * @param int $auditLogId The ID of the audit log entry to delete.
 * @return bool True on success, false on failure.
 */

function deleteAuditLog($pdo, int $auditLogId): bool
{
    try {
        // First, check if the audit log exists and retrieve its is_system value
        $stmt = $pdo->prepare("SELECT is_system FROM audit_logs WHERE id = ?");
        $stmt->execute([$auditLogId]);
        $auditLog = $stmt->fetch(PDO::FETCH_ASSOC);

        if (!$auditLog) {
            throw new InvalidArgumentException('Audit log not found.');
        }

        if ((int)$auditLog['is_system'] === 1) {
            throw new InvalidArgumentException('Cannot delete system-generated audit logs.');
        }

        // Proceed to delete if is_system = 0
        $stmt = $pdo->prepare("DELETE FROM audit_logs WHERE id = ?");
        return $stmt->execute([$auditLogId]);
    } catch (InvalidArgumentException $e) {
        logError($pdo, 'Validation Error in Delete Audit Log: ' . $e->getMessage());
        return false;
    } catch (PDOException $e) {
        logError($pdo, 'Delete Audit Log Error: ' . $e->getMessage());
        return false;
    }
}



// Ensure the $pdo variable is available for the functions
if (!isset($pdo)) {
    try {
        $pdo = new PDO('sqlite:' . DB_FILE);
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
    } catch (PDOException $e) {
        logError($pdo, 'Database Connection Error: ' . $e->getMessage());
    }
}
?>
