<?php
require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/audit_logs.php');
require_once(__DIR__ . '/error_handling.php');
require_once(__DIR__ . '/financial_reports.php');
require_once(__DIR__ . '/accounts.php');
require_once(__DIR__ . '/journal_entries.php');
require_once(__DIR__ . '/transactions.php');

/**
 * Create a new financial period.
 */
function createFinancialPeriod($pdo, string $name, string $startDate, string $endDate, int $userId): array
{
    try {
        if (empty($name) || empty($startDate) || empty($endDate)) {
            throw new InvalidArgumentException('Cannot create financial period: Name, start date, and end date are required.');
        }

        if (strtotime($endDate) < strtotime($startDate)) {
            throw new InvalidArgumentException('Cannot create financial period: End date must be after the start date.');
        }

        if (checkFinancialPeriodOverlap($pdo, $startDate, $endDate)) {
            throw new InvalidArgumentException('Cannot create financial period: The specified date range overlaps with an existing financial period.');
        }

        $stmt = $pdo->prepare("INSERT INTO financial_periods (name, start_date, end_date) VALUES (?, ?, ?);");
        $stmt->execute([$name, $startDate, $endDate]);
        $financialPeriodId = (int)$pdo->lastInsertId();

        createAuditLog($pdo, $userId, 'create', 'financial_periods', $financialPeriodId, "Created Financial Period: $name");

        return ['success' => true, 'message' => 'Financial period created successfully.'];
    } catch (InvalidArgumentException $e) {
        logError($pdo, 'Create Financial Period Validation Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    } catch (PDOException $e) {
        logError($pdo, 'Create Financial Period Database Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'A database error occurred while creating the financial period.'];
    } catch (Exception $e) {
        logError($pdo, 'Create Financial Period Unexpected Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'An unexpected error occurred while creating the financial period.'];
    }
}

function checkFinancialPeriodOverlap($pdo, string $startDate, string $endDate): bool
{
    try {
        $stmt = $pdo->prepare("
            SELECT COUNT(*) FROM financial_periods
            WHERE is_deleted = 0 AND (
                (start_date <= ? AND end_date >= ?)
                OR (start_date <= ? AND end_date >= ?)
                OR (start_date >= ? AND end_date <= ?)
            );
        ");
        $stmt->execute([$startDate, $startDate, $endDate, $endDate, $startDate, $endDate]);
        return ((int)$stmt->fetchColumn()) > 0;
    } catch (PDOException $e) {
        logError($pdo, 'Check Financial Period Overlap Error: ' . $e->getMessage());
        return true;
    }
}

function getFinancialPeriod($pdo, int $financialPeriodId): array
{
    try {
        $stmt = $pdo->prepare("SELECT * FROM financial_periods WHERE id = ? AND is_deleted = 0;");
        $stmt->execute([$financialPeriodId]);
        return $stmt->fetch(PDO::FETCH_ASSOC) ?: [];
    } catch (PDOException $e) {
        logError($pdo, 'Get Financial Period Error: ' . $e->getMessage());
        return [];
    }
}

function getFinancialPeriodIdByDate($pdo, string $date): ?int
{
    try {
        $stmt = $pdo->prepare("SELECT id FROM financial_periods WHERE start_date <= ? AND end_date >= ? AND is_deleted = 0 LIMIT 1;");
        $stmt->execute([$date, $date]);
        $result = $stmt->fetch(PDO::FETCH_ASSOC);
        return $result ? (int)$result['id'] : null;
    } catch (PDOException $e) {
        logError($pdo, 'Get Financial Period ID by Date Error: ' . $e->getMessage());
        return null;
    }
}

function listFinancialPeriods($pdo, array $filters = []): array
{
    try {
        $query = "SELECT * FROM financial_periods WHERE is_deleted = 0";
        $params = [];

        if (isset($filters['is_closed'])) {
            $query .= " AND is_closed = ?";
            $params[] = $filters['is_closed'] ? 1 : 0;
        }
        if (isset($filters['start_date'])) {
            $query .= " AND start_date >= ?";
            $params[] = $filters['start_date'];
        }
        if (isset($filters['end_date'])) {
            $query .= " AND end_date <= ?";
            $params[] = $filters['end_date'];
        }
        if (isset($filters['name'])) {
            $query .= " AND name LIKE ?";
            $params[] = '%' . $filters['name'] . '%';
        }

        $query .= " ORDER BY start_date ASC";
        $stmt = $pdo->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'List Financial Periods Error: ' . $e->getMessage());
        return [];
    }
}

function updateFinancialPeriod($pdo, int $financialPeriodId, string $name, string $startDate, string $endDate, int $userId): array
{
    try {
        if (empty($name) || empty($startDate) || empty($endDate)) {
            throw new InvalidArgumentException('Cannot update financial period: Name, start date, and end date are required.');
        }

        if (strtotime($endDate) < strtotime($startDate)) {
            throw new InvalidArgumentException('Cannot update financial period: End date must be after start date.');
        }

        $currentPeriod = getFinancialPeriod($pdo, $financialPeriodId);
        if (empty($currentPeriod)) {
            throw new InvalidArgumentException('Cannot update: Financial period not found.');
        }

        if ($currentPeriod['is_closed']) {
            throw new InvalidArgumentException('Cannot update a closed financial period.');
        }

        $overlaps = checkFinancialPeriodOverlapExcluding($pdo, $startDate, $endDate, $financialPeriodId);
        if ($overlaps) {
            throw new InvalidArgumentException('Cannot update financial period: The date range overlaps with another period.');
        }

        $stmt = $pdo->prepare("UPDATE financial_periods SET name = ?, start_date = ?, end_date = ? WHERE id = ?;");
        $stmt->execute([$name, $startDate, $endDate, $financialPeriodId]);

        createAuditLog($pdo, $userId, 'update', 'financial_periods', $financialPeriodId, "Updated Financial Period: $name");

        return ['success' => true, 'message' => 'Financial period updated successfully.'];
    } catch (InvalidArgumentException $e) {
        logError($pdo, 'Update Financial Period Validation Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    } catch (PDOException $e) {
        logError($pdo, 'Update Financial Period DB Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'A database error occurred while updating the financial period.'];
    } catch (Exception $e) {
        logError($pdo, 'Update Financial Period Unexpected Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'An unexpected error occurred while updating the financial period.'];
    }
}

function checkFinancialPeriodOverlapExcluding($pdo, string $startDate, string $endDate, int $excludeId): bool
{
    try {
        $stmt = $pdo->prepare("
            SELECT COUNT(*) FROM financial_periods
            WHERE is_deleted = 0 AND id != ?
              AND (
                (start_date <= ? AND end_date >= ?)
                OR (start_date <= ? AND end_date >= ?)
                OR (start_date >= ? AND end_date <= ?)
              );
        ");
        $stmt->execute([$excludeId, $startDate, $startDate, $endDate, $endDate, $startDate, $endDate]);
        return ((int)$stmt->fetchColumn()) > 0;
    } catch (PDOException $e) {
        logError($pdo, 'Check Overlap Excluding Error: ' . $e->getMessage());
        return true;
    }
}

function isUnallocatedAccountZero($pdo, int $financialPeriodId): array
{
    try {
        $period = getFinancialPeriod($pdo, $financialPeriodId);
        if (empty($period)) {
            return ['success' => false, 'message' => 'Unable to check M13 balance: Financial period not found.'];
        }
        $asOfDate = $period['end_date'];

        $stmt = $pdo->prepare("
            SELECT
                COALESCE(d.total_debit, 0) as total_debit,
                COALESCE(c.total_credit, 0) as total_credit
            FROM accounts a
            LEFT JOIN (
                SELECT debit_account, SUM(amount) as total_debit
                FROM transactions
                WHERE transaction_date <= ? AND is_deleted = 0
                GROUP BY debit_account
            ) d ON a.code = d.debit_account
            LEFT JOIN (
                SELECT credit_account, SUM(amount) as total_credit
                FROM transactions
                WHERE transaction_date <= ? AND is_deleted = 0
                GROUP BY credit_account
            ) c ON a.code = c.credit_account
            WHERE a.code = '" . UNALLOCATED_ACC_CODE . "';
        ");
        $stmt->execute([$asOfDate, $asOfDate]);
        $result = $stmt->fetch(PDO::FETCH_ASSOC);

        if ($result) {
            $balance = $result['total_debit'] - $result['total_credit'];
            if (abs($balance) < 0.0001) {
                return ['success' => true, 'message' => 'M13 is zero.'];
            } else {
                return ['success' => false, 'message' => 'Cannot close financial period: Unallocated Account (' . UNALLOCATED_ACC_CODE . ') is not zero. Balance: ' . number_format($balance, 2)];
            }
        }

        return ['success' => false, 'message' => 'M13 account could not be found or calculated.'];
    } catch (PDOException $e) {
        logError($pdo, 'Check M13 Zero Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'Database error occurred while checking M13 balance.'];
    }
}

/**
 * Close a financial period:
 * 1. Check M13 (unallocated account) balance is zero.
 * 2. Create a single journal entry for the closing process (module_id = -1).
 *    - For each revenue account, debit the revenue account and credit Net Income (5200) to zero it out.
 *    - For each expense account, credit the expense account and debit Net Income (5200) to zero it out.
 *    - After all revenues and expenses are zeroed to Net Income, transfer Net Income to Retained Earnings (B15).
 */
function closeFinancialPeriod($pdo, int $financialPeriodId, int $userId): array
{
    try {
        $currentPeriod = getFinancialPeriod($pdo, $financialPeriodId);
        if (empty($currentPeriod)) {
            throw new InvalidArgumentException('Cannot close period: Financial period not found.');
        }

        if ($currentPeriod['is_closed']) {
            throw new InvalidArgumentException('Cannot close period: Financial period is already closed.');
        }
        if (hasOpenVATPeriodInRange($pdo, $currentPeriod['start_date'], $currentPeriod['end_date'])) {
            throw new InvalidArgumentException('Cannot close this financial period: At least one overlapping VAT period is still open. Please close all relevant VAT periods first.');
        }
        // Check M13 (unallocated) balance
        $m13Check = isUnallocatedAccountZero($pdo, $financialPeriodId);
        if (!$m13Check['success']) {
            throw new InvalidArgumentException($m13Check['message']);
        }

        $pdo->beginTransaction();

        $startDate = $currentPeriod['start_date'];
        $endDate = $currentPeriod['end_date'];
        $incomeStmt = generateIncomeStatement($pdo, $startDate, $endDate);

        // Create a journal entry for the close
        $jeDescription = "Period Close - Financial Period ID: $financialPeriodId";
        $journalEntryId = createJournalEntry($pdo, $jeDescription, $endDate, $financialPeriodId, $userId);
        if (!$journalEntryId) {
            throw new Exception('Failed to create closing journal entry.');
        }

        // Set module_id = -1 for the closing JE
        $stmt = $pdo->prepare("UPDATE journal_entries SET module_id = -1 WHERE id = ?");
        $stmt->execute([$journalEntryId]);

        // Use constants for Net Income and Retained Earnings
        $netIncomeAccount = NET_INCOME_ACC_CODE;
        $retainedEarningsAccount = RETAINED_EARNINGS_ACC_CODE;

        $totalRevenue = 0;
        $totalExpense = 0;

        // Zero out revenue and expense accounts by offsetting to Net Income
        foreach ($incomeStmt as $acct) {
            $type = strtolower($acct['account_type']);
            $amount = (float)$acct['total_amount'];
            if (abs($amount) < 0.0001) {
                continue; // skip zero amounts
            }

            if (in_array($type, ['revenue', 'income'])) {
                // Debit revenue, credit net income
                $tid = createTransaction($pdo, $journalEntryId, $acct['code'], $netIncomeAccount, $amount, "Close Revenue: {$acct['name']}", $endDate, $financialPeriodId, $userId);
                $stmt = $pdo->prepare("UPDATE transactions SET module_id = -1 WHERE id = ?");
                $stmt->execute([$tid]);
                $totalRevenue += $amount;
            } elseif ($type === 'expense') {
                // Debit net income, credit expense
                $tid = createTransaction($pdo, $journalEntryId, $netIncomeAccount, $acct['code'], $amount, "Close Expense: {$acct['name']}", $endDate, $financialPeriodId, $userId);
                $stmt = $pdo->prepare("UPDATE transactions SET module_id = -1 WHERE id = ?");
                $stmt->execute([$tid]);
                $totalExpense += $amount;
            }
        }

        $netIncome = $totalRevenue - $totalExpense;

        // Transfer net income to Retained Earnings
        if (abs($netIncome) > 0.0001) {
            if ($netIncome > 0) {
                // Profit: debit Net Income, credit Retained Earnings
                $tid = createTransaction($pdo, $journalEntryId, $netIncomeAccount, $retainedEarningsAccount, $netIncome, "Transfer Net Income to Retained Earnings", $endDate, $financialPeriodId, $userId);
                $stmt = $pdo->prepare("UPDATE transactions SET module_id = -1 WHERE id = ?");
                $stmt->execute([$tid]);
            } else {
                // Loss: credit Net Income, debit Retained Earnings
                $lossAmount = abs($netIncome);
                $tid = createTransaction($pdo, $journalEntryId, $retainedEarningsAccount, $netIncomeAccount, $lossAmount, "Transfer Net Loss to Retained Earnings", $endDate, $financialPeriodId, $userId);
                $stmt = $pdo->prepare("UPDATE transactions SET module_id = -1 WHERE id = ?");
                $stmt->execute([$tid]);
            }
        }

        $stmt = $pdo->prepare("UPDATE financial_periods SET is_closed = 1 WHERE id = ?;");
        $stmt->execute([$financialPeriodId]);

        createAuditLog($pdo, $userId, 'close', 'financial_periods', $financialPeriodId, "Closed Financial Period ID: $financialPeriodId");

        $pdo->commit();
        return ['success' => true, 'message' => 'Financial period closed successfully.'];
    } catch (InvalidArgumentException $e) {
        if ($pdo->inTransaction()) {
            $pdo->rollBack();
        }
        logError($pdo, 'Close Financial Period Validation Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    } catch (PDOException $e) {
        if ($pdo->inTransaction()) {
            $pdo->rollBack();
        }
        logError($pdo, 'Close Financial Period DB Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'A database error occurred while closing the financial period.'];
    } catch (Exception $e) {
        if ($pdo->inTransaction()) {
            $pdo->rollBack();
        }
        logError($pdo, 'Close Financial Period Unexpected Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'An unexpected error occurred while closing the financial period.'];
    }
}

/**
 * Reopen a financial period:
 * When reopening, we delete the closing journal entry and all its transactions (those with module_id = -1).
 */
function reopenFinancialPeriod($pdo, int $financialPeriodId, int $userId): array
{
    try {
        $currentPeriod = getFinancialPeriod($pdo, $financialPeriodId);
        if (empty($currentPeriod)) {
            throw new InvalidArgumentException('Cannot reopen period: Financial period not found.');
        }

        if (!$currentPeriod['is_closed']) {
            throw new InvalidArgumentException('Cannot reopen period: This financial period is already open.');
        }

        // Find the closing journal entry
        $desc = "Period Close - Financial Period ID: $financialPeriodId";
        $stmt = $pdo->prepare("SELECT * FROM journal_entries WHERE financial_period_id = ? AND description = ? AND is_deleted = 0 AND module_id = -1");
        $stmt->execute([$financialPeriodId, $desc]);
        $closingJE = $stmt->fetch(PDO::FETCH_ASSOC);

        if ($closingJE) {
            $closingJEId = (int)$closingJE['id'];

            $pdo->beginTransaction();
            // Delete all transactions for this JE
            $delTrans = $pdo->prepare("DELETE FROM transactions WHERE journal_entry_id = ?");
            $delTrans->execute([$closingJEId]);

            // Delete the journal entry
            $delJE = $pdo->prepare("DELETE FROM journal_entries WHERE id = ?");
            $delJE->execute([$closingJEId]);

            // Set the financial period to open
            $stmt = $pdo->prepare("UPDATE financial_periods SET is_closed = 0 WHERE id = ?;");
            $stmt->execute([$financialPeriodId]);

            createAuditLog($pdo, $userId, 'reopen', 'financial_periods', $financialPeriodId, "Reopened Financial Period ID: $financialPeriodId");

            $pdo->commit();
        } else {
            // If no closing JE found, just mark the period as open
            $stmt = $pdo->prepare("UPDATE financial_periods SET is_closed = 0 WHERE id = ?;");
            $stmt->execute([$financialPeriodId]);
            createAuditLog($pdo, $userId, 'reopen', 'financial_periods', $financialPeriodId, "Reopened Financial Period ID: $financialPeriodId");
        }

        return ['success' => true, 'message' => 'Financial period reopened successfully.'];
    } catch (InvalidArgumentException $e) {
        if ($pdo->inTransaction()) {
            $pdo->rollBack();
        }
        logError($pdo, 'Reopen Financial Period Validation Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    } catch (PDOException $e) {
        if ($pdo->inTransaction()) {
            $pdo->rollBack();
        }
        logError($pdo, 'Reopen Financial Period DB Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'A database error occurred while reopening the financial period.'];
    } catch (Exception $e) {
        if ($pdo->inTransaction()) {
            $pdo->rollBack();
        }
        logError($pdo, 'Reopen Financial Period Unexpected Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'An unexpected error occurred while reopening the financial period.'];
    }
}

function deleteFinancialPeriod($pdo, int $financialPeriodId, int $userId): array
{
    try {
        $currentPeriod = getFinancialPeriod($pdo, $financialPeriodId);
        if (empty($currentPeriod)) {
            throw new InvalidArgumentException('Cannot delete: Financial period not found.');
        }

        $stmt = $pdo->prepare("SELECT COUNT(*) FROM transactions WHERE period_id = ? AND is_deleted = 0;");
        $stmt->execute([$financialPeriodId]);
        $transactionCount = (int)$stmt->fetchColumn();
        if ($transactionCount > 0) {
            throw new InvalidArgumentException('Cannot delete financial period: There are existing transactions associated with this period.');
        }

        $stmt = $pdo->prepare("UPDATE financial_periods SET is_deleted = 1 WHERE id = ?;");
        $stmt->execute([$financialPeriodId]);

        createAuditLog($pdo, $userId, 'delete', 'financial_periods', $financialPeriodId, "Deleted Financial Period ID: $financialPeriodId");

        return ['success' => true, 'message' => 'Financial period deleted successfully.'];
    } catch (InvalidArgumentException $e) {
        logError($pdo, 'Delete Financial Period Validation Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    } catch (PDOException $e) {
        logError($pdo, 'Delete Financial Period DB Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'A database error occurred while deleting the financial period.'];
    } catch (Exception $e) {
        logError($pdo, 'Delete Financial Period Unexpected Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'An unexpected error occurred while deleting the financial period.'];
    }
}

function isFinancialPeriodClosed($pdo, int $financialPeriodId): bool
{
    try {
        $stmt = $pdo->prepare("SELECT is_closed FROM financial_periods WHERE id = ? AND is_deleted = 0;");
        $stmt->execute([$financialPeriodId]);
        $result = $stmt->fetch(PDO::FETCH_ASSOC);
        return $result ? (bool)$result['is_closed'] : false;
    } catch (PDOException $e) {
        logError($pdo, 'Check Financial Period Closed Error: ' . $e->getMessage());
        return false;
    }
}
function hasOpenVATPeriodInRange(PDO $pdo, string $startDate, string $endDate): bool
{
    $stmt = $pdo->prepare("
        SELECT COUNT(*) FROM vat_periods
        WHERE is_deleted = 0
          AND status = 'open'
          AND (
              (start_date <= :endDate AND end_date >= :startDate)
           )
        LIMIT 1
    ");
    $stmt->execute([':startDate' => $startDate, ':endDate' => $endDate]);
    return ((int)$stmt->fetchColumn() > 0);
}
try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    logError(null, 'Database Connection Error: ' . $e->getMessage());
}
