<?php
require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/accounts.php');
require_once(__DIR__ . '/error_handling.php');
require_once(__DIR__ . '/vat_periods.php');

/**
 * ------------------------------------------------------------------
 * Define VAT account constants if not already defined in config.php
 * ------------------------------------------------------------------
 */
if (!defined('VAT_PAYABLE_ACC_CODE')) {
    define('VAT_PAYABLE_ACC_CODE', 'B75'); // Default
}

if (!defined('VAT_RECEIVABLE_ACC_CODE')) {
    define('VAT_RECEIVABLE_ACC_CODE', 'B76'); // Default
}

/**
 * Calculate the Trial Balance for all accounts (cumulative, no date range)
 */
function calculateTrialBalance($pdo): array
{
    try {
        $stmt = $pdo->prepare("
            SELECT
                a.account_type,
                a.code,
                a.name,
                SUM(CASE WHEN t.debit_account = a.code THEN t.amount ELSE 0 END) as total_debit,
                SUM(CASE WHEN t.credit_account = a.code THEN t.amount ELSE 0 END) as total_credit
            FROM accounts a
            LEFT JOIN transactions t ON (t.debit_account = a.code OR t.credit_account = a.code)
                AND t.is_deleted = 0
            GROUP BY a.code, a.account_type, a.name
            ORDER BY
                CASE
                    WHEN a.account_type = 'Asset' THEN 1
                    WHEN a.account_type = 'Liability' THEN 2
                    WHEN a.account_type = 'Equity' THEN 3
                    WHEN a.account_type = 'Revenue' THEN 4
                    WHEN a.account_type = 'Expense' THEN 5
                    ELSE 6
                END,
                a.code ASC
        ");
        $stmt->execute();
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'Calculate Trial Balance Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * Calculate the Trial Balance for a specific date range
 */
function calculateTrialBalanceForDateRange($pdo, string $startDate, string $endDate): array
{
    try {
        $stmt = $pdo->prepare("
            SELECT
                a.account_type,
                a.code,
                a.name,
                a.parent,
                COALESCE(SUM(CASE WHEN t.debit_account = a.code THEN t.amount ELSE 0 END), 0) AS total_debit,
                COALESCE(SUM(CASE WHEN t.credit_account = a.code THEN t.amount ELSE 0 END), 0) AS total_credit
            FROM accounts a
            LEFT JOIN transactions t ON (
                (t.debit_account = a.code OR t.credit_account = a.code)
                AND t.transaction_date BETWEEN ? AND ?
                AND t.is_deleted = 0
            )
            GROUP BY a.code, a.account_type, a.name, a.parent
            ORDER BY
                CASE
                    WHEN a.account_type = 'Asset' THEN 1
                    WHEN a.account_type = 'Liability' THEN 2
                    WHEN a.account_type = 'Equity' THEN 3
                    WHEN a.account_type = 'Revenue' THEN 4
                    WHEN a.account_type = 'Expense' THEN 5
                    ELSE 7
                END,
                a.code ASC;
        ");
        $stmt->execute([$startDate, $endDate]);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'Calculate Trial Balance For Date Range Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * Generate Income Statement (Profit & Loss) for a given date range.
 */
function generateIncomeStatement($pdo, string $startDate, string $endDate): array
{
    try {
        $stmt = $pdo->prepare("
            SELECT
                a.account_type,
                a.code,
                a.name,
                a.parent,
                COALESCE(SUM(
                    CASE
                        WHEN LOWER(a.account_type) IN ('revenue', 'income') THEN
                            CASE
                                WHEN t.credit_account = a.code THEN t.amount
                                WHEN t.debit_account = a.code THEN -t.amount
                                ELSE 0
                            END
                        WHEN LOWER(a.account_type) = 'expense' THEN
                            CASE
                                WHEN t.debit_account = a.code THEN t.amount
                                WHEN t.credit_account = a.code THEN -t.amount
                                ELSE 0
                            END
                        ELSE 0
                    END
                ), 0) as total_amount
            FROM accounts a
            LEFT JOIN transactions t ON (t.debit_account = a.code OR t.credit_account = a.code)
               AND t.transaction_date BETWEEN ? AND ?
               AND t.is_deleted = 0
            WHERE LOWER(a.account_type) IN ('income','revenue','expense')
            GROUP BY a.code, a.account_type, a.name, a.parent
            ORDER BY
                CASE
                    WHEN LOWER(a.account_type) IN ('revenue','income') THEN 1
                    WHEN LOWER(a.account_type) = 'expense' THEN 2
                    ELSE 3
                END,
                a.code
        ");
        $stmt->execute([$startDate, $endDate]);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'Generate Income Statement Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * Generate Income Statement Report that ignores transactions linked to module_id = -1.
 */
function generateIncomeStatementReport($pdo, string $startDate, string $endDate): array
{
    try {
        $stmt = $pdo->prepare("
            SELECT
                a.account_type,
                a.code,
                a.name,
                a.parent,
                COALESCE(SUM(
                    CASE
                        WHEN LOWER(a.account_type) IN ('revenue', 'income') THEN
                            CASE
                                WHEN t.credit_account = a.code
                                     AND (t.journal_entry_id = 0 OR je.module_id IS NULL OR je.module_id != -1)
                                    THEN t.amount
                                WHEN t.debit_account = a.code
                                     AND (t.journal_entry_id = 0 OR je.module_id IS NULL OR je.module_id != -1)
                                    THEN -t.amount
                                ELSE 0
                            END
                        WHEN LOWER(a.account_type) = 'expense' THEN
                            CASE
                                WHEN t.debit_account = a.code
                                     AND (t.journal_entry_id = 0 OR je.module_id IS NULL OR je.module_id != -1)
                                    THEN t.amount
                                WHEN t.credit_account = a.code
                                     AND (t.journal_entry_id = 0 OR je.module_id IS NULL OR je.module_id != -1)
                                    THEN -t.amount
                                ELSE 0
                            END
                        ELSE 0
                    END
                ), 0) AS total_amount
            FROM accounts a
            LEFT JOIN transactions t ON (t.debit_account = a.code OR t.credit_account = a.code)
               AND t.transaction_date BETWEEN ? AND ?
               AND t.is_deleted = 0
            LEFT JOIN journal_entries je ON je.id = t.journal_entry_id
            WHERE LOWER(a.account_type) IN ('income','revenue','expense')
            GROUP BY a.code, a.account_type, a.name, a.parent
            ORDER BY
                CASE
                    WHEN LOWER(a.account_type) IN ('revenue','income') THEN 1
                    WHEN LOWER(a.account_type) = 'expense' THEN 2
                    ELSE 3
                END,
                a.code
        ");
        $stmt->execute([$startDate, $endDate]);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'Generate Income Statement Report Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * Generate Balance Sheet as of a given date
 */
function generateBalanceSheet($pdo, string $asOfDate): array
{
    try {
        $stmt = $pdo->prepare("
            SELECT
                a.account_type,
                a.code,
                a.name,
                a.parent,
                COALESCE(debits.total_debit, 0) as total_debit,
                COALESCE(credits.total_credit, 0) as total_credit,
                CASE
                    WHEN LOWER(a.account_type) = 'asset' THEN
                        COALESCE(debits.total_debit, 0) - COALESCE(credits.total_credit, 0)
                    WHEN LOWER(a.account_type) IN ('liability', 'equity') THEN
                        COALESCE(credits.total_credit, 0) - COALESCE(debits.total_debit, 0)
                    ELSE 0
                END as total_amount
            FROM accounts a
            LEFT JOIN (
                SELECT debit_account AS account_code, SUM(amount) as total_debit
                FROM transactions
                WHERE transaction_date <= ? AND is_deleted = 0
                GROUP BY debit_account
            ) debits ON a.code = debits.account_code
            LEFT JOIN (
                SELECT credit_account AS account_code, SUM(amount) as total_credit
                FROM transactions
                WHERE transaction_date <= ? AND is_deleted = 0
                GROUP BY credit_account
            ) credits ON a.code = credits.account_code
            WHERE LOWER(a.account_type) IN ('asset', 'liability', 'equity')
            ORDER BY
                CASE
                    WHEN LOWER(a.account_type) = 'asset' THEN 1
                    WHEN LOWER(a.account_type) = 'liability' THEN 2
                    WHEN LOWER(a.account_type) = 'equity' THEN 3
                    ELSE 4
                END,
                a.code ASC
        ");
        $stmt->execute([$asOfDate, $asOfDate]);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'Generate Balance Sheet Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * Generate Cash Flow Statement for the given date range
 */
function generateCashFlowStatement($pdo, string $startDate, string $endDate): array
{
    try {
        // Include journal_entry_id to filter out module_id = -1 if needed
        $stmt = $pdo->prepare("
            SELECT t.transaction_date, t.debit_account, t.credit_account, t.amount, t.description, t.journal_entry_id
            FROM transactions t
            WHERE t.transaction_date BETWEEN ? AND ?
              AND t.is_deleted = 0
            ORDER BY t.transaction_date ASC
        ");
        $stmt->execute([$startDate, $endDate]);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'Generate Cash Flow Statement Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * Generate Cash Flow Statement Report that ignores transactions linked to module_id = -1.
 */
function generateCashFlowStatementReport($pdo, string $startDate, string $endDate): array
{
    try {
        $stmt = $pdo->prepare("
            SELECT t.transaction_date, t.debit_account, t.credit_account, t.amount, t.description
            FROM transactions t
            LEFT JOIN journal_entries je ON je.id = t.journal_entry_id
            WHERE t.transaction_date BETWEEN ? AND ?
              AND t.is_deleted = 0
              AND (t.journal_entry_id = 0 OR je.module_id != -1)
            ORDER BY t.transaction_date ASC
        ");
        $stmt->execute([$startDate, $endDate]);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'Generate Cash Flow Statement Report Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * Generate VAT Report
 */
function generateVATReport($pdo, int $vatPeriodId = null, string $startDate = null, string $endDate = null): array
{
    try {
        // Replace hardcoded ['B75', 'B76'] with constants
        $vatAccounts = [VAT_PAYABLE_ACC_CODE, VAT_RECEIVABLE_ACC_CODE];

        if ($vatPeriodId !== null) {
            $vatPeriod = getVATPeriod($pdo, $vatPeriodId);
            if (empty($vatPeriod)) {
                return [];
            }
            $periodStart = $vatPeriod['start_date'];
            $periodEnd = $vatPeriod['end_date'];
        } elseif (!empty($startDate) && !empty($endDate)) {
            $periodStart = $startDate;
            $periodEnd = $endDate;
        } else {
            return [];
        }

        $placeholders = implode(',', array_fill(0, count($vatAccounts), '?'));
        $params = array_merge([$periodStart, $periodEnd], $vatAccounts);

        $stmt = $pdo->prepare("
            SELECT
                a.code as account_code,
                a.name as account_name,
                a.account_type,
                SUM(CASE WHEN t.debit_account = a.code THEN t.amount ELSE 0 END) as total_debit,
                SUM(CASE WHEN t.credit_account = a.code THEN t.amount ELSE 0 END) as total_credit,
                CASE
                    WHEN LOWER(a.account_type) = 'liability'
                         OR LOWER(a.account_type) = 'equity'
                         OR LOWER(a.account_type) = 'revenue' THEN
                        (SUM(CASE WHEN t.credit_account = a.code THEN t.amount ELSE 0 END)
                         - SUM(CASE WHEN t.debit_account = a.code THEN t.amount ELSE 0 END))
                    ELSE
                        (SUM(CASE WHEN t.debit_account = a.code THEN t.amount ELSE 0 END)
                         - SUM(CASE WHEN t.credit_account = a.code THEN t.amount ELSE 0 END))
                END as net_change
            FROM accounts a
            LEFT JOIN transactions t ON (t.debit_account = a.code OR t.credit_account = a.code)
              AND t.transaction_date BETWEEN ? AND ?
              AND COALESCE(t.meta, '') NOT LIKE '%\"vat_period_id\":%'
              AND t.is_deleted = 0
            WHERE a.code IN ($placeholders)
            GROUP BY a.code, a.name, a.account_type
            ORDER BY a.code
        ");
        $stmt->execute($params);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'Generate VAT Report Error: ' . $e->getMessage());
        return [];
    }
}

// Database connection for this file
try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    logError(null, 'Database Connection Error: ' . $e->getMessage());
}
