<?php
// core/initialization.php

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once(__DIR__ . '/../config/config.php');

/**
 * Initialize the DB schema *WITHOUT* forcibly creating the DB
 * if it already exists. This function is now called with parameters from install.php
 */
function initializeDatabase($adminUsername, $adminPassword, $adminEmail): bool
{
    try {
        // 1) Create or open the SQLite DB file:
        $pdo = new PDO('sqlite:' . DB_FILE);
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        // 2) Create the necessary tables & their default data
        $pdo->exec("CREATE TABLE IF NOT EXISTS account_types (
            type TEXT PRIMARY KEY
        );");

        // Insert data into account_types
        $pdo->exec("INSERT OR IGNORE INTO account_types (type) VALUES
            ('Asset'),
            ('Liability'),
            ('Equity'),
            ('Revenue'),
            ('Expense');");

        // Create accounts table
        $pdo->exec("CREATE TABLE IF NOT EXISTS accounts (
            code TEXT PRIMARY KEY,
            name TEXT NOT NULL,
            description TEXT,
            account_type TEXT NOT NULL,
            parent TEXT,
            is_system INTEGER DEFAULT 0,
            module_id INTEGER NOT NULL DEFAULT 0,
            FOREIGN KEY (account_type) REFERENCES account_types(type),
            FOREIGN KEY (parent) REFERENCES accounts(code) ON DELETE SET NULL
        );");

        // Create journal_entries table
        $pdo->exec("CREATE TABLE IF NOT EXISTS journal_entries (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            description TEXT NOT NULL,
            date DATE NOT NULL,
            financial_period_id INTEGER NOT NULL,
            created_by INTEGER NOT NULL,
            is_deleted TINYINT(1) NOT NULL DEFAULT 0,
            module_id INTEGER NOT NULL DEFAULT 0,
            FOREIGN KEY (financial_period_id) REFERENCES financial_periods(id),
            FOREIGN KEY (created_by) REFERENCES users(id)
        );");

        // Create transactions table
        $pdo->exec("CREATE TABLE IF NOT EXISTS transactions (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            journal_entry_id INTEGER NOT NULL,
            debit_account TEXT NOT NULL,
            credit_account TEXT NOT NULL,
            amount REAL NOT NULL,
            description TEXT,
            transaction_date DATE NOT NULL,
            period_id INTEGER NOT NULL,
            created_by INTEGER NOT NULL,
            is_deleted INTEGER NOT NULL DEFAULT 0,
            module_id INTEGER NOT NULL DEFAULT 0,
            vat_rates_id INTEGER DEFAULT 0,
            meta TEXT,
            FOREIGN KEY (journal_entry_id) REFERENCES journal_entries(id),
            FOREIGN KEY (debit_account) REFERENCES accounts(code),
            FOREIGN KEY (credit_account) REFERENCES accounts(code),
            FOREIGN KEY (period_id) REFERENCES financial_periods(id),
            FOREIGN KEY (created_by) REFERENCES users(id)
        );");

        // Create financial_periods table
        $pdo->exec("CREATE TABLE IF NOT EXISTS financial_periods (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            name TEXT NOT NULL,
            start_date DATE NOT NULL,
            end_date DATE NOT NULL,
            is_closed BOOLEAN DEFAULT 0,
            is_deleted BOOLEAN DEFAULT 0
        );");

        // Create vat_periods table
        $pdo->exec("CREATE TABLE IF NOT EXISTS vat_periods (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            name TEXT NOT NULL,
            start_date DATE NOT NULL,
            end_date DATE NOT NULL,
            status TEXT NOT NULL DEFAULT 'open',
            is_deleted INTEGER NOT NULL DEFAULT 0
        );");

        // Create vat_rates table
        $pdo->exec("CREATE TABLE IF NOT EXISTS vat_rates (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            rate_name TEXT NOT NULL UNIQUE,
            rate_value REAL NOT NULL,
            is_default INTEGER NOT NULL DEFAULT 0,
            is_deleted INTEGER NOT NULL DEFAULT 0
        );");

        // Create users table
        $pdo->exec("CREATE TABLE IF NOT EXISTS users (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            username TEXT NOT NULL UNIQUE,
            password_hash TEXT NOT NULL,
            email TEXT NOT NULL UNIQUE,
            is_auth INTEGER NOT NULL DEFAULT 0,
            is_deleted BOOLEAN DEFAULT 0,
            is_system INTEGER DEFAULT 0,
            is_reset INTEGER DEFAULT 0,
            created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
            updated_at DATETIME DEFAULT CURRENT_TIMESTAMP
        );");

        // Create roles table
        $pdo->exec("CREATE TABLE IF NOT EXISTS roles (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            name TEXT NOT NULL UNIQUE,
            is_system INTEGER DEFAULT 0
        );");

        // Insert roles data (if not exist)
        $pdo->exec("INSERT OR IGNORE INTO roles (id, name, is_system) VALUES
            (1, 'admin', 1),
            (2, 'accounts', 1),
            (3, 'accounts_add', 1),
            (4, 'accounts_edit', 1),
            (5, 'accounts_delete', 1),
            (6, 'accounts_import', 1),
            (7, 'accounts_export', 1),
            (8, 'users', 1),
            (9, 'users_add', 1),
            (10, 'users_edit', 1),
            (11, 'users_delete', 1),
            (12, 'users_reset', 1),
            (13, 'roles', 1),
            (14, 'roles_add', 1),
            (15, 'roles_delete', 1),
            (16, 'financial_period', 1),
            (17, 'financial_period_add', 1),
            (18, 'financial_period_update', 1),
            (19, 'financial_period_delete', 1),
            (20, 'financial_period_close', 1),
            (21, 'financial_period_reopen', 1),
            (22, 'vat_period', 1),
            (23, 'vat_period_add', 1),
            (24, 'vat_period_update', 1),
            (25, 'vat_period_delete', 1),
            (26, 'vat_period_close', 1),
            (27, 'vat_period_reopen', 1),
            (28, 'vat_rates', 1),
            (29, 'vat_rates_add', 1),
            (30, 'vat_rates_update', 1),
            (31, 'vat_rates_delete', 1),
            (32, 'vat_rates_default', 1),
            (33, 'audit_logs', 1),
            (34, 'audit_logs_add', 1),
            (35, 'audit_logs_delete', 1),
            (36, 'error_handling', 1),
            (37, 'error_handling_log', 1),
            (38, 'error_handling_delete', 1),
            (39, 'error_handling_handle', 1),
            (40, 'journal_entries', 1),
            (41, 'journal_entries_add', 1),
            (42, 'journal_entries_update', 1),
            (43, 'journal_entries_delete', 1),
            (44, 'transactions', 1),
            (45, 'balance_sheet', 1),
            (46, 'trial_balance', 1),
            (47, 'cash_flow_statement', 1),
            (48, 'income_statement', 1),
            (49, 'vat_report', 1),
            (50, 'header_menu_edit', 1),
            (51, 'document_editor', 1),
            (52, 'module_manager', 1),
            (53, 'module_install', 1),
            (54, 'module_activate', 1),
            (55, 'module_deactivate', 1),
            (56, 'module_delete', 1)
        ");

        // Create user_roles table
        $pdo->exec("CREATE TABLE IF NOT EXISTS user_roles (
            user_id INTEGER,
            role_id INTEGER,
            FOREIGN KEY (user_id) REFERENCES users(id) ON DELETE CASCADE,
            FOREIGN KEY (role_id) REFERENCES roles(id) ON DELETE CASCADE,
            PRIMARY KEY (user_id, role_id)
        );");

        // Create audit_logs table
        $pdo->exec("CREATE TABLE IF NOT EXISTS audit_logs (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            user_id INTEGER NOT NULL,
            action TEXT NOT NULL,
            entity TEXT NOT NULL,
            entity_id INTEGER NOT NULL,
            additional_info TEXT,
            is_system INTEGER DEFAULT 1,
            created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
            FOREIGN KEY (user_id) REFERENCES users(id)
        );");

        // Create errors table
        $pdo->exec("CREATE TABLE IF NOT EXISTS errors (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            message TEXT NOT NULL,
            context TEXT,
            error_level TEXT NOT NULL DEFAULT 'error',
            created_at DATETIME DEFAULT CURRENT_TIMESTAMP
        );");

        // Create menu_items table
        $pdo->exec("CREATE TABLE IF NOT EXISTS menu_items (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            name TEXT NOT NULL,
            url TEXT,
            parent_id INTEGER,
            order_num INTEGER DEFAULT 0,
            is_dropdown INTEGER DEFAULT 0,
            is_visible INTEGER DEFAULT 1,
            is_system INTEGER DEFAULT 0,
            required_role TEXT,
            FOREIGN KEY (parent_id) REFERENCES menu_items(id)
        );");

        // Insert or ignore default menu items
        $pdo->exec("INSERT OR IGNORE INTO menu_items (id, name, url, parent_id, order_num, is_dropdown, is_visible, is_system, required_role) VALUES
            (1, 'Dashboard', 'index.php', NULL, 1, 0, 1,1, NULL),
            (2, 'Accounting', NULL, NULL, 2, 1, 1,0, NULL),
            (3, 'Accounts', 'accounts.php', 2, 1, 0, 1,1, 'accounts'),
            (4, 'Journal Entries', 'journal_entries.php', 2, 2, 0, 1,1, 'journal_entries'),
            (5, 'Transactions', 'transactions.php', 2, 3, 0, 1,1, 'transactions'),
            (6, 'Financial Periods', 'financial_periods.php', 2, 4, 0, 1,1, 'financial_period'),
            (7, 'VAT', NULL, NULL, 3, 1, 1,0, NULL),
            (8, 'VAT Rates', 'vat_rates.php', 7, 1, 0, 1,1, 'vat_rates'),
            (9, 'VAT Periods', 'vat_periods.php', 7, 2, 0, 1,1, 'vat_period'),
            (10, 'Reports', NULL, NULL, 4, 1, 1,0, NULL),
            (11, 'Balance Sheet', 'balancesheet.php', 10, 1, 0, 1,1, 'balance_sheet'),
            (12, 'Trial Balance', 'trialbalance.php', 10, 2, 0, 1,1, 'trial_balance'),
            (13, 'Income Statement', 'incomestatement.php', 10, 3, 0, 1,1, 'income_statement'),
            (14, 'Cash Flow Statement', 'cashflowstatement.php', 10, 4, 0, 1,1, 'cash_flow_statement'),
            (15, 'VAT Report', 'vatreport.php', 10, 5, 0, 1,1, 'vat_report'),
            (16, 'Administrator', NULL, NULL, 5, 1, 1,0, NULL),
            (17, 'Users', 'users.php', 16, 1, 0, 1,1, 'users'),
            (18, 'Menu Builder', 'menu_builder.php', 16, 2, 0, 1,1, 'users'),
            (19, 'Audit Logs', 'audit_logs.php', 16, 3, 0, 1,1, 'audit_logs'),
            (20, 'Errors', 'error_handling.php', 16, 4, 0, 1,1, 'error_handling'),
            (21, 'Document Editor', 'document_editor.php', 16, 5, 0, 1,1, 'document_editor'),
            (22, 'Profile', NULL, NULL, 999, 1, 1,0, NULL),
            (23, 'Logout', 'logout.php', 22, 3, 0, 1,1, NULL),
            (24, 'Module Manager', 'module_manager.php', 16, 6, 0, 1,1, 'module_install');
        ");

        // Create modules table
        $pdo->exec("CREATE TABLE IF NOT EXISTS modules (
            module_name TEXT NOT NULL UNIQUE,
            menu_item_ids TEXT NOT NULL,
            redirect_transaction TEXT DEFAULT '',
            redirect_journal_entries TEXT DEFAULT '',
            redirect_accounts TEXT DEFAULT ''
        );");

        // Create default system accounts
        $defaultAccounts = [
            // Assets (1xxx)
            ['code' => '1000', 'name' => 'Assets', 'description' => 'Assets Account', 'account_type' => 'Asset', 'parent' => null],
            ['code' => '1100', 'name' => 'Current Assets', 'description' => 'Current Assets Category', 'account_type' => 'Asset', 'parent' => '1000'],
            ['code' => '1110', 'name' => 'Inventory', 'description' => 'Goods and materials held for resale.', 'account_type' => 'Asset', 'parent' => '1100'],
            ['code' => '1120', 'name' => 'Accounts Receivable', 'description' => 'Money owed to the business by its clients.', 'account_type' => 'Asset', 'parent' => '1100'],
            ['code' => '1130', 'name' => 'Cash/Bank Account', 'description' => 'Records of all cash and bank accounts.', 'account_type' => 'Asset', 'parent' => '1100'],
            ['code' => '1200', 'name' => 'Non-current Assets', 'description' => 'Non-current Assets Category', 'account_type' => 'Asset', 'parent' => '1000'],
            ['code' => '1210', 'name' => 'Fixed Assets', 'description' => 'Fixed Assets', 'account_type' => 'Asset', 'parent' => '1200'],
            ['code' => '1220', 'name' => 'Investments', 'description' => 'Investments', 'account_type' => 'Asset', 'parent' => '1200'],
            ['code' => '1230', 'name' => 'Other Fixed Assets', 'description' => 'Other Fixed Assets', 'account_type' => 'Asset', 'parent' => '1200'],
            ['code' => '1300', 'name' => 'Unallocated Account/Temporary Account', 'description' => 'Unallocated or Temporary Account', 'account_type' => 'Asset', 'parent' => '1000'],

            // Liabilities (2xxx)
            ['code' => '2000', 'name' => 'Liabilities', 'description' => 'Liabilities Account', 'account_type' => 'Liability', 'parent' => null],
            ['code' => '2100', 'name' => 'Current Liabilities', 'description' => 'Current Liabilities Category', 'account_type' => 'Liability', 'parent' => '2000'],
            ['code' => '2110', 'name' => 'Accounts Payable', 'description' => 'Money owed by the business to its suppliers.', 'account_type' => 'Liability', 'parent' => '2100'],
            ['code' => '2120', 'name' => 'VAT Control Account', 'description' => 'A liability account that records the net VAT position...', 'account_type' => 'Liability', 'parent' => '2100'],
            ['code' => '2121', 'name' => 'VAT Payable CLIENT', 'description' => 'VAT collected from customers', 'account_type' => 'Liability', 'parent' => '2120'],
            ['code' => '2122', 'name' => 'VAT Receivable SUP', 'description' => 'VAT refundable from tax authorities', 'account_type' => 'Liability', 'parent' => '2120'],
            ['code' => '2200', 'name' => 'Non-current Liabilities', 'description' => 'Non-Current Liabilities Category', 'account_type' => 'Liability', 'parent' => '2000'],
            ['code' => '2210', 'name' => 'Long Term Borrowings', 'description' => 'Long Term Borrowings', 'account_type' => 'Liability', 'parent' => '2200'],
            ['code' => '2220', 'name' => 'Other Long Term Liabilities', 'description' => 'Other Long Term Liabilities', 'account_type' => 'Liability', 'parent' => '2200'],

            // Equity (3xxx)
            ['code' => '3000', 'name' => 'Equity', 'description' => 'Equity Account', 'account_type' => 'Equity', 'parent' => null],
            ['code' => '3100', 'name' => 'Capital Contribution', 'description' => 'Capital Contribution Account', 'account_type' => 'Equity', 'parent' => '3000'],
            ['code' => '3200', 'name' => 'Retained Earnings', 'description' => 'Retained Earnings Account', 'account_type' => 'Equity', 'parent' => '3000'],
            ['code' => '3250', 'name' => 'Net Income', 'description' => 'Net Income Account', 'account_type' => 'Equity', 'parent' => '3200'],
            ['code' => '3300', 'name' => 'Shareholders Loans', 'description' => 'Shareholders Loans Account', 'account_type' => 'Equity', 'parent' => '3000'],
            ['code' => '3400', 'name' => 'Withdraw', 'description' => 'Withdraw Account', 'account_type' => 'Equity', 'parent' => '3000'],

            // Revenue (4xxx)
            ['code' => '4000', 'name' => 'Income', 'description' => 'Income Account', 'account_type' => 'Revenue', 'parent' => null],
            ['code' => '4100', 'name' => 'Sales', 'description' => 'Sales', 'account_type' => 'Revenue', 'parent' => '4000'],
            ['code' => '4200', 'name' => 'Other Income', 'description' => 'Other Income', 'account_type' => 'Revenue', 'parent' => '4000'],

            // Expenses (5xxx)
            ['code' => '5000', 'name' => 'Expenses', 'description' => 'Expenses Parent Account', 'account_type' => 'Expense', 'parent' => null],
            ['code' => '5100', 'name' => 'Cost of Sales', 'description' => 'Cost of Sales accounts', 'account_type' => 'Expense', 'parent' => '5000'],
            ['code' => '5200', 'name' => 'General Expenses', 'description' => 'General Expenses accounts.', 'account_type' => 'Expense', 'parent' => '5000']
        ];

        // Insert default accounts with is_system=1
        $stmtAcc = $pdo->prepare("INSERT OR IGNORE INTO accounts (code, name, description, account_type, parent, is_system) VALUES (?, ?, ?, ?, ?, 1);");
        foreach ($defaultAccounts as $acc) {
            $stmtAcc->execute([$acc['code'], $acc['name'], $acc['description'], $acc['account_type'], $acc['parent']]);
        }

        // 5) Create license_info table
        $pdo->exec("CREATE TABLE IF NOT EXISTS license_info (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            unique_registration_number TEXT NOT NULL,
            license_type TEXT NOT NULL DEFAULT 'unregistered',
            start_date DATE DEFAULT NULL,
            end_date DATE DEFAULT NULL,
            registration_code TEXT DEFAULT NULL
        );");

        // If no license_info row yet, insert one with license_type='unregistered'
        $stmtLicense = $pdo->query("SELECT COUNT(*) FROM license_info");
        $countLicense = (int)$stmtLicense->fetchColumn();
        if ($countLicense === 0) {
            $insertLicStmt = $pdo->prepare("
                INSERT INTO license_info (unique_registration_number, license_type)
                VALUES ('', 'unregistered')
            ");
            $insertLicStmt->execute();
        }

        // Add new columns if not exist
        $pdo->exec("ALTER TABLE license_info ADD COLUMN name_or_company TEXT DEFAULT ''");
        $pdo->exec("ALTER TABLE license_info ADD COLUMN email TEXT DEFAULT ''");
        $pdo->exec("ALTER TABLE license_info ADD COLUMN pay_date DATE DEFAULT NULL");

        // 6) Insert admin user
        $hashedPass = password_hash($adminPassword, PASSWORD_BCRYPT);
        $pdo->exec("INSERT OR IGNORE INTO users (username, password_hash, email, is_auth, is_deleted, is_system)
                    VALUES ('$adminUsername', '$hashedPass', '$adminEmail', 1, 0, 1)");
        $checkUser = $pdo->prepare("SELECT id FROM users WHERE username = :u");
        $checkUser->execute([':u' => $adminUsername]);
        $userRow = $checkUser->fetch(PDO::FETCH_ASSOC);
        if ($userRow) {
            $stmtUpd = $pdo->prepare("UPDATE users SET password_hash=?, email=?, is_auth=1, is_system=1 WHERE id=?");
            $stmtUpd->execute([$hashedPass, $adminEmail, $userRow['id']]);

            // Grant admin role if not already
            $stmtUR = $pdo->prepare("INSERT OR IGNORE INTO user_roles (user_id, role_id) VALUES (?,1)");
            $stmtUR->execute([$userRow['id']]);
        }

        return true;
    } catch (PDOException $e) {
        echo 'Database Initialization Error: ' . $e->getMessage();
        return false;
    }
}
