<?php
// public/install.php
// Basic Installation Script
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once(__DIR__ . '/../core/initialization.php'); // has initializeDatabase()
require_once(__DIR__ . '/../core/users.php'); // for createUser or updateUser
require_once(__DIR__ . '/../core/error_handling.php');
require_once(__DIR__ . '/../config/config.php'); // DB_FILE, etc.

// Check if DB already exists
if (file_exists(__DIR__ . '/../data/accounting_system.sqlite')) {
    echo "<h2>Database already exists. Installation not needed.</h2>";
    exit;
}

// If no DB file, run initialization
if (!initializeDatabase()) {
    die("<p>Failed to initialize database. Check logs or permissions.</p>");
}

// Create or read local unique app id
$localAppIdFile = __DIR__ . '/../data/local_app_id.txt';
if (!file_exists($localAppIdFile)) {
    $localAppId = bin2hex(random_bytes(8)); // random 8 bytes
    file_put_contents($localAppIdFile, $localAppId);
} else {
    $localAppId = trim(file_get_contents($localAppIdFile));
}

// Process form
$errors = [];
$successMsg = '';

if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['install_submit'])) {
    $choose          = $_POST['user_choice'] ?? 'new';
    $masterUser      = trim($_POST['master_username'] ?? '');
    $masterPass      = trim($_POST['master_password'] ?? '');
    $masterPassConf  = trim($_POST['master_password_conf'] ?? '');
    $masterEmail     = trim($_POST['master_email'] ?? '');

    if (!$masterUser || !$masterPass || !$masterPassConf || !$masterEmail) {
        $errors[] = "All fields are required.";
    } elseif ($masterPass !== $masterPassConf) {
        $errors[] = "Passwords do not match.";
    } else {
        // Connect to DB
        try {
            $pdo = new PDO('sqlite:' . DB_FILE);
            $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
        } catch (PDOException $e) {
            $errors[] = "Database connect error: " . $e->getMessage();
        }
        if (empty($errors)) {
            // Check if an admin user row already exists
            $adminRow = getUserByUsername($pdo, 'Admin');
            // We'll update or re-create the Admin user
            $hashed   = password_hash($masterPass, PASSWORD_BCRYPT);
            if ($adminRow) {
                // Update existing Admin row
                $stmt = $pdo->prepare("UPDATE users SET username=?, password_hash=?, email=? WHERE id=?");
                $stmt->execute([$masterUser, $hashed, $masterEmail, $adminRow['id']]);
            } else {
                // create new user
                $createResult = createUser($pdo, $masterUser, $masterPass, $masterEmail, 1); // admin user = 1
                // optionally assign admin role
            }
            // License logic
            $uniqueRegNumber = '';
            if ($choose === 'new') {
                // Try to get a unique reg number from donnotec.com
                // If no net, use 'DONNO-defaultregkey'
                $checkUrl = 'https://donnotec.com/request_new_reg_number.php'; // hypothetical
                $opts = [
                    'http' => ['timeout' => 5]
                ];
                $context = stream_context_create($opts);
                $res = @file_get_contents($checkUrl, false, $context);
                if ($res !== false && strpos($res, 'SUCCESS:') !== false) {
                    // Suppose it returns "SUCCESS: DONNO-xxxxx"
                    $uniqueRegNumber = trim(str_replace('SUCCESS:', '', $res));
                } else {
                    $uniqueRegNumber = 'DONNO-defaultregkey';
                }
            } else {
                // Existing user
                // We ask them for existing unique registration number in the form, or do we?
                $uniqueRegNumber = trim($_POST['existing_reg_number'] ?? 'DONNO-defaultregkey');
                // Possibly also send localAppId to donnotec.com so it can store
                // ...
            }

            // Store or update in local license_info
            $stmtLic = $pdo->prepare("UPDATE license_info SET unique_registration_number=? WHERE id=1");
            $stmtLic->execute([$uniqueRegNumber]);

            $successMsg = "Installation completed! <br>Local App ID: $localAppId <br>Unique Reg Number: $uniqueRegNumber";
        }
    }
}
?>
<!DOCTYPE html>
<html>
<head>
    <meta charset="utf-8">
    <title>DONNOTEC Installation</title>
</head>
<body>
<h1>DONNOTEC Installation</h1>
<?php if (!empty($errors)): ?>
    <div style="color:red;">
    <ul>
    <?php foreach($errors as $e) echo "<li>".htmlspecialchars($e)."</li>"; ?>
    </ul>
    </div>
<?php endif; ?>
<?php if (!empty($successMsg)): ?>
    <div style="color:green;"><?php echo $successMsg; ?></div>
<?php else: ?>
<p>Welcome! Please configure your master (admin) user:</p>
<form method="POST" action="install.php">
    <label>
        Are you a new or existing user?
        <select name="user_choice">
            <option value="new">New user</option>
            <option value="existing">Existing user</option>
        </select>
    </label>
    <br><br>
    <label for="master_username">Master Username:</label>
    <input type="text" name="master_username" id="master_username" required><br><br>

    <label for="master_password">Master Password:</label>
    <input type="password" name="master_password" id="master_password" required><br><br>

    <label for="master_password_conf">Confirm Password:</label>
    <input type="password" name="master_password_conf" id="master_password_conf" required><br><br>

    <label for="master_email">Master Email:</label>
    <input type="email" name="master_email" id="master_email" required><br><br>

    <!-- If 'existing' user, ask for existing reg number -->
    <label for="existing_reg_number">Existing Unique Registration Number (only if you chose Existing user):</label>
    <input type="text" name="existing_reg_number" id="existing_reg_number">

    <br><br>
    <button type="submit" name="install_submit" value="1">Install</button>
</form>
<?php endif; ?>
</body>
</html>

