<?php
require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/accounts.php');
require_once(__DIR__ . '/transactions.php');
require_once(__DIR__ . '/audit_logs.php');
require_once(__DIR__ . '/error_handling.php');

// We need these function references to check closed periods (like in transactions.php)
require_once(__DIR__ . '/financial_periods.php');
require_once(__DIR__ . '/vat_periods.php');      // or a function that can check 'isDateInClosedVATPeriod(...)'
// Or if you rely on the one inside transactions.php, ensure we can call it:
require_once(__DIR__ . '/transactions.php'); // for isDateInClosedVATPeriod

function createJournalEntry($pdo, string $description, string $date, int $financialPeriodId, int $userId): int
{
    try {
        $stmt = $pdo->prepare("INSERT INTO journal_entries (description, date, financial_period_id, created_by) VALUES (?, ?, ?, ?);");
        $stmt->execute([$description, $date, $financialPeriodId, $userId]);
        $journalEntryId = (int)$pdo->lastInsertId();

        // Audit log
        createAuditLog($pdo, $userId, 'create', 'journal_entries', $journalEntryId, "Created Journal Entry: $description");

        return $journalEntryId;
    } catch (PDOException $e) {
        logError($pdo, 'Create Journal Entry Error: ' . $e->getMessage());
        return 0;
    }
}

function getJournalEntry($pdo, int $journalEntryId): array
{
    try {
        $stmt = $pdo->prepare("SELECT * FROM journal_entries WHERE id = ?;");
        $stmt->execute([$journalEntryId]);
        return $stmt->fetch(PDO::FETCH_ASSOC) ?: [];
    } catch (PDOException $e) {
        logError($pdo, 'Get Journal Entry Error: ' . $e->getMessage());
        return [];
    }
}

function listJournalEntries($pdo, array $filters = []): array
{
    try {
        $query = "SELECT * FROM journal_entries WHERE is_deleted = 0";
        $params = [];
        if (isset($filters['financial_period_id'])) {
            $query .= " AND financial_period_id = ?";
            $params[] = $filters['financial_period_id'];
        }
        if (isset($filters['date_from'])) {
            $query .= " AND date >= ?";
            $params[] = $filters['date_from'];
        }
        if (isset($filters['date_to'])) {
            $query .= " AND date <= ?";
            $params[] = $filters['date_to'];
        }
        $query .= " ORDER BY date DESC";
        $stmt = $pdo->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'List Journal Entries Error: ' . $e->getMessage());
        return [];
    }
}

function updateJournalEntry($pdo, int $journalEntryId, string $description, string $date, int $userId): bool
{
    try {
        // 1) Check if the financial period is closed
        $entry = getJournalEntry($pdo, $journalEntryId);
        if (empty($entry)) {
            throw new InvalidArgumentException('Journal entry not found.');
        }
        $periodId = (int)$entry['financial_period_id'];
        if (isFinancialPeriodClosed($pdo, $periodId)) {
            throw new InvalidArgumentException('Cannot update a journal entry in a closed financial period.');
        }

        // 2) Check if new date is in a closed VAT period
        if (isDateInClosedVATPeriod($pdo, $date)) {
            throw new InvalidArgumentException('Cannot update a journal entry to a date in a closed VAT period.');
        }

        $stmt = $pdo->prepare("UPDATE journal_entries SET description = ?, date = ? WHERE id = ?;");
        $stmt->execute([$description, $date, $journalEntryId]);

        // Audit log
        createAuditLog($pdo, $userId, 'update', 'journal_entries', $journalEntryId, "Updated Journal Entry: $description");

        return true;
    } catch (InvalidArgumentException $e) {
        logError($pdo, 'Validation Error in Update Journal Entry: ' . $e->getMessage());
        return false;
    } catch (PDOException $e) {
        logError($pdo, 'Update Journal Entry Error: ' . $e->getMessage());
        return false;
    }
}

function deleteJournalEntry($pdo, int $journalEntryId, int $userId): bool
{
    try {
        // Get transactions associated with this journal entry
        $result = listTransactions($pdo, ['journal_entry_id' => $journalEntryId]);
        $activeTransactions = $result['transactions'] ?? [];

        // Filter out any deleted transactions and check if any remain
        $activeTransactions = array_filter($activeTransactions, function($transaction) {
            return isset($transaction['is_deleted']) && $transaction['is_deleted'] == 0;
        });

        if (count($activeTransactions) > 0) {
            // Cannot delete the journal entry if it still has active (non-deleted) transactions
            return false;
        }

        // 3) Additional check for period or VAT closure
        $entry = getJournalEntry($pdo, $journalEntryId);
        if (empty($entry)) {
            throw new InvalidArgumentException('Journal entry not found.');
        }
        $periodId = (int)$entry['financial_period_id'];
        if (isFinancialPeriodClosed($pdo, $periodId)) {
            throw new InvalidArgumentException('Cannot delete a journal entry in a closed financial period.');
        }
        if (isDateInClosedVATPeriod($pdo, $entry['date'])) {
            throw new InvalidArgumentException('Cannot delete a journal entry whose date is in a closed VAT period.');
        }

        $stmt = $pdo->prepare("UPDATE journal_entries SET is_deleted = 1 WHERE id = ?;");
        $stmt->execute([$journalEntryId]);

        // Audit log
        createAuditLog($pdo, $userId, 'delete', 'journal_entries', $journalEntryId, "Deleted Journal Entry ID: $journalEntryId");

        return true;
    } catch (InvalidArgumentException $e) {
        logError($pdo, 'Validation Error in Delete Journal Entry: ' . $e->getMessage());
        return false;
    } catch (PDOException $e) {
        logError($pdo, 'Delete Journal Entry Error: ' . $e->getMessage());
        return false;
    }
}

// Initialize PDO connection
try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    logError(null, 'Database Connection Error: ' . $e->getMessage());
}
