<?php
require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/journal_entries.php');
require_once(__DIR__ . '/accounts.php');
require_once(__DIR__ . '/audit_logs.php');
require_once(__DIR__ . '/error_handling.php');
require_once(__DIR__ . '/vat_periods.php'); // Ensure we can query VAT periods

/**
 * Create a new transaction record.
 *
 * Updated: Removed the restriction that $amount must be > 0 (now 0 is allowed).
 */
function createTransaction(
    $pdo,
    int $journalEntryId,
    string $debitAccount,
    string $creditAccount,
    float $amount,
    string $description,
    string $transactionDate,
    int $periodId,
    int $userId,
    int $moduleId = 0,    // Optional module ID, defaults to 0
    int $vatRatesId = 0,   // Optional VAT rate ID, defaults to 0
    string $meta = ''
): int {
    try {
        // Convert null to 0
        // Input Validation
        if (empty($debitAccount) || empty($creditAccount) || empty($transactionDate) || empty($periodId) || empty($userId)) {
            $_SESSION['transaction_error'] = 'Missing required transaction fields.';
            throw new InvalidArgumentException('Missing required transaction fields.');
        }

        // Debit/Credit accounts cannot be the same
        if ($debitAccount === $creditAccount) {
            $_SESSION['transaction_error'] = 'Debit and Credit accounts cannot be the same.';
            throw new InvalidArgumentException('Debit and Credit accounts cannot be the same.');
        }

        // Validate Accounts
        $debitAccountData = getAccount($pdo, $debitAccount);
        $creditAccountData = getAccount($pdo, $creditAccount);
        if (empty($debitAccountData) || empty($creditAccountData)) {
            $_SESSION['transaction_error'] = 'Invalid debit or credit account.';
            throw new InvalidArgumentException('Invalid debit or credit account.');
        }

        // Validate Journal Entry
        if (empty($journalEntryId)) {
            $journalEntryId = 0;
        }

        // Validate Financial Period
        if (isFinancialPeriodClosed($pdo, $periodId)) {
            $_SESSION['transaction_error'] = 'The financial period is closed.';
            throw new InvalidArgumentException('The financial period is closed.');
        }

        if (isDateInClosedVATPeriod($pdo, $transactionDate)) {
            $_SESSION['transaction_error'] = 'Could not add transaction because the VAT period is closed.';
            throw new InvalidArgumentException('Could not add transaction because the VAT period is closed.');
        }

        // Additional check: if vatRatesId != 0, then date must lie in an open VAT period
        if ($vatRatesId != 0) {
            if (!isDateInOpenVATPeriod($pdo, $transactionDate)) {
                $_SESSION['transaction_error'] = 'No open VAT period covers this transaction date.';
                throw new InvalidArgumentException('Cannot add transaction with VAT to a date not in an open VAT period.');
            }
        }

        // Insert transaction (adding module_id and vat_rates_id)
        $stmt = $pdo->prepare("
            INSERT INTO transactions (
                journal_entry_id,
                debit_account,
                credit_account,
                amount,
                description,
                transaction_date,
                period_id,
                created_by,
                module_id,
                vat_rates_id,
                meta
            ) VALUES (
                ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?
            );
        ");
        $stmt->execute([
            $journalEntryId,
            $debitAccount,
            $creditAccount,
            $amount,
            $description,
            $transactionDate,
            $periodId,
            $userId,
            $moduleId,
            $vatRatesId,
            $meta
        ]);
        $transactionId = (int)$pdo->lastInsertId();

        // Audit log
        createAuditLog($pdo, $userId, 'create', 'transactions', $transactionId, "Created Transaction: $description");

        return $transactionId;
    } catch (InvalidArgumentException $e) {
        logError($pdo, 'Validation Error in Create Transaction: ' . $e->getMessage());
        return 0;
    } catch (PDOException $e) {
        logError($pdo, 'Create Transaction Error: ' . $e->getMessage());
        return 0;
    }
}

/**
 * Fetch a single transaction by ID.
 */
function getTransaction($pdo, int $transactionId): array {
    try {
        $stmt = $pdo->prepare("SELECT * FROM transactions WHERE id = ? AND is_deleted = 0;");
        $stmt->execute([$transactionId]);
        return $stmt->fetch(PDO::FETCH_ASSOC) ?: [];
    } catch (PDOException $e) {
        logError($pdo, 'Get Transaction Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * List transactions with optional filters + pagination.
 */
function listTransactions($pdo, array $filters = [], int $page = 1, int $pageSize = 5): array {
    try {
        $offset = ($page - 1) * $pageSize;
        $query = "SELECT * FROM transactions WHERE is_deleted = 0";
        $params = [];

        // Filters
        if (isset($filters['journal_entry_id'])) {
            $query .= " AND journal_entry_id = ?";
            $params[] = $filters['journal_entry_id'];
        }
        if (isset($filters['debit_account'])) {
            $query .= " AND debit_account = ?";
            $params[] = $filters['debit_account'];
        }
        if (isset($filters['credit_account'])) {
            $query .= " AND credit_account = ?";
            $params[] = $filters['credit_account'];
        }
        if (isset($filters['transaction_date'])) {
            $query .= " AND transaction_date = ?";
            $params[] = $filters['transaction_date'];
        }
        if (isset($filters['start_date']) && isset($filters['end_date'])) {
            $query .= " AND transaction_date BETWEEN ? AND ?";
            $params[] = $filters['start_date'];
            $params[] = $filters['end_date'];
        }
        if (isset($filters['description'])) {
            $query .= " AND description LIKE ?";
            $params[] = '%' . $filters['description'] . '%';
        }
        if (isset($filters['amount_min']) && isset($filters['amount_max'])) {
            $query .= " AND amount BETWEEN ? AND ?";
            $params[] = $filters['amount_min'];
            $params[] = $filters['amount_max'];
        }

        // Pagination
        $query .= " ORDER BY transaction_date DESC LIMIT ? OFFSET ?";
        $params[] = $pageSize;
        $params[] = $offset;

        $stmt = $pdo->prepare($query);
        $stmt->execute($params);

        $transactions = $stmt->fetchAll(PDO::FETCH_ASSOC);

        // Get total count for pagination
        $countQuery = "SELECT COUNT(*) FROM transactions WHERE is_deleted = 0";
        $countParams = [];

        // Same filters for count
        if (isset($filters['journal_entry_id'])) {
            $countQuery .= " AND journal_entry_id = ?";
            $countParams[] = $filters['journal_entry_id'];
        }
        if (isset($filters['debit_account'])) {
            $countQuery .= " AND debit_account = ?";
            $countParams[] = $filters['debit_account'];
        }
        if (isset($filters['credit_account'])) {
            $countQuery .= " AND credit_account = ?";
            $countParams[] = $filters['credit_account'];
        }
        if (isset($filters['transaction_date'])) {
            $countQuery .= " AND transaction_date = ?";
            $countParams[] = $filters['transaction_date'];
        }
        if (isset($filters['start_date']) && isset($filters['end_date'])) {
            $countQuery .= " AND transaction_date BETWEEN ? AND ?";
            $countParams[] = $filters['start_date'];
            $countParams[] = $filters['end_date'];
        }
        if (isset($filters['description'])) {
            $countQuery .= " AND description LIKE ?";
            $countParams[] = '%' . $filters['description'] . '%';
        }
        if (isset($filters['amount_min']) && isset($filters['amount_max'])) {
            $countQuery .= " AND amount BETWEEN ? AND ?";
            $countParams[] = $filters['amount_min'];
            $countParams[] = $filters['amount_max'];
        }

        $countStmt = $pdo->prepare($countQuery);
        $countStmt->execute($countParams);
        $totalCount = (int)$countStmt->fetchColumn();

        return [
            'transactions' => $transactions,
            'total_count'  => $totalCount,
            'current_page' => $page,
            'page_size'    => $pageSize,
            'total_pages'  => ceil($totalCount / $pageSize)
        ];
    } catch (PDOException $e) {
        logError($pdo, 'List Transactions Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * Update an existing transaction record.
 */
function updateTransaction($pdo, int $transactionId, array $data, int $userId): bool {
    try {
        // Input Validation
        if (empty($transactionId) || empty($data)) {
            throw new InvalidArgumentException('Transaction ID and data are required.');
        }

        $transaction = getTransaction($pdo, $transactionId);
        if (empty($transaction)) {
            throw new InvalidArgumentException('Transaction not found.');
        }

        // Prepare update fields
        $fields = [];
        $params = [];

        if (isset($data['description'])) {
            $fields[] = 'description = ?';
            $params[] = $data['description'];
        }
        if (isset($data['amount'])) {
            // Removed the check forbidding 0 or negative amounts
            $fields[] = 'amount = ?';
            $params[] = $data['amount'];
        }
        if (isset($data['transaction_date'])) {
            $fields[] = 'transaction_date = ?';
            $params[] = $data['transaction_date'];
        }

        if (empty($fields)) {
            throw new InvalidArgumentException('No valid fields to update.');
        }

        $params[] = $transactionId;

        $stmt = $pdo->prepare("UPDATE transactions SET " . implode(', ', $fields) . " WHERE id = ?;");
        $stmt->execute($params);

        // Audit log
        createAuditLog($pdo, $userId, 'update', 'transactions', $transactionId, "Updated Transaction ID: $transactionId");

        return true;
    } catch (InvalidArgumentException $e) {
        logError($pdo, 'Validation Error in Update Transaction: ' . $e->getMessage());
        return false;
    } catch (PDOException $e) {
        logError($pdo, 'Update Transaction Error: ' . $e->getMessage());
        return false;
    }
}

/**
 * Soft-deletes a transaction record.
 */
function deleteTransaction($pdo, int $transactionId, int $userId): bool {
    try {
        $transaction = getTransaction($pdo, $transactionId);
        if (empty($transaction)) {
            throw new InvalidArgumentException('Transaction not found.');
        }

        // Check if financial period is closed
        if (isFinancialPeriodClosed($pdo, $transaction['period_id'])) {
            throw new InvalidArgumentException('Cannot delete transaction in a closed financial period.');
        }

        // Check if the transaction date is in a closed VAT period
        if (isDateInClosedVATPeriod($pdo, $transaction['transaction_date'])) {
            throw new InvalidArgumentException('Cannot delete transaction in a closed VAT period.');
        }
        $stmt = $pdo->prepare("UPDATE transactions SET is_deleted = 1 WHERE id = ?;");
        $stmt->execute([$transactionId]);

        // Audit log
        createAuditLog($pdo, $userId, 'delete', 'transactions', $transactionId, "Deleted Transaction ID: $transactionId");

        return true;
    } catch (InvalidArgumentException $e) {
        logError($pdo, 'Validation Error in Delete Transaction: ' . $e->getMessage());
        return false;
    } catch (PDOException $e) {
        logError($pdo, 'Delete Transaction Error: ' . $e->getMessage());
        return false;
    }
}

function isDateInClosedVATPeriod(PDO $pdo, string $date): bool
{
    $stmt = $pdo->prepare("
        SELECT COUNT(*) FROM vat_periods
        WHERE is_deleted = 0
          AND status = 'closed'
          AND start_date <= ?
          AND end_date >= ?
        LIMIT 1
    ");
    $stmt->execute([$date, $date]);
    return ((int)$stmt->fetchColumn() > 0);
}

/**
 * Check if the given date lies in an OPEN VAT period
 */
function isDateInOpenVATPeriod(PDO $pdo, string $date): bool
{
    $stmt = $pdo->prepare("
        SELECT COUNT(*) FROM vat_periods
        WHERE is_deleted = 0
          AND status = 'open'
          AND start_date <= ?
          AND end_date >= ?
        LIMIT 1
    ");
    $stmt->execute([$date, $date]);
    return ((int)$stmt->fetchColumn() > 0);
}

/**
 * Return an array of open date ranges (YYYY-MM-DD) where:
 *   - The financial period is open,
 *   - And NOT inside any closed VAT period.
 * Example return value:
 * [
 *   ['start' => '2024-01-01', 'end' => '2024-07-31'],
 *   ['start' => '2024-10-01', 'end' => '2024-12-31'],
 *   ...
 * ]
 */
function getAllowedTransactionDateRanges(PDO $pdo): array
{
    // 1) Fetch all open financial periods
    $stmt = $pdo->query("SELECT start_date, end_date FROM financial_periods
                         WHERE is_deleted = 0 AND is_closed = 0
                         ORDER BY start_date ASC;");
    $openFinPeriods = $stmt->fetchAll(PDO::FETCH_ASSOC);

    // 2) Fetch all CLOSED VAT periods
    $stmt2 = $pdo->query("SELECT start_date, end_date FROM vat_periods
                          WHERE is_deleted = 0 AND status = 'closed'
                          ORDER BY start_date ASC;");
    $closedVatPeriods = $stmt2->fetchAll(PDO::FETCH_ASSOC);

    // Start with the open financial periods as our "allowed" ranges
    $allowed = [];
    foreach ($openFinPeriods as $f) {
        $allowed[] = [
            'start' => $f['start_date'],
            'end'   => $f['end_date']
        ];
    }

    // 3) Subtract out each closed VAT period from our $allowed ranges
    // We remove any overlap with [closedVatStart, closedVatEnd].
    foreach ($closedVatPeriods as $cvat) {
        $closedStart = $cvat['start_date'];
        $closedEnd   = $cvat['end_date'];

        $newAllowed = [];
        foreach ($allowed as $r) {
            $rStart = $r['start'];
            $rEnd   = $r['end'];

            // If no overlap, keep entire range
            if ($closedEnd < $rStart || $closedStart > $rEnd) {
                $newAllowed[] = $r;
                continue;
            }

            // Overlap => we may split into up to two sub-ranges
            // left side: from rStart up to (closedStart - 1)
            if ($closedStart > $rStart) {
                $leftEnd = date('Y-m-d', strtotime($closedStart . ' -1 day'));
                if (strtotime($leftEnd) >= strtotime($rStart)) {
                    $newAllowed[] = [
                        'start' => $rStart,
                        'end'   => $leftEnd
                    ];
                }
            }
            // right side: from (closedEnd+1) up to rEnd
            if ($closedEnd < $rEnd) {
                $rightStart = date('Y-m-d', strtotime($closedEnd . ' +1 day'));
                if (strtotime($rightStart) <= strtotime($rEnd)) {
                    $newAllowed[] = [
                        'start' => $rightStart,
                        'end'   => $rEnd
                    ];
                }
            }
        }
        $allowed = $newAllowed;
    }

    return $allowed;
}

// Initialize PDO connection
try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    logError(null, 'Database Connection Error: ' . $e->getMessage());
}
