<?php
require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/audit_logs.php');
require_once(__DIR__ . '/error_handling.php');

function createUser($pdo, string $username, string $password, string $email, int $userId): array
{
    // Input validation
    $errors = [];

    // Validate username
    if (empty($username) || !is_string($username)) {
        $errors[] = 'Username is required and must be a string.';
    } elseif (strlen($username) > 50) {
        $errors[] = 'Username must be less than 50 characters.';
    } elseif (!preg_match('/^[a-zA-Z0-9_]+$/', $username)) {
        $errors[] = 'Username can only contain letters, numbers, and underscores.';
    }

    // Validate password
    if (empty($password) || !is_string($password)) {
        $errors[] = 'Password is required.';
    } elseif (strlen($password) < 8) {
        $errors[] = 'Password must be at least 8 characters long.';
    }

    // Validate email
    if (empty($email) || !filter_var($email, FILTER_VALIDATE_EMAIL)) {
        $errors[] = 'A valid email address is required.';
    } elseif (strlen($email) > 100) {
        $errors[] = 'Email must be less than 100 characters.';
    }

    // Validate userId
    if ($userId <= 0) {
        $errors[] = 'Invalid user ID.';
    }

    if (!empty($errors)) {
        // Log errors
        logError($pdo, 'Create User Validation Errors: ' . implode('; ', $errors));
        return ['success' => false, 'message' => implode(' ', $errors)];
    }

    try {
        // Check if username already exists
        $stmt = $pdo->prepare("SELECT id, is_deleted FROM users WHERE username = ?;");
        $stmt->execute([$username]);
        $existingUser = $stmt->fetch(PDO::FETCH_ASSOC);

        $passwordHash = password_hash($password, PASSWORD_BCRYPT);
        $currentTimestamp = date('Y-m-d H:i:s');

        if ($existingUser) {
            if ($existingUser['is_deleted'] == 0) {
                // User exists and is active, return error
                return ['success' => false, 'message' => 'A user with this username already exists.'];
            } else {
                // User exists and is deleted, reactivate/update the user
                $stmt = $pdo->prepare("UPDATE users SET
                    password_hash = ?,
                    email = ?,
                    is_reset = 1,
                    is_deleted = 0,
                    is_auth = 1,
                    updated_at = ?
                    WHERE id = ?;");
                $stmt->execute([$passwordHash, $email, $currentTimestamp, $existingUser['id']]);

                // Audit log
                createAuditLog($pdo, $userId, 'update', 'users', $existingUser['id'], "Reactivated User: $username");

                return ['success' => true, 'message' => 'User reactivated successfully.', 'user_id' => $existingUser['id']];
            }
        } else {
            // User does not exist, insert a new record
            $stmt = $pdo->prepare("INSERT INTO users (username, password_hash, email, is_reset, is_deleted, is_auth, created_at, updated_at) VALUES (?, ?, ?, ?, 0, 1, ?, ?);");
            $stmt->execute([$username, $passwordHash, $email, 1, $currentTimestamp, $currentTimestamp]);
            $newUserId = (int)$pdo->lastInsertId();

            // Audit log
            createAuditLog($pdo, $userId, 'create', 'users', $newUserId, "Created User: $username");

            return ['success' => true, 'message' => 'User created successfully.', 'user_id' => $newUserId];
        }
    } catch (PDOException $e) {
        logError($pdo, 'Create User Error: ' . $e->getMessage());
        return ['success' => false, 'message' => 'Failed to create or update user due to a database error.'];
    }
}

function getUser($pdo, int $userId): array
{
    // Validate userId
    if ($userId <= 0) {
        logError($pdo, 'Get User Error: Invalid user ID.');
        return [];
    }

    try {
        $stmt = $pdo->prepare("SELECT * FROM users WHERE id = ?;");
        $stmt->execute([$userId]);
        return $stmt->fetch(PDO::FETCH_ASSOC) ?: [];
    } catch (PDOException $e) {
        logError($pdo, 'Get User Error: ' . $e->getMessage());
        return [];
    }
}

function getUserByUsername($pdo, string $username): array
{
    // Validate username
    if (empty($username) || !is_string($username)) {
        logError($pdo, 'Get User By Username Error: Invalid username.');
        return [];
    }

    try {
        $stmt = $pdo->prepare("SELECT * FROM users WHERE username = ?;");
        $stmt->execute([$username]);
        return $stmt->fetch(PDO::FETCH_ASSOC) ?: [];
    } catch (PDOException $e) {
        logError($pdo, 'Get User By Username Error: ' . $e->getMessage());
        return [];
    }
}

function listUsers($pdo, array $filters = []): array
{
    try {
        $query = "SELECT * FROM users WHERE is_deleted = 0";
        $params = [];
        if (isset($filters['email'])) {
            $email = $filters['email'];
            // Validate email
            if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
                logError($pdo, 'List Users Error: Invalid email filter.');
                return [];
            }
            $query .= " AND email = ?";
            $params[] = $email;
        }
        if (isset($filters['username'])) {
            $username = $filters['username'];
            // Validate username
            if (!is_string($username) || empty($username)) {
                logError($pdo, 'List Users Error: Invalid username filter.');
                return [];
            }
            $query .= " AND username = ?";
            $params[] = $username;
        }
        $stmt = $pdo->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'List Users Error: ' . $e->getMessage());
        return [];
    }
}

function updateUser($pdo, int $userId, array $data, int $adminUserId): bool
{
    // Validate userId and adminUserId
    if ($userId <= 0 || $adminUserId <= 0) {
        logError($pdo, 'Update User Error: Invalid user ID or admin user ID.');
        return false;
    }

    $fields = [];
    $params = [];

    // Validate and prepare fields
    if (isset($data['username'])) {
        $username = $data['username'];
        if (empty($username) || !is_string($username)) {
            logError($pdo, 'Update User Error: Invalid username.');
            return false;
        } elseif (strlen($username) > 50 || !preg_match('/^[a-zA-Z0-9_]+$/', $username)) {
            logError($pdo, 'Update User Error: Username validation failed.');
            return false;
        }
        $fields[] = "username = ?";
        $params[] = $username;
    }

    if (isset($data['email'])) {
        $email = $data['email'];
        if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
            logError($pdo, 'Update User Error: Invalid email.');
            return false;
        }
        $fields[] = "email = ?";
        $params[] = $email;
    }

    if (isset($data['password'])) {
        $password = $data['password'];
        if (empty($password) || !is_string($password) || strlen($password) < 8) {
            logError($pdo, 'Update User Error: Password validation failed.');
            return false;
        }
        $fields[] = "password_hash = ?";
        $params[] = password_hash($password, PASSWORD_BCRYPT);
    }

    if (isset($data['is_auth'])) {
        $isAuth = $data['is_auth'];
        if (!in_array($isAuth, [0, 1], true)) {
            logError($pdo, 'Update User Error: Invalid is_auth value.');
            return false;
        }
        $fields[] = "is_auth = ?";
        $params[] = $isAuth;
    }

    if (empty($fields)) {
        logError($pdo, 'Update User Error: No valid fields to update.');
        return false;
    }

    $params[] = $userId;

    try {
        $query = "UPDATE users SET " . implode(", ", $fields) . " WHERE id = ?;";
        $stmt = $pdo->prepare($query);
        $stmt->execute($params);

        // Audit log
        createAuditLog($pdo, $adminUserId, 'update', 'users', $userId, "Updated User ID: $userId");

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Update User Error: ' . $e->getMessage());
        return false;
    }
}

function deleteUser($pdo, int $userId, int $adminUserId): bool
{
    // Validate userId and adminUserId
    if ($userId <= 0 || $adminUserId <= 0) {
        logError($pdo, 'Delete User Error: Invalid user ID or admin user ID.');
        return false;
    }

    try {
        $stmt = $pdo->prepare("UPDATE users SET is_deleted = 1 WHERE id = ?;");
        $stmt->execute([$userId]);

        // Audit log
        createAuditLog($pdo, $adminUserId, 'delete', 'users', $userId, "Deleted User ID: $userId");

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Delete User Error: ' . $e->getMessage());
        return false;
    }
}

function authenticateUser($pdo, string $username, string $password): bool
{
    // Validate username and password
    if (empty($username) || empty($password)) {
        logError($pdo, 'Authenticate User Error: Username and password are required.');
        return false;
    }

    try {
        $stmt = $pdo->prepare("SELECT id, password_hash, is_auth FROM users WHERE username = ? AND is_deleted = 0;");
        $stmt->execute([$username]);
        $user = $stmt->fetch(PDO::FETCH_ASSOC);

        if ($user && password_verify($password, $user['password_hash']) && $user['is_auth'] == 1) {
            // Optionally, log successful login
            // createAuditLog($pdo, $user['id'], 'login', 'users', $user['id'], "User logged in.");
            return true;
        } else {
            logError($pdo, 'Authenticate User Error: Invalid credentials or user not authorized.');
            return false;
        }
    } catch (PDOException $e) {
        logError($pdo, 'Authenticate User Error: ' . $e->getMessage());
        return false;
    }
}

function getUserRolesID($pdo, int $userId): array
{
    // Validate userId
    if ($userId <= 0) {
        logError($pdo, 'Get User Roles ID Error: Invalid user ID.');
        return [];
    }

    try {
        $stmt = $pdo->prepare("SELECT r.id FROM roles r
            INNER JOIN user_roles ur ON r.id = ur.role_id
            WHERE ur.user_id = ?;");
        $stmt->execute([$userId]);
        return $stmt->fetchAll(PDO::FETCH_COLUMN);
    } catch (PDOException $e) {
        logError($pdo, 'Get User Roles ID Error: ' . $e->getMessage());
        return [];
    }
}

function getUserRoles($pdo, int $userId): array
{
    // Validate userId
    if ($userId <= 0) {
        logError($pdo, 'Get User Roles Error: Invalid user ID.');
        return [];
    }

    try {
        $stmt = $pdo->prepare("SELECT r.name FROM roles r
            INNER JOIN user_roles ur ON r.id = ur.role_id
            WHERE ur.user_id = ?;");
        $stmt->execute([$userId]);
        return $stmt->fetchAll(PDO::FETCH_COLUMN);
    } catch (PDOException $e) {
        logError($pdo, 'Get User Roles Error: ' . $e->getMessage());
        return [];
    }
}

function getAllRoles($pdo): array
{
    try {
        $stmt = $pdo->prepare("SELECT * FROM roles;");
        $stmt->execute();
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'Get All Roles Error: ' . $e->getMessage());
        return [];
    }
}

function assignUserRole($pdo, int $userId, int $roleId, int $adminUserId): bool
{
    // Validate userId, roleId, adminUserId
    if ($userId <= 0 || $roleId <= 0 || $adminUserId <= 0) {
        logError($pdo, 'Assign User Role Error: Invalid user ID, role ID, or admin user ID.');
        return false;
    }

    try {
        $stmt = $pdo->prepare("INSERT OR IGNORE INTO user_roles (user_id, role_id) VALUES (?, ?);");
        $stmt->execute([$userId, $roleId]);

        // Audit log
        createAuditLog($pdo, $adminUserId, 'assign_role', 'users', $userId, "Assigned Role ID: $roleId");

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Assign User Role Error: ' . $e->getMessage());
        return false;
    }
}

function removeAllUserRoles($pdo, int $userId, int $adminUserId): bool
{
    // Validate userId and adminUserId
    if ($userId <= 0 || $adminUserId <= 0) {
        logError($pdo, 'Remove All User Roles Error: Invalid user ID or admin user ID.');
        return false;
    }

    try {
        $stmt = $pdo->prepare("DELETE FROM user_roles WHERE user_id = ?;");
        $stmt->execute([$userId]);

        // Audit log
        createAuditLog($pdo, $adminUserId, 'remove_roles', 'users', $userId, "Removed all roles from User ID: $userId");

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Remove All User Roles Error: ' . $e->getMessage());
        return false;
    }
}

function createRole($pdo, string $roleName, int $userId): array
{
    // Input validation
    $errors = [];

    // Validate role name
    if (empty($roleName) || !is_string($roleName)) {
        $errors[] = 'Role name is required and must be a string.';
    } elseif (strlen($roleName) > 50) {
        $errors[] = 'Role name must be less than 50 characters.';
    }

    // Validate userId
    if ($userId <= 0) {
        $errors[] = 'Invalid user ID.';
    }

    if (!empty($errors)) {
        // Log errors
        logError($pdo, 'Create Role Validation Errors: ' . implode('; ', $errors));
        return ['success' => false, 'message' => implode(' ', $errors)];
    }

    try {
        $stmt = $pdo->prepare("INSERT INTO roles (name) VALUES (?);");
        $stmt->execute([$roleName]);
        $roleId = (int)$pdo->lastInsertId();

        // Audit log
        createAuditLog($pdo, $userId, 'create', 'roles', $roleId, "Created Role: $roleName");

        return ['success' => true, 'message' => 'Role created successfully.', 'role_id' => $roleId];
    } catch (PDOException $e) {
        // Check if the error is due to a UNIQUE constraint violation
        if ($e->getCode() == '23000') { // SQLSTATE code for integrity constraint violation
            logError($pdo, 'Create Role Error: Role name already exists.');
            return ['success' => false, 'message' => 'Role name already exists.'];
        } else {
            logError($pdo, 'Create Role Error: ' . $e->getMessage());
            return ['success' => false, 'message' => 'Failed to add role due to a database error.'];
        }
    }
}

function deleteRole($pdo, int $roleId, int $userId): bool
{
    // Validate roleId and userId
    if ($roleId <= 0 || $userId <= 0) {
        logError($pdo, 'Delete Role Error: Invalid role ID or user ID.');
        return false;
    }

    try {
        $stmt = $pdo->prepare("DELETE FROM roles WHERE id = ?;");
        $stmt->execute([$roleId]);

        // Audit log
        createAuditLog($pdo, $userId, 'delete', 'roles', $roleId, "Deleted Role ID: $roleId");

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Delete Role Error: ' . $e->getMessage());
        return false;
    }
}

function listRoles($pdo): array
{
    try {
        $stmt = $pdo->prepare("SELECT * FROM roles;");
        $stmt->execute();
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'List Roles Error: ' . $e->getMessage());
        return [];
    }
}

function deleteUserCompletely($pdo, int $userId): bool
{
    try {
        // Delete user roles
        $stmt = $pdo->prepare("DELETE FROM user_roles WHERE user_id = ?");
        $stmt->execute([$userId]);
        $rolesDeleted = $stmt->rowCount();

        // Delete user
        $stmt = $pdo->prepare("DELETE FROM users WHERE id = ?");
        $stmt->execute([$userId]);
        $userDeleted = $stmt->rowCount();

        // Log the number of rows deleted
        logError($pdo, "DeleteUserCompletely: RolesDeleted=$rolesDeleted, UserDeleted=$userDeleted");

        if ($userDeleted === 0) {
            logError($pdo, 'Delete User Error: User not deleted.');
            return false;
        }

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Delete User Error: ' . $e->getMessage());
        return false;
    }
}

function updateUserPassword($pdo, $user_id, $new_password) {
    // Hash the new password
    $hashed_password = password_hash($new_password, PASSWORD_DEFAULT);

    // Update the user's password in the database
    // Replace 'hashed_password' with your actual password column name
    $stmt = $pdo->prepare("UPDATE users SET password_hash = :password WHERE id = :id");
    return $stmt->execute([
        ':password' => $hashed_password,
        ':id' => $user_id
    ]);
}


// Initialize PDO connection if not already initialized
if (!isset($pdo) || !$pdo instanceof PDO) {
    try {
        $pdo = new PDO('sqlite:' . DB_FILE);
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
        $pdo->exec("PRAGMA foreign_keys = ON;");
    } catch (PDOException $e) {
        logError(null, 'Database Connection Error: ' . $e->getMessage());
    }
}
?>
