<?php
require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/error_handling.php');

function validateAccountData(array $data): bool
{
    if (empty($data['code']) || empty($data['name']) || empty($data['account_type'])) {
        return false;
    }
    return true;
}

function validateJournalEntryData(array $data): bool
{
    if (empty($data['description']) || empty($data['date']) || empty($data['financial_period_id'])) {
        return false;
    }
    return true;
}

function validateTransactionData(array $data): bool
{
    if (empty($data['journal_entry_id']) || empty($data['debit_account']) || empty($data['credit_account']) || empty($data['amount']) || empty($data['transaction_date'])) {
        return false;
    }
    return true;
}

function validateUserData(array $data): bool
{
    if (empty($data['username']) || empty($data['email']) || empty($data['password'])) {
        return false;
    }
    return true;
}

function formatCurrency(float $amount): string
{
    return number_format($amount, 2, '.', ',');
}

function hashPassword(string $password): string
{
    return password_hash($password, PASSWORD_BCRYPT);
}

function verifyPassword(string $password, string $hash): bool
{
    return password_verify($password, $hash);
}

function generateAuthToken(int $userId): string
{
    return base64_encode(random_bytes(32)) . '.' . $userId;
}

function verifyAuthToken(string $token): bool
{
    return !empty($token);
}

function isFinancialPeriodClosed($pdo, int $financialPeriodId): bool
{
    try {
        $stmt = $pdo->prepare("SELECT is_closed FROM financial_periods WHERE id = ?;");
        $stmt->execute([$financialPeriodId]);
        return (bool)$stmt->fetchColumn();
    } catch (PDOException $e) {
        logError($pdo, 'Is Financial Period Closed Error: ' . $e->getMessage());
        return false;
    }
}

function isVATPeriodSubmitted($pdo, int $vatPeriodId): bool
{
    try {
        $stmt = $pdo->prepare("SELECT status FROM vat_periods WHERE id = ?;");
        $stmt->execute([$vatPeriodId]);
        $status = $stmt->fetchColumn();
        return $status === 'closed';
    } catch (PDOException $e) {
        logError($pdo, 'Is VAT Period Submitted Error: ' . $e->getMessage());
        return false;
    }
}

function getCurrentFinancialPeriod($pdo): array
{
    try {
        $stmt = $pdo->prepare("SELECT * FROM financial_periods WHERE is_closed = 0 ORDER BY start_date DESC LIMIT 1;");
        $stmt->execute();
        return $stmt->fetch(PDO::FETCH_ASSOC) ?: [];
    } catch (PDOException $e) {
        logError($pdo, 'Get Current Financial Period Error: ' . $e->getMessage());
        return [];
    }
}

function getCurrentVATPeriod($pdo): array
{
    try {
        $stmt = $pdo->prepare("SELECT * FROM vat_periods WHERE status = 'open' ORDER BY start_date DESC LIMIT 1;");
        $stmt->execute();
        return $stmt->fetch(PDO::FETCH_ASSOC) ?: [];
    } catch (PDOException $e) {
        logError($pdo, 'Get Current VAT Period Error: ' . $e->getMessage());
        return [];
    }
}

try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    logError($pdo, 'Database Connection Error: ' . $e->getMessage());
}
?>
