<?php
require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/audit_logs.php');
require_once(__DIR__ . '/error_handling.php');
require_once(__DIR__ . '/journal_entries.php');
require_once(__DIR__ . '/transactions.php');
require_once(__DIR__ . '/financial_periods.php');

/**
 * ------------------------------------------------------------------
 * Define VAT account constants if not already defined in config.php
 * ------------------------------------------------------------------
 */
if (!defined('VAT_PAYABLE_ACC_CODE')) {
    define('VAT_PAYABLE_ACC_CODE', 'B75'); // Default if missing
}
if (!defined('VAT_RECEIVABLE_ACC_CODE')) {
    define('VAT_RECEIVABLE_ACC_CODE', 'B76'); // Default if missing
}
if (!defined('VAT_CONTROL_ACC_CODE')) {
    define('VAT_CONTROL_ACC_CODE', 'B76'); // Fallback if not defined in config
}

/**
 * Create a new VAT Period.
 */
function createVATPeriod($pdo, string $name, string $startDate, string $endDate, int $userId): int
{
    try {
        // Validate inputs
        if (empty($name)) {
            throw new Exception('VAT Period name cannot be empty.');
        }

        // Validate dates
        $startDateObj = DateTime::createFromFormat('Y-m-d', $startDate);
        $endDateObj   = DateTime::createFromFormat('Y-m-d', $endDate);
        if (!$startDateObj || !$endDateObj) {
            throw new Exception('Invalid date format. Must be YYYY-MM-DD.');
        }
        if ($startDateObj > $endDateObj) {
            throw new Exception('Start date cannot be after end date.');
        }

        // Check overlapping
        $stmt = $pdo->prepare("SELECT COUNT(*) FROM vat_periods
            WHERE is_deleted = 0
              AND (
                (start_date BETWEEN ? AND ?)
                OR (end_date BETWEEN ? AND ?)
                OR (? BETWEEN start_date AND end_date)
                OR (? BETWEEN start_date AND end_date)
              )");
        $stmt->execute([$startDate, $endDate, $startDate, $endDate, $startDate, $endDate]);
        $overlapCount = (int)$stmt->fetchColumn();
        if ($overlapCount > 0) {
            throw new Exception('VAT Period overlaps with existing VAT Periods.');
        }

        // Insert
        $stmt = $pdo->prepare("INSERT INTO vat_periods (name, start_date, end_date, status) VALUES (?, ?, ?, 'open');");
        $stmt->execute([$name, $startDate, $endDate]);
        $vatPeriodId = (int)$pdo->lastInsertId();

        createAuditLog($pdo, $userId, 'create', 'vat_periods', $vatPeriodId, "Created VAT Period: $name");
        return $vatPeriodId;
    } catch (Exception $e) {
        logError($pdo, 'Create VAT Period Error: ' . $e->getMessage());
        return 0;
    } catch (PDOException $e) {
        logError($pdo, 'Create VAT Period Database Error: ' . $e->getMessage());
        return 0;
    }
}

/**
 * Retrieve one VAT Period by ID.
 */
function getVATPeriod($pdo, int $vatPeriodId): array
{
    try {
        $stmt = $pdo->prepare("SELECT * FROM vat_periods WHERE id = ? AND is_deleted = 0;");
        $stmt->execute([$vatPeriodId]);
        return $stmt->fetch(PDO::FETCH_ASSOC) ?: [];
    } catch (PDOException $e) {
        logError($pdo, 'Get VAT Period Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * List VAT Periods with optional filters.
 */
function listVATPeriods($pdo, array $filters = []): array
{
    try {
        $query  = "SELECT * FROM vat_periods WHERE is_deleted = 0";
        $params = [];
        if (isset($filters['status'])) {
            $query .= " AND status = ?";
            $params[] = $filters['status'];
        }
        if (isset($filters['start_date'])) {
            $query .= " AND start_date >= ?";
            $params[] = $filters['start_date'];
        }
        if (isset($filters['end_date'])) {
            $query .= " AND end_date <= ?";
            $params[] = $filters['end_date'];
        }
        $stmt = $pdo->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        logError($pdo, 'List VAT Periods Error: ' . $e->getMessage());
        return [];
    }
}

/**
 * Update an existing VAT Period's name/dates.
 */
function updateVATPeriod($pdo, int $vatPeriodId, string $name, string $startDate, string $endDate, int $userId): bool
{
    try {
        $vatPeriod = getVATPeriod($pdo, $vatPeriodId);
        if (empty($vatPeriod)) {
            throw new Exception('VAT Period not found.');
        }
        if (empty($name)) {
            throw new Exception('VAT Period name cannot be empty.');
        }

        // Validate new dates
        $startDateObj = DateTime::createFromFormat('Y-m-d', $startDate);
        $endDateObj   = DateTime::createFromFormat('Y-m-d', $endDate);
        if (!$startDateObj || !$endDateObj) {
            throw new Exception('Invalid date format. Must be YYYY-MM-DD.');
        }
        if ($startDateObj > $endDateObj) {
            throw new Exception('Start date cannot be after end date.');
        }

        // Check overlap excluding current
        $stmt = $pdo->prepare("SELECT COUNT(*) FROM vat_periods
            WHERE is_deleted = 0 AND id != ?
              AND (
                (start_date BETWEEN ? AND ?)
                OR (end_date BETWEEN ? AND ?)
                OR (? BETWEEN start_date AND end_date)
                OR (? BETWEEN start_date AND end_date)
              )");
        $stmt->execute([$vatPeriodId, $startDate, $endDate, $startDate, $endDate, $startDate, $endDate]);
        $overlapCount = (int)$stmt->fetchColumn();
        if ($overlapCount > 0) {
            throw new Exception('VAT Period overlaps with existing VAT Periods.');
        }

        // Check if transactions are outside new date range
        $stmt = $pdo->prepare("
            SELECT COUNT(*) FROM transactions
            WHERE is_deleted=0
              AND vat_rates_id != 0
              AND (transaction_date < ? OR transaction_date > ?)
              AND transaction_date >= ?
              AND transaction_date <= ?
        ");
        $stmt->execute([
            $startDate,
            $endDate,
            $vatPeriod['start_date'],
            $vatPeriod['end_date']
        ]);
        $invalidCount = (int)$stmt->fetchColumn();
        if ($invalidCount > 0) {
            throw new Exception('Cannot update VAT Period dates; some VAT transactions fall outside the new range.');
        }

        // Update
        $stmt = $pdo->prepare("UPDATE vat_periods SET name=?, start_date=?, end_date=? WHERE id=?;");
        $stmt->execute([$name, $startDate, $endDate, $vatPeriodId]);

        createAuditLog($pdo, $userId, 'update', 'vat_periods', $vatPeriodId, "Updated VAT Period: $name");
        return true;
    } catch (Exception $e) {
        logError($pdo, 'Update VAT Period Error: ' . $e->getMessage());
        return false;
    } catch (PDOException $e) {
        logError($pdo, 'Update VAT Period DB Error: ' . $e->getMessage());
        return false;
    }
}

/**
 * Helper function to get net debit-credit for a single account in a date range.
 * Positive => net DEBIT, Negative => net CREDIT
 */
function calculateVatAccountNet(PDO $pdo, string $accountCode, string $startDate, string $endDate): float
{
    $sql = "
        SELECT
          COALESCE(SUM(CASE WHEN debit_account = :acct THEN amount ELSE 0 END),0)
          - COALESCE(SUM(CASE WHEN credit_account = :acct THEN amount ELSE 0 END),0)
          AS net
        FROM transactions
        WHERE is_deleted = 0
          AND transaction_date >= :startDate
          AND transaction_date <= :endDate
    ";
    $st = $pdo->prepare($sql);
    $st->execute([
        ':acct'      => $accountCode,
        ':startDate' => $startDate,
        ':endDate'   => $endDate
    ]);
    $row = $st->fetch(PDO::FETCH_ASSOC);
    return isset($row['net']) ? (float)$row['net'] : 0.0;
}

/**
 * Close a VAT Period by transferring net balances from Payable/Receivable
 * into VAT_CONTROL_ACC_CODE. Then mark 'closed'.
 */
function closeVATPeriod($pdo, int $vatPeriodId, int $userId): array
{
    try {
        $vatPeriod = getVATPeriod($pdo, $vatPeriodId);
        if (empty($vatPeriod)) {
            throw new Exception('VAT Period not found.');
        }
        if ($vatPeriod['status'] === 'closed') {
            throw new Exception('VAT Period is already closed.');
        }

        // 1) Calculate net for VAT Payable and VAT Receivable in the date range
        $payableNet    = calculateVatAccountNet($pdo, VAT_PAYABLE_ACC_CODE,     $vatPeriod['start_date'], $vatPeriod['end_date']);
        $receivableNet = calculateVatAccountNet($pdo, VAT_RECEIVABLE_ACC_CODE, $vatPeriod['start_date'], $vatPeriod['end_date']);

        // 2) Create a journal entry
        $description = "VAT Settlement for Period [{$vatPeriodId}] ({$vatPeriod['name']})";
        $periodId = getFinancialPeriodIdByDate($pdo, $vatPeriod['end_date']);
        if (!$periodId) {
            throw new Exception('No financial period matches the VAT Period end date.');
        }
        $journalEntryId = createJournalEntry($pdo, $description, $vatPeriod['end_date'], $periodId, $userId);
        if (!$journalEntryId) {
            throw new Exception('Failed to create a journal entry for VAT settlement.');
        }

        // 3) Transfer net amounts to/from VAT_CONTROL_ACC_CODE
        $vatControl = defined('VAT_CONTROL_ACC_CODE') ? VAT_CONTROL_ACC_CODE : 'B76';

        // 3a) VAT Payable net (positive => net debit => DR control, CR payable; negative => net credit => DR payable, CR control)
        if (abs($payableNet) > 0.00001) {
            if ($payableNet > 0) {
                // net DEBIT => DR control, CR payable
                $amt = $payableNet;
                createTransaction(
                    $pdo,
                    $journalEntryId,
                    $vatControl,
                    VAT_PAYABLE_ACC_CODE,
                    $amt,
                    $description,
                    $vatPeriod['end_date'],
                    $periodId,
                    $userId,
                    0,
                    0,
                    json_encode(['vat_period_id' => $vatPeriodId])
                );
            } else {
                // net CREDIT => DR payable, CR control
                $amt = abs($payableNet);
                createTransaction(
                    $pdo,
                    $journalEntryId,
                    VAT_PAYABLE_ACC_CODE,
                    $vatControl,
                    $amt,
                    $description,
                    $vatPeriod['end_date'],
                    $periodId,
                    $userId,
                    0,
                    0,
                    json_encode(['vat_period_id' => $vatPeriodId])
                );
            }
        }

        // 3b) VAT Receivable net (positive => net debit => DR control, CR receivable; negative => net credit => DR receivable, CR control)
        if (abs($receivableNet) > 0.00001) {
            if ($receivableNet > 0) {
                // net DEBIT => DR control, CR receivable
                $amt = $receivableNet;
                createTransaction(
                    $pdo,
                    $journalEntryId,
                    $vatControl,
                    VAT_RECEIVABLE_ACC_CODE,
                    $amt,
                    $description,
                    $vatPeriod['end_date'],
                    $periodId,
                    $userId,
                    0,
                    0,
                    json_encode(['vat_period_id' => $vatPeriodId])
                );
            } else {
                // net CREDIT => DR receivable, CR control
                $amt = abs($receivableNet);
                createTransaction(
                    $pdo,
                    $journalEntryId,
                    VAT_RECEIVABLE_ACC_CODE,
                    $vatControl,
                    $amt,
                    $description,
                    $vatPeriod['end_date'],
                    $periodId,
                    $userId,
                    0,
                    0,
                    json_encode(['vat_period_id' => $vatPeriodId])
                );
            }
        }

        // 4) Mark Period closed
        $stmt = $pdo->prepare("UPDATE vat_periods SET status = 'closed' WHERE id = ?;");
        $stmt->execute([$vatPeriodId]);

        createAuditLog($pdo, $userId, 'update', 'vat_periods', $vatPeriodId, "Closed VAT Period ID: $vatPeriodId");
        return ['success' => true, 'message' => "VAT Period [$vatPeriodId] closed successfully."];
    } catch (Exception $e) {
        logError($pdo, 'Close VAT Period Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    } catch (PDOException $e) {
        logError($pdo, 'Close VAT Period DB Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    }
}

/**
 * Reopen a VAT Period by deleting the settlement transactions
 * (found via the meta column) and marking it open again.
 */
function reopenVATPeriod($pdo, int $vatPeriodId, int $userId): array
{
    try {
        $vatPeriod = getVATPeriod($pdo, $vatPeriodId);
        if (empty($vatPeriod)) {
            throw new Exception('VAT Period not found.');
        }
        if ($vatPeriod['status'] === 'open') {
            throw new Exception('VAT Period is already open.');
        }

        $stmt = $pdo->prepare("UPDATE vat_periods SET status = 'open' WHERE id = ?;");
        $stmt->execute([$vatPeriodId]);

        // Remove any settlement transactions from meta
        $stmt2 = $pdo->prepare("
            SELECT id FROM transactions
            WHERE is_deleted = 0
              AND meta LIKE :search
        ");
        $stmt2->execute([
            ':search' => '%"vat_period_id":' . $vatPeriodId . '%'
        ]);
        $settlementTxs = $stmt2->fetchAll(PDO::FETCH_ASSOC);
        foreach ($settlementTxs as $txRow) {
            deleteTransaction($pdo, (int)$txRow['id'], $userId);
        }

        createAuditLog($pdo, $userId, 'update', 'vat_periods', $vatPeriodId, "Reopened VAT Period ID: $vatPeriodId");
        return ['success' => true, 'message' => "VAT Period [$vatPeriodId] reopened successfully."];
    } catch (Exception $e) {
        logError($pdo, 'Reopen VAT Period Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    } catch (PDOException $e) {
        logError($pdo, 'Reopen VAT Period DB Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    }
}

/**
 * Delete a VAT Period if no VAT transactions exist in its range.
 */
function deleteVATPeriod($pdo, int $vatPeriodId, int $userId): array
{
    try {
        $vatPeriod = getVATPeriod($pdo, $vatPeriodId);
        if (empty($vatPeriod)) {
            throw new Exception('VAT Period not found.');
        }

        // Check for transactions in date range
        $stmt = $pdo->prepare("
            SELECT COUNT(*)
            FROM transactions
            WHERE is_deleted = 0
              AND vat_rates_id != 0
              AND transaction_date >= ?
              AND transaction_date <= ?
        ");
        $stmt->execute([$vatPeriod['start_date'], $vatPeriod['end_date']]);
        $count = (int)$stmt->fetchColumn();

        if ($count > 0) {
            throw new Exception('Cannot delete VAT Period - existing VAT transactions in that date range.');
        }

        // Soft-delete the VAT period
        $stmt = $pdo->prepare("UPDATE vat_periods SET is_deleted = 1 WHERE id = ?;");
        $stmt->execute([$vatPeriodId]);

        createAuditLog($pdo, $userId, 'delete', 'vat_periods', $vatPeriodId, "Deleted VAT Period ID: $vatPeriodId");
        return ['success' => true, 'message' => "VAT Period [$vatPeriodId] deleted successfully."];
    } catch (Exception $e) {
        logError($pdo, 'Delete VAT Period Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    } catch (PDOException $e) {
        logError($pdo, 'Delete VAT Period DB Error: ' . $e->getMessage());
        return ['success' => false, 'message' => $e->getMessage()];
    }
}

// Initialize DB
try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    logError(null, 'Database Connection Error: ' . $e->getMessage());
}
