<?php
require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/audit_logs.php');
require_once(__DIR__ . '/error_handling.php');

function createVATRate($pdo, $rateName, $rateValue, $userId, $isDefault = 0) {
    // Validate inputs
    $errors = [];
    if (empty($rateName)) {
        $errors[] = "VAT Rate Name is required.";
    }
    if (!is_numeric($rateValue)) {
        $errors[] = "VAT Rate Value must be a numeric value.";
    } elseif ($rateValue < 0) {
        $errors[] = "VAT Rate Value cannot be negative.";
    }
    if (!is_int($userId) || $userId <= 0) {
        $errors[] = "Invalid User ID.";
    }
    $isDefault = $isDefault ? 1 : 0;

    if (!empty($errors)) {
        // Log the error
        $errorMessage = implode(" ", $errors);
        logError($pdo, 'Create VAT Rate Validation Error: ' . $errorMessage);
        $_SESSION['vat_rate_error'] = $errorMessage;
        return false;
    }

    try {
        // Start transaction
        $pdo->beginTransaction();

        // Check if rateName already exists
        $stmt = $pdo->prepare("SELECT id, is_deleted FROM vat_rates WHERE rate_name = ?");
        $stmt->execute([$rateName]);
        $existingRate = $stmt->fetch(PDO::FETCH_ASSOC);

        if ($existingRate) {
            $vatRateId = $existingRate['id'];

            if ($existingRate['is_deleted'] == 1) {
                // 'Undelete' the VAT rate by setting is_deleted to 0 and updating rate_value and is_default
                if ($isDefault) {
                    // Set all other VAT rates to is_default = 0
                    $stmt = $pdo->prepare("UPDATE vat_rates SET is_default = 0 WHERE is_default = 1");
                    $stmt->execute();
                }
                $stmt = $pdo->prepare("UPDATE vat_rates SET is_deleted = 0, rate_value = ?, is_default = ? WHERE id = ?");
                $stmt->execute([$rateValue, $isDefault, $vatRateId]);

                // Audit log
                createAuditLog($pdo, $userId, 'undelete', 'vat_rates', $vatRateId, "Undeleted and updated VAT Rate: $rateName");

                $pdo->commit();
                return $vatRateId;
            } else {
                // VAT rate with same name already exists and is not deleted
                $errorMessage = "A VAT Rate with the name '$rateName' already exists.";
                logError($pdo, 'Create VAT Rate Error: ' . $errorMessage);
                $_SESSION['vat_rate_error'] = $errorMessage;
                $pdo->rollBack();
                return false;
            }
        } else {
            if ($isDefault) {
                $stmt = $pdo->prepare("UPDATE vat_rates SET is_default = 0 WHERE is_default = 1");
                $stmt->execute();
            }
            $stmt = $pdo->prepare("INSERT INTO vat_rates (rate_name, rate_value, is_default, is_deleted) VALUES (?, ?, ?, 0)");
            $stmt->execute([$rateName, $rateValue, $isDefault]);
            $vatRateId = $pdo->lastInsertId();

            // Audit log
            createAuditLog($pdo, $userId, 'create', 'vat_rates', $vatRateId, "Created VAT Rate: $rateName");

            $pdo->commit();
            return $vatRateId;
        }
    } catch (PDOException $e) {
        $pdo->rollBack();
        $_SESSION['vat_rate_error'] = $e->getMessage();
        logError($pdo, 'Create VAT Rate Error: ' . $e->getMessage());
        return false;
    }
}

function getVATRate($pdo, $vatRateId) {
    // Validate vatRateId
    if (!is_int($vatRateId) || $vatRateId <= 0) {
        logError($pdo, 'Get VAT Rate Error: Invalid VAT Rate ID.');
        return false;
    }
    try {
        $stmt = $pdo->prepare("SELECT * FROM vat_rates WHERE id = ? AND is_deleted = 0");
        $stmt->execute([$vatRateId]);
        $result = $stmt->fetch(PDO::FETCH_ASSOC);
        if (!$result) {
            logError($pdo, 'Get VAT Rate Error: VAT Rate not found for ID ' . $vatRateId);
            return false;
        }
        return $result;
    } catch (PDOException $e) {
        logError($pdo, 'Get VAT Rate Error: ' . $e->getMessage());
        return false;
    }
}

function listVATRates($pdo, $filters = []) {
    try {
        $sql = "SELECT * FROM vat_rates WHERE is_deleted = 0";
        $params = [];
        if (isset($filters['rate_name'])) {
            $sql .= " AND rate_name LIKE ?";
            $params[] = '%' . $filters['rate_name'] . '%';
        }
        $stmt = $pdo->prepare($sql);
        $stmt->execute($params);
        $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
        if (!$results) {
            return [];
        }
        return $results;
    } catch (PDOException $e) {
        logError($pdo, 'List VAT Rates Error: ' . $e->getMessage());
        return false;
    }
}

function updateVATRate($pdo, $vatRateId, $rateName, $rateValue, $userId) {
    // Validate inputs
    $errors = [];
    if (!is_int($vatRateId) || $vatRateId <= 0) {
        $errors[] = "Invalid VAT Rate ID.";
    }
    if (empty($rateName)) {
        $errors[] = "VAT Rate Name is required.";
    }
    if (!is_numeric($rateValue)) {
        $errors[] = "VAT Rate Value must be a numeric value.";
    } elseif ($rateValue < 0) {
        $errors[] = "VAT Rate Value cannot be negative.";
    }
    if (!is_int($userId) || $userId <= 0) {
        $errors[] = "Invalid User ID.";
    }

    if (!empty($errors)) {
        $errorMessage = implode(" ", $errors);
        logError($pdo, 'Update VAT Rate Validation Error: ' . $errorMessage);
        $_SESSION['vat_rate_error'] = $errorMessage;
        return false;
    }

    try {
        // Start transaction
        $pdo->beginTransaction();

        // Check if VAT Rate exists
        $stmt = $pdo->prepare("SELECT * FROM vat_rates WHERE id = ? AND is_deleted = 0");
        $stmt->execute([$vatRateId]);
        $existingRate = $stmt->fetch(PDO::FETCH_ASSOC);
        if (!$existingRate) {
            $errorMessage = "VAT Rate not found for ID $vatRateId.";
            logError($pdo, 'Update VAT Rate Error: ' . $errorMessage);
            $_SESSION['vat_rate_error'] = $errorMessage;
            $pdo->rollBack();
            return false;
        }

        // Check if rateName is unique
        $stmt = $pdo->prepare("SELECT COUNT(*) FROM vat_rates WHERE rate_name = ? AND id != ? AND is_deleted = 0");
        $stmt->execute([$rateName, $vatRateId]);
        if ($stmt->fetchColumn() > 0) {
            $errorMessage = "Another VAT Rate with the name '$rateName' already exists.";
            logError($pdo, 'Update VAT Rate Error: ' . $errorMessage);
            $_SESSION['vat_rate_error'] = $errorMessage;
            $pdo->rollBack();
            return false;
        }

        $stmt = $pdo->prepare("UPDATE vat_rates SET rate_name = ?, rate_value = ? WHERE id = ? AND is_deleted = 0");
        $stmt->execute([$rateName, $rateValue, $vatRateId]);

        // Audit log
        createAuditLog($pdo, $userId, 'update', 'vat_rates', $vatRateId, "Updated VAT Rate: $rateName");

        $pdo->commit();
        return true;
    } catch (PDOException $e) {
        $pdo->rollBack();
        logError($pdo, 'Update VAT Rate Error: ' . $e->getMessage());
        return false;
    }
}

function deleteVATRate($pdo, $vatRateId, $userId) {
    // Validate inputs
    if (!is_int($vatRateId) || $vatRateId <= 0) {
        $errorMessage = "Invalid VAT Rate ID.";
        logError($pdo, 'Delete VAT Rate Error: ' . $errorMessage);
        $_SESSION['vat_rate_error'] = $errorMessage;
        return false;
    }
    if (!is_int($userId) || $userId <= 0) {
        $errorMessage = "Invalid User ID.";
        logError($pdo, 'Delete VAT Rate Error: ' . $errorMessage);
        $_SESSION['vat_rate_error'] = $errorMessage;
        return false;
    }

    try {
        // Check if VAT Rate exists
        $stmt = $pdo->prepare("SELECT * FROM vat_rates WHERE id = ? AND is_deleted = 0");
        $stmt->execute([$vatRateId]);
        $existingRate = $stmt->fetch(PDO::FETCH_ASSOC);
        if (!$existingRate) {
            $errorMessage = "VAT Rate not found for ID $vatRateId.";
            logError($pdo, 'Delete VAT Rate Error: ' . $errorMessage);
            $_SESSION['vat_rate_error'] = $errorMessage;
            return false;
        }

        $stmt = $pdo->prepare("UPDATE vat_rates SET is_deleted = 1 WHERE id = ?");
        $stmt->execute([$vatRateId]);

        // Audit log
        createAuditLog($pdo, $userId, 'delete', 'vat_rates', $vatRateId, "Deleted VAT Rate ID: $vatRateId");

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Delete VAT Rate Error: ' . $e->getMessage());
        return false;
    }
}

function setDefaultVATRate($pdo, $vatRateId, $userId) {
    // Validate inputs
    if (!is_int($vatRateId) || $vatRateId <= 0) {
        $errorMessage = "Invalid VAT Rate ID.";
        logError($pdo, 'Set Default VAT Rate Error: ' . $errorMessage);
        $_SESSION['vat_rate_error'] = $errorMessage;
        return false;
    }
    if (!is_int($userId) || $userId <= 0) {
        $errorMessage = "Invalid User ID.";
        logError($pdo, 'Set Default VAT Rate Error: ' . $errorMessage);
        $_SESSION['vat_rate_error'] = $errorMessage;
        return false;
    }

    try {
        // Check if VAT Rate exists
        $stmt = $pdo->prepare("SELECT * FROM vat_rates WHERE id = ? AND is_deleted = 0");
        $stmt->execute([$vatRateId]);
        $existingRate = $stmt->fetch(PDO::FETCH_ASSOC);
        if (!$existingRate) {
            $errorMessage = "VAT Rate not found for ID $vatRateId.";
            logError($pdo, 'Set Default VAT Rate Error: ' . $errorMessage);
            $_SESSION['vat_rate_error'] = $errorMessage;
            return false;
        }

        // Set all is_default to 0
        $stmt = $pdo->prepare("UPDATE vat_rates SET is_default = 0 WHERE is_default = 1");
        $stmt->execute();

        // Set the selected VAT Rate as default
        $stmt = $pdo->prepare("UPDATE vat_rates SET is_default = 1 WHERE id = ?");
        $stmt->execute([$vatRateId]);

        // Audit log
        createAuditLog($pdo, $userId, 'update', 'vat_rates', $vatRateId, "Set Default VAT Rate ID: $vatRateId");

        return true;
    } catch (PDOException $e) {
        logError($pdo, 'Set Default VAT Rate Error: ' . $e->getMessage());
        return false;
    }
}
