<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Include necessary files
require_once('../core/accounts.php');
require_once('components/auth.php'); // Ensure this path is correct
require_once('components/header.php');
// *** CHANGED/ADDED *** Require the updated account_functions:
require_once('components/account_functions.php');

$errors = [];

// **Role Check: Ensure the user has 'accounts' role or 'admin'**
if (!checkRole(['accounts'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to view this page.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Manage Accounts</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

/**
 * Renders a table based on provided data and column definitions.
 *
 * @param array       $data           The data to display in the table.
 * @param array       $columns        An array of columns with 'header' and 'field' keys.
 * @param array       $actions        (Optional) An array of actions with 'label', 'url', 'field', 'class', and 'required_role' keys.
 * @param array       $moduleInfo     (Optional) An array indexed by module_id with 'module_name' and 'redirect_accounts'.
 * @param string|null $highlightCode  If set, will highlight the row whose 'code' matches this value.
 */
function renderTable($data, $columns, $actions = [], $moduleInfo = [], $highlightCode = null) {
    echo '<table>';
    echo '<thead><tr>';
    foreach ($columns as $column) {
        $header = $column['header'] ?? '';
        echo '<th>' . htmlspecialchars((string)$header) . '</th>';
    }
    // Always have an actions column
    echo '<th>Actions</th>';
    echo '</tr></thead>';
    echo '<tbody>';

    foreach ($data as $row) {
        // Build an ID for the row so we can scroll to it or link to it via #account-row-<code>
        $rowId = 'account-row-' . htmlspecialchars($row['code'] ?? '');

        // Determine if we should highlight this row
        $extraClass = '';
        if ($highlightCode !== null && isset($row['code']) && $row['code'] === $highlightCode) {
            $extraClass = 'highlighted-row';
        } elseif (!empty($row['row_class'])) {
            $extraClass = htmlspecialchars($row['row_class']);
        }

        echo '<tr id="' . $rowId . '"' . ($extraClass ? ' class="' . $extraClass . '"' : '') . '>';

        foreach ($columns as $column) {
            $cellData = $row[$column['field']] ?? '';

            // Convert account_type to uppercase
            if ($column['field'] === 'account_type') {
                $cellData = strtoupper($cellData);
            }
            if ($column['field'] === 'is_system') {
                $cellData = $cellData ? 'Yes' : 'No';
            }
            if ($column['field'] === 'parent') {
                $cellData = $cellData ?: '';
            }
            if ($column['field'] === 'name' && isset($row['depth'])) {
                // Indent child rows
                $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $row['depth']);
                $cellData = $indent . htmlspecialchars((string)$cellData);
            } else {
                $cellData = htmlspecialchars((string)$cellData);
            }
            echo '<td>' . $cellData . '</td>';
        }

        // Actions cell
        echo '<td>';
        // If module_id == 0, show edit/delete actions if applicable
        if (empty($row['module_id']) || $row['module_id'] == 0) {
            if (!isset($row['is_system']) || !$row['is_system']) {
                // Loop through each defined action
                foreach ($actions as $action) {
                    if (hasRole($action['required_role'])) {
                        $actionFieldValue = isset($row[$action['field']]) ? urlencode($row[$action['field']]) : '';

                        // If label is "Delete", add an onClick to store scroll pos
                        $onClick = '';
                        if (strtolower($action['label']) === 'delete') {
                            $onClick = 'onclick="storeScrollPosition()"';
                        }

                        echo '<a class="' . htmlspecialchars($action['class'] ?? 'button') . '" href="'
                             . htmlspecialchars($action['url']) . '?id=' . $actionFieldValue . '" '
                             . $onClick . '>'
                             . htmlspecialchars($action['label'])
                             . '</a> ';
                    }
                }
            }
        }

        // If module_id != 0, show the module button (if available)
        if (!empty($row['module_id']) && isset($moduleInfo[$row['module_id']])) {
            $mInfo = $moduleInfo[$row['module_id']];
            if (!empty($mInfo['redirect_accounts'])) {
                $moduleName = htmlspecialchars($mInfo['module_name']);
                $redirectUrl = htmlspecialchars($mInfo['redirect_accounts']);
                echo '<a class="button" href="' . $redirectUrl . '">' . $moduleName . '</a> ';
            }
        }

        echo '</td>';
        echo '</tr>';
    }

    echo '</tbody>';
    echo '</table>';
}

// Fetch and process accounts
$accounts = listAccounts($pdo);
if (empty($accounts)) {
    $_SESSION['error_message'] = 'No accounts found in the database.';
}

// We want them grouped by type in order: Asset, Liability, Equity, Revenue, Expense
$accountTypeOrder = ['asset', 'liability', 'equity', 'revenue', 'expense'];

// Decorate each account with row_class, check for 'parent' == 'N/A'
foreach ($accounts as &$acct) {
    switch (strtolower($acct['account_type'])) {
        case 'asset':
            $acct['row_class'] = 'account-asset';
            break;
        case 'liability':
            $acct['row_class'] = 'account-liability';
            break;
        case 'equity':
            $acct['row_class'] = 'account-equity';
            break;
        case 'revenue':
            $acct['row_class'] = 'account-revenue';
            break;
        case 'expense':
            $acct['row_class'] = 'account-expense';
            break;
        default:
            $acct['row_class'] = '';
            break;
    }
    $acct['is_system'] = isset($acct['is_system']) ? (int)$acct['is_system'] : 0;
    if (!empty($acct['parent']) && strtolower($acct['parent']) === 'n/a') {
        $acct['parent'] = null;
    }
}
unset($acct);

// We'll group each type, sort by code ascending, build a tree, then flatten in code order
$groupedByType = [];
foreach ($accountTypeOrder as $type) {
    // Filter
    $groupedByType[$type] = array_filter($accounts, function($a) use ($type) {
        return strtolower($a['account_type']) === $type;
    });
    // Sort them by code ascending
    usort($groupedByType[$type], function($a, $b) {
        return strcmp($a['code'], $b['code']);
    });
}

$sortedAccounts = [];
foreach ($accountTypeOrder as $type) {
    $typeAccounts = array_values($groupedByType[$type]); // reindex
    // Build the tree from these code-sorted accounts
    $tree = buildAccountTree($typeAccounts, null);
    // Then flatten
    traverseAccountTree($tree, $sortedAccounts);
}

// Gather module_ids
$moduleIds = [];
foreach ($sortedAccounts as $acct) {
    if (!empty($acct['module_id']) && $acct['module_id'] != 0) {
        $moduleIds[] = (int)$acct['module_id'];
    }
}
$moduleIds = array_unique($moduleIds);

// Fetch module info
$moduleInfo = [];
if (!empty($moduleIds)) {
    $inPlaceholders = implode(',', array_fill(0, count($moduleIds), '?'));
    $stmt = $pdo->prepare("SELECT rowid as id, module_name, redirect_accounts FROM modules WHERE rowid IN ($inPlaceholders)");
    $stmt->execute($moduleIds);
    while ($m = $stmt->fetch(\PDO::FETCH_ASSOC)) {
        $moduleInfo[(int)$m['id']] = [
            'module_name' => $m['module_name'],
            'redirect_accounts' => $m['redirect_accounts']
        ];
    }
}

// Define table columns
$columns = [
    ['header' => 'Code', 'field' => 'code'],
    ['header' => 'Name', 'field' => 'name'],
    ['header' => 'Description', 'field' => 'description'],
    ['header' => 'Account Type', 'field' => 'account_type'],
    ['header' => 'Parent', 'field' => 'parent']
];

// Define actions with required roles
$actions = [
    ['label' => 'Edit', 'url' => 'edit_account.php', 'field' => 'code', 'class' => 'button edit-button', 'required_role' => 'accounts_edit'],
    ['label' => 'Delete', 'url' => 'delete_account.php', 'field' => 'code', 'class' => 'button delete-button', 'required_role' => 'accounts_delete']
];

// Capture highlight_code param if present
$highlightCode = isset($_GET['highlight_code']) ? $_GET['highlight_code'] : null;
?>
<h1>Manage Accounts</h1>
<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>
<div class="button-container">
    <?php if (hasRole('accounts_add')): ?>
        <a href="add_account.php" class="button">Add New Account</a>
    <?php endif; ?>
    <?php if (hasRole('accounts_import')): ?>
        <a href="import_accounts.php" class="button">Import Accounts</a>
    <?php endif; ?>
    <?php if (hasRole('accounts_export')): ?>
        <a href="export_accounts.php" class="button">Export Accounts</a>
    <?php endif; ?>
</div>
<?php
if (isset($_SESSION['success_message'])):
?>
    <div class="modal success">
        <span class="close-modal">&times;</span>
        <p><?php echo htmlspecialchars($_SESSION['success_message']); ?></p>
    </div>
<?php
unset($_SESSION['success_message']);
endif;

if (isset($_SESSION['error_message'])):
?>
    <div class="modal error">
        <span class="close-modal">&times;</span>
        <p><?php echo htmlspecialchars($_SESSION['error_message']); ?></p>
    </div>
<?php
unset($_SESSION['error_message']);
endif;

if (isset($_SESSION['import_message'])):
?>
    <div class="modal import-result">
        <span class="close-modal">&times;</span>
        <p><?php echo nl2br(htmlspecialchars($_SESSION['import_message'])); ?></p>
    </div>
<?php
unset($_SESSION['import_message']);
endif;

if (!empty($sortedAccounts)) {
    renderTable($sortedAccounts, $columns, $actions, $moduleInfo, $highlightCode);
} else {
    echo '<p>No accounts to display. Please add some accounts.</p>';
}
require_once('components/footer.php');
?>

<!-- OPTIONAL: Add some styling for the highlighted row -->
<style>
.highlighted-row {
  background-color: #fffa9e !important; /* a light yellow highlight */
}
.account-asset {
  /* or color-coded styling for each type if you wish */
}
.account-liability {}
.account-equity {}
.account-revenue {}
.account-expense {}
</style>

<script>
// Optionally, you can scroll into view if highlight_code is present
document.addEventListener('DOMContentLoaded', function() {
    const urlParams = new URLSearchParams(window.location.search);

    // 1) Restore scroll position from localStorage (if any)
    const savedScroll = localStorage.getItem('accountScrollPos');
    if (savedScroll) {
        localStorage.removeItem('accountScrollPos');
        // Convert to integer and scroll
        window.scrollTo(0, parseInt(savedScroll, 10));
    }

    // 2) Scroll to highlight_code if present
    if (urlParams.has('highlight_code')) {
        const code = urlParams.get('highlight_code');
        const targetRow = document.getElementById('account-row-' + code);
        if (targetRow) {
            targetRow.scrollIntoView({ behavior: 'smooth', block: 'center' });
        }
    }
});

// 3) Store scroll pos in localStorage on "Delete"
function storeScrollPosition() {
  localStorage.setItem('accountScrollPos', window.scrollY);
}
</script>
