<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);
require_once('../core/accounts.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

$errors = [];

// Capture old form values to repopulate if there's an error
$codeValue = isset($_POST['code']) ? $_POST['code'] : '';
$nameValue = isset($_POST['name']) ? $_POST['name'] : '';
$descriptionValue = isset($_POST['description']) ? $_POST['description'] : '';
$accountTypeValue = isset($_POST['account_type']) ? $_POST['account_type'] : '';
$parentValue = isset($_POST['parent']) ? $_POST['parent'] : '';

// Add role check for 'accounts_add'
if (!checkRole(['accounts_add'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Add New Account</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

/**
 * Build a hierarchical tree of accounts **in ascending code order**,
 * but if an account has `module_id != 0`, we do **not** recurse into children.
 *
 * @param array       $accounts   Array of accounts (already code-sorted).
 * @param string|null $parentCode Parent code (null for top-level).
 * @return array Hierarchical tree
 */
function buildAccountTree(array $accounts, string $parentCode = null): array {
    // 1) Find direct children of $parentCode
    $children = array_filter($accounts, function($acc) use ($parentCode) {
        return $acc['parent'] === $parentCode;
    });

    // 2) They should already be code-sorted at this stage, but you can re-check:
    usort($children, function($a, $b) {
        return strcmp($a['code'], $b['code']);
    });

    $branch = [];
    foreach ($children as $child) {
        // If this child has module_id != 0, we do not build further children
        // We simply add the child as a leaf node with no children.
        if (!empty($child['module_id']) && $child['module_id'] != 0) {
            $branch[] = $child;
        } else {
            // Recurse normally
            $child['children'] = buildAccountTree($accounts, $child['code']);
            $branch[] = $child;
        }
    }
    return $branch;
}

/**
 * Flatten the tree to a list with 'depth' info.
 * We do not skip children here unless they've been pruned already above.
 *
 * @param array $tree
 * @param array &$sortedList
 * @param int   $depth
 */
function traverseAccountTree(array $tree, array &$sortedList, int $depth = 0): void {
    foreach ($tree as $node) {
        $node['depth'] = $depth;
        $sortedList[] = $node;

        // Recurse if children exist
        if (!empty($node['children'])) {
            traverseAccountTree($node['children'], $sortedList, $depth + 1);
        }
    }
}

/**
 * Render account options for a <select> in hierarchical code-sorted order.
 * If an account has module_id != 0, we do not recurse further *and* disable the option.
 *
 * @param array $tree
 * @param int   $level
 * @return void
 */
function renderAccountOptions(array $tree, int $level = 0): void {
    foreach ($tree as $account) {
        $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $level);

        switch (strtolower($account['account_type'])) {
            case 'asset':
                $color = '#1E90FF';
                break;
            case 'liability':
                $color = '#FF4500';
                break;
            case 'equity':
                $color = '#32CD32';
                break;
            case 'revenue':
                $color = '#FFD700';
                break;
            case 'expense':
                $color = '#FF6347';
                break;
            default:
                $color = '#000000';
        }

        $codeSafe  = htmlspecialchars($account['code']);
        $nameSafe  = htmlspecialchars($account['name']);
        $typeSafe  = htmlspecialchars(strtolower($account['account_type']));

        // *** NEW: If module_id != 0, disable this <option> so it can't be selected.
        $disabled = (!empty($account['module_id']) && $account['module_id'] != 0) ? ' disabled' : '';

        echo '<option value="' . $codeSafe . '" data-account-type="' . $typeSafe . '" style="color: ' . $color . ';"' . $disabled . '>';
        echo $indent . $nameSafe . ' (' . $codeSafe . ')';
        echo '</option>';

        // If module_id != 0, do NOT recurse further
        if (!empty($account['module_id']) && $account['module_id'] != 0) {
            continue;
        }
        // Otherwise, render children
        if (!empty($account['children'])) {
            renderAccountOptions($account['children'], $level + 1);
        }
    }
}

// Handle form submission
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $code = trim($_POST['code']);
    $name = trim($_POST['name']);
    $description = trim($_POST['description']);
    $account_type = trim($_POST['account_type']);
    $parent = trim($_POST['parent']) ?: null;
    $isSystem = false;

    if (empty($code) || empty($name) || empty($account_type)) {
        $errors[] = 'Code, Name, and Account Type are required.';
    }
    if (empty($parent)) {
        $errors[] = 'Parent Account is required.';
    }

    $validAccountTypes = ['asset', 'liability', 'equity', 'revenue', 'expense'];
    if (!in_array(strtolower($account_type), $validAccountTypes)) {
        $errors[] = 'Invalid Account Type selected.';
    }

    if ($parent !== null) {
        $parentAccount = getAccount($pdo, $parent);
        if (!$parentAccount) {
            $errors[] = "Parent account [$parent] does not exist.";
        } else {
            // If parent is disabled (module_id != 0), we should not allow selection
            if (!empty($parentAccount['module_id']) && $parentAccount['module_id'] != 0) {
                $errors[] = 'The selected Parent Account cannot be chosen because it is linked to a module.';
            }
        }
    }

    if (empty($errors)) {
        $existingAccount = getAccount($pdo, $code);
        if ($existingAccount) {
            $errors[] = "An account with code [$code] already exists.";
        }
    }

    if (empty($errors)) {
        $result = createAccount($pdo, $code, $name, $account_type, $_SESSION['user_id'], $description, $parent, $isSystem);
        if ($result) {
            // *** Here we redirect with highlight_code param so we highlight the new account. ***
            $_SESSION['success_message'] = "Account [$code] $name has been successfully added!";
            header('Location: accounts.php?highlight_code=' . urlencode($code) . '#account-row-' . urlencode($code));
            exit;
        } else {
            $errors[] = 'Failed to create account. Please try again.';
        }
    }

    // If we reach here, there are errors; the form will be re-displayed with old values
}

$accountTypes = [
    'asset' => 'Asset',
    'liability' => 'Liability',
    'equity' => 'Equity',
    'revenue' => 'Revenue',
    'expense' => 'Expense'
];

$accounts = listAccounts($pdo);
$accountTree = buildAccountTree($accounts);

// Gather module_ids
$moduleIds = [];
foreach ($accounts as $acct) {
    if (!empty($acct['module_id']) && $acct['module_id'] != 0) {
        $moduleIds[] = (int)$acct['module_id'];
    }
}
$moduleIds = array_unique($moduleIds);

// Fetch module info
$moduleInfo = [];
if (!empty($moduleIds)) {
    $inPlaceholders = implode(',', array_fill(0, count($moduleIds), '?'));
    $stmt = $pdo->prepare("SELECT rowid as id, module_name, redirect_accounts FROM modules WHERE rowid IN ($inPlaceholders)");
    $stmt->execute($moduleIds);
    while ($m = $stmt->fetch(PDO::FETCH_ASSOC)) {
        $moduleInfo[(int)$m['id']] = [
            'module_name' => $m['module_name'],
            'redirect_accounts' => $m['redirect_accounts']
        ];
    }
}
?>
<h1>Add New Account</h1>

<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>

<?php if (isset($_SESSION['success_message'])): ?>
<div class="success">
    <p><?php echo htmlspecialchars($_SESSION['success_message']); ?></p>
</div>
<?php unset($_SESSION['success_message']); ?>
<?php endif; ?>

<div class="form-container">
    <form action="add_account.php" method="POST">
        <?php
        // Use the variables we captured above to repopulate fields
        renderInput('text', 'code', 'Code', $codeValue, true);
        renderInput('text', 'name', 'Name', $nameValue, true);
        renderTextarea('description', 'Description', $descriptionValue);
        renderSelect('account_type', 'Account Type', $accountTypes, $accountTypeValue, true);

        echo '<label for="parent">Parent Account</label>';
        echo '<select id="parent" name="parent" required>';
        echo '<option value="">Select Parent Account</option>';
        renderAccountOptions($accountTree, 0, $moduleInfo, $parentValue);
        echo '</select>';
        ?>
        <div class="button-group">
            <button type="submit">Add Account</button>
            <a href="accounts.php" class="button">Back to Manage Accounts</a>
        </div>
    </form>
</div>

<?php require_once('components/footer.php'); ?>
