<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('../core/users.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

// Ensure $pdo is available. If not, include the initialization file.
if (!isset($pdo)) {
    require_once('../core/utilities.php'); // Adjust the path as needed
}

$errors = [];
$success = '';

// Check if the user has the 'header_menu_edit' role
if (!checkRole(['header_menu_edit'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to edit the menu.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Menu Builder</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Handle form submission for adding a new menu item
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['action']) && $_POST['action'] === 'add') {
        $name = trim($_POST['name']);
        $url = trim($_POST['url']);
        $parent_id = !empty($_POST['parent_id']) ? (int)$_POST['parent_id'] : null;
        $order_num = isset($_POST['order_num']) ? (int)$_POST['order_num'] : 0;
        $is_dropdown = isset($_POST['is_dropdown']) ? 1 : 0;
        $is_visible = isset($_POST['is_visible']) ? 1 : 0;
        $required_role = trim($_POST['required_role']) ?: null;

        // Validation
        if (empty($name)) {
            $errors[] = 'Name is required.';
        }

        if ($is_dropdown && !empty($url)) {
            $errors[] = 'Dropdown menus should not have a URL.';
        }

        if (!$is_dropdown && empty($url)) {
            $errors[] = 'URL is required for menu items.';
        }

        // Optional: Prevent assigning a menu item as its own parent
        // This is more relevant during editing, but good to check
        if ($parent_id) {
            // Check if parent_id exists and is a dropdown
            $stmt = $pdo->prepare("SELECT is_dropdown FROM menu_items WHERE id = ?");
            $stmt->execute([$parent_id]);
            $parentMenu = $stmt->fetch(PDO::FETCH_ASSOC);
            if (!$parentMenu || !$parentMenu['is_dropdown']) {
                $errors[] = 'Selected parent menu is invalid.';
            }
        }

        if (empty($errors)) {
            // Insert new menu item
            $stmt = $pdo->prepare("INSERT INTO menu_items (name, url, parent_id, order_num, is_dropdown, is_visible, required_role) VALUES (?, ?, ?, ?, ?, ?, ?)");
            $result = $stmt->execute([$name, $url ?: null, $parent_id, $order_num, $is_dropdown, $is_visible, $required_role]);

            if ($result) {
                $success = 'Menu item added successfully.';
                // Optionally, redirect back to menu_builder.php
                header('Location: menu_builder.php');
                exit;
            } else {
                $errors[] = 'Failed to add menu item.';
            }
        }
    }
}

// Fetch all dropdown menu items to populate the Parent Menu dropdown
$stmt = $pdo->prepare("SELECT * FROM menu_items WHERE is_dropdown = 1 ORDER BY name ASC");
$stmt->execute();
$dropdownMenus = $stmt->fetchAll(PDO::FETCH_ASSOC);

// Fetch roles for the required_role field
$roles = getAllRoles($pdo);
?>
<h1>Add New Menu Item</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
    <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success); ?></p>
</div>
<?php endif; ?>

<!-- Add Menu Item Form -->
<div class="form-container">
    <form action="add_menu_builder.php" method="POST">
        <input type="hidden" name="action" value="add">
        <?php
        renderInput('text', 'name', 'Name', '', true);
        renderInput('text', 'url', 'URL');
        ?>
        <label for="parent_id">Parent Menu</label>
        <select id="parent_id" name="parent_id">
            <option value="">None</option>
            <?php foreach ($dropdownMenus as $menu): ?>
                <option value="<?php echo htmlspecialchars($menu['id']); ?>"><?php echo htmlspecialchars($menu['name']); ?></option>
            <?php endforeach; ?>
        </select>
        <?php
        renderInput('number', 'order_num', 'Order Number', '0');
        renderCheckbox('is_dropdown', 'Is Dropdown');
        renderCheckbox('is_visible', 'Is Visible', true);
        ?>
        <label for="required_role">Required Role</label>
        <select id="required_role" name="required_role">
            <option value="">None</option>
            <?php foreach ($roles as $role): ?>
                <option value="<?php echo htmlspecialchars($role['name']); ?>"><?php echo htmlspecialchars($role['name']); ?></option>
            <?php endforeach; ?>
        </select>
        <div class="button-group">
            <button type="submit">Add Menu Item</button>
            <a href="menu_builder.php" class="button">Back to Menu Builder</a>
        </div>
    </form>
</div>

<?php require_once('components/footer.php'); ?>

