<?php
require_once('../core/users.php');
require_once('components/auth.php'); // session_start() is called here
require_once('components/header.php');

// Initialize session error, success, and form data messages if not set
if (!isset($_SESSION['user_error'])) {
    $_SESSION['user_error'] = [];
}
if (!isset($_SESSION['user_success'])) {
    $_SESSION['user_success'] = '';
}
if (!isset($_SESSION['form_data'])) {
    $_SESSION['form_data'] = [];
}

$rolesList = getAllRoles($pdo);

// Handle form submission
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // Trim and retrieve form inputs
    $username = trim($_POST['username']);
    $email = trim($_POST['email']);
    $isAuth = isset($_POST['is_auth']) ? 1 : 0;
    $roles = $_POST['roles'] ?? [];

    // Validate username and email
    $errors = [];

    if (empty($username) || empty($email)) {
        $errors[] = 'Username and Email are required.';
    } else {
        // Additional validation for username
        if (!preg_match('/^[a-zA-Z0-9_]+$/', $username)) {
            $errors[] = 'Username can only contain letters, numbers, and underscores.';
        }
        if (strlen($username) > 50) {
            $errors[] = 'Username must be less than 50 characters.';
        }

        // Validate email format
        if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
            $errors[] = 'Please provide a valid email address.';
        }
        if (strlen($email) > 100) {
            $errors[] = 'Email must be less than 100 characters.';
        }
    }

    // Proceed only if no validation errors
    if (empty($errors)) {
        // Generate a secure random password (10 characters)
        try {
            $randomBytes = random_bytes(5); // 5 bytes = 10 hex characters
            $generatedPassword = bin2hex($randomBytes);
        } catch (Exception $e) {
            $errors[] = 'Failed to generate a secure password.';
            logError($pdo, 'Password Generation Error: ' . $e->getMessage());
        }

        if (empty($errors)) {
            // Create or update user with the generated password
            $createResult = createUser($pdo, $username, $generatedPassword, $email, $_SESSION['user_id']);
            if ($createResult['success']) {
                $userId = $createResult['user_id'];

                // Remove existing roles if reactivating or updating
                $stmt = $pdo->prepare("DELETE FROM user_roles WHERE user_id = ?");
                $stmt->execute([$userId]);

                // Assign roles to the user
                $assignRolesSuccess = true;
                foreach ($roles as $roleId) {
                    $assignResult = assignUserRole($pdo, $userId, $roleId, $_SESSION['user_id']);
                    if (!$assignResult) {
                        $assignRolesSuccess = false;
                        $errors[] = 'Failed to assign one or more roles.';
                        break;
                    }
                }

                if ($assignRolesSuccess) {
                    // Store success message in session
                    $_SESSION['user_success'] = ($createResult['message'] === 'User created successfully.' ? 'User created' : 'User updated') . ' successfully. Generated password: <strong>' . htmlspecialchars($generatedPassword) . '</strong>';
                    // Clear any previously stored form data
                    $_SESSION['form_data'] = [];
                    header('Location: add_user.php'); // Redirect to prevent form resubmission
                    exit;
                } else {
                    // If role assignment failed, log the error
                    logError($pdo, 'Role Assignment Failed for User ID: ' . $userId);
                }
            } else {
                // Store specific error message from createUser
                $errors[] = $createResult['message'];
            }
        }
    }

    // If there are errors, store them and the form data in the session
    if (!empty($errors)) {
        $_SESSION['user_error'] = $errors;
        // Store the submitted form data to repopulate the form
        $_SESSION['form_data'] = [
            'username' => $username,
            'email' => $email,
            'is_auth' => $isAuth,
            'roles' => $roles
        ];
        header('Location: add_user.php'); // Redirect to display errors and repopulate the form
        exit;
    }
}
?>
<h1>Add User</h1>

<?php
// Initialize flags to check if any message exists
$hasSuccess = false;
$hasError = false;

// Display success message if exists
if (!empty($_SESSION['user_success'])):
    $hasSuccess = true;
?>
    <div class="success">
        <p><?php echo $_SESSION['user_success']; ?></p>
        <a href="users.php" class="button">Back to Users</a>
    </div>
    <?php
    // Clear the success message after displaying
    unset($_SESSION['user_success']);
endif;

// Display error messages if exists
if (!empty($_SESSION['user_error'])):
    $hasError = true;
?>
    <div class="alert">
        <?php foreach ($_SESSION['user_error'] as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php
    // Clear the error messages after displaying
    unset($_SESSION['user_error']);
endif;

// Determine if form should be displayed
// Show the form if there's no success message
// Always show the form on errors to allow corrections
$showForm = !$hasSuccess;
?>
<?php if ($showForm): ?>
    <div class="form-container">
        <form action="add_user.php" method="POST">
            <label for="username">Username</label>
            <input type="text" id="username" name="username"
                value="<?php
                    echo isset($_SESSION['form_data']['username']) ? htmlspecialchars($_SESSION['form_data']['username']) : (isset($username) ? htmlspecialchars($username) : '');
                ?>"
                required>

            <label for="email">Email</label>
            <input type="email" id="email" name="email"
                value="<?php
                    echo isset($_SESSION['form_data']['email']) ? htmlspecialchars($_SESSION['form_data']['email']) : (isset($email) ? htmlspecialchars($email) : '');
                ?>"
                required>

            <label for="is_auth">Authorize User</label>
            <input type="checkbox" id="is_auth" name="is_auth"
                <?php
                    $isAuthValue = isset($_SESSION['form_data']['is_auth']) ? $_SESSION['form_data']['is_auth'] : (isset($isAuth) ? $isAuth : 0);
                    echo $isAuthValue ? 'checked' : '';
                ?>>

            <label>Roles</label>
            <?php
                // Retrieve previously selected roles
                $selectedRoles = isset($_SESSION['form_data']['roles']) ? $_SESSION['form_data']['roles'] : (isset($roles) ? $roles : []);
                foreach ($rolesList as $role):
            ?>
                <label>
                    <input type="checkbox" name="roles[]" value="<?php echo htmlspecialchars($role['id']); ?>"
                        <?php echo in_array($role['id'], $selectedRoles) ? 'checked' : ''; ?>>
                    <?php echo htmlspecialchars($role['name']); ?>
                </label><br>
            <?php endforeach; ?>

            <div class="button-group">
                <button type="submit">Add User</button>
                <a href="users.php" class="button">Back</a>
            </div>
        </form>
    </div>
    <?php
        // Clear the form data after repopulating the form
        unset($_SESSION['form_data']);
    ?>
<?php endif; ?>

<?php require_once('components/footer.php'); ?>
