<?php
require_once('../core/audit_logs.php');
require_once('components/auth.php');
require_once('components/header.php');

$errors = [];
$success = [];

// Check if the user has the necessary role to view audit logs
if (!checkRole(['audit_logs'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to view audit logs.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Audit Logs</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Handle POST requests (e.g., delete action)
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['delete_audit_log_id']) && checkRole(['audit_logs_delete'])) {
        $auditLogId = (int)$_POST['delete_audit_log_id'];
        if ($auditLogId > 0) {
            $result = deleteAuditLog($pdo, $auditLogId);
            if ($result) {
                $success[] = 'Audit log has been deleted successfully.';
            } else {
                $errors[] = 'Failed to delete audit log.';
            }
        } else {
            $errors[] = 'Invalid audit log ID.';
        }
    }
}

// Define allowed page sizes
$allowedPageSizes = [10, 20, 50, 100];

// Capture page size from GET parameters, default to 20 if not set or invalid
$pageSize = isset($_GET['page_size']) && in_array((int)$_GET['page_size'], $allowedPageSizes) ? (int)$_GET['page_size'] : 20;

// Ensure $pageSize is within allowed values
if (!in_array($pageSize, $allowedPageSizes)) {
    $pageSize = 20;
}

// Pagination
$page = isset($_GET['page']) && is_numeric($_GET['page']) && $_GET['page'] > 0 ? (int)$_GET['page'] : 1;

// Sorting parameters
$sortBy = isset($_GET['sort_by']) ? $_GET['sort_by'] : 'audit_logs.created_at';
$sortOrder = isset($_GET['sort_order']) && strtolower($_GET['sort_order']) === 'asc' ? 'ASC' : 'DESC';

// Define allowed sort columns to prevent SQL injection
$allowedSortColumns = ['audit_logs.id', 'audit_logs.user_id', 'audit_logs.action', 'audit_logs.entity', 'audit_logs.entity_id', 'audit_logs.created_at'];
if (!in_array($sortBy, $allowedSortColumns)) {
    $sortBy = 'audit_logs.created_at';
}

// Fetch all filter inputs from GET
$filters = [];
$appliedFilters = [];

if (isset($_GET['user_id']) && $_GET['user_id'] !== '') {
    $filters['user_id'] = $_GET['user_id'];
    $appliedFilters[] = 'User ID: ' . htmlspecialchars($_GET['user_id']);
}
if (isset($_GET['action']) && $_GET['action'] !== '') {
    $filters['action'] = $_GET['action'];
    $appliedFilters[] = 'Action: ' . htmlspecialchars($_GET['action']);
}
if (isset($_GET['entity']) && $_GET['entity'] !== '') {
    $filters['entity'] = $_GET['entity'];
    $appliedFilters[] = 'Entity: ' . htmlspecialchars($_GET['entity']);
}
if (isset($_GET['entity_id']) && $_GET['entity_id'] !== '') {
    $filters['entity_id'] = $_GET['entity_id'];
    $appliedFilters[] = 'Entity ID: ' . htmlspecialchars($_GET['entity_id']);
}
if (isset($_GET['additional_info']) && $_GET['additional_info'] !== '') {
    $filters['additional_info'] = $_GET['additional_info'];
    $appliedFilters[] = 'Additional Info: ' . htmlspecialchars($_GET['additional_info']);
}
if (isset($_GET['start_date']) && $_GET['start_date'] !== '') {
    $filters['start_date'] = $_GET['start_date'];
    $appliedFilters[] = 'Start Date: ' . htmlspecialchars($_GET['start_date']);
}
if (isset($_GET['end_date']) && $_GET['end_date'] !== '') {
    $filters['end_date'] = $_GET['end_date'];
    $appliedFilters[] = 'End Date: ' . htmlspecialchars($_GET['end_date']);
}

$filters['sort_by'] = $sortBy;
$filters['sort_order'] = $sortOrder;

// Fetch audit logs with filters and pagination
$auditLogsData = listAuditLogs($pdo, $filters, $page, $pageSize);
$auditLogs = $auditLogsData['audit_logs'] ?? [];
$totalRecords = $auditLogsData['total_records'] ?? 0;
$totalPages = $auditLogsData['total_pages'] ?? 1;

// Function to build query string with updated parameters
function buildQuery($params, $exclude = []) {
    foreach ($exclude as $key) {
        unset($params[$key]);
    }
    return http_build_query($params);
}
?>
<h1>Audit Logs</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
    <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if (!empty($success)): ?>
<div class="success">
    <?php foreach ($success as $message): ?>
    <p><?php echo htmlspecialchars($message); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>

<!-- Button Container with Toggle Filters Button -->
<div class="button-container">
    <?php if (checkRole(['audit_logs_add'])): ?>
        <a href="add_audit_logs.php" class="button">Add Audit Log</a>
    <?php endif; ?>
    <button type="button" class="button button-right" id="toggle-filters-button">Toggle Filters</button>
</div>

<!-- Filter Form -->
<form class="filter-form" method="GET" action="audit_logs.php" id="filter-form">
    <div>
        <label for="user_id">User ID:</label>
        <input type="number" id="user_id" name="user_id" value="<?php echo htmlspecialchars($_GET['user_id'] ?? ''); ?>">
    </div>
    <div>
        <label for="action">Action:</label>
        <input type="text" id="action" name="action" value="<?php echo htmlspecialchars($_GET['action'] ?? ''); ?>">
    </div>
    <div>
        <label for="entity">Entity:</label>
        <input type="text" id="entity" name="entity" value="<?php echo htmlspecialchars($_GET['entity'] ?? ''); ?>">
    </div>
    <div>
        <label for="entity_id">Entity ID:</label>
        <input type="number" id="entity_id" name="entity_id" value="<?php echo htmlspecialchars($_GET['entity_id'] ?? ''); ?>">
    </div>
    <div>
        <label for="additional_info">Additional Info:</label>
        <input type="text" id="additional_info" name="additional_info" value="<?php echo htmlspecialchars($_GET['additional_info'] ?? ''); ?>">
    </div>
    <div>
        <label for="start_date">Start Date:</label>
        <input type="date" id="start_date" name="start_date" value="<?php echo htmlspecialchars($_GET['start_date'] ?? ''); ?>">
    </div>
    <div>
        <label for="end_date">End Date:</label>
        <input type="date" id="end_date" name="end_date" value="<?php echo htmlspecialchars($_GET['end_date'] ?? ''); ?>">
    </div>
    <div style="margin-top: 10px;">
        <button type="submit" class="button">Filter</button>
        <button type="button" class="button" id="reset-filters-button">Reset</button>
    </div>
</form>

<!-- Applied Filters Message -->
<div class="applied-filters">
    <?php
    if (!empty($appliedFilters)) {
        echo 'Applied Filters: ' . implode(', ', $appliedFilters);
    } else {
        echo 'No filters applied.';
    }
    ?>
</div>

<!-- Audit Logs Table -->
<table>
    <thead>
        <tr>
            <?php
            // Define table headers with sorting links
            $headers = [
                'users.username' => 'User',
                'audit_logs.action' => 'Action',
                'audit_logs.entity' => 'Entity',
                'audit_logs.entity_id' => 'Entity ID',
                'audit_logs.additional_info' => 'Additional Info',
                'audit_logs.created_at' => 'Timestamp'
            ];

            foreach ($headers as $key => $label) {
                $currentSort = ($sortBy === $key) ? 'sort-' . strtolower($sortOrder) : '';
                echo '<th class="' . $currentSort . '">';
                echo '<a href="?' . buildQuery(array_merge($_GET, [
                    'sort_by' => $key,
                    'sort_order' => ($sortBy === $key && $sortOrder === 'ASC') ? 'desc' : 'asc',
                    'page' => 1
                ])) . '">';
                echo htmlspecialchars($label);
                echo '</a>';
                echo '</th>';
            }
            if (checkRole(['audit_logs_delete'])) {
                echo '<th>Actions</th>';
            }
            ?>
        </tr>
    </thead>
    <tbody>
        <?php if (!empty($auditLogs)): ?>
            <?php foreach ($auditLogs as $log): ?>
            <tr>
                <td><?php echo htmlspecialchars($log['username'] ?? 'System'); ?></td>
                <td><?php echo htmlspecialchars($log['action']); ?></td>
                <td><?php echo htmlspecialchars($log['entity']); ?></td>
                <td><?php echo htmlspecialchars($log['entity_id']); ?></td>
                <td><?php echo htmlspecialchars($log['additional_info']); ?></td>
                <td><?php echo htmlspecialchars($log['created_at']); ?></td>
                        <!-- Other columns -->
        <?php if (checkRole(['audit_logs_delete'])): ?>
        <td>
            <?php if ((int)$log['is_system'] === 0): ?>
            <form action="audit_logs.php" method="POST" style="display:inline;">
                <input type="hidden" name="delete_audit_log_id" value="<?php echo htmlspecialchars($log['id']); ?>">
                <button type="submit" class="button delete-button" onclick="return confirm('Are you sure you want to delete this audit log?');">Delete</button>
            </form>
            <?php else: ?>
            <!-- No delete option for system logs -->
            <span style="color: grey;">System Log</span>
            <?php endif; ?>
        </td>
        <?php endif; ?>
            </tr>
            <?php endforeach; ?>
        <?php else: ?>
        <tr>
            <td colspan="<?php echo checkRole(['audit_logs_delete']) ? '7' : '6'; ?>">No audit logs found.</td>
        </tr>
        <?php endif; ?>
    </tbody>
</table>

<!-- Pagination Controls and Page Size Dropdown -->
<div class="pagination-controls">
    <!-- Pagination Links -->
    <div class="pagination">
        <?php
        // Define the range of pages to show
        $range = 2; // Number of pages to show on either side of the current page
        $start = max(1, $page - $range);
        $end = min($totalPages, $page + $range);

        // Previous Page Link
        if ($page > 1) {
            echo '<a href="?' . buildQuery(array_merge($_GET, ['page' => $page - 1])) . '">&laquo; Prev</a>';
        }

        // First Page Link
        if ($start > 1) {
            echo '<a href="?' . buildQuery(array_merge($_GET, ['page' => 1])) . '">1</a>';
            if ($start > 2) {
                echo '<span>...</span>';
            }
        }

        // Page Number Links
        for ($i = $start; $i <= $end; $i++) {
            if ($i == $page) {
                echo '<a href="?' . buildQuery(array_merge($_GET, ['page' => $i])) . '" class="active">' . $i . '</a>';
            } else {
                echo '<a href="?' . buildQuery(array_merge($_GET, ['page' => $i])) . '">' . $i . '</a>';
            }
        }

        // Last Page Link
        if ($end < $totalPages) {
            if ($end < $totalPages - 1) {
                echo '<span>...</span>';
            }
            echo '<a href="?' . buildQuery(array_merge($_GET, ['page' => $totalPages])) . '">' . $totalPages . '</a>';
        }

        // Next Page Link
        if ($page < $totalPages) {
            echo '<a href="?' . buildQuery(array_merge($_GET, ['page' => $page + 1])) . '">Next &raquo;</a>';
        }
        ?>
    </div>

    <!-- Page Size Dropdown within a Form -->
    <form method="GET" action="audit_logs.php" id="page_size_form">
        <?php
        // Preserve existing GET parameters except 'page_size' and 'page'
        $currentParams = $_GET;
        unset($currentParams['page_size']);
        unset($currentParams['page']);
        foreach ($currentParams as $key => $value) {
            echo '<input type="hidden" name="' . htmlspecialchars($key) . '" value="' . htmlspecialchars($value) . '">';
        }
        ?>
        <input type="hidden" name="page" value="1"> <!-- Reset to first page when page size changes -->
        <label for="page_size">Show
            <select id="page_size" name="page_size" onchange="document.getElementById('page_size_form').submit()" style="padding: 5px;">
                <?php
                foreach ($allowedPageSizes as $size) {
                    $selected = ($size === $pageSize) ? 'selected' : '';
                    echo '<option value="' . $size . '" ' . $selected . '>' . $size . '</option>';
                }
                ?>
            </select>
            results per page
        </label>
    </form>
</div>

<!-- Toggle Filters Script -->
<script>
document.addEventListener('DOMContentLoaded', function() {
    var toggleFiltersBtn = document.getElementById('toggle-filters-button');
    var filterForm = document.getElementById('filter-form');
    var resetBtn = document.getElementById('reset-filters-button');

    // Function to check if any filters are applied
    function areFiltersApplied() {
        // List of possible filter fields
        var filterFields = ['user_id', 'action', 'entity', 'entity_id', 'additional_info', 'start_date', 'end_date'];
        for (var i = 0; i < filterFields.length; i++) {
            var field = document.getElementById(filterFields[i]);
            if (field) {
                if (field.type === 'select-one' || field.type === 'select-multiple') {
                    if (field.value !== '') return true;
                } else if (field.type === 'checkbox' || field.type === 'radio') {
                    if (field.checked) return true;
                } else {
                    if (field.value.trim() !== '') return true;
                }
            }
        }
        return false;
    }

    // Show filter form if filters are applied
    if (areFiltersApplied()) {
        filterForm.style.display = 'block';
    }

    // Toggle Filters visibility
    if(toggleFiltersBtn && filterForm){
        toggleFiltersBtn.addEventListener('click', function() {
            if(filterForm.style.display === 'none' || filterForm.style.display === ''){
                filterForm.style.display = 'block';
            } else {
                filterForm.style.display = 'none';
            }
        });
    }

    // Reset Filters
    if(resetBtn){
        resetBtn.addEventListener('click', function() {
            window.location.href = 'audit_logs.php';
        });
    }
});
</script>

<style>
    /* Basic styling for table and forms */
    th, td {
        border: 1px solid #ddd;
        padding: 8px;
        vertical-align: middle;
    }
    th {
        cursor: pointer;
        background-color: #f2f2f2;
        position: relative;
        color: #000; /* Set text color to black */
    }
    th a {
        text-decoration: none;
        color: inherit; /* Inherit the color from th */
        display: block;
    }
    th.sort-asc::after {
        content: " ▲";
        position: absolute;
        top:10px;
        right: 8px;
    }
    th.sort-desc::after {
        content: " ▼";
        position: absolute;
        top:10px;
        right: 8px;
    }
    .alert {
        background-color: #f8d7da;
        color: #721c24;
        padding: 10px;
        margin-bottom: 15px;
        border-radius: 4px;
    }
    .success {
        background-color: #d4edda;
        color: #155724;
        padding: 10px;
        margin-bottom: 15px;
        border-radius: 4px;
    }
    .filter-form {
        margin-bottom: 20px;
        display: none; /* Initially hidden */
        padding: 15px;
        border: 1px solid #ccc;
        border-radius: 4px;
        background-color: #fafafa;
    }
    .filter-form div {
        margin-bottom: 10px;
    }
    .filter-form label {
        margin-right: 10px;
        display: inline-block;
        width: 150px;
        font-weight: bold;
    }
    .filter-form input, .filter-form select {
        margin-bottom: 10px;
        padding: 5px;
        width: 200px;
        display: inline-block;
    }
    .pagination-controls {
        display: flex;
        justify-content: space-between;
        align-items: center;
        margin-top: 20px;
    }
    .pagination a, .pagination span {
        margin: 0 5px;
        text-decoration: none;
        padding: 5px 10px;
        border: 1px solid #ddd;
        border-radius: 4px;
        color: #007bff;
    }
    .pagination a.active {
        background-color: #007bff;
        color: white;
        border-color: #007bff;
    }
    .pagination span {
        padding: 5px 10px;
        color: #555;
    }
    .button-right {
        margin-left: 10px;
        float:right !important;
    }
    .button:hover {
        background-color: #0056b3;
    }
    /* Applied Filters Message */
    .applied-filters {
        margin-top: 10px;
        padding: 10px;
        background-color: #e2e3e5;
        border: 1px solid #d6d8db;
        border-radius: 4px;
        color: #383d41;
    }
</style>

<?php require_once('components/footer.php'); ?>
