<?php
// File: public/reports/balancesheet.php

require_once('../core/financial_reports.php');
require_once('../core/financial_periods.php');
require_once('components/auth.php');

require_once('../config/config.php'); // Added line to include config.php

/***********************************************************************
 * ADD THIS ROUTE AT THE TOP TO LIST BALANCESHEET LAYOUTS
 **********************************************************************/
if (isset($_GET['action']) && $_GET['action'] === 'list_bs_layouts') {
    header('Content-Type: application/json; charset=utf-8');

    $layoutsDir = __DIR__ . '/documents';

    // Helper function to find .layout files with "type":"balancesheet"
    function findBalanceSheetLayouts($dir) {
        $results = [];
        if (!is_dir($dir)) {
            return $results;
        }
        $files = scandir($dir);
        foreach ($files as $f) {
            if (strtolower(pathinfo($f, PATHINFO_EXTENSION)) === 'layout') {
                $fullPath = $dir . '/' . $f;
                $jsonData = @json_decode(file_get_contents($fullPath), true);
                if (is_array($jsonData) && isset($jsonData['type']) && $jsonData['type'] === 'balancesheet') {
                    $results[] = $f;
                }
            }
        }
        return $results;
    }

    $bsLayouts = findBalanceSheetLayouts($layoutsDir);
    if (count($bsLayouts) === 0) {
        echo json_encode(['success' => false, 'message' => 'No layout for balancesheet']);
        exit;
    }
    echo json_encode(['success' => true, 'layouts' => $bsLayouts]);
    exit; // Stop
}

$errors = [];
require_once('components/header.php');
require_once('components/form_component.php');
// Check if the user has the 'balance_sheet' role
if (!checkRole(['balance_sheet'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to view this page.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Balance Sheet</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

$asOfDate = date('Y-m-d'); // Default to today
$financialPeriodId = null;
$showTable = false;
$balanceSheet = [];
$closedPeriod = false;
$footnote = '';

// If we detect ?ajax=periods, return JSON of financial periods
if (isset($_GET['ajax']) && $_GET['ajax'] === 'periods') {
    header('Content-Type: application/json');

    // Retrieve from DB
    $financialPeriods = listFinancialPeriods($pdo);

    // Format them as JSON
    $output = [];
    foreach ($financialPeriods as $fp) {
        $output[] = [
            'id'         => (string)$fp['id'],
            'name'       => $fp['name'],
            'start_date' => $fp['start_date'],
            'end_date'   => $fp['end_date']
        ];
    }

    echo json_encode($output);
    exit; // STOP
}

// Handle form submission
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['financial_period_id']) && !empty($_POST['financial_period_id'])) {
        $financialPeriodId = (int)$_POST['financial_period_id'];
        // Fetch financial period dates
        $financialPeriod = getFinancialPeriod($pdo, $financialPeriodId);
        if ($financialPeriod) {
            $asOfDate = $financialPeriod['end_date'];
            $showTable = true;
            $closedPeriod = isFinancialPeriodClosed($pdo, $financialPeriodId);
        } else {
            $errors[] = 'Invalid financial period selected.';
        }
    } elseif (isset($_POST['as_of_date']) && !empty($_POST['as_of_date'])) {
        $asOfDate = $_POST['as_of_date'];
        $showTable = true;
        // Without a selected period, consider not closed
        $closedPeriod = false;
    } else {
        $errors[] = 'Please select a financial period or enter an "As of Date".';
    }

    if ($showTable) {
        // Generate balance sheet data
        $balanceSheet = generateBalanceSheet($pdo, $asOfDate);

        if (empty($balanceSheet)) {
            $errors[] = 'No data available for the selected date or period.';
        } else {
            // Calculate net income
            $netIncome = 0;
            if ($financialPeriodId !== null) {
                $periodData = getFinancialPeriod($pdo, $financialPeriodId);
                if ($periodData) {
                    $incomeStmt = generateIncomeStatement($pdo, $periodData['start_date'], $periodData['end_date']);
                    // Calculate net income
                    $totalRevenue = 0;
                    $totalExpense = 0;
                    foreach ($incomeStmt as $acct) {
                        $type = strtolower($acct['account_type']);
                        if (in_array($type, ['revenue','income'])) {
                            $totalRevenue += (float)$acct['total_amount'];
                        } elseif ($type === 'expense') {
                            $totalExpense += (float)$acct['total_amount'];
                        }
                    }
                    $netIncome = $totalRevenue - $totalExpense;
                }
            } else {
                // No period selected: from start of year to asOfDate
                $yearStart = date('Y-01-01', strtotime($asOfDate));
                $incomeStmt = generateIncomeStatement($pdo, $yearStart, $asOfDate);
                $totalRevenue = 0;
                $totalExpense = 0;
                foreach ($incomeStmt as $acct) {
                    $type = strtolower($acct['account_type']);
                    if (in_array($type, ['revenue','income'])) {
                        $totalRevenue += (float)$acct['total_amount'];
                    } elseif ($type === 'expense') {
                        $totalExpense += (float)$acct['total_amount'];
                    }
                }
                $netIncome = $totalRevenue - $totalExpense;
            }

            // Determine Net Income line name
            if ($financialPeriodId !== null) {
                if ($closedPeriod) {
                    // Closed period: final net income
                    $netIncomeName = 'Net Income';
                    $footnote = "Accounting period closed. This is the final figure.";
                } else {
                    // Not closed
                    $netIncomeName = 'Net Income (Temporary)';
                    $footnote = "Accounting period not closed. Figures are not final.";
                }
            } else {
                // No period selected
                $netIncomeName = 'Net Income (Temporary)';
                $footnote = "No financial period selected. Figures might not represent a finalized period.";
            }

            // Check if account code NET_INCOME_ACC_CODE already exists
            $netIncomeLineIndex = null;
            foreach ($balanceSheet as $idx => $acc) {
                if (isset($acc['code']) && $acc['code'] === NET_INCOME_ACC_CODE) {
                    $netIncomeLineIndex = $idx;
                    break;
                }
            }

            // Update or add the net income line
            if ($netIncomeLineIndex !== null) {
                $balanceSheet[$netIncomeLineIndex]['name'] = $netIncomeName;
                $balanceSheet[$netIncomeLineIndex]['total_amount'] = $netIncome;
            } else {
                $balanceSheet[] = [
                    'account_type' => 'Equity',
                    'code'         => NET_INCOME_ACC_CODE,
                    'name'         => $netIncomeName,
                    'parent'       => null,
                    'total_debit'  => 0,
                    'total_credit' => 0,
                    'total_amount' => $netIncome
                ];
            }
        }
    }
}

// Fetch financial periods for the dropdown
$financialPeriods = listFinancialPeriods($pdo);
?>
<h1>Balance Sheet</h1>

<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>

<div class="form-container">
    <form action="balancesheet.php" method="POST">
        <?php
        // Financial Period Dropdown
        $periodOptions = [];
        foreach ($financialPeriods as $period) {
            $periodOptions[$period['id']] = htmlspecialchars($period['name'] . ' (' . $period['start_date'] . ' to ' . $period['end_date'] . ')');
        }
        renderSelect('financial_period_id', 'Financial Period', $periodOptions, $financialPeriodId);
        renderInput('date', 'as_of_date', 'As of Date', htmlspecialchars($asOfDate));
        ?>
        <div class="button-group">
            <button type="submit">Generate Balance Sheet</button>
            <!-- NEW ORANGE BUTTON FOR PDF -->
            <button type="button"
                    id="generate_bs_pdf_button"
                    style="background-color: orange; color: #fff; border: none; padding: 8px 12px; cursor: pointer;">
                Generate Balance Sheet PDF
            </button>
        </div>
    </form>
</div>

<?php
if ($showTable && !empty($balanceSheet) && empty($errors)) {
    // Columns definition
    $columns = [
        ['header' => 'Account Type', 'field' => 'account_type'],
        ['header' => 'Code', 'field' => 'code'],
        ['header' => 'Name', 'field' => 'name'],
        ['header' => 'Balance', 'field' => 'total_amount']
    ];

    function buildAccountTree(array $accounts, $parent = null): array {
        $branch = [];
        foreach ($accounts as $account) {
            $accountParent = (!empty($account['parent']) && strtolower($account['parent']) !== 'n/a') ? $account['parent'] : null;
            if ($accountParent === $parent) {
                $children = buildAccountTree($accounts, $account['code']);
                if ($children) {
                    $account['children'] = $children;
                }
                $branch[] = $account;
            }
        }
        return $branch;
    }

    function traverseAccountTree(array $tree, array &$sortedList, $depth = 0): void {
        foreach ($tree as $node) {
            $node['depth'] = $depth;
            $sortedList[] = $node;
            if (isset($node['children'])) {
                traverseAccountTree($node['children'], $sortedList, $depth + 1);
            }
        }
    }

    function renderTable($data, $columns) {
        echo '<table>';
        echo '<thead><tr>';
        foreach ($columns as $column) {
            $header = $column['header'] ?? '';
            echo '<th>' . htmlspecialchars((string)$header) . '</th>';
        }
        echo '</tr></thead>';
        echo '<tbody>';
        foreach ($data as $row) {
            $rowClass = htmlspecialchars($row['row_class'] ?? '');
            echo '<tr' . ($rowClass ? ' class="' . $rowClass . '"' : '') . '>';
            foreach ($columns as $column) {
                $field = $column['field'];
                $cellData = $row[$field] ?? '';

                if ($field === 'account_type') {
                    $cellData = strtoupper($cellData);
                }
                if ($field === 'name' && isset($row['depth'])) {
                    $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $row['depth']);
                    $cellData = $indent . htmlspecialchars((string)$cellData);
                } else {
                    $cellData = htmlspecialchars((string)$cellData);
                }

                if ($field === 'total_amount') {
                    $cellData = number_format((float)$cellData, 2);
                }

                echo '<td>' . $cellData . '</td>';
            }
            echo '</tr>';
        }
        echo '</tbody>';
        echo '</table>';
    }

    // Build and traverse the account tree
    $accountTree = buildAccountTree($balanceSheet);
    $sortedAccounts = [];
    traverseAccountTree($accountTree, $sortedAccounts);

    // Apply classes and compute subtotals
    $totals = [
        'asset' => 0.0,
        'liability' => 0.0,
        'equity' => 0.0,
    ];

    $processedAccounts = [];
    $currentType = null;

    foreach ($sortedAccounts as $account) {
        $type = strtolower($account['account_type']);
        // Set row class
        switch ($type) {
            case 'asset':
                $account['row_class'] = 'account-asset';
                break;
            case 'liability':
                $account['row_class'] = 'account-liability';
                break;
            case 'equity':
                $account['row_class'] = 'account-equity';
                break;
            default:
                $account['row_class'] = '';
        }

        if (in_array($type, ['asset','liability','equity'])) {
            if ($currentType !== $type) {
                // Insert subtotal of previous type
                if ($currentType !== null) {
                    $processedAccounts[] = [
                        'account_type' => 'Subtotal',
                        'code' => '',
                        'name' => 'Total ' . ucfirst($currentType),
                        'total_amount' => $totals[$currentType],
                        'row_class' => 'subtotal',
                        'depth' => 0
                    ];
                }
                $currentType = $type;
            }
            $totals[$type] += (float)$account['total_amount'];
        }
        $processedAccounts[] = $account;
    }

    if ($currentType !== null && isset($totals[$currentType])) {
        $processedAccounts[] = [
            'account_type' => 'Subtotal',
            'code' => '',
            'name' => 'Total ' . ucfirst($currentType),
            'total_amount' => $totals[$currentType],
            'row_class' => 'subtotal',
            'depth' => 0
        ];
    }

    // Render the table
    if (!empty($processedAccounts)) {
        renderTable($processedAccounts, $columns);
    }

    // Print footnote
    if (!empty($footnote)) {
        echo '<p style="text-align:center; font-weight:bold;">' . htmlspecialchars($footnote) . '</p>';
    }
}
?>

<!-- HIDDEN ALERT IF NO BALANCESHEET LAYOUTS -->
<div id="bs-no-layout-error" class="alert" style="display:none;">
    <p>No saved layout found for <strong>balancesheet</strong>.
       <a href="document_editor.php">Create a layout here</a>.
    </p>
</div>

<!-- MODAL FOR MULTIPLE BALANCESHEET LAYOUTS -->
<div id="bs-modal-overlay" style="
    display: none;
    position: fixed;
    top: 0; left: 0;
    width: 100%; height: 100%;
    background: rgba(0,0,0,0.4);
    align-items: center;
    justify-content: center;
    z-index: 9999;">
  <div id="bs-modal-content" style="
      background: #fff;
      padding: 20px;
      border-radius: 4px;
      min-width: 300px;">
    <h2>Select a Balance Sheet Layout</h2>
    <div id="bs-modal-error" style="color: red; font-weight: bold; display:none;"></div>
    <label for="bs-modal-select">Layouts:</label>
    <select id="bs-modal-select" style="width:100%; margin-bottom:15px;"></select>
    <div style="text-align:right;">
      <button id="bs-modal-cancel" style="margin-right:10px;">Cancel</button>
      <button id="bs-modal-ok" style="background-color: orange; color: #fff; border:none; padding:5px 12px; cursor:pointer;">
          OK
      </button>
    </div>
  </div>
</div>

<script>
document.addEventListener('DOMContentLoaded', function() {

    const pdfBtn = document.getElementById('generate_bs_pdf_button');
    if (pdfBtn) {
        pdfBtn.addEventListener('click', function(e) {
            e.preventDefault();
            // Validate form inputs
            const periodSelect = document.querySelector('[name="financial_period_id"]');
            const asOfDateInput = document.querySelector('[name="as_of_date"]');

            if (!periodSelect.value && !asOfDateInput.value) {
                alert("Please select a financial period or enter an 'As of Date'.");
                return;
            }

            // Fetch layout info
            fetch('balancesheet.php?action=list_bs_layouts')
            .then(r => r.json())
            .then(json => {
                if (!json.success) {
                    // Show a hidden alert about no layout
                    document.getElementById('bs-no-layout-error').style.display = 'block';
                    return;
                }
                const layouts = json.layouts || [];
                if (layouts.length === 0) {
                    document.getElementById('bs-no-layout-error').style.display = 'block';
                    return;
                }
                if (layouts.length === 1) {
                    openBsPdfInNewTab(layouts[0]);
                } else {
                    showBsModal(layouts);
                }
            })
            .catch(err => {
                console.error("Error fetching balancesheet layouts:", err);
                alert("Failed to list balancesheet layouts. See console.");
            });
        });
    }

function openBsPdfInNewTab(layoutName) {
  // CHANGED: gather form inputs
  const periodVal = document.querySelector('[name="financial_period_id"]').value.trim();
  const asOfVal   = document.querySelector('[name="as_of_date"]').value.trim();

  // 1) Fetch the actual layout JSON from 'documents/' + layoutName
  fetch('documents/' + layoutName)  // CHANGED line
    .then(r => r.json())
    .then(fullLayout => {
      // 2) Override or add the dialogData with user period/date inputs
      fullLayout.dialogData = {
        financial_period_id: periodVal,
        as_of_date: asOfVal
      };
      // (Optionally ensure orientation is present if you want a fallback:)
      if (!fullLayout.orientation) {
        fullLayout.orientation = 'P'; // fallback orientation if missing
      }

      // 3) Now POST that entire layout JSON to document_display_pdf.php
      const f = document.createElement('form');
      f.method = 'POST';
      f.action = 'document_display_pdf.php';
      f.target = '_blank';

      const hiddenInp = document.createElement('input');
      hiddenInp.type = 'hidden';
      hiddenInp.name = 'layoutJSON';
      hiddenInp.value = JSON.stringify(fullLayout);
      f.appendChild(hiddenInp);

      document.body.appendChild(f);
      f.submit();

      // OPTIONAL: Remove or comment out any forced reload that might have caused partial output:
      // setTimeout(() => { window.location.reload(); }, 500);
    })
    .catch(err => {
      console.error("Cannot load layout file:", err);
      alert("Error: Failed to load layout JSON for " + layoutName);
    });
}

    function showBsModal(listOfLayouts) {
        const overlay   = document.getElementById('bs-modal-overlay');
        const modalSel  = document.getElementById('bs-modal-select');
        const cancelBtn = document.getElementById('bs-modal-cancel');
        const okBtn     = document.getElementById('bs-modal-ok');
        const errDiv    = document.getElementById('bs-modal-error');

        // Reset
        modalSel.innerHTML = '';
        errDiv.textContent = '';
        errDiv.style.display = 'none';

        listOfLayouts.forEach(lay => {
            const opt = document.createElement('option');
            opt.value = lay;
            opt.textContent = lay;
            modalSel.appendChild(opt);
        });

        overlay.style.display = 'flex';

        cancelBtn.onclick = () => {
            overlay.style.display = 'none';
        };

        okBtn.onclick = () => {
            const chosen = modalSel.value.trim();
            if (!chosen) {
                errDiv.textContent = "Please select a layout.";
                errDiv.style.display = 'block';
                return;
            }
            overlay.style.display = 'none';
            openBsPdfInNewTab(chosen);
        };
    }
});
</script>

<?php require_once('components/footer.php'); ?>
