<?php
require_once('../core/financial_reports.php');
require_once('../core/accounts.php'); // To fetch account types
require_once('../core/financial_periods.php');


/***********************************************************************
 * ADD THIS ROUTE AT THE VERY TOP TO LIST CASHFLOW LAYOUTS
 **********************************************************************/
if (isset($_GET['action']) && $_GET['action'] === 'list_cf_layouts') {
    header('Content-Type: application/json; charset=utf-8');

    $layoutsDir = __DIR__ . '/documents';

    // Helper function to find .layout files with "type":"cashflow"
    function findCashFlowLayouts($dir) {
        $results = [];
        if (!is_dir($dir)) {
            return $results;
        }
        $files = scandir($dir);
        foreach ($files as $f) {
            if (strtolower(pathinfo($f, PATHINFO_EXTENSION)) === 'layout') {
                $fullPath = $dir . '/' . $f;
                $jsonData = @json_decode(file_get_contents($fullPath), true);
                if (is_array($jsonData) && isset($jsonData['type']) && $jsonData['type'] === 'cashflow') {
                    $results[] = $f;
                }
            }
        }
        return $results;
    }

    $cfLayouts = findCashFlowLayouts($layoutsDir);
    if (count($cfLayouts) === 0) {
        echo json_encode(['success' => false, 'message' => 'No layout for cashflow']);
        exit;
    }
    echo json_encode(['success' => true, 'layouts' => $cfLayouts]);
    exit; // Stop
}

$errors = [];

require_once('components/auth.php');
require_once('components/header.php');

$cashFlowStatement = null;
$selectedPeriodId = '';
$startDate = '';
$endDate = '';
$closedPeriod = false;

if (!checkRole(['cash_flow_statement'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to view this page.';
    unset($_SESSION['error_message']);
}

// Fetch financial periods for dropdown
$financialPeriods = listFinancialPeriods($pdo);

if ($_SERVER['REQUEST_METHOD'] === 'POST' && empty($errors)) {
    // Determine if a Financial Period is selected
    if (!empty($_POST['financial_period_id'])) {
        $selectedPeriodId = (int)$_POST['financial_period_id'];
        $period = getFinancialPeriod($pdo, $selectedPeriodId);
        if ($period) {
            $startDate = $period['start_date'];
            $endDate   = $period['end_date'];
            $closedPeriod = isFinancialPeriodClosed($pdo, $selectedPeriodId);
        } else {
            $errors[] = 'Selected financial period not found.';
        }
    } else {
        // Handle custom date range
        $startDate = trim($_POST['start_date'] ?? '');
        $endDate   = trim($_POST['end_date'] ?? '');

        // Validate dates
        if (empty($startDate) || empty($endDate)) {
            $errors[] = 'Both start date and end date are required.';
        } elseif ($startDate > $endDate) {
            $errors[] = 'Start date cannot be later than end date.';
        }
    }

    if (empty($errors)) {
        // Fetch transactions using the appropriate function based on whether the period is closed
        if ($selectedPeriodId && $closedPeriod) {
            // Use generateCashFlowStatementReport to exclude transactions linked to module_id = -1
            // (Below function is presumably defined in your code; fallback if not found.)
            if (function_exists('generateCashFlowStatementReport')) {
                $transactions = generateCashFlowStatementReport($pdo, $startDate, $endDate);
            } else {
                // fallback
                $transactions = generateCashFlowStatement($pdo, $startDate, $endDate);
            }
        } else {
            // Use standard generateCashFlowStatement
            $transactions = generateCashFlowStatement($pdo, $startDate, $endDate);
        }

        if (empty($transactions)) {
            $errors[] = 'No transactions found for the selected period.';
        } else {
            // Fetch all accounts to map account codes to their types
            $accounts = listAccounts($pdo);
            $accountTypes = [];
            foreach ($accounts as $account) {
                $accountTypes[$account['code']] = strtolower($account['account_type']);
            }

            // Initialize categories
            $cashFlowStatement = [
                'operating_activities' => 0.00,
                'investing_activities' => 0.00,
                'financing_activities' => 0.00
            ];

            // Categorize transactions
            foreach ($transactions as $transaction) {
                $debitAccount  = $transaction['debit_account'];
                $creditAccount = $transaction['credit_account'];
                $amount        = (float)$transaction['amount'];

                $debitType  = $accountTypes[$debitAccount]  ?? '';
                $creditType = $accountTypes[$creditAccount] ?? '';

                // Simple categorization logic
                $isOperating = false;
                $isInvesting = false;
                $isFinancing = false;

                // Define types that belong to each category
                $investingTypes = ['investment', 'fixed asset'];
                $financingTypes = ['liability', 'equity'];

                if (in_array($debitType, $investingTypes) || in_array($creditType, $investingTypes)) {
                    $isInvesting = true;
                } elseif (in_array($debitType, $financingTypes) || in_array($creditType, $financingTypes)) {
                    $isFinancing = true;
                } else {
                    // Default to operating
                    $isOperating = true;
                }

                if ($isOperating) {
                    $cashFlowStatement['operating_activities'] += $amount;
                } elseif ($isInvesting) {
                    $cashFlowStatement['investing_activities'] += $amount;
                } elseif ($isFinancing) {
                    $cashFlowStatement['financing_activities'] += $amount;
                }
            }
        }
    }
}
?>
<h1>Cash Flow Statement</h1>

<div class="form-container">
    <form method="POST" action="cashflowstatement.php">
        <h2>Select Financial Period or Date Range</h2>
        <label for="financial_period_id">Financial Period:</label>
        <select id="financial_period_id" name="financial_period_id">
            <option value="">-- Select Financial Period --</option>
            <?php foreach ($financialPeriods as $period): ?>
                <option value="<?php echo htmlspecialchars($period['id']); ?>" <?php echo ($selectedPeriodId == $period['id']) ? 'selected' : ''; ?>>
                    <?php echo htmlspecialchars($period['name'] . ' (' . $period['start_date'] . ' to ' . $period['end_date'] . ')'); ?>
                </option>
            <?php endforeach; ?>
        </select>
        <p>Or enter a custom date range:</p>
        <label for="start_date">Start Date:</label>
        <input type="date" id="start_date" name="start_date" value="<?php echo htmlspecialchars($startDate); ?>">

        <label for="end_date">End Date:</label>
        <input type="date" id="end_date" name="end_date" value="<?php echo htmlspecialchars($endDate); ?>">

        <div class="button-group">
            <button type="submit">Generate</button>
            <!-- NEW ORANGE BUTTON FOR PDF -->
            <button type="button"
                    id="generate_cf_pdf_button"
                    style="background-color: orange; color: #fff; border: none; padding: 8px 12px; cursor: pointer;">
                Generate Cash Flow PDF
            </button>
        </div>
    </form>
</div>

<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>

<?php if ($cashFlowStatement !== null && empty($errors)): ?>
<table>
    <thead>
        <tr>
            <th>Cash Flow Activities</th>
            <th>Amount</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td>Operating Activities</td>
            <td><?php echo number_format($cashFlowStatement['operating_activities'], 2); ?></td>
        </tr>
        <tr>
            <td>Investing Activities</td>
            <td><?php echo number_format($cashFlowStatement['investing_activities'], 2); ?></td>
        </tr>
        <tr>
            <td>Financing Activities</td>
            <td><?php echo number_format($cashFlowStatement['financing_activities'], 2); ?></td>
        </tr>
        <tr>
            <th>Net Increase in Cash</th>
            <th><?php
                echo number_format(
                    $cashFlowStatement['operating_activities'] +
                    $cashFlowStatement['investing_activities'] +
                    $cashFlowStatement['financing_activities'],
                2);
            ?></th>
        </tr>
    </tbody>
</table>
<?php endif; ?>

<!-- HIDDEN ALERT IF NO CASHFLOW LAYOUTS -->
<div id="cf-no-layout-error" class="alert" style="display:none;">
    <p>No saved layout found for <strong>cashflow</strong>.
       <a href="document_editor.php">Create a layout here</a>.
    </p>
</div>

<!-- MODAL FOR MULTIPLE CASHFLOW LAYOUTS -->
<div id="cf-modal-overlay" style="
    display: none;
    position: fixed;
    top: 0; left: 0;
    width: 100%; height: 100%;
    background: rgba(0,0,0,0.4);
    align-items: center;
    justify-content: center;
    z-index: 9999;">
  <div id="cf-modal-content" style="
      background: #fff;
      padding: 20px;
      border-radius: 4px;
      min-width: 300px;">
    <h2>Select a Cash Flow Layout</h2>
    <div id="cf-modal-error" style="color: red; font-weight: bold; display:none;"></div>
    <label for="cf-modal-select">Layouts:</label>
    <select id="cf-modal-select" style="width:100%; margin-bottom:15px;"></select>
    <div style="text-align:right;">
      <button id="cf-modal-cancel" style="margin-right:10px;">Cancel</button>
      <button id="cf-modal-ok" style="background-color: orange; color: #fff; border:none; padding:5px 12px; cursor:pointer;">
          OK
      </button>
    </div>
  </div>
</div>

<script>
document.addEventListener('DOMContentLoaded', function() {
    const pdfBtn = document.getElementById('generate_cf_pdf_button');
    if (pdfBtn) {
        pdfBtn.addEventListener('click', function(e) {
            e.preventDefault();
            // Validate the existing form inputs
            const periodSelect = document.querySelector('[name="financial_period_id"]');
            const startDateInput = document.querySelector('[name="start_date"]');
            const endDateInput   = document.querySelector('[name="end_date"]');

            // Must have either period or both dates:
            if (!periodSelect.value && (!startDateInput.value || !endDateInput.value)) {
                alert("Please select a financial period or enter a custom date range.");
                return;
            }

            // Query layout files for type:"cashflow"
            fetch('cashflowstatement.php?action=list_cf_layouts')
            .then(r => r.json())
            .then(json => {
                if (!json.success) {
                    // Show a hidden alert about no layout
                    document.getElementById('cf-no-layout-error').style.display = 'block';
                    return;
                }
                const layouts = json.layouts || [];
                if (layouts.length === 0) {
                    document.getElementById('cf-no-layout-error').style.display = 'block';
                    return;
                }
                if (layouts.length === 1) {
                    openCfPdfInNewTab(layouts[0]);
                } else {
                    showCfModal(layouts);
                }
            })
            .catch(err => {
                console.error("Error fetching cashflow layouts:", err);
                alert("Failed to list cashflow layouts. See console.");
            });
        });
    }

    function openCfPdfInNewTab(layoutName) {
        // Gather form inputs
        const periodVal = document.querySelector('[name="financial_period_id"]').value.trim();
        const sdVal     = document.querySelector('[name="start_date"]').value.trim();
        const edVal     = document.querySelector('[name="end_date"]').value.trim();

        // 1) Load the layout from /documents/<layoutName>
        fetch('documents/' + layoutName)
          .then(r => r.json())
          .then(fullLayout => {
              // 2) Insert dialogData from the user input
              fullLayout.dialogData = {
                  financial_period_id: periodVal,
                  start_date: sdVal,
                  end_date: edVal
              };
              if (!fullLayout.orientation) {
                  fullLayout.orientation = 'P';
              }

              // 3) Submit to document_display_pdf.php
              const f = document.createElement('form');
              f.method = 'POST';
              f.action = 'document_display_pdf.php';
              f.target = '_blank';

              const hiddenInp = document.createElement('input');
              hiddenInp.type = 'hidden';
              hiddenInp.name = 'layoutJSON';
              hiddenInp.value = JSON.stringify(fullLayout);
              f.appendChild(hiddenInp);
              document.body.appendChild(f);
              f.submit();
          })
          .catch(err => {
              console.error("Cannot load layout file:", err);
              alert("Error: Failed to load layout JSON for " + layoutName);
          });
    }

    function showCfModal(listOfLayouts) {
        const overlay   = document.getElementById('cf-modal-overlay');
        const modalSel  = document.getElementById('cf-modal-select');
        const cancelBtn = document.getElementById('cf-modal-cancel');
        const okBtn     = document.getElementById('cf-modal-ok');
        const errDiv    = document.getElementById('cf-modal-error');

        modalSel.innerHTML = '';
        errDiv.textContent = '';
        errDiv.style.display = 'none';

        listOfLayouts.forEach(lay => {
            const opt = document.createElement('option');
            opt.value = lay;
            opt.textContent = lay;
            modalSel.appendChild(opt);
        });

        overlay.style.display = 'flex';

        cancelBtn.onclick = () => {
            overlay.style.display = 'none';
        };

        okBtn.onclick = () => {
            const chosen = modalSel.value.trim();
            if (!chosen) {
                errDiv.textContent = "Please select a layout.";
                errDiv.style.display = 'block';
                return;
            }
            overlay.style.display = 'none';
            openCfPdfInNewTab(chosen);
        };
    }
});
</script>

<?php require_once('components/footer.php'); ?>
