<?php
/**
 * Build a hierarchical tree of accounts **in ascending code order**,
 * but if an account has `module_id != 0`, we do **not** recurse into children.
 *
 * @param array       $accounts   Array of accounts (already code-sorted).
 * @param string|null $parentCode Parent code (null for top-level).
 * @return array Hierarchical tree
 */
function buildAccountTree(array $accounts, string $parentCode = null): array {
    // 1) Find direct children of $parentCode
    $children = array_filter($accounts, function($acc) use ($parentCode) {
        return $acc['parent'] === $parentCode;
    });

    // 2) They should already be code-sorted at this stage, but you can re-check:
    usort($children, function($a, $b) {
        return strcmp($a['code'], $b['code']);
    });

    $branch = [];
    foreach ($children as $child) {
        // If this child has module_id != 0, we do not build further children
        // We simply add the child as a leaf node with no children.
        if (!empty($child['module_id']) && $child['module_id'] != 0) {
            $branch[] = $child;
        } else {
            // Recurse normally
            $child['children'] = buildAccountTree($accounts, $child['code']);
            $branch[] = $child;
        }
    }
    return $branch;
}

/**
 * Flatten the tree to a list with 'depth' info.
 * We do not skip children here unless they've been pruned already above.
 *
 * @param array $tree
 * @param array &$sortedList
 * @param int   $depth
 */
function traverseAccountTree(array $tree, array &$sortedList, int $depth = 0): void {
    foreach ($tree as $node) {
        $node['depth'] = $depth;
        $sortedList[] = $node;

        // Recurse if children exist
        if (!empty($node['children'])) {
            traverseAccountTree($node['children'], $sortedList, $depth + 1);
        }
    }
}

/**
 * Render account options for a <select> in hierarchical code-sorted order.
 * If an account has module_id != 0, we do not recurse further *and* disable the option.
 *
 * @param array $tree
 * @param int   $level
 * @return void
 */
function renderAccountOptions(array $tree, int $level = 0): void {
    foreach ($tree as $account) {
        $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $level);

        switch (strtolower($account['account_type'])) {
            case 'asset':
                $color = '#1E90FF';
                break;
            case 'liability':
                $color = '#FF4500';
                break;
            case 'equity':
                $color = '#32CD32';
                break;
            case 'revenue':
                $color = '#FFD700';
                break;
            case 'expense':
                $color = '#FF6347';
                break;
            default:
                $color = '#000000';
        }

        $codeSafe  = htmlspecialchars($account['code']);
        $nameSafe  = htmlspecialchars($account['name']);
        $typeSafe  = htmlspecialchars(strtolower($account['account_type']));

        // *** NEW: If module_id != 0, disable this <option> so it can't be selected.
        $disabled = (!empty($account['module_id']) && $account['module_id'] != 0) ? ' disabled' : '';

        echo '<option value="' . $codeSafe . '" data-account-type="' . $typeSafe . '" style="color: ' . $color . ';"' . $disabled . '>';
        echo $indent . $nameSafe . ' (' . $codeSafe . ')';
        echo '</option>';

        // If module_id != 0, do NOT recurse further
        if (!empty($account['module_id']) && $account['module_id'] != 0) {
            continue;
        }
        // Otherwise, render children
        if (!empty($account['children'])) {
            renderAccountOptions($account['children'], $level + 1);
        }
    }
}
?>
