<?php
/**
 * document_display_pdf.php
 *
 * Receives layout JSON from POST and generates a PDF with FPDF + optional watermark.
 * We'll use "bind" property (header|footer|watermark|none) for each element.
 * Also uses margins so multi-page content doesn't overlap header or footer.
 */

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('components/auth.php');
require_once('../config/config.php');
require_once('../lib/fpdf186/fpdf.php');
require_once('../lib/fpdf186/rotation.php');

if (!checkRole(['accounts'])) {
    die('Access denied.');
}
if (!isset($_POST['layoutJSON'])) {
    die("No layout data received.");
}
$layoutData = json_decode($_POST['layoutJSON'], true);
if (!$layoutData) {
    die("Invalid layout JSON.");
}

// ADDED: Load type's fpdf.php if present
$type = isset($layoutData['type']) ? $layoutData['type'] : '';
$dialogData = isset($layoutData['dialogData']) ? $layoutData['dialogData'] : [];
if ($type) {
    $typeFpdfFile = __DIR__ . "/documents/type/$type/fpdf.php";
    if (file_exists($typeFpdfFile)) {
        require_once $typeFpdfFile;
    }
}

$pageWidth  = (float)($layoutData['pageWidth']  ?? 210);
$pageHeight = (float)($layoutData['pageHeight'] ?? 297);
$orientation= ($layoutData['orientation'] === 'L') ? 'L' : 'P';
$allElements = $layoutData['elements'] ?? [];
$headerElements    = [];
$footerElements    = [];
$watermarkElements = [];
$mainElements      = [];
foreach ($allElements as $elem) {
    $bindVal = isset($elem['bind']) ? strtolower($elem['bind']) : 'none';
    switch ($bindVal) {
        case 'header':
            $headerElements[] = $elem;
            break;
        case 'footer':
            $footerElements[] = $elem;
            break;
        case 'watermark':
            $watermarkElements[] = $elem;
            break;
        default:
            $mainElements[] = $elem;
    }
}
class MyPDF extends PDF_Rotate {
    public $headerItems    = [];
    public $footerItems    = [];
    public $watermarkItems = [];
    public $mainItems      = [];
    public $headerBottomY = 0;
    public $footerTopY    = 0;
    public $PDFName        = 'output.pdf';
    public function computeHeaderBottom() {
        $maxY = 0;
        foreach ($this->headerItems as $el) {
            $type = $el['type'] ?? '';
            switch ($type) {
                case 'Line':
                    $y1 = (float)($el['y1'] ?? 0);
                    $y2 = (float)($el['y2'] ?? 0);
                    $candidate = max($y1, $y2);
                    if ($candidate > $maxY) {
                        $maxY = $candidate;
                    }
                    break;
                case 'Rect':
                case 'Cell':
                case 'MultiCell':
                case 'Image':
                case 'Text':
                    $y = (float)($el['y'] ?? 0);
                    if (isset($el['height'])) {
                        $candidate = $y + (float)$el['height'];
                        if ($candidate > $maxY) {
                            $maxY = $candidate;
                        }
                    } else {
                        if ($y > $maxY) {
                            $maxY = $y;
                        }
                    }
                    break;
            }
        }
        $this->headerBottomY = $maxY + 2;
    }
    public function computeFooterTop($pageHeight) {
        $minY = $pageHeight;
        foreach ($this->footerItems as $el) {
            $type = $el['type'] ?? '';
            switch ($type) {
                case 'Line':
                    $y1 = (float)($el['y1'] ?? 0);
                    $y2 = (float)($el['y2'] ?? 0);
                    $candidate = min($y1, $y2);
                    if ($candidate < $minY) {
                        $minY = $candidate;
                    }
                    break;
                case 'Rect':
                case 'Cell':
                case 'MultiCell':
                case 'Image':
                case 'Text':
                    $y = (float)($el['y'] ?? 0);
                    if ($y < $minY) {
                        $minY = $y;
                    }
                    break;
            }
        }
        $this->footerTopY = $minY - 2;
        if ($this->footerTopY < 0) {
            $this->footerTopY = 0;
        }
    }
    public function Header() {
        foreach ($this->watermarkItems as $item) {
            $this->renderElement($item, true);
        }
        foreach ($this->headerItems as $item) {
            $this->renderElement($item, true);
        }
    }
    public function Footer() {
        foreach ($this->footerItems as $item) {
            $this->renderElement($item, true);
        }
    }
    public function renderElement($el, $isHeaderFooter=false) {
        $type = $el['type'] ?? '';
        switch ($type) {
            case 'Line':
                $x1 = (float)($el['x1'] ?? 0);
                $y1 = (float)($el['y1'] ?? 0);
                $x2 = (float)($el['x2'] ?? 0);
                $y2 = (float)($el['y2'] ?? 0);
                $lw = (float)($el['lineWidth'] ?? 0.2);
                $r  = (int)($el['r'] ?? 0);
                $g  = (int)($el['g'] ?? 0);
                $b  = (int)($el['b'] ?? 0);
                $this->SetLineWidth($lw);
                $this->SetDrawColor($r,$g,$b);
                $this->Line($x1,$y1,$x2,$y2);
                break;
            case 'Rect':
                $x   = (float)($el['x'] ?? 0);
                $y   = (float)($el['y'] ?? 0);
                $w   = (float)($el['width'] ?? 0);
                $h   = (float)($el['height'] ?? 0);
                $sty = $el['style'] ?? 'D';
                $lw  = (float)($el['lineWidth'] ?? 0.2);
                $dr  = (int)($el['r'] ?? 0);
                $dg  = (int)($el['g'] ?? 0);
                $db  = (int)($el['b'] ?? 0);
                $fr  = (int)($el['fillR'] ?? 255);
                $fg  = (int)($el['fillG'] ?? 255);
                $fb  = (int)($el['fillB'] ?? 255);
                $this->SetLineWidth($lw);
                $this->SetDrawColor($dr,$dg,$db);
                if (stripos($sty,'F')!==false) {
                    $this->SetFillColor($fr,$fg,$fb);
                }
                if (!in_array($sty,['D','F','DF','FD'])) {
                    $sty='D';
                }
                $this->Rect($x,$y,$w,$h,$sty);
                break;
            case 'Cell':
            {
                $x=(float)($el['x']??0);
                $y=(float)($el['y']??0);
                $w=(float)($el['width']??0);
                $h=(float)($el['height']??0);
                $txt=$el['value']??'';
                $borderSides='';
                if(!empty($el['borderLeft']))$borderSides.='L';
                if(!empty($el['borderTop']))$borderSides.='T';
                if(!empty($el['borderRight']))$borderSides.='R';
                if(!empty($el['borderBottom']))$borderSides.='B';
                $ln=(int)($el['ln']??0);
                $align=$el['align']??'L';
                $fill=(bool)($el['fill']??false);
                $fFam=$el['fontFamily']??'Arial';
                $fSty=$el['fontStyle']??'';
                $fSize=(float)($el['fontSize']??12);
                $lw=(float)($el['lineWidth']??0.2);
                $dr=(int)($el['r']??0);$dg=(int)($el['g']??0);$db=(int)($el['b']??0);
                $fr=(int)($el['fillR']??255);$fg=(int)($el['fillG']??255);$fb=(int)($el['fillB']??255);
                $tr=(int)($el['textR']??0);$tg=(int)($el['textG']??0);$tb=(int)($el['textB']??0);
                $this->SetFont($fFam,$fSty,$fSize);
                $this->SetLineWidth($lw);
                $this->SetDrawColor($dr,$dg,$db);
                $this->SetFillColor($fr,$fg,$fb);
                $this->SetTextColor($tr,$tg,$tb);
                $this->SetXY($x,$y);
                $this->Cell($w,$h,$txt,$borderSides,$ln,$align,$fill);
            }
            break;
            case 'MultiCell':
            {
                $x=(float)($el['x']??0);
                $y=(float)($el['y']??0);
                $w=(float)($el['width']??0);
                $lineHeight=(float)($el['height']??5);
                $txt=$el['value']??'';
                $borderSides='';
                if(!empty($el['borderLeft']))$borderSides.='L';
                if(!empty($el['borderTop']))$borderSides.='T';
                if(!empty($el['borderRight']))$borderSides.='R';
                if(!empty($el['borderBottom']))$borderSides.='B';
                $align=$el['align']??'L';
                $fill=(bool)($el['fill']??false);
                $fFam=$el['fontFamily']??'Arial';
                $fSty=$el['fontStyle']??'';
                $fSize=(float)($el['fontSize']??12);
                $lw=(float)($el['lineWidth']??0.2);
                $dr=(int)($el['r']??0);$dg=(int)($el['g']??0);$db=(int)($el['b']??0);
                $fr=(int)($el['fillR']??255);$fg=(int)($el['fillG']??255);$fb=(int)($el['fillB']??255);
                $tr=(int)($el['textR']??0);$tg=(int)($el['textG']??0);$tb=(int)($el['textB']??0);
                $this->SetFont($fFam,$fSty,$fSize);
                $this->SetLineWidth($lw);
                $this->SetDrawColor($dr,$dg,$db);
                $this->SetFillColor($fr,$fg,$fb);
                $this->SetTextColor($tr,$tg,$tb);
                $this->SetXY($x,$y);
                $this->MultiCell($w,$lineHeight,$txt,$borderSides,$align,$fill);
            }
            break;
            case 'Image':
            {
                $x=(float)($el['x']??0);
                $y=(float)($el['y']??0);
                $w=(float)($el['width']??0);
                $h=(float)($el['height']??0);
                $imgFile=$el['image']??($el['value']??'');
                $imgType=$el['imgType']??'';
                $fullPath=__DIR__.'/documents/images/'.basename($imgFile);
                if(file_exists($fullPath)){
                    $this->Image($fullPath,$x,$y,$w,$h,$imgType);
                }else{
                    $this->SetXY($x,$y);
                    $this->Write(5,"[Missing image: $imgFile]");
                }
            }
            break;
            case 'Text':
            {
                $x=(float)($el['x']??0);
                $y=(float)($el['y']??0);
                $txt=$el['value']??'(no text)';
                $fFam=$el['fontFamily']??'Arial';
                $fSty=$el['fontStyle']??'';
                $fSize=(float)($el['fontSize']??12);
                $tr=(int)($el['r']??0);$tg=(int)($el['g']??0);$tb=(int)($el['b']??0);
                $this->SetFont($fFam,$fSty,$fSize);
                $this->SetTextColor($tr,$tg,$tb);
                $this->SetXY($x,$y);
                $this->Write(5,$txt);
            }
            break;
            default:
            // ADDED: If there's a function named render{type} from the type's fpdf
            // pass $dialogData
            $funcName='render'.$type;
            if(function_exists($funcName)){
                $funcName($this,$el,$GLOBALS['dialogData']);
            }else{
                $fn='render'.$el['type'];
                if(function_exists($fn)){
                    $fn($this,$el,$GLOBALS['dialogData']);
                }
            }
            break;
        }
    }
}
$pdf=new MyPDF($orientation,'mm',[$pageWidth,$pageHeight]);
$pdf->headerItems=$headerElements;
$pdf->footerItems=$footerElements;
$pdf->watermarkItems=$watermarkElements;
$pdf->mainItems=$mainElements;
$pdf->computeHeaderBottom();
$pdf->computeFooterTop($pageHeight);
$pdf->SetMargins(0,$pdf->headerBottomY,0);
$pdf->SetAutoPageBreak(true,($pageHeight-$pdf->footerTopY));
$pdf->AddPage();
$pdf->SetFont('Arial','',12);
foreach($pdf->mainItems as $element){
    $pdf->renderElement($element,false);
}

// if internal viewer is disabled, do the old method
if (!INTERNAL_PDF_VIEWER) {
    $pdf->SetTitle($pdf->PDFName,true);
    $pdf->Output('I',$pdf->PDFName);
    exit;
}

// if internal viewer is enabled, store in temp/{username}/
session_start(); // ensure we have $_SESSION
$userName = isset($_SESSION['username']) ? $_SESSION['username'] : 'guest';
$userTempDir = __DIR__ . '/temp/' . $userName;
if (!is_dir($userTempDir)) {
    mkdir($userTempDir, 0777, true);
}
$filePath = $userTempDir . '/' . $pdf->PDFName;
$pdfContent = $pdf->Output('S');
file_put_contents($filePath, $pdfContent);

// redirect to pdfjs viewer
//header('Location: pdfjs/web/viewer.html?file=../../pdf_proxy.php?f=' . urlencode($pdf->PDFName) . '&u=' . urlencode($userName));
//exit;

$combined = $pdf->PDFName . '@@' . $userName;

// 2) URL-encode it
$encoded = rawurlencode($combined);

// 3) Redirect to PDF.js viewer passing only "?f="
header('Location: pdfjs/web/viewer.html?file=../../pdf_proxy.php?f=' . $encoded);
exit;
