<?php
require_once('../config/config.php');
require_once('../core/financial_reports.php');
require_once('../core/financial_periods.php');
require_once('../core/accounts.php');

/**
 * Renders the CF Table object onto the PDF using $dialogData from the type dialog.
 * This calls your existing 'cashflowstatement.php' logic style (generateCashFlowStatement).
 * Because we only place final 4 lines: Operating, Investing, Financing, Net Increase in Cash.
 */
function renderCFTable($pdf, $element, $dialogData)
{
    global $pdo;

    // Grab table style from $element
    $borderShow     = !empty($element['bordershow']);
    $borderColorHex = !empty($element['bordercolor']) ? $element['bordercolor'] : '#000000';
    $borderWidth    = isset($element['borderwidth']) ? floatval($element['borderwidth']) : 1.0;

    $headerFontFam  = $element['headerfontfam']  ?? 'Courier';
    $headerFontSty  = $element['headerFontSty']  ?? ''; // e.g. "B" "I" "U"
    $headerFontSize = isset($element['headerfontsize']) ? floatval($element['headerfontsize']) : 12;
    $headerFontColor= $element['headerfontcolor'] ?? '#000000';
    $headerFill     = !empty($element['headerfill']);
    $headerFillColor= $element['headerfillcolor'] ?? '#cccccc';

    $dataFontFam    = $element['datafontfam']  ?? 'Courier';
    $dataFontSty    = $element['dataFontSty']  ?? ''; // e.g. "B" "I" "U"
    $dataFontSize   = isset($element['datafontsize']) ? floatval($element['datafontsize']) : 12;
    $dataFontColor  = $element['datafontcolor'] ?? '#000000';
    $dataFill       = $element['datafill']       ?? 'stripe';
    $dataFillColor1 = $element['datafillcolor']  ?? '#ffffff';
    $dataFillColor2 = $element['datafillcolor2'] ?? '#f3f3f3';

    $x = isset($element['x']) ? floatval($element['x']) : 10;
    $y = isset($element['y']) ? floatval($element['y']) : 10;
    $w = isset($element['width'])  ? floatval($element['width'])  : 60;
    $h = isset($element['height']) ? floatval($element['height']) : 50;

    // Convert hex => rgb
    $bc = hexToRgb($borderColorHex);
    // Prepare the PDF position
    $pdf->SetLineWidth($borderWidth);
    $pdf->SetDrawColor($bc['r'], $bc['g'], $bc['b']);
    $pdf->SetXY($x, $y);

    if ($borderShow) {
        $pdf->Rect($x, $y, $w, $h, 'D'); // Outer bounding box
    }

    // Determine the startDate, endDate, period from dialog
    $selectedPeriodId = isset($dialogData['financial_period_id']) ? (int)$dialogData['financial_period_id'] : 0;
    $startDate        = trim($dialogData['start_date'] ?? '');
    $endDate          = trim($dialogData['end_date']   ?? '');

    // If a valid period is chosen, override the date range with that period
    if ($selectedPeriodId > 0) {
        $fp = getFinancialPeriod($pdo, $selectedPeriodId);
        if ($fp) {
            $startDate = $fp['start_date'];
            $endDate   = $fp['end_date'];
        }
    }
    // If still empty, fallback to something safe
    if (empty($startDate)) $startDate = date('Y-01-01');
    if (empty($endDate))   $endDate   = date('Y-m-d');

    // Possibly rename the PDF
    $pdf->PDFName = 'CashFlowStatement_' . $startDate . '_to_' . $endDate . '.pdf';

    // We'll do a simplified 'generateCashFlowStatement' approach:
    $statementData = generateCashFlowStatement($pdo, $startDate, $endDate);
    // Summarize:
    $flow = [
        'Operating Activities' => 0.0,
        'Investing Activities' => 0.0,
        'Financing Activities' => 0.0
    ];

    // We'll do naive classification or you can customize:
    // (In a real scenario, we'd rely on the user classification or your built logic.)
    require_once('../core/accounts.php');
    $acctList = listAccounts($pdo);
    $typesMap = [];
    foreach ($acctList as $ac) {
        $typesMap[$ac['code']] = strtolower($ac['account_type']);
    }

    $investingKeywords  = ['investment','fixed asset'];
    $financingKeywords  = ['liability','equity'];
    foreach ($statementData as $txn) {
        $amt = floatval($txn['amount']);
        $dt  = $typesMap[$txn['debit_account']]  ?? '';
        $ct  = $typesMap[$txn['credit_account']] ?? '';

        $isInvesting = false;
        $isFinancing = false;
        $isOperating = false;
        if (in_array($dt, $investingKeywords) || in_array($ct, $investingKeywords)) {
            $isInvesting = true;
        } elseif (in_array($dt, $financingKeywords) || in_array($ct, $financingKeywords)) {
            $isFinancing = true;
        } else {
            $isOperating = true;
        }

        if ($isOperating)  $flow['Operating Activities']  += $amt;
        if ($isInvesting)  $flow['Investing Activities']  += $amt;
        if ($isFinancing)  $flow['Financing Activities']  += $amt;
    }
    $netIncrease = $flow['Operating Activities'] + $flow['Investing Activities'] + $flow['Financing Activities'];

    // We'll create a simple 2-col table
    $rows = [
        ['label' => 'Operating Activities',   'value' => $flow['Operating Activities']],
        ['label' => 'Investing Activities',   'value' => $flow['Investing Activities']],
        ['label' => 'Financing Activities',   'value' => $flow['Financing Activities']],
        ['label' => 'Net Increase in Cash',   'value' => $netIncrease]
    ];

    // Set up columns
    $col1W = 0.6 * $w;  // "Cash Flow Activities"
    $col2W = 0.4 * $w;  // "Amount"
    $rowHeight = 8;

    // Render the table header
    // Header style
    $hc  = hexToRgb($headerFontColor);
    $pdf->SetFont($headerFontFam, $headerFontSty, $headerFontSize);
    $pdf->SetTextColor($hc['r'], $hc['g'], $hc['b']);
    $hf  = hexToRgb($headerFillColor);
    if ($headerFill) {
        $pdf->SetFillColor($hf['r'], $hf['g'], $hf['b']);
    } else {
        $pdf->SetFillColor(255,255,255);
    }
    $pdf->SetXY($x, $y);
    $borderOpt = $borderShow ? 1 : 0;

    // Headers
    $pdf->Cell($col1W, $rowHeight, 'Cash Flow Activities', $borderOpt, 0, 'C', $headerFill);
    $pdf->Cell($col2W, $rowHeight, 'Amount',              $borderOpt, 0, 'C', $headerFill);
    $pdf->Ln($rowHeight);

    // Data style
    $dfc = hexToRgb($dataFontColor);
    $pdf->SetFont($dataFontFam, $dataFontSty, $dataFontSize);
    $pdf->SetTextColor($dfc['r'], $dfc['g'], $dfc['b']);

    $f1 = hexToRgb($dataFillColor1);
    $f2 = hexToRgb($dataFillColor2);

    $rowIndex = 0;
    foreach ($rows as $r) {
        $rowIndex++;
        // fill color
        $useFillColor = '255,255,255';
        if ($dataFill === 'solid') {
            $useFillColor = "{$f1['r']},{$f1['g']},{$f1['b']}";
        } elseif ($dataFill === 'stripe') {
            if ($rowIndex % 2 === 1) {
                $useFillColor = "{$f1['r']},{$f1['g']},{$f1['b']}";
            } else {
                $useFillColor = "{$f2['r']},{$f2['g']},{$f2['b']}";
            }
        }
        list($fr, $fg, $fb) = explode(',', $useFillColor);
        $pdf->SetFillColor($fr, $fg, $fb);

        $pdf->SetX($x);
        $pdf->Cell($col1W, $rowHeight, $r['label'], $borderOpt, 0, 'L', true);
        $pdf->Cell($col2W, $rowHeight, number_format($r['value'], 2), $borderOpt, 0, 'R', true);
        $pdf->Ln($rowHeight);
    }

    // done
}

/**
 * Helper: convert hex color => array [r,g,b]
 */
function hexToRgb($hex) {
    $hex = str_replace('#','',$hex);
    if(strlen($hex)===3){
        $hex=$hex[0].$hex[0].$hex[1].$hex[1].$hex[2].$hex[2];
    }
    $val=hexdec($hex);
    return [
        'r' => ($val >> 16) & 255,
        'g' => ($val >> 8)  & 255,
        'b' => $val & 255
    ];
}

