<?php
require_once('../config/config.php');
require_once('../core/financial_reports.php');
require_once('../core/financial_periods.php');

/**
 * Build a minimal hierarchical tree for Income Statement entries.
 * Each item must have 'code' and optionally 'parent'.
 */
function buildAccountTreeIS(array $items, $parent=null)
{
    $branch = [];
    foreach ($items as $item) {
        $itemParent = $item['parent'] ?? null;
        if ($itemParent === $parent) {
            // Recursively find children
            $children = buildAccountTreeIS($items, $item['code']);
            if ($children) {
                $item['children'] = $children;
            }
            $branch[] = $item;
        }
    }
    return $branch;
}

/**
 * Flatten the tree into a single array while assigning a 'level' for indentation
 */
function traverseAccountTreeIS(array $tree, array &$flat, int $depth=0)
{
    foreach ($tree as $node) {
        $node['level'] = $depth;
        $flat[] = $node;
        if (isset($node['children'])) {
            traverseAccountTreeIS($node['children'], $flat, $depth+1);
        }
    }
}

/**
 * Render the "ISTable" object in an Income Statement PDF.
 */
function renderISTable($pdf, $element, $dialogData)
{
    global $pdo;

    // Basic table styling
    $tblBorderShow  = isset($element['borderShow'])  ? (bool)$element['borderShow']  : true;
    $tblBorderColor = isset($element['borderColor']) ? $element['borderColor']       : '#000000';
    $tblBorderWidth = isset($element['borderWidth']) ? floatval($element['borderWidth']) : 1.0;

    // Header font
    $hdrFontFam   = $element['headerFontFam']   ?? 'Courier';
    $hdrFontSty   = $element['headerFontSty']   ?? '';
    $hdrFontSize  = (float)($element['headerFontSize']  ?? 12);
    $hdrFontColor = $element['headerFontColor'] ?? '#000000';
    $hdrFill      = isset($element['headerFill'])      ? (bool)$element['headerFill'] : true;
    $hdrFillColor = $element['headerFillColor'] ?? '#cccccc';

    // Data font
    $dataFontFam   = $element['dataFontFam']   ?? 'Courier';
    $dataFontSty   = $element['dataFontSty']   ?? '';
    $dataFontSize  = (float)($element['dataFontSize']  ?? 12);
    $dataFontColor = $element['dataFontColor'] ?? '#000000';
    $dataFill      = $element['dataFill']       ?? 'stripe';
    $dataFillColor  = $element['dataFillColor']  ?? '#ffffff';
    $dataFillColor2 = $element['dataFillColor2'] ?? '#f3f3f3';

    // Additional flags
    $hideNoBalance  = isset($element['hideNoBalance']) ? (bool)$element['hideNoBalance'] : false;
    $dataHierarchy  = isset($element['dataHierarchy'])  ? (bool)$element['dataHierarchy']  : false;
    $dataMulticolor = isset($element['dataMulticolor']) ? (bool)$element['dataMulticolor'] : true;

    // Unique for Income Statement:
    $colorRevenue = $element['colorRevenue'] ?? '#20c997';
    $colorExpense = $element['colorExpense'] ?? '#dc3545';
    $colorTotals  = $element['colorTotals']  ?? '#000000';

    // Table position/size
    $x = (float)($element['x'] ?? 10);
    $y = (float)($element['y'] ?? 10);
    $w = (float)($element['width']  ?? 60);
    $h = (float)($element['height'] ?? 50);

    // Convert border color to RGB
    $bc = hexToRgbIS($tblBorderColor);
    $borderR = $bc['r'];
    $borderG = $bc['g'];
    $borderB = $bc['b'];

    // Gather user dialog data
    $fpId      = $dialogData['financial_period_id'] ?? '';
    $startDate = $dialogData['start_date'] ?? '';
    $endDate   = $dialogData['end_date']   ?? '';

    // Decide which date range
    if (!empty($fpId)) {
        // If user selected a period, override with that period's boundaries
        $fp = getFinancialPeriod($pdo, (int)$fpId);
        if ($fp) {
            $startDate = $fp['start_date'];
            $endDate   = $fp['end_date'];
        }
    }
    if (empty($startDate)) {
        $startDate = date('Y-01-01');
    }
    if (empty($endDate)) {
        $endDate = date('Y-m-d');
    }

    $pdf->PDFName = 'Income Statement ' . $startDate . '_to_' . $endDate . '.pdf';

    // Retrieve data
    $rawIS = generateIncomeStatement($pdo, $startDate, $endDate);
    if (empty($rawIS)) {
        // No data, just skip
        return;
    }

    // Summation
    $totalRevenue  = 0.0;
    $totalExpenses = 0.0;
    // Temporary storage of items
    $tempList = [];
    foreach ($rawIS as $acct) {
        $t    = strtolower($acct['account_type'] ?? '');
        $amt  = (float)($acct['total_amount'] ?? 0);
        $code = $acct['code']  ?? null; // store code
        $parent = $acct['parent'] ?? null; // store parent if any

        if ($t==='revenue' || $t==='income') {
            $totalRevenue  += $amt;
            $group = 'R';
        } elseif ($t==='expense') {
            $totalExpenses += $amt;
            $group = 'E';
        } else {
            $group = 'X'; // not used
        }

        $tempList[] = [
            'code'   => $code,
            'parent' => $parent,
            'group'  => $group,
            'name'   => $acct['name'] ?? '',
            'amount' => $amt
        ];
    }
    $netIncome = $totalRevenue - $totalExpenses;

    // Build final table row list
    $rows = [];

    if ($dataHierarchy) {
        // Build a tree if we have code/parent
        $builtTree = buildAccountTreeIS($tempList, null);
        traverseAccountTreeIS($builtTree, $rows, 0);
    } else {
        // No hierarchy, just linear
        $rows = $tempList;
    }

    // Then append total lines
    $rows[] = [ 'group'=>'T','code'=>'','parent'=>null,'name'=>'Total Revenue','amount'=>$totalRevenue ];
    $rows[] = [ 'group'=>'T','code'=>'','parent'=>null,'name'=>'Total Expenses','amount'=>$totalExpenses ];
    $rows[] = [ 'group'=>'T','code'=>'','parent'=>null,'name'=>'Net Income','amount'=>$netIncome ];

    // Draw outer rectangle if needed
    $pdf->SetLineWidth($tblBorderWidth);
    $pdf->SetDrawColor($borderR, $borderG, $borderB);
    $pdf->SetXY($x, $y);
    if ($tblBorderShow) {
        $pdf->Rect($x, $y, $w, $h, 'D');
    }
    $tblBorder = $tblBorderShow ? 1 : 0;

    // Setup the header font in FPDF
    $pdf->SetFont($hdrFontFam, $hdrFontSty, $hdrFontSize);
    // Convert header font color
    $hc = hexToRgbIS($hdrFontColor);
    $pdf->SetTextColor($hc['r'], $hc['g'], $hc['b']);

    // Possibly fill color for header row
    $hf = hexToRgbIS($hdrFillColor);
    if ($hdrFill) {
        $pdf->SetFillColor($hf['r'], $hf['g'], $hf['b']);
    } else {
        $pdf->SetFillColor(255,255,255);
    }

    // Table columns
    $rowHeight = 8;
    $amountColW = 25.0;
    $nameColW   = $w - $amountColW;
    if ($nameColW < 20) {
        $nameColW = 20;
    }

    // Print header
    $pdf->SetX($x);
    // "Account Name" - center aligned
    $pdf->Cell($nameColW, $rowHeight, 'Account Name', $tblBorder, 0, 'C', $hdrFill);
    // "Amount" - center aligned
    $pdf->Cell($amountColW, $rowHeight, 'Amount', $tblBorder, 0, 'C', $hdrFill);
    $pdf->Ln($rowHeight);

    // Data font
    $pdf->SetFont($dataFontFam, $dataFontSty, $dataFontSize);
    $dfc = hexToRgbIS($dataFontColor);
    $pdf->SetTextColor($dfc['r'], $dfc['g'], $dfc['b']);

    $f1 = hexToRgbIS($dataFillColor);
    $f2 = hexToRgbIS($dataFillColor2);

    $lineCount = 0;
    foreach ($rows as $r) {
        // skip zero-balance row if hideNoBalance
        if ($hideNoBalance && abs($r['amount']) < 0.00001) {
            continue;
        }
        $lineCount++;

        // fill color logic (solid / stripe)
        if ($dataFill==='solid') {
            $pdf->SetFillColor($f1['r'], $f1['g'], $f1['b']);
        } elseif ($dataFill==='stripe') {
            if ($lineCount % 2 === 1) {
                $pdf->SetFillColor($f1['r'], $f1['g'], $f1['b']);
            } else {
                $pdf->SetFillColor($f2['r'], $f2['g'], $f2['b']);
            }
        } else {
            $pdf->SetFillColor(255,255,255);
        }

        // row color logic
        $rowColor = $dataFontColor; // default
        if ($dataMulticolor) {
            if ($r['group'] === 'R') {
                $rowColor = $colorRevenue;
            } elseif ($r['group'] === 'E') {
                $rowColor = $colorExpense;
            } elseif ($r['group'] === 'T') {
                $rowColor = $colorTotals;
            }
        }
        $rc = hexToRgbIS($rowColor);
        $pdf->SetTextColor($rc['r'], $rc['g'], $rc['b']);

        // If dataHierarchy is true, indent based on 'level'
        $level = (int)($r['level'] ?? 0);
        $indentSpaces = $dataHierarchy ? str_repeat('  ', $level) : '';
        $displayName  = $indentSpaces . $r['name'];

        // Output row
        $pdf->SetX($x);
        $pdf->Cell($nameColW, $rowHeight, $displayName, $tblBorder, 0, 'L', true);

        $amtStr = number_format($r['amount'], 2);
        $pdf->Cell($amountColW, $rowHeight, $amtStr, $tblBorder, 0, 'R', true);

        $pdf->Ln($rowHeight);

        // revert text color
        $pdf->SetTextColor($dfc['r'], $dfc['g'], $dfc['b']);
    }
    $pdf->Ln(3);
}

/**
 * Helper: convert hex color #RRGGBB to array
 */
function hexToRgbIS($hex)
{
    $hex = str_replace('#', '', $hex);
    if (strlen($hex) === 3) {
        $hex = $hex[0].$hex[0].$hex[1].$hex[1].$hex[2].$hex[2];
    }
    $val = hexdec($hex);
    return [
        'r' => ($val >> 16) & 255,
        'g' => ($val >> 8)  & 255,
        'b' => $val & 255
    ];
}
