<?php
require_once('../config/config.php');
require_once('../core/financial_reports.php');
require_once('../core/financial_periods.php');

/**
 * Render function for the "TBTable" object.
 *
 * We display two columns for Debit / Credit rather than "Total Debit" / "Total Credit".
 * Also color Revenue and Expense rows if dataMulticolor is true, using the newly added
 * colorRevenue (#20c997) and colorExpense (#dc3545).
 */
function renderTBTable($pdf, $element, $dialogData)
{

    // Set the PDF name based on element properties or dialog data


    global $pdo;

    // Basic table config:
    $tblBorderShow  = isset($element['borderShow'])  ? (bool)$element['borderShow']  : true;
    $tblBorderColor = isset($element['borderColor']) ? $element['borderColor']       : '#000000';
    $tblBorderWidth = isset($element['borderWidth']) ? floatval($element['borderWidth']) : 1.0;

    // Header style:
    $hdrFontFam   = $element['headerFontFam']   ?? 'Courier';
    $hdrFontSty   = $element['headerFontSty']   ?? '';
    $hdrFontSize  = floatval($element['headerFontSize']  ?? 12);
    $hdrFontColor = $element['headerFontColor'] ?? '#000000';
    $hdrFill      = isset($element['headerFill']) ? (bool)$element['headerFill'] : true;
    $hdrFillColor = $element['headerFillColor']  ?? '#cccccc';

    // Data style:
    $dataFontFam   = $element['dataFontFam']   ?? 'Courier';
    $dataFontSty   = $element['dataFontSty']   ?? '';
    $dataFontSize  = floatval($element['dataFontSize']  ?? 12);
    $dataFontColor = $element['dataFontColor'] ?? '#000000';
    $dataFill      = $element['dataFill']      ?? 'stripe';
    $dataFillColor  = $element['dataFillColor']  ?? '#ffffff';
    $dataFillColor2 = $element['dataFillColor2'] ?? '#f3f3f3';

    // Colors for different account types:
    $dataHierarchy = isset($element['dataHierarchy']) ? (bool)$element['dataHierarchy'] : true;
    $dataMulticolor= isset($element['dataMulticolor'])? (bool)$element['dataMulticolor']: true;
    $colorAsset    = $element['colorAsset']    ?? '#26A269';
    $colorLiab     = $element['colorLiab']     ?? '#1A5FB4';
    $colorEquity   = $element['colorEquity']   ?? '#DC8ADD';
    $colorTotals   = $element['colorTotals']   ?? '#000000';
    $colorRevenue  = $element['colorRevenue']  ?? '#20c997'; // newly added
    $colorExpense  = $element['colorExpense']  ?? '#dc3545'; // newly added

    // Additional Hide columns toggles
    $hideCode     = isset($element['hideCode'])     ? (bool)$element['hideCode']     : false;
    $hideAcctType = isset($element['hideAcctType']) ? (bool)$element['hideAcctType'] : false;
    $hideNoBalance= isset($element['hideNoBalance'])? (bool)$element['hideNoBalance'] : false;

    // Position & size
    $x = floatval($element['x'] ?? 10);
    $y = floatval($element['y'] ?? 10);
    $w = floatval($element['width']  ?? 60);
    $h = floatval($element['height'] ?? 50);

    // Convert border color from hex
    $bc        = hexToRgb($tblBorderColor);
    $borderR   = $bc['r'];
    $borderG   = $bc['g'];
    $borderB   = $bc['b'];

    // Gather the input for trialbalance: either period or start/end date
    $periodId = (int)($dialogData['financial_period_id'] ?? 0);
    $startDate = trim($dialogData['start_date'] ?? '');
    $endDate   = trim($dialogData['end_date']   ?? '');

    // Decide which date range to use
    $calcStart = '';
    $calcEnd   = '';
    if ($periodId > 0) {
        $fp = getFinancialPeriod($pdo, $periodId);
        if ($fp) {
            $calcStart = $fp['start_date'];
            $calcEnd   = $fp['end_date'];
        }
    }
    if (!$calcStart || !$calcEnd) {
        if (!empty($startDate) && !empty($endDate)) {
            $calcStart = $startDate;
            $calcEnd   = $endDate;
        } else {
            // fallback if nothing provided
            $calcStart = date('Y-01-01');
            $calcEnd   = date('Y-m-d');
        }
    }

    $pdf->PDFName = 'Trial Balance '.$calcStart.' - '.$calcEnd.'.pdf'; // Default name if not specified
    // get the Trial Balance array
    $trialBalance = calculateTrialBalanceForDateRange($pdo, $calcStart, $calcEnd);
    if (empty($trialBalance)) {
        // Nothing to draw, just return
        return;
    }

    // Build a tree, then flatten it for indentation
    $accountTree = buildTBAccountTree($trialBalance);
    $flatRows    = [];
    traverseTBTree($accountTree, $flatRows);

    // Prepare the rectangle for the table
    $pdf->SetLineWidth($tblBorderWidth);
    $pdf->SetDrawColor($borderR, $borderG, $borderB);
    $pdf->SetXY($x, $y);

    if ($tblBorderShow) {
        $pdf->Rect($x, $y, $w, $h, 'D');
    }
    $tblBorder = $tblBorderShow ? 1 : 0;

    // We'll display columns: "Type", "Code", "Name", "Debit", "Credit"
    $rowHeight = 8;
    // Decide column widths
    $acctTypeW = 25.0;
    $codeW     = 20.0;
    $debitW    = 30.0;
    $creditW   = 30.0;
    $fixedTotal= $debitW + $creditW;

    if (!$hideAcctType) {
        $fixedTotal += $acctTypeW;
    }
    if (!$hideCode) {
        $fixedTotal += $codeW;
    }

    // The remainder is for Name
    $nameW = $w - $fixedTotal;
    if ($nameW < 10) {
        $nameW = 10;
    }

    $cols = [];
    if (!$hideAcctType) {
        $cols[] = ['id' => 'acctType', 'label' => 'Type',   'width' => $acctTypeW];
    }
    if (!$hideCode) {
        $cols[] = ['id' => 'code',     'label' => 'Code',   'width' => $codeW];
    }
    $cols[] = ['id' => 'name',   'label' => 'Name',   'width' => $nameW];
    $cols[] = ['id' => 'debit',  'label' => 'Debit',  'width' => $debitW];
    $cols[] = ['id' => 'credit', 'label' => 'Credit', 'width' => $creditW];

    // Print header row
    $hc = hexToRgb($hdrFontColor);
    $pdf->SetFont($hdrFontFam, $hdrFontSty, $hdrFontSize);
    $pdf->SetTextColor($hc['r'], $hc['g'], $hc['b']);
    $hf = hexToRgb($hdrFillColor);
    if ($hdrFill) {
        $pdf->SetFillColor($hf['r'], $hf['g'], $hf['b']);
    } else {
        $pdf->SetFillColor(255, 255, 255);
    }
    $pdf->SetX($x);
    foreach ($cols as $c) {
        $pdf->Cell($c['width'], $rowHeight, $c['label'], $tblBorder, 0, 'C', $hdrFill);
    }
    $pdf->Ln($rowHeight);

    // Data row styling
    $dfc = hexToRgb($dataFontColor);
    $pdf->SetFont($dataFontFam, $dataFontSty, $dataFontSize);
    $pdf->SetTextColor($dfc['r'], $dfc['g'], $dfc['b']);
    $f1 = hexToRgb($dataFillColor);
    $f2 = hexToRgb($dataFillColor2);
    $rowCount = 0;

    // Render each row
    foreach ($flatRows as $r) {
        $debVal  = floatval($r['total_debit']  ?? 0);
        $credVal = floatval($r['total_credit'] ?? 0);

        // If hideNoBalance is set, skip row if both debit/credit are zero
        if ($hideNoBalance && abs($debVal) < 0.00001 && abs($credVal) < 0.00001) {
            continue;
        }

        $rowCount++;
        // fill color
        if ($dataFill === 'solid') {
            $pdf->SetFillColor($f1['r'], $f1['g'], $f1['b']);
        } elseif ($dataFill === 'stripe') {
            if ($rowCount % 2 === 1) {
                $pdf->SetFillColor($f1['r'], $f1['g'], $f1['b']);
            } else {
                $pdf->SetFillColor($f2['r'], $f2['g'], $f2['b']);
            }
        } else {
            // none
            $pdf->SetFillColor(255, 255, 255);
        }

        // figure out text color based on type
        $typ = strtolower($r['account_type'] ?? '');
        $rowColor = $dataFontColor; // default
        if ($dataMulticolor) {
            if ($typ === 'asset')       $rowColor = $colorAsset;
            elseif ($typ === 'liability') $rowColor = $colorLiab;
            elseif ($typ === 'equity')    $rowColor = $colorEquity;
            elseif ($typ === 'revenue' || $typ === 'income') $rowColor = $colorRevenue;
            elseif ($typ === 'expense')   $rowColor = $colorExpense;
            // If you have "is_total" or something, you could do rowColor=$colorTotals
        }

        $rc = hexToRgb($rowColor);
        $pdf->SetTextColor($rc['r'], $rc['g'], $rc['b']);

        $depth = intval($r['depth'] ?? 0);
        $indent = $dataHierarchy ? str_repeat('   ', $depth) : '';
        $acctName = $r['name'] ?? '';
        $acctType = ucfirst(strtolower($r['account_type'] ?? ''));
        $acctCode = $r['code'] ?? '';
        $debStr   = number_format($debVal, 2);
        $credStr  = number_format($credVal, 2);

        $pdf->SetX($x);
        foreach ($cols as $c) {
            $cellTxt = '';
            $align   = 'L';
            switch ($c['id']) {
                case 'acctType':
                    $cellTxt = $acctType;
                    break;
                case 'code':
                    $cellTxt = $acctCode;
                    break;
                case 'name':
                    $cellTxt = $indent . $acctName;
                    break;
                case 'debit':
                    $cellTxt = $debStr;
                    $align   = 'R';
                    break;
                case 'credit':
                    $cellTxt = $credStr;
                    $align   = 'R';
                    break;
            }
            $pdf->Cell($c['width'], $rowHeight, $cellTxt, $tblBorder, 0, $align, true);
        }
        $pdf->Ln($rowHeight);

        // restore default data font color
        $pdf->SetTextColor($dfc['r'], $dfc['g'], $dfc['b']);
    }

    // -- Add final TOTAL row at bottom:
    $totalDebitSum  = 0;
    $totalCreditSum = 0;
    foreach ($flatRows as $r) {
        $totalDebitSum  += floatval($r['total_debit']  ?? 0);
        $totalCreditSum += floatval($r['total_credit'] ?? 0);
    }
    $tc = hexToRgb($colorTotals);
    $pdf->SetTextColor($tc['r'], $tc['g'], $tc['b']);
    $pdf->SetX($x);

    foreach ($cols as $c) {
        $cellTxt = '';
        $align   = 'L';
        switch ($c['id']) {
            case 'acctType':
                $cellTxt = '';
                break;
            case 'code':
                $cellTxt = '';
                break;
            case 'name':
                $cellTxt = 'Total';
                break;
            case 'debit':
                $align   = 'R';
                $cellTxt = number_format($totalDebitSum, 2);
                break;
            case 'credit':
                $align   = 'R';
                $cellTxt = number_format($totalCreditSum, 2);
                break;
        }
        $pdf->Cell($c['width'], $rowHeight, $cellTxt, $tblBorder, 0, $align, true);
    }
    $pdf->Ln($rowHeight);
    // restore text color to black
    $pdf->SetTextColor(0,0,0);
}

/**
 * Build a tree from the raw trialBalance array so we can do indentation.
 */
function buildTBAccountTree(array $accounts, $parent=null): array
{
    $branch = [];
    foreach ($accounts as $acct) {
        $acctParent = (!empty($acct['parent']) && strtolower($acct['parent']) !== 'n/a')
            ? $acct['parent']
            : null;
        if ($acctParent === $parent) {
            $children = buildTBAccountTree($accounts, $acct['code']);
            if (!empty($children)) {
                $acct['children'] = $children;
            }
            $branch[] = $acct;
        }
    }
    return $branch;
}

/**
 * Flatten the tree with a depth indicator, so we can indent in PDF.
 */
function traverseTBTree(array $tree, array &$sortedList, $depth=0): void
{
    foreach ($tree as $node) {
        $node['depth'] = $depth;
        $sortedList[] = $node;
        if (isset($node['children'])) {
            traverseTBTree($node['children'], $sortedList, $depth+1);
        }
    }
}

/**
 * Convert #rrggbb => array('r'=>...,'g'=>...,'b'=>...).
 */
function hexToRgb($hex)
{
    $hex = str_replace('#','',$hex);
    if (strlen($hex) === 3) {
        // short notation
        $hex = $hex[0].$hex[0].$hex[1].$hex[1].$hex[2].$hex[2];
    }
    $val = hexdec($hex);
    return [
        'r' => ($val >> 16) & 255,
        'g' => ($val >> 8)  & 255,
        'b' => $val & 255
    ];
}
