window.typeObjectCreators = window.typeObjectCreators || {};

/**
 * Define a single object type: "TBTable"
 * This stands for "Trial Balance Table".
 */
window.typeObjectCreators['TBTable'] = function(nextId) {
  return {
    id: nextId,
    type: 'TBTable',
    x: 10,
    y: 10,
    width: 60,
    height: 50,
    borderShow: true,
    borderColor: '#000000',
    borderWidth: 1.0,

    headerFontFam: 'Courier',
    headerFontSty: '',
    headerFontSize: 12,
    headerFontColor: '#000000',
    headerFill: true,
    headerFillColor: '#cccccc',

    dataFontFam: 'Courier',
    dataFontSty: '',
    dataFontSize: 12,
    dataFontColor: '#000000',
    dataFill: 'stripe',    // 'none','solid','stripe'
    dataFillColor: '#ffffff',
    dataFillColor2: '#f3f3f3',

    dataHierarchy: true,
    dataMulticolor: true,

    colorAsset:   '#26A269',
    colorLiab:    '#1A5FB4',
    colorEquity:  '#DC8ADD',
    colorTotals:  '#000000',
    colorRevenue: '#20c997', // newly added for TB
    colorExpense: '#dc3545', // newly added for TB

    hideCode: false,
    hideAcctType: false,
    hideNoBalance: false
  };
};

/**
 * Provide a custom preview for "TBTable" on the editor canvas
 */
window.typeObjectCreators['_renderElementPreview'] = window.typeObjectCreators['_renderElementPreview'] || function(el, idx) {
  // If we see "TBTable", render it. Otherwise, return null to skip.
  if (el.type !== 'TBTable') {
    // If another type also uses `_renderElementPreview`, we won't break it—just return null
    return null;
  }

  const div = document.createElement('div');
  div.className = 'element';
  div.setAttribute('data-index', idx);

  // approximate bounding box
  div.style.border = '1px dashed #aaa';
  div.style.margin = '0';
  div.style.padding= '0';

  // compute position & size (just like in balancesheet)
  const leftPx   = (el.x || 0)   * 3;
  const topPx    = (el.y || 0)   * 3;
  const widthPx  = (el.width  || 60) * 3;
  const heightPx = (el.height || 50) * 3;

  div.style.left   = leftPx + 'px';
  div.style.top    = topPx  + 'px';
  div.style.width  = widthPx + 'px';
  div.style.height = heightPx + 'px';
  div.style.position = 'absolute';
  div.style.background = '#fff';
  div.style.overflow   = 'hidden';

  // create a table preview
  const table = document.createElement('table');
  table.style.width           = '100%';
  table.style.height          = '100%';
  table.style.borderCollapse  = 'collapse';
  table.style.boxSizing       = 'border-box';

  // border
  const pxWidth = Math.round((el.borderWidth || 1.0) * 3);
  const color   = el.borderColor || '#000000';
  if (el.borderShow) {
    table.style.border = pxWidth + 'px solid ' + color;
  } else {
    table.style.border = 'none';
  }

  // We want columns: [Account Type] [Code] [Name] [Total Debit] [Total Credit]
  let colDefs = [];
  if (!el.hideAcctType) {
    colDefs.push({ id: 'acctType', header: 'Account Type' });
  }
  if (!el.hideCode) {
    colDefs.push({ id: 'code', header: 'Code' });
  }
  colDefs.push({ id: 'name',   header: 'Name' });
  colDefs.push({ id: 'debit',  header: 'Total Debit' });
  colDefs.push({ id: 'credit', header: 'Total Credit' });

  // table header row
  const thead = document.createElement('thead');
  const thr   = document.createElement('tr');
  const thBg  = el.headerFill ? (el.headerFillColor || '#cccccc') : 'transparent';
  const thColor = el.headerFontColor || '#000000';

  // parse font style: B, I, U
  const hFontWeight   = el.headerFontSty.includes('B') ? 'bold'   : 'normal';
  const hFontStyle    = el.headerFontSty.includes('I') ? 'italic' : 'normal';
  const hFontDecoration = el.headerFontSty.includes('U') ? 'underline' : 'none';
  const hFontSize = (el.headerFontSize || 12) + 'px';

  colDefs.forEach(def => {
    const th = document.createElement('th');
    th.textContent = def.header;
    if (el.borderShow) {
      th.style.border = pxWidth + 'px solid ' + color;
    } else {
      th.style.border = 'none';
    }
    th.style.background     = thBg;
    th.style.color          = thColor;
    th.style.fontFamily     = el.headerFontFam;
    th.style.fontSize       = hFontSize;
    th.style.fontWeight     = hFontWeight;
    th.style.fontStyle      = hFontStyle;
    th.style.textDecoration = hFontDecoration;
    th.style.textAlign      = 'center';
    thr.appendChild(th);
  });
  thead.appendChild(thr);
  table.appendChild(thead);

  // sample data rows
  const tbody = document.createElement('tbody');
  const fill1 = el.dataFillColor  || '#ffffff';
  const fill2 = el.dataFillColor2 || '#f3f3f3';
  const fillMode = el.dataFill || 'stripe';

  // parse data font style
  const dFontWeight    = el.dataFontSty.includes('B') ? 'bold'   : 'normal';
  const dFontStyle     = el.dataFontSty.includes('I') ? 'italic' : 'normal';
  const dFontDecoration= el.dataFontSty.includes('U') ? 'underline' : 'none';
  const dFontSize = (el.dataFontSize || 12) + 'px';

  // sample TR
  const sampleRows = [
    // account_type, code, name, total_debit, total_credit
    { type:'asset',     code:'1000',  depth:0, name:'Assets',              debit:'0.00', credit:'0.00' },
    { type:'asset',     code:'1100',  depth:1, name:'Current Assets',      debit:'0.00', credit:'0.00' },
    { type:'asset',     code:'1110',  depth:2, name:'Inventory',           debit:'0.00', credit:'0.00' },
    { type:'asset',     code:'1120',  depth:2, name:'Accounts Receivable', debit:'0.00', credit:'0.00' },
    { type:'liability', code:'2000',  depth:0, name:'Liabilities',         debit:'0.00', credit:'0.00' },
    { type:'liability', code:'2100',  depth:1, name:'Current Liabilities', debit:'0.00', credit:'0.00' },
    { type:'liability', code:'2110',  depth:2, name:'Accounts Payable',    debit:'0.00', credit:'0.00' },
    { type:'equity',    code:'3000',  depth:0, name:'Equity',              debit:'0.00', credit:'0.00' },
    { type:'equity',    code:'3200',  depth:1, name:'Retained Earnings',   debit:'351941.15', credit:'0.00' },
    { type:'revenue',   code:'4000',  depth:0, name:'Income',             debit:'0.00',  credit:'0.00' },
    { type:'revenue',   code:'4100',  depth:1, name:'Sales',              debit:'3984.60', credit:'497987.36' },
    { type:'expense',   code:'5000',  depth:0, name:'Expenses',           debit:'0.00', credit:'0.00' },
    { type:'expense',   code:'5100',  depth:1, name:'Cost of Sales',      debit:'257201.99', credit:'904.61' },
    { type:'expense',   code:'5201',  depth:2, name:'Accounting',         debit:'16435.19', credit:'0.00' },
    { type:'total',     code:'',      depth:0, name:'Total',              debit:'3407716.56', credit:'3407716.56'}
  ];

  let rowCount = 0;
  sampleRows.forEach(r => {
    rowCount++;
    const tr = document.createElement('tr');
    // determine fill
    let rowBg = '#ffffff';
    if (fillMode === 'solid') {
      rowBg = fill1;
    } else if (fillMode === 'stripe') {
      rowBg = (rowCount % 2 === 1 ? fill1 : fill2);
    }
    tr.style.background = rowBg;

    // if dataMulticolor is true, color row by type
    let rowColor = el.dataFontColor || '#000000';
    if (el.dataMulticolor) {
      const t = r.type.toLowerCase();
      if (t === 'asset')      rowColor = el.colorAsset   || '#26A269';
      else if (t === 'liability') rowColor = el.colorLiab|| '#1A5FB4';
      else if (t === 'equity')    rowColor = el.colorEquity || '#DC8ADD';
      else if (t === 'revenue')   rowColor = el.colorRevenue|| '#20c997';
      else if (t === 'expense')   rowColor = el.colorExpense|| '#dc3545';
      else if (t === 'total')     rowColor = el.colorTotals || '#000000';
    }

    tr.style.color          = rowColor;
    tr.style.fontFamily     = el.dataFontFam;
    tr.style.fontSize       = dFontSize;
    tr.style.fontWeight     = dFontWeight;
    tr.style.fontStyle      = dFontStyle;
    tr.style.textDecoration = dFontDecoration;

    colDefs.forEach(def => {
      const td = document.createElement('td');
      if (el.borderShow) {
        td.style.border = pxWidth + 'px solid ' + color;
      } else {
        td.style.border = 'none';
      }
      let cellText = '';
      let align    = 'left';
      switch (def.id) {
        case 'acctType':
          cellText = (r.type === 'total' ? 'Total' : capitalize(r.type));
          break;
        case 'code':
          cellText = r.code || '';
          break;
        case 'name':
          if (el.dataHierarchy) {
            const indent = '   '.repeat(r.depth || 0);
            cellText = indent + (r.name || '');
          } else {
            cellText = r.name || '';
          }
          break;
        case 'debit':
          cellText = r.debit || '0.00';
          align = 'right';
          break;
        case 'credit':
          cellText = r.credit || '0.00';
          align = 'right';
          break;
      }
      td.textContent   = cellText;
      td.style.textAlign = align;
      tr.appendChild(td);
    });
    tbody.appendChild(tr);
  });
  table.appendChild(tbody);

  div.appendChild(table);

  // Add a resize handle
  const resizeHandle = document.createElement('div');
  resizeHandle.className = 'resize-br-handle';
  div.appendChild(resizeHandle);
  resizeHandle.addEventListener('mousedown', e => {
    e.stopPropagation();
    e.preventDefault();
    activeElementIndex = idx;
    resizingElementIndex = idx;
    resizingCorner = 'BR';
    offsetX = e.offsetX;
    offsetY = e.offsetY;
  });

  div.addEventListener('mousedown', e => {
    activeElementDiv = div;
    isDragging = true;
    offsetX = e.offsetX;
    offsetY = e.offsetY;
    activeElementIndex = idx;
    loadProperties(idx);
    e.preventDefault();
    objectListSel.value = (el.id || '') + '';
  });

  return div;
};

// Helper function to capitalize first letter
function capitalize(str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
}

window.typeFunctions = window.typeFunctions || {};
window.typeFunctions['trialbalance'] = window.typeFunctions['trialbalance'] || {};

/**
 * Update property function called when user changes a field in property box
 */
window.typeFunctions['trialbalance'].updateProperty = function(el, propId, newValue) {
  if (!el || el.type !== 'TBTable') return;
  switch (propId) {
    case 'tbtable-x':            el.x = parseFloat(newValue) || 0;  break;
    case 'tbtable-y':            el.y = parseFloat(newValue) || 0;  break;
    case 'tbtable-w':            el.width  = parseFloat(newValue) || 60;  break;
    case 'tbtable-h':            el.height = parseFloat(newValue) || 50;  break;
    case 'tbtable-bordershow':   el.borderShow = (newValue === 'on');     break;
    case 'tbtable-bordercolor':  el.borderColor = newValue;               break;
    case 'tbtable-borderwidth':  el.borderWidth = parseFloat(newValue)||1;break;

    // header font style
    case 'tbtable-headerBold':
      if (newValue === 'on') {
        if (!el.headerFontSty.includes('B')) el.headerFontSty += 'B';
      } else {
        el.headerFontSty = el.headerFontSty.replace(/B/g, '');
      }
      break;
    case 'tbtable-headerItalic':
      if (newValue === 'on') {
        if (!el.headerFontSty.includes('I')) el.headerFontSty += 'I';
      } else {
        el.headerFontSty = el.headerFontSty.replace(/I/g, '');
      }
      break;
    case 'tbtable-headerUnderline':
      if (newValue === 'on') {
        if (!el.headerFontSty.includes('U')) el.headerFontSty += 'U';
      } else {
        el.headerFontSty = el.headerFontSty.replace(/U/g, '');
      }
      break;
    case 'tbtable-headerfontfam':   el.headerFontFam   = newValue; break;
    case 'tbtable-headerfontsize':  el.headerFontSize  = parseFloat(newValue)||12; break;
    case 'tbtable-headerfontcolor': el.headerFontColor = newValue; break;
    case 'tbtable-headerfill':      el.headerFill      = (newValue === 'on'); break;
    case 'tbtable-headerfillcolor': el.headerFillColor = newValue; break;

    // data font style
    case 'tbtable-dataBold':
      if (newValue === 'on') {
        if (!el.dataFontSty.includes('B')) el.dataFontSty += 'B';
      } else {
        el.dataFontSty = el.dataFontSty.replace(/B/g, '');
      }
      break;
    case 'tbtable-dataItalic':
      if (newValue === 'on') {
        if (!el.dataFontSty.includes('I')) el.dataFontSty += 'I';
      } else {
        el.dataFontSty = el.dataFontSty.replace(/I/g, '');
      }
      break;
    case 'tbtable-dataUnderline':
      if (newValue === 'on') {
        if (!el.dataFontSty.includes('U')) el.dataFontSty += 'U';
      } else {
        el.dataFontSty = el.dataFontSty.replace(/U/g, '');
      }
      break;
    case 'tbtable-datafontfam':    el.dataFontFam   = newValue; break;
    case 'tbtable-datafontsize':   el.dataFontSize  = parseFloat(newValue)||12; break;
    case 'tbtable-datafontcolor':  el.dataFontColor = newValue; break;
    case 'tbtable-datafill':       el.dataFill      = newValue; break;
    case 'tbtable-datafillcolor':  el.dataFillColor = newValue; break;
    case 'tbtable-datafillcolor2': el.dataFillColor2= newValue; break;
    case 'tbtable-datahierarchy':  el.dataHierarchy = (newValue === 'on'); break;
    case 'tbtable-datamulticolor': el.dataMulticolor= (newValue === 'on'); break;

    // type colors
    case 'tbtable-colorasset':   el.colorAsset   = newValue; break;
    case 'tbtable-colorliab':    el.colorLiab    = newValue; break;
    case 'tbtable-colorequity':  el.colorEquity  = newValue; break;
    case 'tbtable-colortotals':  el.colorTotals  = newValue; break;
    case 'tbtable-colorrevenue': el.colorRevenue = newValue; break;
    case 'tbtable-colorexpense': el.colorExpense = newValue; break;

    // Hide columns
    case 'tbtable-hidecode':     el.hideCode     = (newValue === 'on'); break;
    case 'tbtable-hideaccttype': el.hideAcctType = (newValue === 'on'); break;
    case 'tbtable-hidenobalance':el.hideNoBalance= (newValue === 'on'); break;

    default:
      console.warn("Unhandled TBTable prop:", propId, newValue);
  }
};

/**
 * Called when we load the property panel for a "TBTable".
 */
window.typeFunctions['trialbalance'].populateProperties = function(el) {
  if (!el || el.type !== 'TBTable') return;

  // x,y,w,h
  const xInput = document.getElementById('tbtable-x');
  if (xInput) xInput.value = el.x || 0;
  const yInput = document.getElementById('tbtable-y');
  if (yInput) yInput.value = el.y || 0;
  const wInput = document.getElementById('tbtable-w');
  if (wInput) wInput.value = el.width || 60;
  const hInput = document.getElementById('tbtable-h');
  if (hInput) hInput.value = el.height || 50;

  // border
  const borderShow    = document.getElementById('tbtable-bordershow');
  if (borderShow) borderShow.checked = !!el.borderShow;
  const borderColor   = document.getElementById('tbtable-bordercolor');
  if (borderColor) borderColor.value = el.borderColor || '#000000';
  const borderWidth   = document.getElementById('tbtable-borderwidth');
  if (borderWidth) borderWidth.value = el.borderWidth || 1;

  // header
  const hFam   = document.getElementById('tbtable-headerfontfam');
  if (hFam) hFam.value = el.headerFontFam || 'Courier';
  const hSize  = document.getElementById('tbtable-headerfontsize');
  if (hSize) hSize.value = el.headerFontSize || 12;
  const hColor = document.getElementById('tbtable-headerfontcolor');
  if (hColor) hColor.value = el.headerFontColor || '#000000';
  const hFill  = document.getElementById('tbtable-headerfill');
  if (hFill) hFill.checked = !!el.headerFill;
  const hFillC = document.getElementById('tbtable-headerfillcolor');
  if (hFillC) hFillC.value = el.headerFillColor || '#cccccc';

  // header styles B,I,U
  const hBold   = document.getElementById('tbtable-headerBold');
  const hItal   = document.getElementById('tbtable-headerItalic');
  const hUnder  = document.getElementById('tbtable-headerUnderline');
  if (hBold)  hBold.checked   = (el.headerFontSty || '').includes('B');
  if (hItal)  hItal.checked   = (el.headerFontSty || '').includes('I');
  if (hUnder) hUnder.checked  = (el.headerFontSty || '').includes('U');

  // data
  const dFam    = document.getElementById('tbtable-datafontfam');
  if (dFam) dFam.value = el.dataFontFam || 'Courier';
  const dSize   = document.getElementById('tbtable-datafontsize');
  if (dSize) dSize.value = el.dataFontSize || 12;
  const dColor  = document.getElementById('tbtable-datafontcolor');
  if (dColor) dColor.value = el.dataFontColor || '#000000';
  const dFill   = document.getElementById('tbtable-datafill');
  if (dFill) dFill.value = el.dataFill || 'stripe';
  const dFillC  = document.getElementById('tbtable-datafillcolor');
  if (dFillC) dFillC.value = el.dataFillColor || '#ffffff';
  const dFillC2 = document.getElementById('tbtable-datafillcolor2');
  if (dFillC2) dFillC2.value = el.dataFillColor2 || '#f3f3f3';

  // data style B,I,U
  const dBold   = document.getElementById('tbtable-dataBold');
  const dItal   = document.getElementById('tbtable-dataItalic');
  const dUnder  = document.getElementById('tbtable-dataUnderline');
  if (dBold)  dBold.checked   = (el.dataFontSty || '').includes('B');
  if (dItal)  dItal.checked   = (el.dataFontSty || '').includes('I');
  if (dUnder) dUnder.checked  = (el.dataFontSty || '').includes('U');

  // data checkboxes
  const dHier = document.getElementById('tbtable-datahierarchy');
  if (dHier) dHier.checked = !!el.dataHierarchy;
  const dMulti= document.getElementById('tbtable-datamulticolor');
  if (dMulti) dMulti.checked = !!el.dataMulticolor;

  // colorAsset colorLiab colorEquity colorTotals
  const cA = document.getElementById('tbtable-colorasset');
  if (cA) cA.value = el.colorAsset || '#26A269';
  const cL = document.getElementById('tbtable-colorliab');
  if (cL) cL.value = el.colorLiab || '#1A5FB4';
  const cE = document.getElementById('tbtable-colorequity');
  if (cE) cE.value = el.colorEquity || '#DC8ADD';
  const cT = document.getElementById('tbtable-colortotals');
  if (cT) cT.value = el.colorTotals || '#000000';

  // new: revenue & expense
  const cRev = document.getElementById('tbtable-colorrevenue');
  if (cRev) cRev.value = el.colorRevenue || '#20c997';
  const cExp = document.getElementById('tbtable-colorexpense');
  if (cExp) cExp.value = el.colorExpense || '#dc3545';

  // hide columns
  const hCode = document.getElementById('tbtable-hidecode');
  if (hCode) hCode.checked = !!el.hideCode;
  const hAct  = document.getElementById('tbtable-hideaccttype');
  if (hAct)  hAct.checked  = !!el.hideAcctType;
  const hNB   = document.getElementById('tbtable-hidenobalance');
  if (hNB)   hNB.checked   = !!el.hideNoBalance;
};

