<?php
require_once('../config/config.php');
require_once('../core/financial_reports.php');
require_once('../core/vat_periods.php');

/**
 * Render function for the VRTable object.
 * - Now uses additional "H1" style props to format headings:
 *   "Transaction Listing by VAT Rate", each $vatGroup['rate_name'], and "VAT Report Results".
 * - Also hides the listing portion if summaryOnly is checked.
 * - Adds center remark under total net change.
 * - Adds new data fill logic for row backgrounds (none|solid|stripe).
 */
function renderVRTable($pdf, $element, $dialogData)
{
    global $pdo;

    // ============ Extract VRTable element properties ============

    // H1 styling (for big headings)
    $h1FontFam   = $element['h1FontFam']   ?? 'Courier';
    $h1FontSty   = $element['h1FontSty']   ?? 'B';
    $h1FontSize  = isset($element['h1FontSize']) ? floatval($element['h1FontSize']) : 16.0;
    $h1FontColor = $element['h1FontColor'] ?? '#000000';
    $h1Fill      = isset($element['h1Fill']) ? (bool)$element['h1Fill'] : true;
    $h1FillColor = $element['h1FillColor'] ?? '#ffffff';

    // Table border
    $borderShow  = isset($element['borderShow'])  ? (bool)$element['borderShow']  : true;
    $borderColor = isset($element['borderColor']) ? $element['borderColor']       : '#000000';
    $borderWidth = isset($element['borderWidth']) ? floatval($element['borderWidth']) : 1.0;

    // Table header styling (these apply to the inner column headers, not the big H1)
    $hdrFontFam   = isset($element['headerFontFam'])   ? $element['headerFontFam']   : 'Courier';
    $hdrFontSty   = isset($element['headerFontSty'])   ? $element['headerFontSty']   : '';
    $hdrFontSize  = isset($element['headerFontSize'])  ? floatval($element['headerFontSize']) : 12;
    $hdrFontColor = isset($element['headerFontColor']) ? $element['headerFontColor'] : '#000000';
    $hdrFill      = isset($element['headerFill'])      ? (bool)$element['headerFill'] : true;
    $hdrFillColor = isset($element['headerFillColor']) ? $element['headerFillColor']  : '#cccccc';

    // Data styling
    $dataFontFam   = isset($element['dataFontFam'])   ? $element['dataFontFam']      : 'Courier';
    $dataFontSty   = isset($element['dataFontSty'])   ? $element['dataFontSty']      : '';
    $dataFontSize  = isset($element['dataFontSize'])  ? floatval($element['dataFontSize']) : 12;
    $dataFontColor = isset($element['dataFontColor']) ? $element['dataFontColor']    : '#000000';

    // Data fill
    $dataFill      = isset($element['dataFill'])      ? $element['dataFill']         : 'none';    // 'none','solid','stripe'
    $dataFillColor = isset($element['dataFillColor']) ? $element['dataFillColor']    : '#ffffff';
    $dataFillColor2= isset($element['dataFillColor2'])? $element['dataFillColor2']   : '#f3f3f3';

    // "Summary only" property
    $summaryOnly = isset($element['summaryOnly']) ? (bool)$element['summaryOnly'] : false;

    // Coordinates
    $x = isset($element['x']) ? floatval($element['x']) : 10;
    $y = isset($element['y']) ? floatval($element['y']) : 10;
    $w = isset($element['width']) ? floatval($element['width']) : 80;
    $h = isset($element['height'])? floatval($element['height']) : 60;

    // Convert border color
    $bc = hexToRgb($borderColor);
    $borderR = $bc['r'];
    $borderG = $bc['g'];
    $borderB = $bc['b'];

    // Determine date range from dialogData
    $vatPeriodId = !empty($dialogData['vat_period_id']) ? (int)$dialogData['vat_period_id'] : 0;
    $startDate   = trim($dialogData['start_date'] ?? '');
    $endDate     = trim($dialogData['end_date']   ?? '');

    // If user selected a VAT period, override start/end
    if ($vatPeriodId > 0) {
        $vp = getVATPeriod($pdo, $vatPeriodId);
        if ($vp) {
            $startDate = $vp['start_date'];
            $endDate   = $vp['end_date'];
        }
    }

    if ($element['summaryOnly']){
        $pdf->PDFName = 'VAT_Report_' . ($startDate ?: '...') . '_to_' . ($endDate ?: '...') . ' Summary.pdf';
    }else{
        $pdf->PDFName = 'VAT_Report_' . ($startDate ?: '...') . '_to_' . ($endDate ?: '...') . '.pdf';
    }
    // Optionally draw bounding rectangle if $borderShow
    if ($borderShow) {
        $pdf->SetLineWidth($borderWidth);
        $pdf->SetDrawColor($borderR, $borderG, $borderB);
        // We won't do a big $pdf->Rect(...) unless needed.
        // But you can uncomment if you want a bounding box:
        // $pdf->Rect($x, $y, $w, $h, 'D');
    }

    // We'll set a local left margin so the content doesn't exceed width $w
    $pdf->SetXY($x, $y);
    $pdf->SetLeftMargin($x);
    $pdf->SetRightMargin($pdf->GetPageWidth() - ($x + $w));

    // Prepare the "H1" style for big headings
    $pdf->SetFont($h1FontFam, $h1FontSty, $h1FontSize);
    $h1rgb = hexToRgb($h1FontColor);
    $pdf->SetTextColor($h1rgb['r'],$h1rgb['g'],$h1rgb['b']);

    // 1) Transaction Listing by VAT Rate
    if (!$summaryOnly && !empty($startDate) && !empty($endDate)) {
        // Possibly fill color behind heading
        if ($h1Fill) {
            $hf = hexToRgb($h1FillColor);
            $pdf->SetFillColor($hf['r'],$hf['g'],$hf['b']);
            $pdf->Cell($w, 10, 'Transaction Listing by VAT Rate', 0, 1, 'C', true);
        } else {
            $pdf->Cell($w, 10, 'Transaction Listing by VAT Rate', 0, 1, 'C');
        }

        $grouped = fetchVATTransactionsByRate($pdo, $startDate, $endDate);

        // For each group => $vatGroup['rate_name'] heading
        foreach ($grouped as $rateId => $vatGroup) {
            // Slightly smaller or keep same
            $pdf->SetFont($h1FontFam, $h1FontSty, max($h1FontSize-1, 10));
            $label = $vatGroup['rate_name'] . ' (' . number_format($vatGroup['rate_value'],2) . '%)';
            if ($h1Fill) {
                $pdf->Cell($w, 8, $label, 0, 1, 'C', true);
            } else {
                $pdf->Cell($w, 8, $label, 0, 1, 'C');
            }

            // Back to normal table header styling
            $pdf->SetFont($hdrFontFam, $hdrFontSty, $hdrFontSize);
            $hdrC = hexToRgb($hdrFontColor);
            $pdf->SetTextColor($hdrC['r'],$hdrC['g'],$hdrC['b']);
            if ($hdrFill) {
                $hf2 = hexToRgb($hdrFillColor);
                $pdf->SetFillColor($hf2['r'],$hf2['g'],$hf2['b']);
            } else {
                $pdf->SetFillColor(255,255,255);
            }

            // Original column widths total ~190 => scale to fit $w
            $colTotal = 190.0;
            $scale    = $w / $colTotal;
            $widths   = [35*$scale, 40*$scale, 40*$scale, 25*$scale, 50*$scale];
            $headerCols = ['Transaction Date','Debit Account','Credit Account','Amount','Description'];

            // Print table header
            foreach ($headerCols as $i => $hcTitle) {
                $pdf->Cell($widths[$i], 6, $hcTitle, 1, 0, 'C', true);
            }
            $pdf->Ln(6);

            // Now the data rows
            $pdf->SetFont($dataFontFam, $dataFontSty, $dataFontSize);
            $dfc = hexToRgb($dataFontColor);
            $pdf->SetTextColor($dfc['r'],$dfc['g'],$dfc['b']);

            $rowCount=0;
            foreach ($vatGroup['transactions'] as $tx) {
                $rowCount++;
                // data fill logic
                $rowBg = ['r'=>255,'g'=>255,'b'=>255];
                if ($dataFill==='solid') {
                    $rowBg = hexToRgb($dataFillColor);
                } elseif ($dataFill==='stripe') {
                    $rowBg = ($rowCount % 2===1)
                        ? hexToRgb($dataFillColor)
                        : hexToRgb($dataFillColor2);
                }
                $pdf->SetFillColor($rowBg['r'],$rowBg['g'],$rowBg['b']);

                $debitName  = $tx['debit_name']  ?: $tx['debit_account'];
                $creditName = $tx['credit_name'] ?: $tx['credit_account'];

                $pdf->Cell($widths[0], 6, $tx['transaction_date'], 1, 0, 'L', true);
                $pdf->Cell($widths[1], 6, $debitName,             1, 0, 'L', true);
                $pdf->Cell($widths[2], 6, $creditName,            1, 0, 'L', true);
                $pdf->Cell($widths[3], 6, number_format($tx['amount'],2), 1, 0, 'R', true);
                $pdf->Cell($widths[4], 6, $tx['description'],     1, 0, 'L', true);
                $pdf->Ln(6);
            }
            $pdf->Ln(4);

            // reset H1 font for next group if needed
            $pdf->SetFont($h1FontFam, $h1FontSty, $h1FontSize);
        }

        // Some spacing before the summary
        $pdf->SetXY($x, $pdf->GetY() + 5);
    }

    // 2) Summarized VAT Report table
    $vatReport = [];
    if (!empty($startDate) && !empty($endDate)) {
        $vatReport = generateVATReport($pdo, ($vatPeriodId>0 ? $vatPeriodId : null), $startDate, $endDate);
    }

    if (!empty($vatReport)) {
        // Title: "VAT Report Results"
        $pdf->SetFont($h1FontFam, $h1FontSty, $h1FontSize);
        $pdf->SetTextColor($h1rgb['r'],$h1rgb['g'],$h1rgb['b']);
        if ($h1Fill) {
            $hf = hexToRgb($h1FillColor);
            $pdf->SetFillColor($hf['r'],$hf['g'],$hf['b']);
            $pdf->Cell($w, 10, 'VAT Report Results', 0, 1, 'C', true);
        } else {
            $pdf->Cell($w, 10, 'VAT Report Results', 0, 1, 'C');
        }

        // Table columns
        $totalNetChange = 0;
        foreach ($vatReport as $row) {
            $totalNetChange += (float)$row['net_change'];
        }

        // table header
        $pdf->SetFont($hdrFontFam, $hdrFontSty, $hdrFontSize);
        $hdrC = hexToRgb($hdrFontColor);
        $pdf->SetTextColor($hdrC['r'],$hdrC['g'],$hdrC['b']);
        if ($hdrFill) {
            $hf2 = hexToRgb($hdrFillColor);
            $pdf->SetFillColor($hf2['r'],$hf2['g'],$hf2['b']);
        } else {
            $pdf->SetFillColor(255,255,255);
        }

        // base widths sum to ~175 => scale
        $colTotal2 = 175.0;
        $scale2    = $w / $colTotal2;
        $cols = [
            ['header'=>'Account Code', 'field'=>'account_code',  'baseWidth'=>25],
            ['header'=>'Account Name', 'field'=>'account_name',  'baseWidth'=>45],
            ['header'=>'Account Type', 'field'=>'account_type',  'baseWidth'=>30],
            ['header'=>'Total Debit',  'field'=>'total_debit',   'baseWidth'=>25],
            ['header'=>'Total Credit', 'field'=>'total_credit',  'baseWidth'=>25],
            ['header'=>'Net Change',   'field'=>'net_change',    'baseWidth'=>25]
        ];
        $lineHeight = 7;
        foreach ($cols as $c) {
            $colW = $c['baseWidth'] * $scale2;
            $pdf->Cell($colW, $lineHeight, $c['header'], 1, 0, 'C', true);
        }
        $pdf->Ln($lineHeight);

        // table rows
        $pdf->SetFont($dataFontFam, $dataFontSty, $dataFontSize);
        $dfc = hexToRgb($dataFontColor);
        $pdf->SetTextColor($dfc['r'],$dfc['g'],$dfc['b']);

        $rowCount=0;
        foreach ($vatReport as $rw) {
            $rowCount++;
            $rowBg = ['r'=>255,'g'=>255,'b'=>255];
            if ($dataFill==='solid') {
                $rowBg = hexToRgb($dataFillColor);
            } elseif ($dataFill==='stripe') {
                $rowBg = ($rowCount % 2===1)
                    ? hexToRgb($dataFillColor)
                    : hexToRgb($dataFillColor2);
            }
            $pdf->SetFillColor($rowBg['r'],$rowBg['g'],$rowBg['b']);

            foreach ($cols as $c) {
                $colW = $c['baseWidth'] * $scale2;
                $val  = $rw[$c['field']];
                if (in_array($c['field'], ['total_debit','total_credit','net_change'])) {
                    $val = number_format($val,2);
                    $pdf->Cell($colW, $lineHeight, $val, 1, 0, 'R', true);
                } else {
                    $pdf->Cell($colW, $lineHeight, $val, 1, 0, 'L', true);
                }
            }
            $pdf->Ln($lineHeight);
        }

        // final total net change row
        $pdf->SetFont($dataFontFam,'B',$dataFontSize+1);
        $sumWidth = 0;
        for($i=0; $i<5; $i++){
            $sumWidth += $cols[$i]['baseWidth'] * $scale2;
        }
        $pdf->Cell($sumWidth, $lineHeight, 'Total Net Change:', 1, 0, 'R');
        $pdf->Cell($cols[5]['baseWidth'] * $scale2, $lineHeight, number_format($totalNetChange,2), 1, 0, 'R');
        $pdf->Ln($lineHeight);

        // 3) Additional remark below total net change, centered
        $remark = '';
        if ($totalNetChange > 0) {
            $remark = "Pay VAT of " . number_format($totalNetChange, 2);
        } elseif ($totalNetChange < 0) {
            $remark = "Refundable VAT of " . number_format(abs($totalNetChange), 2);
        } else {
            $remark = "No VAT payable or credit at this time.";
        }
        $pdf->Ln(3);
        $pdf->Cell($w, 8, $remark, 0, 1, 'C');
    }
}

/**
 * Helper to fetch transactions grouped by vat_rates_id for the date range
 */
function fetchVATTransactionsByRate(PDO $pdo, string $start, string $end): array
{
    $sql = "SELECT t.*, vr.rate_name, vr.rate_value,
                   accD.name AS debit_name, accC.name AS credit_name
            FROM transactions t
            JOIN vat_rates vr ON t.vat_rates_id = vr.id
            LEFT JOIN accounts accD ON t.debit_account  = accD.code
            LEFT JOIN accounts accC ON t.credit_account = accC.code
            WHERE t.vat_rates_id != 0
              AND t.is_deleted = 0
              AND t.transaction_date BETWEEN :start AND :end
            ORDER BY t.transaction_date ASC";
    $stmt = $pdo->prepare($sql);
    $stmt->execute([':start'=>$start, ':end'=>$end]);

    $results = [];
    while ($row = $stmt->fetch(\PDO::FETCH_ASSOC)) {
        $rateId = (int)$row['vat_rates_id'];
        if (!isset($results[$rateId])) {
            $results[$rateId] = [
                'rate_name' => $row['rate_name'],
                'rate_value'=> $row['rate_value'],
                'transactions'=> []
            ];
        }
        $results[$rateId]['transactions'][] = $row;
    }
    return $results;
}

/** Hex to RGB helper */
function hexToRgb($hex) {
    $hex = str_replace('#','',$hex);
    if(strlen($hex)===3){
        $hex=$hex[0].$hex[0].$hex[1].$hex[1].$hex[2].$hex[2];
    }
    $val=hexdec($hex);
    return [
        'r' => ($val>>16) & 255,
        'g' => ($val>>8)  & 255,
        'b' => $val & 255
    ];
}
