<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('../core/journal_entries.php');
require_once('../core/transactions.php');
require_once('../core/accounts.php');
require_once('../core/financial_periods.php');
require_once('../core/vat_rates.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

$errors = [];
$success = [];

// Role check for 'journal_entries_update'
if (!checkRole(['journal_entries_update'])) {
    $errors[] = 'Access denied. You do not have permission to edit journal entries.';
    echo '<h1>Edit Journal Entry</h1>';
    if (!empty($errors)) {
        echo '<div class="alert">';
        foreach ($errors as $error) {
            echo '<p>' . htmlspecialchars($error) . '</p>';
        }
        echo '</div>';
    }
    require_once('components/footer.php');
    exit;
}

// Validate Journal Entry ID
if (!isset($_GET['id']) || empty(trim($_GET['id']))) {
    $_SESSION['error_message'] = "Invalid journal entry ID.";
    header('Location: journal_entries.php');
    exit;
}

$journalEntryId = (int)$_GET['id'];
$journalEntry   = getJournalEntry($pdo, $journalEntryId);

if (!$journalEntry) {
    $_SESSION['error_message'] = "Journal entry not found.";
    header('Location: journal_entries.php');
    exit;
}

$userId = $_SESSION['user_id'] ?? 0;

// Determine if the journal entry is a system entry (module_id == 0)
$isSystemEntry = isset($journalEntry['module_id']) && $journalEntry['module_id'] == 0;

// Check if financial period or VAT is closed for THIS journal entry
$financialPeriodClosed = isFinancialPeriodClosed($pdo, (int)$journalEntry['financial_period_id']);
$vatClosed             = isDateInClosedVATPeriod($pdo, $journalEntry['date']);

/**
 * Build an account hierarchy (tree).
 */
function buildAccountTree(array $accounts, string $parentCode = null): array {
    $branch = [];
    foreach ($accounts as $account) {
        if ($account['parent'] === $parentCode) {
            $children = buildAccountTree($accounts, $account['code']);
            if ($children) {
                $account['children'] = $children;
            }
            $branch[] = $account;
        }
    }
    return $branch;
}

// Render only "Asset, Liability, Equity" (Primary)
function renderPrimaryAccounts(array $tree, string $selectedCode = '', int $level = 0) {
    foreach ($tree as $account) {
        $acctType = strtolower($account['account_type'] ?? '');
        if (in_array($acctType, ['asset','liability','equity'])) {
            $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $level);
            switch ($acctType) {
                case 'asset':     $color = '#1E90FF';  break;
                case 'liability': $color = '#FF4500';  break;
                case 'equity':    $color = '#32CD32';  break;
                default:          $color = '#000000';  break;
            }
            $selected = ($account['code'] === $selectedCode) ? 'selected' : '';
            echo '<option value="' . htmlspecialchars($account['code']) . '" style="color:' . $color . ';" ' . $selected . '>'
                 . $indent . htmlspecialchars($account['name']) . ' (' . htmlspecialchars($account['code']) . ')</option>';
            if (isset($account['children'])) {
                renderPrimaryAccounts($account['children'], $selectedCode, $level + 1);
            }
        } else {
            if (isset($account['children'])) {
                renderPrimaryAccounts($account['children'], $selectedCode, $level + 1);
            }
        }
    }
}

// Render only "Revenue/Income" and "Expense" (Secondary)
function renderSecondaryAccounts(array $tree, string $selectedCode = '', int $level = 0) {
    foreach ($tree as $account) {
        $acctType = strtolower($account['account_type'] ?? '');
        if (in_array($acctType, ['revenue','income','expense'])) {
            $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $level);
            switch ($acctType) {
                case 'revenue':
                case 'income':    $color = '#FFD700'; break;
                case 'expense':   $color = '#FF6347'; break;
                default:          $color = '#000000'; break;
            }
            $selected = ($account['code'] === $selectedCode) ? 'selected' : '';
            echo '<option value="' . htmlspecialchars($account['code']) . '" style="color:' . $color . ';" ' . $selected . '>'
                 . $indent . htmlspecialchars($account['name']) . ' (' . htmlspecialchars($account['code']) . ')</option>';
            if (isset($account['children'])) {
                renderSecondaryAccounts($account['children'], $selectedCode, $level + 1);
            }
        } else {
            if (isset($account['children'])) {
                renderSecondaryAccounts($account['children'], $selectedCode, $level + 1);
            }
        }
    }
}

/**
 * Render general account options (fallback).
 */
function renderAccountOptions(array $tree, int $level = 0): void {
    foreach ($tree as $account) {
        $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $level);
        switch (strtolower($account['account_type'] ?? '')) {
            case 'asset':     $color = '#1E90FF';  break;
            case 'liability': $color = '#FF4500';  break;
            case 'equity':    $color = '#32CD32';  break;
            case 'revenue':
            case 'income':    $color = '#FFD700';  break;
            case 'expense':   $color = '#FF6347';  break;
            default:          $color = '#000000';  break;
        }
        echo '<option value="' . htmlspecialchars($account['code']) . '" style="color:' . $color . ';">'
             . $indent . htmlspecialchars($account['name']) . ' (' . htmlspecialchars($account['code']) . ')</option>';

        if (isset($account['children'])) {
            renderAccountOptions($account['children'], $level + 1);
        }
    }
}

// -------------------------------------------------------------------
// Check GET parameters for highlight or success messages
// -------------------------------------------------------------------
$highlightTransactionId = null;
if (!empty($_GET['transaction_id']) && is_numeric($_GET['transaction_id'])) {
    $transIdParam       = (int)$_GET['transaction_id'];
    $transactionToCheck = getTransaction($pdo, $transIdParam);
    if ($transactionToCheck && $transactionToCheck['journal_entry_id'] === $journalEntryId) {
        $highlightTransactionId = $transIdParam;
    }
}

if (isset($_GET['added'])) {
    $success[] = 'Transaction added successfully.';
}
if (isset($_GET['deleted'])) {
    $success[] = 'Transaction deleted successfully.';
}

// We'll need the VAT Rates array:
$vatRates = listVATRates($pdo);

// -------------------------------------------------------------------
// Handle POST requests
// -------------------------------------------------------------------
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // Add Transaction
    if (isset($_POST['add_transaction'])) {
        if (!hasRole('transactions')) {
            $errors[] = 'Access denied. You do not have permission to add transactions.';
        } else {
            // Only allow if not closed
            if ($financialPeriodClosed || $vatClosed) {
                $errors[] = 'Cannot add a transaction to a closed financial or VAT period.';
            } else {
                $debitAccount    = trim($_POST['debit_account']);
                $creditAccount   = trim($_POST['credit_account']);
                $amount          = floatval($_POST['amount']);
                $description     = trim($_POST['transaction_description']);
                $transactionDate = $journalEntry['date'];
                $periodId        = $journalEntry['financial_period_id'];

                // Basic validation
                if (empty($debitAccount) || empty($creditAccount)) {
                    $errors[] = 'Both Debit and Credit accounts are required.';
                }
                if (!empty($debitAccount) && !empty($creditAccount) && $debitAccount === $creditAccount) {
                    $errors[] = 'Debit and Credit accounts cannot be the same.';
                }
                if ($amount <= 0) {
                    $errors[] = 'Amount must be greater than zero.';
                }

                if (empty($errors)) {
                    $newTransId = createTransaction(
                        $pdo,
                        $journalEntryId,
                        $debitAccount,
                        $creditAccount,
                        $amount,
                        $description,
                        $transactionDate,
                        $periodId,
                        $userId
                    );
                    if ($newTransId) {
                        header('Location: edit_journal_entry.php?id=' . $journalEntryId
                              . '&transaction_id=' . $newTransId
                              . '&added=1'
                              . '#transaction-row-' . $newTransId);
                        exit;
                    } else {
                        $errors[] = 'Failed to add transaction.';
                    }
                }
            }
        }
    }
    // Delete Transaction
    elseif (isset($_POST['delete_transaction_id'])) {
        if (!hasRole('transactions_delete')) {
            $errors[] = 'Access denied. You do not have permission to delete transactions.';
        } else {
            // Only allow if not closed
            if ($financialPeriodClosed || $vatClosed) {
                $errors[] = 'Cannot delete a transaction in a closed financial or VAT period.';
            } else {
                $delId = (int)$_POST['delete_transaction_id'];
                if ($delId > 0) {
                    $delResult = deleteTransaction($pdo, $delId, $userId);
                    if ($delResult) {
                        header('Location: edit_journal_entry.php?id=' . $journalEntryId . '&deleted=1');
                        exit;
                    } else {
                        $errors[] = 'Failed to delete transaction.';
                    }
                }
            }
        }
    }
    // ADD TRANSACTION WITH VAT
    elseif (isset($_POST['add_transaction_with_vat'])) {
        if (!hasRole('transactions_add')) {
            $errors[] = 'Access denied. You do not have permission to add transactions.';
        } else {
            if ($financialPeriodClosed || $vatClosed) {
                $errors[] = 'Cannot add a transaction with VAT to a closed financial or VAT period.';
            } else {
                // Gather posted data
                $tDate     = trim($_POST['vat_transaction_date'] ?? $journalEntry['date']);
                $pAcc      = trim($_POST['vat_primary_account'] ?? '');
                $pPos      = trim($_POST['vat_primary_position'] ?? '');
                $amtGross  = floatval($_POST['TotalAmount'] ?? 0);
                $desc      = trim($_POST['vat_description'] ?? '');
                $sAcc      = trim($_POST['vat_secondary_account'] ?? '');
                $vatRateId = (int)($_POST['vat_rate_id'] ?? 0);
                $reversal  = isset($_POST['vat_reversal']) ? 1 : 0;

                $periodId = $journalEntry['financial_period_id'];

                // Basic checks
                if (empty($pAcc) || empty($sAcc)) {
                    $errors[] = 'Primary and secondary accounts are required.';
                }
                if (!in_array($pPos, ['debit','credit'])) {
                    $errors[] = 'Primary account position must be Debit or Credit.';
                }
                if ($amtGross <= 0) {
                    $errors[] = 'Amount must be greater than zero (total including VAT).';
                }

                // Check VAT Rate
                $vatRateInfo = null;
                if ($vatRateId > 0) {
                    $vatRateInfo = getVATRate($pdo, $vatRateId);
                    if (!$vatRateInfo) {
                        $errors[] = 'Invalid VAT rate selected.';
                    }
                } else {
                    $errors[] = 'Please select a VAT rate.';
                }

                if (empty($errors)) {
                    $vatValue = (float)$vatRateInfo['rate_value'];
                    $vat = 0.0;
                    if ($vatValue > 0) {
                        $vat = $amtGross * ($vatValue / (100.0 + $vatValue));
                    }

                    // FIRST: Create a line for the TOTAL
                    if ($pPos === 'debit') {
                        // DEBIT pAcc, CREDIT sAcc => amtGross
                        $tidTotal = createTransaction(
                            $pdo,
                            $journalEntryId,
                            $pAcc,
                            $sAcc,
                            $amtGross,
                            $desc,
                            $tDate,
                            $periodId,
                            $userId,
                            0,
                            $vatRateId
                        );
                    } else {
                        // pPos === 'credit'
                        $tidTotal = createTransaction(
                            $pdo,
                            $journalEntryId,
                            $sAcc,
                            $pAcc,
                            $amtGross,
                            $desc,
                            $tDate,
                            $periodId,
                            $userId,
                            0,
                            $vatRateId
                        );
                    }

                    // SECOND: handle line for VAT
                    $secondaryAccountData = getAccount($pdo, $sAcc);
                    $secondaryType        = strtolower($secondaryAccountData['account_type'] ?? '');
                    $tidVat = false;

                    if ($secondaryType === 'expense') {
                        if ($reversal) {
                            // expense + reversal => VAT is credit => VAT_PAYABLE, secondary is debit
                            $tidVat = createTransaction(
                                $pdo,
                                $journalEntryId,
                                $sAcc,                // DEBIT
                                VAT_PAYABLE_ACC_CODE, // CREDIT
                                $vat,
                                $desc . ' (VAT)',
                                $tDate,
                                $periodId,
                                $userId,
                                0,
                                $vatRateId
                            );
                        } else {
                            // expense normal => VAT is debit => VAT_RECEIVABLE, secondary is credit
                            $tidVat = createTransaction(
                                $pdo,
                                $journalEntryId,
                                VAT_RECEIVABLE_ACC_CODE,
                                $sAcc,
                                $vat,
                                $desc . ' (VAT)',
                                $tDate,
                                $periodId,
                                $userId,
                                0,
                                $vatRateId
                            );
                        }
                    } elseif (in_array($secondaryType, ['revenue','income'])) {
                        if ($reversal) {
                            // income + reversal => VAT is debit => VAT_RECEIVABLE, secondary is credit
                            $tidVat = createTransaction(
                                $pdo,
                                $journalEntryId,
                                VAT_RECEIVABLE_ACC_CODE,
                                $sAcc,
                                $vat,
                                $desc . ' (VAT)',
                                $tDate,
                                $periodId,
                                $userId,
                                0,
                                $vatRateId
                            );
                        } else {
                            // income normal => VAT is credit => VAT_PAYABLE, secondary is debit
                            $tidVat = createTransaction(
                                $pdo,
                                $journalEntryId,
                                $sAcc,
                                VAT_PAYABLE_ACC_CODE,
                                $vat,
                                $desc . ' (VAT)',
                                $tDate,
                                $periodId,
                                $userId,
                                0,
                                $vatRateId
                            );
                        }
                    } else {
                        $errors[] = 'Secondary account is not expense or revenue; cannot allocate VAT.';
                    }

                    if (!$tidTotal || !$tidVat || !empty($errors)) {
                        if (!isset($errors[0])) {
                            $errors[] = 'Failed to insert the total or VAT portion of the transaction.';
                        }
                    } else {
                        header('Location: edit_journal_entry.php?id=' . $journalEntryId . '&added=1');
                        exit;
                    }
                }
            }
        }
    }
}

// -------------------------------------------------------------------
// Fetch transactions & accounts
// -------------------------------------------------------------------
$transactionsData = listTransactions($pdo, ['journal_entry_id' => $journalEntryId], 1, 1000);
$transactions     = $transactionsData['transactions'] ?? [];
$accounts         = listAccounts($pdo);
$accountTree      = buildAccountTree($accounts);

// Build code->name map
$accountMap = [];
foreach ($accounts as $acct) {
    $accountMap[$acct['code']] = $acct['name'];
}
?>
<!-- STICKY MESSAGES -->
<div id="sticky-messages">
  <?php if (!empty($errors)): ?>
    <div class="alert">
      <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
      <?php endforeach; ?>
    </div>
  <?php endif; ?>
  <?php if (!empty($success)): ?>
    <div class="success">
      <?php foreach ($success as $msg): ?>
        <p><?php echo htmlspecialchars($msg); ?></p>
      <?php endforeach; ?>
    </div>
  <?php endif; ?>
</div>

<h1>Edit Journal Entry</h1>
<h2>
  <?php
    echo htmlspecialchars($journalEntry['description'])
       . ' (Date: ' . htmlspecialchars($journalEntry['date']) . ')';
  ?>
</h2>

<div class="button-container" style="height:35px;">
  <?php if (!$financialPeriodClosed && !$vatClosed && hasRole('transactions')): ?>
    <button class="button" id="add-transaction-button">Add Transaction</button>
    <button class="button" id="add-transaction-with-vat-button">Add Transaction With VAT</button>
  <?php endif; ?>
  <a href="journal_entries.php" class="button" style="float:right;">Back to Journal Entries</a>
</div>

<?php if ($highlightTransactionId !== null): ?>
  <div class="transaction-message">
    Highlighted transaction #<?php echo (int)$highlightTransactionId; ?> below.
  </div>
<?php endif; ?>

<table>
  <thead>
    <tr>
      <th>ID</th>
      <th>Debit Account</th>
      <th>Credit Account</th>
      <th>Amount</th>
      <th>Description</th>
      <th>Transaction Date</th>
      <th>Actions</th>
    </tr>
  </thead>
  <tbody>
    <?php if (!empty($transactions)): ?>
      <?php foreach ($transactions as $t): ?>
        <?php
          $tid = (int)($t['id'] ?? 0);
          $highlightClass = ($highlightTransactionId === $tid) ? 'highlighted-row' : '';
        ?>
        <tr id="transaction-row-<?php echo $tid; ?>" class="<?php echo $highlightClass; ?>">
          <td><?php echo htmlspecialchars($tid); ?></td>
          <td>
            <?php
              $debCode = $t['debit_account'] ?? '';
              $debName = $accountMap[$debCode] ?? '';
              echo htmlspecialchars($debCode . ($debName ? " ($debName)" : ''));
            ?>
          </td>
          <td>
            <?php
              $credCode = $t['credit_account'] ?? '';
              $credName = $accountMap[$credCode] ?? '';
              echo htmlspecialchars($credCode . ($credName ? " ($credName)" : ''));
            ?>
          </td>
          <td><?php echo htmlspecialchars(number_format($t['amount'] ?? 0, 2)); ?></td>
          <td><?php echo htmlspecialchars($t['description'] ?? ''); ?></td>
          <td><?php echo htmlspecialchars($t['transaction_date'] ?? ''); ?></td>
          <td>
            <?php
            // Hide if closed
            if ($isSystemEntry && hasRole('transactions_delete') && !$financialPeriodClosed && !$vatClosed) {
                ?>
                <form action="edit_journal_entry.php?id=<?php echo (int)$journalEntryId; ?>" method="POST" style="display:inline;">
                  <input type="hidden" name="delete_transaction_id" value="<?php echo $tid; ?>" />
                  <button type="submit" class="button delete-button"
                          onclick="return confirm('Are you sure you want to delete this transaction?');">
                      Delete
                  </button>
                </form>
            <?php } ?>
          </td>
        </tr>
      <?php endforeach; ?>
    <?php else: ?>
      <tr><td colspan="7">No transactions found.</td></tr>
    <?php endif; ?>
  </tbody>
</table>

<!-- ADD TRANSACTION MODAL -->
<?php if ($isSystemEntry && hasRole('transactions')): ?>
<div id="add-transaction-modal" class="transaction-modal">
  <div class="transaction-modal-content form-container">
    <span class="transaction-close-modal">&times;</span>
    <h2>Add Transaction</h2>

    <!-- Template for NO VAT -->
    <label for="no_vat_template" style="width: 100%">Template</label>
    <select id="no_vat_template" name="no_vat_template" style="width: 100%;">
        <option value="">Custom Transaction</option>
    </select>

    <form action="edit_journal_entry.php?id=<?php echo (int)$journalEntryId; ?>" method="POST">
      <label for="debit_account">Debit Account</label>
      <select id="debit_account" name="debit_account" required>
        <option value="">-- Select Debit Account --</option>
        <?php renderAccountOptions($accountTree); ?>
      </select>

      <label for="credit_account">Credit Account</label>
      <select id="credit_account" name="credit_account" required>
        <option value="">-- Select Credit Account --</option>
        <?php renderAccountOptions($accountTree); ?>
      </select>

      <?php
        renderInput('number', 'amount', 'Amount', '', true, ['step'=>'0.01','min'=>'0.01']);
        renderTextarea('transaction_description', 'Description', '');
      ?>

      <input type="hidden" name="add_transaction" value="1">
      <div class="button-group">
        <button type="submit">Add Transaction</button>
      </div>
    </form>
  </div>
</div>

<!-- ADD TRANSACTION WITH VAT MODAL -->
<div id="add-transaction-with-vat-modal" class="transaction-modal">
  <div class="transaction-modal-content form-container">
    <span class="transaction-close-modal vat-modal-close">&times;</span>
    <h2>Add Transaction With VAT</h2>

    <!-- Template for WITH VAT -->
    <label for="vat_template" style="width: 100%">Template</label>
    <select id="vat_template" name="vat_template" style="width: 100%">
        <option value="">Custom Transaction</option>
    </select>

    <form action="edit_journal_entry.php?id=<?php echo (int)$journalEntryId; ?>" method="POST" id="vat-transaction-form">
      <label for="vat_transaction_date">Transaction Date</label>
      <input type="date" id="vat_transaction_date" name="vat_transaction_date"
             value="<?php echo htmlspecialchars($journalEntry['date']); ?>" required>

      <label for="vat_primary_account">Primary Account</label>
      <select id="vat_primary_account" name="vat_primary_account" required>
        <option value="">-- Select Primary Account --</option>
        <?php renderPrimaryAccounts($accountTree); ?>
      </select>

      <label for="vat_primary_position">Primary Account Position</label>
      <select id="vat_primary_position" name="vat_primary_position" required>
        <option value="">-- Select Debit or Credit --</option>
        <option value="debit">Debit</option>
        <option value="credit">Credit</option>
      </select>

      <?php
      renderInput('number', 'TotalAmount', 'Amount (Total)', '', true, ['step'=>'0.01','min'=>'0.01']);
      renderTextarea('vat_description', 'Description', '');
      ?>

      <!-- Reversal checkbox -->
      <label for="vat_reversal" style="display:inline;">
          <input type="checkbox" id="vat_reversal" name="vat_reversal" value="1" />
          Reversal
      </label>

      <br><br>

      <label for="vat_secondary_account">Secondary Account</label>
      <select id="vat_secondary_account" name="vat_secondary_account" required>
        <option value="">-- Select Secondary Account --</option>
        <?php renderSecondaryAccounts($accountTree); ?>
      </select>

      <label for="vat_rate_id">VAT Rate</label>
      <select id="vat_rate_id" name="vat_rate_id" required>
        <option value="">-- Select VAT Rate --</option>
        <?php
        foreach ($vatRates as $vr) {
            $rId   = (int)$vr['id'];
            $rName = htmlspecialchars($vr['rate_name']);
            $rVal  = htmlspecialchars($vr['rate_value']);
            echo "<option value=\"$rId\" data-rate-value=\"$rVal\">$rName ({$rVal}%)</option>";
        }
        ?>
      </select>

      <div id="vat-example-preview" style="background-color:#fafafa;padding:10px;margin-top:10px;border:1px solid #ccc;">
        <strong>Example of the two transactions to be added:</strong>
        <ul style="list-style-type:disc;padding-left:20px;">
          <li id="vat-net-line">Line 1: TBD</li>
          <li id="vat-vat-line">Line 2: TBD</li>
        </ul>
      </div>

      <input type="hidden" name="add_transaction_with_vat" value="1">
      <div class="button-group">
        <button type="submit">Add Transaction With VAT</button>
      </div>
    </form>
  </div>
</div>
<?php endif; ?>

<style>
body {
  margin: 0; padding: 0;
  font-family: Arial, sans-serif;
}
#sticky-messages {
  position: fixed;
  top: 0; left: 0;
  width: 100%;
  z-index: 9999;
}
.alert, .success {
  max-width: 800px;
  margin: 10px auto;
  padding: 10px;
  border-radius: 4px;
  box-shadow: 0 2px 5px rgba(0,0,0,0.2);
  cursor: pointer;
}
.alert {
  background-color: #f8d7da;
  color: #721c24;
}
.success {
  background-color: #d4edda;
  color: #155724;
}
table {
  width: 100%;
  border-collapse: collapse;
  margin-top: 20px;
  margin-bottom: 50px;
  background-color: #fff;
  border-radius: 5px;
  overflow: hidden;
  box-shadow: 0 2px 5px rgba(0,0,0,0.1);
}
table th, table td {
  padding: 12px 15px;
  border-bottom: 1px solid #ddd;
  text-align: left;
}
table th {
  background-color: #2980b9;
  color: #fff;
}
.highlighted-row {
  background-color: #fffa9e !important;
}
.button-container {
  margin-top: 20px;
  margin-bottom: 20px;
}
.transaction-message {
  margin: 10px 0;
  padding: 10px;
  background-color: #e2e3e5;
  border: 1px solid #d6d8db;
  border-radius: 4px;
  color: #383d41;
}
.button {
  padding: 8px 12px;
  background-color: #2980b9;
  color: #fff;
  border: none;
  border-radius: 4px;
  cursor: pointer;
  text-decoration: none;
  margin-right: 5px;
}
.button:hover {
  background-color: #1f6391;
}
.delete-button {
  background-color: #c0392b;
}
.delete-button:hover {
  background-color: #a93226;
}
.transaction-modal {
  display: none;
  position: fixed;
  z-index: 1000;
  left: 0; top: 0;
  width: 100%; height: 100%;
  overflow: auto;
  background-color: rgba(0,0,0,0.4);
}
.transaction-modal-content {
  background-color: #fff;
  margin: 1% auto;
  padding: 20px;
  border: 1px solid #888;
  width: 50%;
  border-radius: 4px;
  position: relative;
}
.transaction-close-modal {
  float: right;
  font-size: 28px;
  font-weight: bold;
  cursor: pointer;
}
.transaction-close-modal:hover,
.transaction-close-modal:focus {
  color: black;
  text-decoration: none;
  cursor: pointer;
}
</style>

<script>
document.addEventListener('DOMContentLoaded', function() {
    // Auto-hide sticky messages
    const sticky = document.getElementById('sticky-messages');
    if(sticky) {
        setTimeout(function(){
            sticky.style.display = 'none';
        }, 7000);
        sticky.addEventListener('click',function(){
            sticky.style.display='none';
        });
    }

    // If user can add transactions => wire up "Add Transaction" modals
    <?php if ($isSystemEntry && hasRole('transactions')): ?>
    const modal        = document.getElementById('add-transaction-modal');
    const btn          = document.getElementById('add-transaction-button');
    const closeSpan    = document.querySelector('.transaction-close-modal');
    const vatBtn       = document.getElementById('add-transaction-with-vat-button');
    const vatModal     = document.getElementById('add-transaction-with-vat-modal');
    const vatCloseSpan = document.querySelector('.vat-modal-close');

    if (btn && modal) {
        btn.onclick = function() {
            modal.style.display = 'block';
        };
    }
    if (closeSpan) {
        closeSpan.onclick = function() {
            modal.style.display = 'none';
        };
    }
    window.onclick = function(e) {
        if (e.target === modal) {
            modal.style.display = 'none';
        }
        if (e.target === vatModal) {
            vatModal.style.display = 'none';
        }
    };

    // Add Transaction With VAT modal
    if (vatBtn && vatModal) {
        vatBtn.onclick = function() {
            vatModal.style.display = 'block';
        };
    }
    if (vatCloseSpan) {
        vatCloseSpan.onclick = function() {
            vatModal.style.display = 'none';
        };
    }

    // We'll fix #vat-example-preview to show TOTAL vs. VAT portion & handle Reversal:
    const totalAmountInput       = document.getElementById('TotalAmount');
    const vatRateSelect          = document.getElementById('vat_rate_id');
    const primaryPosSelect       = document.getElementById('vat_primary_position');
    const primaryAccSelect       = document.getElementById('vat_primary_account');
    const secondaryAccSelect     = document.getElementById('vat_secondary_account');
    const reversalCheckbox       = document.getElementById('vat_reversal');

    const netLineElem            = document.getElementById('vat-net-line');
    const vatLineElem            = document.getElementById('vat-vat-line');

    function updateVatPreview() {
        if (!totalAmountInput || !vatRateSelect) return;
        const total     = parseFloat(totalAmountInput.value)||0;
        const selectedOption = vatRateSelect.options[vatRateSelect.selectedIndex];
        const reversal  = (reversalCheckbox && reversalCheckbox.checked);
        if (!selectedOption) {
            netLineElem.textContent = "Line 1: (Select a VAT Rate)";
            vatLineElem.textContent = "Line 2: (Select a VAT Rate)";
            return;
        }
        const vatVal = parseFloat(selectedOption.getAttribute('data-rate-value')||'0');
        const pPos   = primaryPosSelect.value||'debit';
        const pCode  = primaryAccSelect.value||'Primary';
        const sCode  = secondaryAccSelect.value||'Secondary';

        // Line1 => Always total
        if (pPos==='debit') {
            netLineElem.textContent = "Line 1: DEBIT " + pCode + " " + total.toFixed(2) +
                                      " | CREDIT " + sCode + " " + total.toFixed(2) + " (TOTAL)";
        } else {
            netLineElem.textContent = "Line 1: DEBIT " + sCode + " " + total.toFixed(2) +
                                      " | CREDIT " + pCode + " " + total.toFixed(2) + " (TOTAL)";
        }

        // Calculate VAT portion:
        let vat = 0;
        if (vatVal>0) {
            vat = total * (vatVal/(100+vatVal));
        }

        let vatAccName = reversal ? "VAT_PAYABLE" : "VAT_RECEIVABLE";

        if (reversal) {
            vatLineElem.textContent =
              "Line 2: DEBIT " + sCode + " " + vat.toFixed(2) +
              " | CREDIT " + vatAccName + " " + vat.toFixed(2) + " (VAT)";
        } else {
            vatLineElem.textContent =
              "Line 2: DEBIT " + vatAccName + " " + vat.toFixed(2) +
              " | CREDIT " + sCode + " " + vat.toFixed(2) + " (VAT)";
        }
    }

    // Template select for NO VAT
    const noVatSelect = document.getElementById('no_vat_template');
    if (noVatSelect && window.TransactionsTemplates && window.TransactionsTemplates.noVAT) {
        while (noVatSelect.options.length > 1) {
            noVatSelect.remove(1);
        }
        (window.TransactionsTemplates.noVAT.templates || []).forEach(t => {
            const opt = document.createElement('option');
            opt.value = t.value;
            opt.textContent = t.label;
            noVatSelect.appendChild(opt);
        });
        noVatSelect.addEventListener('change', function() {
            if (typeof TransactionsTemplates.noVAT.applyTemplate === 'function') {
                TransactionsTemplates.noVAT.applyTemplate(
                    noVatSelect.value,
                    document.getElementById('debit_account'),
                    document.getElementById('credit_account')
                );
            }
        });
    }

    // Template select for WITH VAT
    const vatSelect = document.getElementById('vat_template');
    if (vatSelect && window.TransactionsTemplates && window.TransactionsTemplates.withVAT) {
        while (vatSelect.options.length > 1) {
            vatSelect.remove(1);
        }
        (window.TransactionsTemplates.withVAT.templates || []).forEach(t => {
            const opt = document.createElement('option');
            opt.value = t.value;
            opt.textContent = t.label;
            vatSelect.appendChild(opt);
        });
        vatSelect.addEventListener('change', function(){
            if (typeof TransactionsTemplates.withVAT.applyTemplate === 'function') {
                TransactionsTemplates.withVAT.applyTemplate(
                    vatSelect.value,
                    document.getElementById('vat_primary_account'),
                    document.getElementById('vat_primary_position'),
                    document.getElementById('vat_secondary_account')
                );
            }
            updateVatPreview();
        });
    }

    totalAmountInput?.addEventListener('input', updateVatPreview);
    vatRateSelect?.addEventListener('change', updateVatPreview);
    primaryPosSelect?.addEventListener('change', updateVatPreview);
    primaryAccSelect?.addEventListener('change', updateVatPreview);
    secondaryAccSelect?.addEventListener('change', updateVatPreview);
    reversalCheckbox?.addEventListener('change', updateVatPreview);

    updateVatPreview();
    <?php endif; ?>

    // Scroll position
    const urlParams = new URLSearchParams(window.location.search);
    if (!urlParams.has('added') && !urlParams.has('deleted')) {
        // restore scroll only if we didn't just add or delete
        const storedPos = sessionStorage.getItem('journalEntryScrollPosition');
        if (storedPos) {
            window.scrollTo(0, parseInt(storedPos,10));
        }
    }
    window.addEventListener('scroll', function() {
        sessionStorage.setItem('journalEntryScrollPosition', window.scrollY);
    });

    // If highlighting a transaction
    <?php if ($highlightTransactionId !== null): ?>
    const row = document.getElementById('transaction-row-<?php echo $highlightTransactionId; ?>');
    if (row) {
        row.scrollIntoView({behavior:'smooth', block:'center'});
    }
    <?php endif; ?>
});
</script>
<script src="/templates/transactions.php"></script>
<?php require_once('components/footer.php'); ?>
