<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Include necessary files and initialize the database connection
require_once('../core/users.php');      // For getAllRoles()
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

$errors = [];
$success = '';

// Check if the user has the 'header_menu_edit' role
if (!checkRole(['header_menu_edit'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to edit the menu.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Edit Menu Item</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error ?? '', ENT_QUOTES, 'UTF-8'); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Check if 'edit_id' is set in GET parameters
if (!isset($_GET['edit_id']) || empty(trim($_GET['edit_id']))) {
    $errors[] = 'Invalid menu item ID.';
    ?>
    <h1>Edit Menu Item</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error ?? '', ENT_QUOTES, 'UTF-8'); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <a href="menu_builder.php" class="button">Back to Menu Builder</a>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

$editId = (int)trim($_GET['edit_id']);

// Fetch the menu item details
$stmt = $pdo->prepare("SELECT * FROM menu_items WHERE id = ?");
$stmt->execute([$editId]);
$menuItem = $stmt->fetch(PDO::FETCH_ASSOC);

if (!$menuItem) {
    $errors[] = 'Menu item not found.';
    ?>
    <h1>Edit Menu Item</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error ?? '', ENT_QUOTES, 'UTF-8'); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <a href="menu_builder.php" class="button">Back to Menu Builder</a>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Handle form submission
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['action']) && $_POST['action'] === 'edit') {
        $name = trim($_POST['name']);
        $url = isset($_POST['url']) ? trim($_POST['url']) : null;
        $parent_id = isset($_POST['parent_id']) && !empty($_POST['parent_id']) ? (int)$_POST['parent_id'] : null;
        $order_num = isset($_POST['order_num']) ? (int)$_POST['order_num'] : 0;
        $is_dropdown = isset($_POST['is_dropdown']) ? 1 : 0;
        $is_visible = isset($_POST['is_visible']) ? 1 : 0;
        $required_role = isset($_POST['required_role']) ? trim($_POST['required_role']) : null;

        // Validation
        if (empty($name)) {
            $errors[] = 'Name is required.';
        }

        if ($menuItem['is_system']) {
            // For system items, only the name can be edited
            if (empty($errors)) {
                $stmt = $pdo->prepare("UPDATE menu_items SET name = ? WHERE id = ?");
                $result = $stmt->execute([$name, $editId]);

                if ($result) {
                    $success = 'System menu item name updated successfully.';
                    // Redirect back to menu_builder.php after successful update
                    header('Location: menu_builder.php');
                    exit;
                } else {
                    $errors[] = 'Failed to update system menu item.';
                }
            }
        } else {
            // For non-system items, validate URL based on is_dropdown
            if ($is_dropdown && !empty($url)) {
                $errors[] = 'Dropdown menus should not have a URL.';
            }

            if (!$is_dropdown && empty($url)) {
                $errors[] = 'URL is required for menu items.';
            }

            // Prevent assigning the menu item as its own parent
            if ($parent_id === $editId) {
                $errors[] = 'A menu item cannot be its own parent.';
            }

            // Validate parent_id if set
            if ($parent_id) {
                $stmt = $pdo->prepare("SELECT is_dropdown FROM menu_items WHERE id = ?");
                $stmt->execute([$parent_id]);
                $parentMenu = $stmt->fetch(PDO::FETCH_ASSOC);
                if (!$parentMenu || !$parentMenu['is_dropdown']) {
                    $errors[] = 'Selected parent menu is invalid.';
                }
            }

            if (empty($errors)) {
                // Update all fields for non-system items
                $stmt = $pdo->prepare("UPDATE menu_items SET name = ?, url = ?, parent_id = ?, order_num = ?, is_dropdown = ?, is_visible = ?, required_role = ? WHERE id = ?");
                $result = $stmt->execute([$name, $url ?: null, $parent_id, $order_num, $is_dropdown, $is_visible, $required_role, $editId]);

                if ($result) {
                    $success = 'Menu item updated successfully.';
                    // Redirect back to menu_builder.php after successful update
                    header('Location: menu_builder.php');
                    exit;
                } else {
                    $errors[] = 'Failed to update menu item.';
                }
            }
        }
    }
}

// Fetch roles for the required_role field
$roles = getAllRoles($pdo);

// Fetch all dropdown menu items to populate the Parent Menu dropdown
$stmt = $pdo->prepare("SELECT * FROM menu_items WHERE is_dropdown = 1 AND id != ? ORDER BY name ASC");
$stmt->execute([$editId]);
$dropdownMenus = $stmt->fetchAll(PDO::FETCH_ASSOC);
?>
<h1>Edit Menu Item</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
    <p><?php echo htmlspecialchars($error ?? '', ENT_QUOTES, 'UTF-8'); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success ?? '', ENT_QUOTES, 'UTF-8'); ?></p>
</div>
<?php endif; ?>

<!-- Edit Menu Item Form -->
<div class="form-container">
    <form action="edit_menu_builder.php?edit_id=<?php echo htmlspecialchars($editId, ENT_QUOTES, 'UTF-8'); ?>" method="POST">
        <input type="hidden" name="action" value="edit">
        <input type="hidden" name="id" value="<?php echo htmlspecialchars($menuItem['id'], ENT_QUOTES, 'UTF-8'); ?>">

        <?php
        // Render the 'name' field (always editable)
        renderInput('text', 'name', 'Name', htmlspecialchars($menuItem['name'] ?? '', ENT_QUOTES, 'UTF-8'), true);

        if (!$menuItem['is_system']) {
            // Render additional fields for non-system items
            renderInput('text', 'url', 'URL', htmlspecialchars($menuItem['url'] ?? '', ENT_QUOTES, 'UTF-8'));

            echo '<label for="parent_id">Parent Menu</label>';
            echo '<select id="parent_id" name="parent_id">';
            echo '<option value="">None</option>';
            foreach ($dropdownMenus as $menu) {
                $selected = ($menu['id'] == $menuItem['parent_id']) ? 'selected' : '';
                echo '<option value="' . htmlspecialchars($menu['id'], ENT_QUOTES, 'UTF-8') . '" ' . $selected . '>' . htmlspecialchars($menu['name'], ENT_QUOTES, 'UTF-8') . '</option>';
            }
            echo '</select>';

            renderInput('number', 'order_num', 'Order Number', htmlspecialchars($menuItem['order_num'] ?? '0', ENT_QUOTES, 'UTF-8'));

            renderCheckbox('is_dropdown', 'Is Dropdown', $menuItem['is_dropdown']);

            renderCheckbox('is_visible', 'Is Visible', $menuItem['is_visible']);

            echo '<label for="required_role">Required Role</label>';
            echo '<select id="required_role" name="required_role">';
            echo '<option value="">None</option>';
            foreach ($roles as $role) {
                $selected = ($role['name'] == $menuItem['required_role']) ? 'selected' : '';
                echo '<option value="' . htmlspecialchars($role['name'], ENT_QUOTES, 'UTF-8') . '" ' . $selected . '>' . htmlspecialchars($role['name'], ENT_QUOTES, 'UTF-8') . '</option>';
            }
            echo '</select>';
        }
        ?>

        <div class="button-group">
            <button type="submit">Update Menu Item</button>
            <a href="menu_builder.php" class="button">Cancel</a>
        </div>
    </form>
</div>

<?php require_once('components/footer.php'); ?>
