<?php
require_once('../core/users.php');
require_once('components/auth.php');
require_once('components/header.php');
$errors = [];
$success = '';

// Update role check to 'users_edit'
if (!checkRole(['users_edit'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Edit User</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Validate and retrieve user ID from GET parameters
if (!isset($_GET['id']) || empty(trim($_GET['id']))) {
    $_SESSION['error_message'] = "Invalid user ID.";
    header('Location: users.php');
    exit;
}

$editUserId = (int)$_GET['id'];
$userData = getUser($pdo, $editUserId);

// Check if user exists
if (!$userData) {
    $_SESSION['error_message'] = "User not found.";
    header('Location: users.php');
    exit;
}

// Handle form submission
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // Removed username from POST processing to prevent editing
    $email = trim($_POST['email']);
    $isAuth = isset($_POST['is_auth']) ? 1 : 0;
    $roles = $_POST['roles'] ?? [];

    // Updated validation to exclude username
    if (empty($email)) {
        $errors[] = 'Email is required.';
    } elseif (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
        $errors[] = 'Please provide a valid email address.';
    } elseif (strlen($email) > 100) {
        $errors[] = 'Email must be less than 100 characters.';
    }

    // Optional: Check for email uniqueness if the email has changed
    if ($email !== $userData['email']) {
        $existingEmails = listUsers($pdo, ['email' => $email]);
        if (!empty($existingEmails)) {
            $errors[] = 'An account with this email already exists.';
        }
    }

    if (empty($errors)) {
        $updateData = [
            'email' => $email,
            'is_auth' => $isAuth
        ];
        $result = updateUser($pdo, $editUserId, $updateData, $_SESSION['user_id']);
        if ($result) {
            // Remove all existing roles
            $removeRolesResult = removeAllUserRoles($pdo, $editUserId, $_SESSION['user_id']);
            if (!$removeRolesResult) {
                $errors[] = 'Failed to remove existing roles.';
            } else {
                // Assign new roles
                $assignRolesSuccess = true;
                foreach ($roles as $roleId) {
                    $assignResult = assignUserRole($pdo, $editUserId, $roleId, $_SESSION['user_id']);
                    if (!$assignResult) {
                        $assignRolesSuccess = false;
                        $errors[] = 'Failed to assign one or more roles.';
                        break;
                    }
                }

                if ($assignRolesSuccess) {
                    $success = 'User has been updated successfully.';
                    // Refresh user data
                    $userData = getUser($pdo, $editUserId);
                }
            }
        } else {
            $errors[] = 'Failed to update user. Please try again.';
        }
    }
}

// Fetch all roles and the roles assigned to the user
$rolesList = getAllRoles($pdo);
$userRolesList = getUserRolesID($pdo, $editUserId);
?>
<h1>Edit User</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success); ?></p>
    <a href="users.php" class="button">Back to Users</a>
</div>
<?php else: ?>
<div class="form-container">
    <form action="edit_user.php?id=<?php echo $editUserId; ?>" method="POST">
        <!-- Username is displayed as plain text and not editable -->
        <label for="username">Username</label>
        <p><?php echo htmlspecialchars($userData['username']); ?></p>

        <label for="email">Email</label>
        <input type="email" id="email" name="email" value="<?php echo htmlspecialchars($userData['email']); ?>" required>

        <label for="is_auth">Authorize User</label>
        <input type="checkbox" id="is_auth" name="is_auth" <?php echo $userData['is_auth'] == 1 ? 'checked' : ''; ?>>

        <label>Roles</label>
        <?php foreach ($rolesList as $role): ?>
            <label>
                <input type="checkbox" name="roles[]" value="<?php echo $role['id']; ?>" <?php echo in_array($role['id'], $userRolesList) ? 'checked' : ''; ?>>
                <?php echo htmlspecialchars($role['name']); ?>
            </label><br>
        <?php endforeach; ?>

        <div class="button-group">
            <button type="submit">Update User</button>
            <a href="users.php" class="button">Back</a>
        </div>
    </form>
</div>
<?php endif; ?>
<?php require_once('components/footer.php'); ?>
