<?php
require_once('../core/financial_periods.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

$errors = [];
$success = '';

// Check if the user has the 'financial_period' role
if (!checkRole(['financial_period'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to view this page.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Manage Financial Periods</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Helper function to handle results from core functions
function handleOperationResult($result, &$errors, &$success, $defaultError) {
    if (is_array($result) && array_key_exists('success', $result) && array_key_exists('message', $result)) {
        if ($result['success']) {
            // success
            $success = $result['message'];
        } else {
            // detailed error message
            $errors[] = $result['message'];
        }
    } else {
        // If the result is not in expected format
        $errors[] = $defaultError . ' Operation returned unexpected result.';
    }
}

// Handle POST requests for actions: delete, close, reopen
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['delete_period_id']) && hasRole('financial_period_delete')) {
        $periodId = (int)$_POST['delete_period_id'];
        if ($periodId > 0) {
            $result = deleteFinancialPeriod($pdo, $periodId, $_SESSION['user_id']);
            handleOperationResult($result, $errors, $success, 'Failed to delete financial period.');
        } else {
            $errors[] = 'Invalid financial period ID for deletion.';
        }
    } elseif (isset($_POST['close_period_id']) && hasRole('financial_period_close')) {
        $periodId = (int)$_POST['close_period_id'];
        if ($periodId > 0) {
            $result = closeFinancialPeriod($pdo, $periodId, $_SESSION['user_id']);
            handleOperationResult($result, $errors, $success, 'Failed to close financial period.');
        } else {
            $errors[] = 'Invalid financial period ID for closure.';
        }
    } elseif (isset($_POST['reopen_period_id']) && hasRole('financial_period_reopen')) {
        $periodId = (int)$_POST['reopen_period_id'];
        if ($periodId > 0) {
            $result = reopenFinancialPeriod($pdo, $periodId, $_SESSION['user_id']);
            handleOperationResult($result, $errors, $success, 'Failed to reopen financial period.');
        } else {
            $errors[] = 'Invalid financial period ID for reopening.';
        }
    }
}

// Fetch only non-deleted financial periods
$financialPeriods = listFinancialPeriods($pdo, ['is_deleted' => 0]);
?>
<h1>Manage Financial Periods</h1>
<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>
<?php if ($success): ?>
    <div class="success">
        <p><?php echo htmlspecialchars($success); ?></p>
    </div>
<?php endif; ?>
<div class="button-container">
    <?php if (hasRole('financial_period_add')): ?>
        <a href="add_financial_period.php" class="button">Add Financial Period</a>
    <?php endif; ?>
</div>
<table>
    <thead>
        <tr>
            <th>Name</th>
            <th>Start Date</th>
            <th>End Date</th>
            <th>Is Closed</th>
            <th>Actions</th>
        </tr>
    </thead>
    <tbody>
        <?php if (!empty($financialPeriods)): ?>
            <?php foreach ($financialPeriods as $period): ?>
                <tr>
                    <td><?php echo htmlspecialchars($period['name']); ?></td>
                    <td><?php echo htmlspecialchars($period['start_date']); ?></td>
                    <td><?php echo htmlspecialchars($period['end_date']); ?></td>
                    <td><?php echo $period['is_closed'] ? 'Yes' : 'No'; ?></td>
                    <td>
                        <?php if (hasRole('financial_period_update')): ?>
                            <a href="update_financial_period.php?id=<?php echo $period['id']; ?>" class="button edit-button">Edit</a>
                        <?php endif; ?>

                        <?php if ($period['is_closed'] && hasRole('financial_period_reopen')): ?>
                            <form action="financial_periods.php" method="POST" style="display:inline;">
                                <input type="hidden" name="reopen_period_id" value="<?php echo $period['id']; ?>">
                                <button type="submit" class="button reopen-button">Reopen</button>
                            </form>
                        <?php elseif (!$period['is_closed'] && hasRole('financial_period_close')): ?>
                            <form action="financial_periods.php" method="POST" style="display:inline;">
                                <input type="hidden" name="close_period_id" value="<?php echo $period['id']; ?>">
                                <button type="submit" class="button close-button">Close</button>
                            </form>
                        <?php endif; ?>

                        <?php if (hasRole('financial_period_delete')): ?>
                            <form action="financial_periods.php" method="POST" style="display:inline;">
                                <input type="hidden" name="delete_period_id" value="<?php echo $period['id']; ?>">
                                <button type="submit" class="button delete-button" onclick="return confirm('Are you sure you want to delete this financial period?');">Delete</button>
                            </form>
                        <?php endif; ?>
                    </td>
                </tr>
            <?php endforeach; ?>
        <?php else: ?>
            <tr>
                <td colspan="5">No financial periods available.</td>
            </tr>
        <?php endif; ?>
    </tbody>
</table>
<?php require_once('components/footer.php'); ?>
