<?php
// public/incomestatement.php

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('../core/financial_reports.php');
require_once('../core/accounts.php');
require_once('../core/financial_periods.php');


$errors = [];
$incomeStatementData = [];
$sortedAccounts = [];
$totalRevenue = 0;
$totalExpenses = 0;
$netIncome = 0;
$startDate = '';
$endDate = '';
$selectedPeriodId = '';
$closedPeriod = false;

/***********************************************************************
 * ADD THIS ROUTE AT THE TOP TO LIST INCOMESTATEMENT LAYOUTS
 **********************************************************************/
if (isset($_GET['action']) && $_GET['action'] === 'list_is_layouts') {
    header('Content-Type: application/json; charset=utf-8');

    $layoutsDir = __DIR__ . '/documents';

    // Helper function to find .layout files with "type":"incomestatement"
    function findIncomeStatementLayouts($dir) {
        $results = [];
        if (!is_dir($dir)) {
            return $results;
        }
        $files = scandir($dir);
        foreach ($files as $f) {
            if (strtolower(pathinfo($f, PATHINFO_EXTENSION)) === 'layout') {
                $fullPath = $dir . '/' . $f;
                $jsonData = @json_decode(file_get_contents($fullPath), true);
                if (is_array($jsonData) && isset($jsonData['type']) && $jsonData['type'] === 'incomestatement') {
                    $results[] = $f;
                }
            }
        }
        return $results;
    }

    $isLayouts = findIncomeStatementLayouts($layoutsDir);
    if (count($isLayouts) === 0) {
        echo json_encode(['success' => false, 'message' => 'No layout for incomestatement']);
        exit;
    }

    echo json_encode(['success' => true, 'layouts' => $isLayouts]);
    exit;
}
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/account_functions.php'); // Include account functions
// **Role Check: Ensure the user has 'income_statement' role**
if (!checkRole(['income_statement'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to view this page.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Income Statement</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

/**
 * Renders the Income Statement table rows with hierarchy and color coding.
 *
 * @param array $sortedAccounts The sorted list of accounts.
 * @param float &$totalRevenue A reference to the total revenue accumulator.
 * @param float &$totalExpenses A reference to the total expenses accumulator.
 */
function renderIncomeStatementRows(array $sortedAccounts, float &$totalRevenue, float &$totalExpenses): void {
    foreach ($sortedAccounts as $account) {
        $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $account['depth']);
        $accountName = $indent . htmlspecialchars($account['name']);
        $amount = floatval($account['total_amount']);
        $formattedAmount = number_format($amount, 2);
        $accountType = strtolower($account['account_type']);

        // Determine row class based on account_type
        if (in_array($accountType, ['revenue', 'income'])) {
            $rowClass = 'account-revenue';
            $totalRevenue += $amount;
        } elseif ($accountType == 'expense') {
            $rowClass = 'account-expense';
            $totalExpenses += $amount;
        } else {
            $rowClass = '';
        }

        echo '<tr class="' . htmlspecialchars($rowClass) . '">';
        echo '<td>' . $accountName . '</td>';
        echo '<td>' . $formattedAmount . '</td>';
        echo '</tr>';
    }
}

// Fetch all financial periods for the dropdown
$financialPeriods = listFinancialPeriods($pdo);

// Handle form submission
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // Determine if a Financial Period is selected
    if (isset($_POST['financial_period_id']) && !empty($_POST['financial_period_id'])) {
        $selectedPeriodId = (int)$_POST['financial_period_id'];
        $period = getFinancialPeriod($pdo, $selectedPeriodId);
        if ($period) {
            $startDate = $period['start_date'];
            $endDate = $period['end_date'];
            $closedPeriod = isFinancialPeriodClosed($pdo, $selectedPeriodId);
        } else {
            $errors[] = 'Selected financial period not found.';
        }
    } else {
        // Handle custom date range
        $startDate = trim($_POST['start_date']);
        $endDate = trim($_POST['end_date']);
        if (empty($startDate) || empty($endDate)) {
            $errors[] = 'Please provide both start date and end date.';
        } elseif ($startDate > $endDate) {
            $errors[] = 'Start date cannot be after end date.';
        }
    }

    // If no errors, generate the Income Statement using your existing function
    if (empty($errors)) {
        $incomeStatementData = generateIncomeStatementReport($pdo, $startDate, $endDate);

        if (empty($incomeStatementData)) {
            $errors[] = 'No income or expense accounts found for the selected period.';
        } else {
            // Build account hierarchy
            $accountTree = buildAccountTree($incomeStatementData, null);
            $sortedAccounts = [];
            traverseAccountTree($accountTree, $sortedAccounts);
        }
    }
}
?>
<h1>Income Statement</h1>

<div class="form-container">
    <form action="incomestatement.php" method="POST">
        <h2>Select Financial Period or Date Range</h2>
        <label for="financial_period_id">Financial Period:</label>
        <select id="financial_period_id" name="financial_period_id">
            <option value="">-- Select Financial Period --</option>
            <?php foreach ($financialPeriods as $period): ?>
                <option value="<?php echo htmlspecialchars($period['id']); ?>" <?php echo ($selectedPeriodId == $period['id']) ? 'selected' : ''; ?>>
                    <?php echo htmlspecialchars($period['name'] . ' (' . $period['start_date'] . ' to ' . $period['end_date'] . ')'); ?>
                </option>
            <?php endforeach; ?>
        </select>
        <p>Or enter a custom date range:</p>
        <label for="start_date">Start Date:</label>
        <input type="date" id="start_date" name="start_date" value="<?php echo htmlspecialchars($startDate); ?>">
        <label for="end_date">End Date:</label>
        <input type="date" id="end_date" name="end_date" value="<?php echo htmlspecialchars($endDate); ?>">

        <div class="button-group">
            <button type="submit">Generate Income Statement</button>
            <!-- NEW ORANGE BUTTON FOR PDF -->
            <button type="button"
                    id="generate_is_pdf_button"
                    style="background-color: orange; color: #fff; border: none; padding: 8px 12px; cursor: pointer;">
                Generate Income Statement PDF
            </button>
        </div>
    </form>
</div>

<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>

<?php if (!empty($incomeStatementData)): ?>
    <?php
    // Reset totals before rendering
    $totalRevenue = 0;
    $totalExpenses = 0;
    ?>
    <h2>Income Statement Report from <?php echo htmlspecialchars($startDate); ?> to <?php echo htmlspecialchars($endDate); ?></h2>
    <table>
        <thead>
            <tr>
                <th>Account Name</th>
                <th>Amount</th>
            </tr>
        </thead>
        <tbody>
            <?php renderIncomeStatementRows($sortedAccounts, $totalRevenue, $totalExpenses); ?>
        </tbody>
        <tfoot>
            <?php
            $netIncome = $totalRevenue - $totalExpenses;
            ?>
            <tr class="subtotal">
                <td>Total Revenue</td>
                <td><?php echo number_format($totalRevenue, 2); ?></td>
            </tr>
            <tr class="subtotal">
                <td>Total Expenses</td>
                <td><?php echo number_format($totalExpenses, 2); ?></td>
            </tr>
            <tr class="subtotal">
                <td>Net Income</td>
                <td><?php echo number_format($netIncome, 2); ?></td>
            </tr>
        </tfoot>
    </table>
<?php endif; ?>

<!-- HIDDEN ALERT IF NO INCOMESTATEMENT LAYOUTS -->
<div id="is-no-layout-error" class="alert" style="display:none;">
    <p>No saved layout found for <strong>incomestatement</strong>.
       <a href="document_editor.php">Create a layout here</a>.
    </p>
</div>

<!-- MODAL FOR MULTIPLE INCOMESTATEMENT LAYOUTS -->
<div id="is-modal-overlay" style="
    display: none;
    position: fixed;
    top: 0; left: 0;
    width: 100%; height: 100%;
    background: rgba(0,0,0,0.4);
    align-items: center;
    justify-content: center;
    z-index: 9999;">
  <div id="is-modal-content" style="
      background: #fff;
      padding: 20px;
      border-radius: 4px;
      min-width: 300px;">
    <h2>Select an Income Statement Layout</h2>
    <div id="is-modal-error" style="color: red; font-weight: bold; display:none;"></div>
    <label for="is-modal-select">Layouts:</label>
    <select id="is-modal-select" style="width:100%; margin-bottom:15px;"></select>
    <div style="text-align:right;">
      <button id="is-modal-cancel" style="margin-right:10px;">Cancel</button>
      <button id="is-modal-ok" style="background-color: orange; color: #fff; border:none; padding:5px 12px; cursor:pointer;">
          OK
      </button>
    </div>
  </div>
</div>

<script>
document.addEventListener('DOMContentLoaded', function() {
    const pdfBtn = document.getElementById('generate_is_pdf_button');
    if (pdfBtn) {
        pdfBtn.addEventListener('click', function(e) {
            e.preventDefault();
            // Validate the form inputs
            const periodSelect = document.querySelector('[name="financial_period_id"]');
            const startDateInput = document.getElementById('start_date');
            const endDateInput   = document.getElementById('end_date');

            if (!periodSelect.value && (!startDateInput.value || !endDateInput.value)) {
                alert("Please select a financial period or enter a valid start/end date range.");
                return;
            }

            // Step 1: fetch the layout info
            fetch('incomestatement.php?action=list_is_layouts')
            .then(r => r.json())
            .then(data => {
                if (!data.success) {
                    // Show a hidden alert about no layout
                    document.getElementById('is-no-layout-error').style.display = 'block';
                    return;
                }
                const layouts = data.layouts || [];
                if (layouts.length === 0) {
                    document.getElementById('is-no-layout-error').style.display = 'block';
                    return;
                }
                if (layouts.length === 1) {
                    openIncomeStatementPdfInNewTab(layouts[0]);
                } else {
                    showIncomeStatementModal(layouts);
                }
            })
            .catch(err => {
                console.error("Error fetching IncomeStatement layouts:", err);
                alert("Failed to list IncomeStatement layouts. See console for details.");
            });
        });
    }

    function openIncomeStatementPdfInNewTab(layoutName) {
        // Gather user inputs
        const periodVal   = document.querySelector('[name="financial_period_id"]').value.trim();
        const startDateVal= document.getElementById('start_date').value.trim();
        const endDateVal  = document.getElementById('end_date').value.trim();

        // Fetch the layout JSON
        fetch('documents/' + layoutName)
            .then(r => r.json())
            .then(fullLayout => {
                // Insert or override dialogData with user inputs
                fullLayout.dialogData = {
                    financial_period_id: periodVal,
                    start_date: startDateVal,
                    end_date: endDateVal
                };

                // Post that layout JSON to document_display_pdf.php in a new tab
                const f = document.createElement('form');
                f.method = 'POST';
                f.action = 'document_display_pdf.php';
                f.target = '_blank';

                const hiddenInp = document.createElement('input');
                hiddenInp.type = 'hidden';
                hiddenInp.name = 'layoutJSON';
                hiddenInp.value = JSON.stringify(fullLayout);
                f.appendChild(hiddenInp);

                document.body.appendChild(f);
                f.submit();
            })
            .catch(err => {
                console.error("Cannot load layout file:", err);
                alert("Error loading layout JSON for " + layoutName);
            });
    }

    function showIncomeStatementModal(listOfLayouts) {
        const overlay   = document.getElementById('is-modal-overlay');
        const modalSel  = document.getElementById('is-modal-select');
        const cancelBtn = document.getElementById('is-modal-cancel');
        const okBtn     = document.getElementById('is-modal-ok');
        const errDiv    = document.getElementById('is-modal-error');

        // Reset the modal
        modalSel.innerHTML = '';
        errDiv.textContent = '';
        errDiv.style.display = 'none';

        listOfLayouts.forEach(l => {
            const opt = document.createElement('option');
            opt.value = l;
            opt.textContent = l;
            modalSel.appendChild(opt);
        });

        overlay.style.display = 'flex';

        cancelBtn.onclick = () => {
            overlay.style.display = 'none';
        };

        okBtn.onclick = () => {
            const chosen = modalSel.value.trim();
            if (!chosen) {
                errDiv.textContent = "Please select a layout.";
                errDiv.style.display = 'block';
                return;
            }
            overlay.style.display = 'none';
            openIncomeStatementPdfInNewTab(chosen);
        };
    }
});
</script>

<?php require_once('components/footer.php'); ?>
