<?php
require_once('../core/journal_entries.php');
require_once('../core/transactions.php'); // for isDateInClosedVATPeriod(...)
require_once('../core/financial_periods.php'); // for isFinancialPeriodClosed(...)
require_once('components/auth.php');
require_once('components/header.php');

$errors = [];
$success = [];

// Check if the user has the 'journal_entries' role
if (!checkRole(['journal_entries'])) {
    $errors[] = 'Access denied. You do not have permission to view this page.';
    echo '<h1>Manage Journal Entries</h1>';
    if (!empty($errors)) {
        echo '<div class="alert">';
        foreach ($errors as $error) {
            echo '<p>' . htmlspecialchars($error) . '</p>';
        }
        echo '</div>';
    }
    require_once('components/footer.php');
    exit;
}

// For deleting
$userId = $_SESSION['user_id'] ?? 0;

// ------------------------------------------------------------------
// SESSION-BASED FILTER STATE
// ------------------------------------------------------------------
function restoreJournalEntriesStateFromSession(): array {
    if (!isset($_SESSION['journal_entries_state'])) {
        $_SESSION['journal_entries_state'] = [];
    }
    $merged = $_SESSION['journal_entries_state'];
    foreach ($_GET as $key => $value) {
        if ($value !== '') {
            $merged[$key] = $value;
        } else {
            unset($merged[$key]);
        }
    }
    $_SESSION['journal_entries_state'] = $merged;
    return $merged;
}

function buildQuery($params, $exclude = []) {
    foreach ($exclude as $key) {
        unset($params[$key]);
    }
    return http_build_query($params);
}

if (isset($_GET['reset_filters']) && $_GET['reset_filters'] === '1') {
    unset($_SESSION['journal_entries_state']);
    header('Location: journal_entries.php');
    exit;
}

// Merge GET with the session state
$state = restoreJournalEntriesStateFromSession();
$_GET = $state; // so old code can reference $_GET

// ------------------------------------------------------------------
// POST Handling: Deletion
// ------------------------------------------------------------------
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['delete_journal_entry_id']) && hasRole('journal_entries_delete')) {
        $journalEntryId = (int)$_POST['delete_journal_entry_id'];
        if ($journalEntryId > 0) {
            $result = deleteJournalEntry($pdo, $journalEntryId, $userId);
            if ($result) {
                $success[] = 'Journal entry has been deleted successfully.';
            } else {
                $errors[] = 'Cannot delete journal entry because there are active transactions linked to it or it is in a closed period.';
            }
        }
    }
}

// ------------------------------------------------------------------
// FILTERS, SORTING, PAGINATION
// ------------------------------------------------------------------
$filters = [];
$appliedFilters = [];

// Potential filters
if (isset($state['id']) && $state['id'] !== '') {
    $filters['id'] = $state['id'];
    $appliedFilters[] = 'ID: ' . htmlspecialchars($state['id']);
}
if (isset($state['description']) && $state['description'] !== '') {
    $filters['description'] = $state['description'];
    $appliedFilters[] = 'Description: ' . htmlspecialchars($state['description']);
}
if (isset($state['start_date']) && isset($state['end_date']) && $state['start_date'] !== '' && $state['end_date'] !== '') {
    $filters['start_date'] = $state['start_date'];
    $filters['end_date']   = $state['end_date'];
    $appliedFilters[] = 'Date Range: ' . htmlspecialchars($state['start_date']) . ' to ' . htmlspecialchars($state['end_date']);
}
if (isset($state['financial_period_id']) && $state['financial_period_id'] !== '') {
    $filters['financial_period_id'] = $state['financial_period_id'];
    $appliedFilters[] = 'Financial Period: ' . htmlspecialchars($state['financial_period_id']);
}
if (isset($state['created_by']) && $state['created_by'] !== '') {
    $filters['created_by'] = $state['created_by'];
    $appliedFilters[] = 'Created By: ' . htmlspecialchars($state['created_by']);
}

// Sorting & page size
$allowedSortColumns = ['id','description','date','financial_period_id','created_by'];
$sortBy    = $state['sort_by']    ?? 'date';
$sortOrder = (isset($state['sort_order']) && strtolower($state['sort_order']) === 'asc') ? 'ASC' : 'DESC';
if (!in_array($sortBy, $allowedSortColumns)) {
    $sortBy = 'date';
}

$allowedPageSizes = [5, 10, 20, 50, 100];
$pageSize = isset($state['page_size']) && in_array((int)$state['page_size'], $allowedPageSizes)
    ? (int)$state['page_size'] : 20;
if (!in_array($pageSize, $allowedPageSizes)) {
    $pageSize = 20;
}
$page = isset($state['page']) && is_numeric($state['page']) && $state['page'] > 0
    ? (int)$state['page'] : 1;

// ------------------------------------------------------------------
// Build Custom Query for Journal Entries
// ------------------------------------------------------------------
$sqlWhere   = ['is_deleted = 0'];
$sqlParams  = [];

if (isset($filters['id'])) {
    $sqlWhere[] = 'id = :id';
    $sqlParams[':id'] = $filters['id'];
}
if (isset($filters['description'])) {
    $sqlWhere[] = 'description LIKE :description';
    $sqlParams[':description'] = '%' . $filters['description'] . '%';
}
if (isset($filters['start_date']) && isset($filters['end_date'])) {
    $sqlWhere[] = 'date BETWEEN :start_date AND :end_date';
    $sqlParams[':start_date'] = $filters['start_date'];
    $sqlParams[':end_date']   = $filters['end_date'];
}
if (isset($filters['financial_period_id'])) {
    $sqlWhere[] = 'financial_period_id = :financial_period_id';
    $sqlParams[':financial_period_id'] = $filters['financial_period_id'];
}
if (isset($filters['created_by'])) {
    $sqlWhere[] = 'created_by = :created_by';
    $sqlParams[':created_by'] = $filters['created_by'];
}

$whereClause = '';
if (!empty($sqlWhere)) {
    $whereClause = 'WHERE ' . implode(' AND ', $sqlWhere);
}

// Count total
$countSQL = "SELECT COUNT(*) FROM journal_entries $whereClause";
$countStmt= $pdo->prepare($countSQL);
foreach ($sqlParams as $k => $v) {
    $countStmt->bindValue($k, $v);
}
$countStmt->execute();
$totalRecords = $countStmt->fetchColumn();
$totalPages   = ceil($totalRecords / $pageSize);

$orderSQL = "ORDER BY $sortBy $sortOrder";
$offset   = ($page - 1) * $pageSize;
$limitSQL = "LIMIT :limit OFFSET :offset";

// Fetch data
$dataSQL = "SELECT * FROM journal_entries $whereClause $orderSQL $limitSQL";
$dataStmt = $pdo->prepare($dataSQL);
foreach ($sqlParams as $k => $v) {
    $dataStmt->bindValue($k, $v);
}
$dataStmt->bindValue(':limit', (int)$pageSize, PDO::PARAM_INT);
$dataStmt->bindValue(':offset', (int)$offset, PDO::PARAM_INT);
$dataStmt->execute();
$journalEntries = $dataStmt->fetchAll(PDO::FETCH_ASSOC);

// ------------------------------------------------------------------
// Build Module Info
// ------------------------------------------------------------------
$moduleIds = [];
foreach ($journalEntries as $entry) {
    if (!empty($entry['module_id']) && $entry['module_id'] != 0) {
        $moduleIds[] = (int)$entry['module_id'];
    }
}
$moduleIds = array_unique($moduleIds);

$moduleInfo = [];
if (!empty($moduleIds)) {
    $inPlaceholders = implode(',', array_fill(0, count($moduleIds), '?'));
    $stmt = $pdo->prepare("SELECT rowid as id, module_name, redirect_journal_entries
                           FROM modules WHERE rowid IN ($inPlaceholders)");
    $stmt->execute($moduleIds);
    while ($m = $stmt->fetch(PDO::FETCH_ASSOC)) {
        $moduleInfo[(int)$m['id']] = [
            'module_name'             => $m['module_name'],
            'redirect_journal_entries'=> $m['redirect_journal_entries']
        ];
    }
}
?>

<h1>Manage Journal Entries</h1>
<?php
// Errors
if (!empty($errors)) {
    echo '<div class="alert">';
    foreach ($errors as $error) {
        echo '<p>' . htmlspecialchars($error) . '</p>';
    }
    echo '</div>';
}
// Success
if (!empty($success)) {
    echo '<div class="success">';
    foreach ($success as $msg) {
        echo '<p>' . htmlspecialchars($msg) . '</p>';
    }
    echo '</div>';
}
?>
<div class="button-container">
    <?php if (hasRole('journal_entries_add')): ?>
        <a href="add_journal_entry.php" class="button">Add Journal Entry</a>
    <?php endif; ?>

    <button type="button" class="button button-right" id="toggle-filters-button">Toggle Filters</button>
</div>


<!-- Filter Form -->
<form class="filter-form" method="GET" action="journal_entries.php" id="filter-form">
    <div>
        <label for="id">ID:</label>
        <input type="number" name="id" id="id"
               value="<?php echo htmlspecialchars($state['id'] ?? ''); ?>">
    </div>
    <div>
        <label for="description">Description:</label>
        <input type="text" name="description" id="description"
               value="<?php echo htmlspecialchars($state['description'] ?? ''); ?>">
    </div>
    <div>
        <label for="start_date">Start Date:</label>
        <input type="date" name="start_date" id="start_date"
               value="<?php echo htmlspecialchars($state['start_date'] ?? ''); ?>">
    </div>
    <div>
        <label for="end_date">End Date:</label>
        <input type="date" name="end_date" id="end_date"
               value="<?php echo htmlspecialchars($state['end_date'] ?? ''); ?>">
    </div>
    <div>
        <label for="financial_period_id">Financial Period:</label>
        <input type="number" name="financial_period_id" id="financial_period_id"
               value="<?php echo htmlspecialchars($state['financial_period_id'] ?? ''); ?>">
    </div>
    <div>
        <label for="created_by">Created By (User ID):</label>
        <input type="number" name="created_by" id="created_by"
               value="<?php echo htmlspecialchars($state['created_by'] ?? ''); ?>">
    </div>
    <div style="margin-top: 10px;">
        <button type="submit" class="button">Filter</button>
        <a href="journal_entries.php?reset_filters=1" class="button">Reset</a>
    </div>
</form>

<div class="applied-filters">
    <?php
    if (!empty($appliedFilters)) {
        echo 'Applied Filters: ' . implode(', ', $appliedFilters);
    } else {
        echo 'No filters applied.';
    }
    ?>
</div>

<table>
    <thead>
    <tr>
        <?php
        $headers = [
            'id'                  => 'ID',
            'description'         => 'Description',
            'date'                => 'Date',
            'financial_period_id' => 'Financial Period',
            'created_by'          => 'Created By'
        ];
        foreach ($headers as $key => $label) {
            $currentSort = ($sortBy === $key) ? 'sort-' . strtolower($sortOrder) : '';
            echo '<th class="' . $currentSort . '" style="position:relative;">';
            echo '<a href="?' . buildQuery(array_merge($state, [
                'sort_by' => $key,
                'sort_order' => ($sortBy === $key && $sortOrder === 'ASC') ? 'desc' : 'asc',
                'page' => 1
            ])) . '">';
            echo htmlspecialchars($label);
            echo '</a>';
            echo '</th>';
        }
        ?>
        <th>Actions</th>
    </tr>
    </thead>
    <tbody>
    <?php if (!empty($journalEntries)): ?>
        <?php foreach ($journalEntries as $entry): ?>
            <tr>
                <td><?php echo htmlspecialchars($entry['id']); ?></td>
                <td><?php echo htmlspecialchars($entry['description']); ?></td>
                <td><?php echo htmlspecialchars($entry['date']); ?></td>
                <td><?php echo htmlspecialchars($entry['financial_period_id']); ?></td>
                <td><?php echo htmlspecialchars($entry['created_by']); ?></td>
                <td>
                    <?php
                    if (!empty($entry['module_id']) && $entry['module_id'] != 0 && isset($moduleInfo[$entry['module_id']])) {
                        // If module, show module redirect
                        $mName     = htmlspecialchars($moduleInfo[$entry['module_id']]['module_name']);
                        $mRedirect = htmlspecialchars($moduleInfo[$entry['module_id']]['redirect_journal_entries']);
                        // Edit if allowed
                        if (hasRole('journal_entries_update')) {
                            echo '<a href="edit_journal_entry.php?id=' . htmlspecialchars($entry['id']) . '" class="button edit-button">Edit</a> ';
                        }
                        echo '<a href="' . $mRedirect . '?id=' . htmlspecialchars($entry['id']) . '" class="button">' . $mName . '</a>';
                    } else {
                        // Normal
                        if (hasRole('journal_entries_update')) {
                            echo '<a href="edit_journal_entry.php?id=' . htmlspecialchars($entry['id']) . '" class="button edit-button">Edit</a> ';
                        }

                        // Hide delete if period or VAT is closed
                        $periodClosed = isFinancialPeriodClosed($pdo, (int)$entry['financial_period_id']);
                        $vatClosed    = isDateInClosedVATPeriod($pdo, $entry['date']);

                        if (hasRole('journal_entries_delete') && $entry['module_id'] != -1 && !$periodClosed && !$vatClosed) {
                            echo '<form action="journal_entries.php" method="POST" style="display:inline;">';
                            echo '<input type="hidden" name="delete_journal_entry_id" value="' . htmlspecialchars($entry['id']) . '">';
                            echo '<button type="submit" class="button delete-button" onclick="return confirm(\'Are you sure you want to delete this journal entry?\');">Delete</button>';
                            echo '</form>';
                        }
                    }
                    ?>
                </td>
            </tr>
        <?php endforeach; ?>
    <?php else: ?>
        <tr><td colspan="6">No journal entries found.</td></tr>
    <?php endif; ?>
    </tbody>
</table>

<!-- Pagination -->
<div class="pagination-controls">
    <div class="pagination">
        <?php
        $range = 2;
        $start = max(1, $page - $range);
        $end   = min($totalPages, $page + $range);

        if ($page > 1) {
            echo '<a href="?' . buildQuery(array_merge($state, ['page' => $page - 1])) . '">&laquo; Prev</a>';
        }
        if ($start > 1) {
            echo '<a href="?' . buildQuery(array_merge($state, ['page' => 1])) . '">1</a>';
            if ($start > 2) {
                echo '<span>...</span>';
            }
        }
        for ($i = $start; $i <= $end; $i++) {
            if ($i == $page) {
                echo '<a href="?' . buildQuery(array_merge($state, ['page' => $i])) . '" class="active">' . $i . '</a>';
            } else {
                echo '<a href="?' . buildQuery(array_merge($state, ['page' => $i])) . '">' . $i . '</a>';
            }
        }
        if ($end < $totalPages) {
            if ($end < $totalPages - 1) {
                echo '<span>...</span>';
            }
            echo '<a href="?' . buildQuery(array_merge($state, ['page' => $totalPages])) . '">' . $totalPages . '</a>';
        }
        if ($page < $totalPages) {
            echo '<a href="?' . buildQuery(array_merge($state, ['page' => $page + 1])) . '">Next &raquo;</a>';
        }
        ?>
    </div>
    <form method="GET" action="journal_entries.php" id="page_size_form">
        <?php
        // Keep all existing parameters except page_size, page
        $currentParams = $state;
        unset($currentParams['page_size'], $currentParams['page']);
        foreach ($currentParams as $key => $val) {
            echo '<input type="hidden" name="' . htmlspecialchars($key) . '" value="' . htmlspecialchars($val) . '">';
        }
        ?>
        <input type="hidden" name="page" value="1">
        <label for="page_size">Show
            <select id="page_size" name="page_size" onchange="document.getElementById('page_size_form').submit()" style="padding: 5px;">
                <?php
                foreach ([5, 10, 20, 50, 100] as $sz) {
                    $selected = ($sz === $pageSize) ? 'selected' : '';
                    echo '<option value="' . $sz . '" ' . $selected . '>' . $sz . '</option>';
                }
                ?>
            </select>
            results per page
        </label>
    </form>
</div>

<script>
document.addEventListener('DOMContentLoaded', function() {
    // Toggle filter form
    var toggleBtn = document.getElementById('toggle-filters-button');
    var filterForm = document.getElementById('filter-form');

    function setFilterFormDisplay(isVisible) {
        filterForm.style.display = isVisible ? 'block' : 'none';
        sessionStorage.setItem('journalEntriesFilterFormVisible', isVisible ? 'true' : 'false');
    }

    function areFiltersApplied() {
        var fields = ['id','description','start_date','end_date','financial_period_id','created_by'];
        for (var i=0; i<fields.length; i++){
            var el = document.getElementById(fields[i]);
            if(el && el.value && el.value.trim() !== '') {
                return true;
            }
        }
        return false;
    }

    var savedState = sessionStorage.getItem('journalEntriesFilterFormVisible');
    if (savedState === null) {
        if (areFiltersApplied()) {
            setFilterFormDisplay(true);
        } else {
            setFilterFormDisplay(false);
        }
    } else {
        setFilterFormDisplay(savedState === 'true');
    }

    if(toggleBtn && filterForm) {
        toggleBtn.addEventListener('click', function() {
            var currentlyVisible = (filterForm.style.display !== 'none' && filterForm.style.display !== '');
            setFilterFormDisplay(!currentlyVisible);
        });
    }

    // Scroll position
    if (sessionStorage.getItem('journalEntriesScrollPosition')) {
        window.scrollTo(0, parseInt(sessionStorage.getItem('journalEntriesScrollPosition'), 10));
    }
    window.addEventListener('scroll', function() {
        sessionStorage.setItem('journalEntriesScrollPosition', window.scrollY);
    });
});
</script>

<style>
.alert {
    background-color: #f8d7da;
    color: #721c24;
    padding: 10px;
    margin-bottom: 15px;
    border-radius: 4px;
}
.success {
    background-color: #d4edda;
    color: #155724;
    padding: 10px;
    margin-bottom: 15px;
    border-radius: 4px;
}
.filter-form {
    margin-bottom: 20px;
    display: none;
    padding: 15px;
    border: 1px solid #ccc;
    border-radius: 4px;
    background-color: #fafafa;
}
.filter-form div {
    margin-bottom: 10px;
}
.filter-form label {
    margin-right: 10px;
    display: inline-block;
    width: 150px;
    font-weight: bold;
}
.filter-form input, .filter-form select {
    margin-bottom: 10px;
    padding: 5px;
    width: 200px;
    display: inline-block;
}
.applied-filters {
    margin-top: 10px;
    padding: 10px;
    background-color: #e2e3e5;
    border: 1px solid #d6d8db;
    border-radius: 4px;
    color: #383d41;
}
.pagination-controls {
    display: flex;
    justify-content: space-between;
    align-items: center;
    margin-top: 20px;
}
.pagination a, .pagination span {
    margin: 0 5px;
    text-decoration: none;
    padding: 5px 10px;
    border: 1px solid #ddd;
    border-radius: 4px;
    color: #007bff;
}
.pagination a.active {
    background-color: #007bff;
    color: white;
    border-color: #007bff;
}
.pagination span {
    padding: 5px 10px;
    color: #555;
}
.button-right {
    margin-left: 10px;
    float: right !important;
}
th {
    position: relative;
    background-color: #f2f2f2;
}
th a {
    text-decoration: none;
    color: #ffffff;
    display: block;
    padding-right: 20px; /* space for the sort arrow */
}
th.sort-asc::after {
    content: " ▲";
    position: absolute;
    top: 10px;
    right: 8px;
}
th.sort-desc::after {
    content: " ▼";
    position: absolute;
    top: 10px;
    right: 8px;
}
.delete-button {
    background-color: #c0392b;
}
.delete-button:hover {
    background-color: #a93226;
}
</style>

<?php require_once('components/footer.php'); ?>
