<?php
require_once('../core/accounts.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('../config/config.php');

// Load Bank Module configuration
$bankModuleConfig   = require_once __DIR__ . '/../modules/Bank/config.php';
$bankAccountParent  = $bankModuleConfig['bank_account_parent'] ?? 'B60';

try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    die("Database Connection Failed: " . $e->getMessage());
}

$errors = [];
$success = '';

if (!checkRole(['bank_accounts'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to view this page.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Manage Bank Accounts</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['delete_account'])) {
    if (!checkRole(['bank_accounts_delete'])) {
        $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to delete bank accounts.';
    } else {
        $code = trim($_POST['delete_account']);
        $account = getAccount($pdo, $code);
        if (!$account) {
            $errors[] = "Bank account not found.";
        } else {
            $hasTransactions = checkIfAccountHasTransactions($pdo, $code);
            if ($hasTransactions) {
                $errors[] = "Cannot delete bank account [$code] {$account['name']} because it has associated transactions.";
            } else {
                $deleted = deleteAccount($pdo, $code);
                if ($deleted) {
                    $success = "Bank account [$code] has been successfully deleted.";
                } else {
                    $errors[] = "Failed to delete bank account [$code]. Please try again.";
                }
            }
        }
    }
}

function getDescendantAccounts(PDO $pdo, $parentCode) {
    $accounts = [];
    $stmt = $pdo->prepare("SELECT * FROM accounts WHERE parent = :parent");
    $stmt->execute([':parent' => $parentCode]);
    $children = $stmt->fetchAll(PDO::FETCH_ASSOC);

    foreach ($children as $child) {
        $accounts[] = $child;
        $descendants = getDescendantAccounts($pdo, $child['code']);
        $accounts = array_merge($accounts, $descendants);
    }
    return $accounts;
}

// Retrieve all bank accounts descending from the configured parent code
$bankAccounts = getDescendantAccounts($pdo, $bankAccountParent);

/**
 * Checks if an account has any associated transactions.
 *
 * @param PDO    $pdo
 * @param string $code
 * @return bool
 */
function checkIfAccountHasTransactions(PDO $pdo, string $code): bool {
    try {
        $stmt = $pdo->prepare("SELECT COUNT(*) FROM transactions WHERE debit_account = :code OR credit_account = :code");
        $stmt->execute([':code' => $code]);
        $count = $stmt->fetchColumn();
        return $count > 0;
    } catch (PDOException $e) {
        // LOG_FILE should be defined in your system
        error_log('Check Transactions Error: ' . $e->getMessage(), 3, LOG_FILE);
        return false;
    }
}
?>
<h1>Manage Bank Accounts</h1>

<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>

<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success); ?></p>
</div>
<?php endif; ?>

<?php if (hasRole('bank_accounts_add')): ?>
<div class="button-container">
    <a href="add_bank_account.php" class="button">Add Bank Account</a>
</div>
<?php endif; ?>

<?php if (!empty($bankAccounts)): ?>
<table>
    <thead>
        <tr>
            <th>Code</th>
            <th>Name</th>
            <th>Description</th>
            <?php if (hasRole('bank_accounts_delete')): ?>
                <th>Actions</th>
            <?php endif; ?>
        </tr>
    </thead>
    <tbody>
    <?php foreach ($bankAccounts as $account): ?>
        <tr>
            <td><?php echo htmlspecialchars($account['code']); ?></td>
            <td><?php echo htmlspecialchars($account['name']); ?></td>
            <td><?php echo htmlspecialchars($account['description']); ?></td>
            <?php if (hasRole('bank_accounts_delete')): ?>
                <td>
                    <form action="manage_bank_accounts.php" method="POST" onsubmit="return confirm('Are you sure you want to delete this bank account?');">
                        <input type="hidden" name="delete_account" value="<?php echo htmlspecialchars($account['code']); ?>">
                        <button type="submit" class="button delete-button">Delete</button>
                    </form>
                </td>
            <?php endif; ?>
        </tr>
    <?php endforeach; ?>
    </tbody>
</table>
<?php else: ?>
<p>No bank accounts found. Please add a bank account.</p>
<?php endif; ?>

<?php require_once('components/footer.php'); ?>
