<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Include necessary files and initialize the database connection
require_once('../core/utilities.php'); // Ensure this initializes $pdo
require_once('../core/users.php'); // For getAllRoles()
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

$errors = [];
$success = '';

// Check if the user has the 'header_menu_edit' role
if (!checkRole(['header_menu_edit'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to edit the menu.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Menu Builder</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error ?? '', ENT_QUOTES, 'UTF-8'); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Handle form submissions
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // Delete Menu Item
    if (isset($_POST['action']) && $_POST['action'] === 'delete') {
        $id = isset($_POST['id']) ? (int)$_POST['id'] : null;

        if ($id) {
            // Check if the menu item is not a system item
            $stmt = $pdo->prepare("SELECT is_system FROM menu_items WHERE id = ?");
            $stmt->execute([$id]);
            $menuItem = $stmt->fetch(PDO::FETCH_ASSOC);

            if ($menuItem && !$menuItem['is_system']) {
                // Delete menu item
                $stmt = $pdo->prepare("DELETE FROM menu_items WHERE id = ?");
                $result = $stmt->execute([$id]);

                if ($result) {
                    $success = 'Menu item deleted successfully.';
                } else {
                    $errors[] = 'Failed to delete menu item.';
                }
            } else {
                $errors[] = 'Cannot delete a system menu item.';
            }
        } else {
            $errors[] = 'Invalid menu item ID.';
        }
    }
}

// Fetch all menu items for display
$stmt = $pdo->prepare("SELECT * FROM menu_items ORDER BY parent_id ASC, order_num ASC");
$stmt->execute();
$menuItems = $stmt->fetchAll(PDO::FETCH_ASSOC);

// Fetch roles for the required_role field (if needed in future enhancements)
$roles = getAllRoles($pdo);

// Build a menu tree for display
function buildMenuTree($menuItems, $parentId = null) {
    $branch = [];
    foreach ($menuItems as $item) {
        if ($item['parent_id'] == $parentId) {
            $children = buildMenuTree($menuItems, $item['id']);
            if ($children) {
                $item['children'] = $children;
            }
            $branch[] = $item;
        }
    }
    return $branch;
}

$menuTree = buildMenuTree($menuItems);
?>
<h1>Menu Builder</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
    <p><?php echo htmlspecialchars($error ?? '', ENT_QUOTES, 'UTF-8'); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success ?? '', ENT_QUOTES, 'UTF-8'); ?></p>
</div>
<?php endif; ?>

<!-- Button to Add New Menu Item -->
<div class="button-container">
    <a href="add_menu_builder.php" class="button">Add New Menu Item</a>
</div>

<!-- Display Menu Items -->
<h2>Menu Items</h2>
<table>
    <thead>
        <tr>
            <th>Name</th>
            <th>URL</th>
            <th>Parent</th>
            <th>Order</th>
            <th>Dropdown</th>
            <th>Visible</th>
            <th>Required Role</th>
            <th>Actions</th>
        </tr>
    </thead>
    <tbody>
        <?php
        function renderMenuTable($menuTree, $parentName = '', $level = 0) {
            foreach ($menuTree as $item) {
                $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $level);
                $isSystem = $item['is_system'] ? 'Yes' : 'No';
                $parentNameDisplay = $parentName ?: 'None';
                $isDropdown = $item['is_dropdown'] ? 'Yes' : 'No';
                $isVisible = $item['is_visible'] ? 'Yes' : 'No';
                $requiredRole = htmlspecialchars($item['required_role'] ?? '', ENT_QUOTES, 'UTF-8');
                ?>
                <tr>
                    <td><?php echo $indent . htmlspecialchars($item['name'] ?? '', ENT_QUOTES, 'UTF-8'); ?></td>
                    <td><?php echo htmlspecialchars($item['url'] ?? '', ENT_QUOTES, 'UTF-8'); ?></td>
                    <td><?php echo htmlspecialchars($parentNameDisplay, ENT_QUOTES, 'UTF-8'); ?></td>
                    <td><?php echo htmlspecialchars($item['order_num'] ?? '', ENT_QUOTES, 'UTF-8'); ?></td>
                    <td><?php echo $isDropdown; ?></td>
                    <td><?php echo $isVisible; ?></td>
                    <td><?php echo $requiredRole; ?></td>
                    <td>
                        <!-- Edit Button -->
                        <a href="edit_menu_builder.php?edit_id=<?php echo htmlspecialchars($item['id'], ENT_QUOTES, 'UTF-8'); ?>" class="button edit-button">Edit</a>
                        <?php if (!$item['is_system']): ?>
                        <!-- Delete Button -->
                        <form action="menu_builder.php" method="POST" style="display:inline;" onsubmit="return confirm('Are you sure you want to delete this menu item?');">
                            <input type="hidden" name="action" value="delete">
                            <input type="hidden" name="id" value="<?php echo htmlspecialchars($item['id'], ENT_QUOTES, 'UTF-8'); ?>">
                            <button type="submit" class="button delete-button">Delete</button>
                        </form>
                        <?php endif; ?>
                    </td>
                </tr>
                <?php
                if (!empty($item['children'])) {
                    renderMenuTable($item['children'], $item['name'], $level + 1);
                }
            }
        }

        renderMenuTable($menuTree);
        ?>
    </tbody>
</table>

<?php require_once('components/footer.php'); ?>
