<?php
require_once('components/auth.php');
require_once('../core/users.php');       // for roles check
require_once('../core/error_handling.php'); // if you want to log errors
require_once('../config/config.php');    // Use config for DB_FILE, etc.
require_once('components/header.php');

/*
    Roles needed:
     - module_install     => to upload & install new .dmod
     - module_activate    => to run init.php
     - module_deactivate  => to run deinit.php
     - module_delete      => to remove physical folder
*/

// Quick role-check:
function canInstall()     { return hasRole('module_install'); }
function canActivate()    { return hasRole('module_activate'); }
function canDeactivate()  { return hasRole('module_deactivate'); }
function canDeleteModule(){ return hasRole('module_delete'); }

$errors = [];
$success = '';

if (!canInstall() && !canActivate() && !canDeactivate() && !canDeleteModule()) {
    $errors[] = 'Access denied. No module management permissions.';
    ?>
    <h1>Module Manager</h1>
    <?php if (!empty($errors)): ?>
      <div class="alert">
        <?php foreach ($errors as $e): ?>
          <p><?php echo htmlspecialchars($e); ?></p>
        <?php endforeach; ?>
      </div>
    <?php endif; ?>
    <?php require_once('components/footer.php');
    exit;
}

// Database connect using config
try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    $errors[] = "Database Connection Failed: " . $e->getMessage();
}

// Helper to recursively delete folder
function rrmdir($dirPath) {
    if (!is_dir($dirPath)) return;
    $it = new RecursiveIteratorIterator(
        new RecursiveDirectoryIterator($dirPath, RecursiveDirectoryIterator::SKIP_DOTS),
        RecursiveIteratorIterator::CHILD_FIRST
    );
    foreach ($it as $file) {
        if ($file->isDir()) {
            rmdir($file->getRealPath());
        } else {
            unlink($file->getRealPath());
        }
    }
    rmdir($dirPath);
}

/**
 * Find the common root folder (if any) for all items in a PclZip list.
 * Returns the folder name if there's exactly one top folder, otherwise '' (empty).
 */
function findCommonRootFolder(array $archiveList): string {
    if (empty($archiveList)) return '';
    // Collect top-level dirs
    $paths = [];
    foreach ($archiveList as $item) {
        $filename = $item['filename'] ?? '';
        if (!$filename) continue;
        $parts = explode('/', $filename);
        if (!empty($parts[0])) {
            $paths[] = $parts[0];
        }
    }
    $paths = array_unique($paths);
    if (count($paths) === 1) {
        // We have exactly one top-level folder
        return reset($paths);
    }
    return '';
}

// modules folder location
$modulesDir = dirname(__DIR__) . '/modules';

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // 1) Install module
    if (isset($_POST['action']) && $_POST['action'] === 'install' && canInstall()) {
        if (!isset($_FILES['module_file']) || $_FILES['module_file']['error'] != UPLOAD_ERR_OK) {
            $errors[] = 'Please select a valid .dmod file to upload.';
        } else {
            $tmpPath      = $_FILES['module_file']['tmp_name'];
            $originalName = $_FILES['module_file']['name'];
            if (strtolower(pathinfo($originalName, PATHINFO_EXTENSION)) !== 'dmod') {
                $errors[] = 'File must have .dmod extension.';
            } else {
                // Load PclZip library (zlib-based)
                require_once('../lib/pclzip/pclzip.lib.php');

                $archive = new PclZip($tmpPath);
                $tempExtractName = '_temp_extract_' . uniqid();
                $extractPath     = $modulesDir . '/' . $tempExtractName;

                if (!mkdir($extractPath, 0777, true)) {
                    $errors[] = 'Could not create temporary extraction folder.';
                } else {
                    // Gather a list of all files/folders in the archive
                    $archiveList = $archive->listContent();
                    $commonRoot  = findCommonRootFolder($archiveList);

                    if ($commonRoot) {
                        // Remove the single top-level folder, preserving subfolders
                        $result = $archive->extract(
                            PCLZIP_OPT_PATH, $extractPath,
                            PCLZIP_OPT_REMOVE_PATH, $commonRoot
                        );
                    } else {
                        // Otherwise extract as is
                        $result = $archive->extract(PCLZIP_OPT_PATH, $extractPath);
                    }

                    if ($result == 0) {
                        $errors[] = 'Extraction error: ' . $archive->errorInfo(true);
                    } else {
                        // Attempt to parse $moduleName from init.php
                        $moduleName = '';
                        $possibleInit = $extractPath . '/init.php';
                        if (file_exists($possibleInit)) {
                            $content = file_get_contents($possibleInit);
                            if (preg_match("/\\\$moduleName\\s*=\\s*'([^']+)'/i", $content, $m)) {
                                $moduleName = trim($m[1]);
                            }
                        }
                        // fallback if not found
                        if (!$moduleName) {
                            $moduleName = 'Bank';
                        }

                        // final target: /modules/<moduleName>
                        $finalPath = $modulesDir . '/' . $moduleName;
                        if (is_dir($finalPath)) {
                            // Remove the temp folder
                            rrmdir($extractPath);
                            $errors[] = "A folder named '$moduleName' already exists in /modules. Cannot install duplicate.";
                        } else {
                            if (!rename($extractPath, $finalPath)) {
                                rrmdir($extractPath);
                                $errors[] = "Failed to rename temp folder to '$moduleName'. Installation aborted.";
                            } else {
                                $success = "Module '$moduleName' extracted successfully. You may now activate it.";
                            }
                        }
                    }
                }
            }
        }
    }

    // 2) Activate module
    if (isset($_POST['action']) && $_POST['action'] === 'activate' && canActivate()) {
        $modName = trim($_POST['module_name'] ?? '');
        if (!$modName) {
            $errors[] = 'No module name specified.';
        } else {
            $initFile = $modulesDir . '/' . $modName . '/init.php';
            if (!file_exists($initFile)) {
                $errors[] = "init.php not found in $modName folder.";
            } else {
                ob_start();
                include($initFile);
                $output = ob_get_clean();
                $success = "Module [$modName] activated. Output:\n" . $output;
            }
        }
    }

    // 3) Deactivate module
    if (isset($_POST['action']) && $_POST['action'] === 'deactivate' && canDeactivate()) {
        $modName = trim($_POST['module_name'] ?? '');
        if (!$modName) {
            $errors[] = 'No module name specified.';
        } else {
            $deinitFile = $modulesDir . '/' . $modName . '/deinit.php';
            if (!file_exists($deinitFile)) {
                $errors[] = "deinit.php not found in $modName folder.";
            } else {
                ob_start();
                include($deinitFile);
                $output = ob_get_clean();
                $success = "Module [$modName] deactivated. Output:\n" . $output;
            }
        }
    }

    // 4) Remove module folder
    if (isset($_POST['action']) && $_POST['action'] === 'delete' && canDeleteModule()) {
        $modName = trim($_POST['module_name'] ?? '');
        if (!$modName) {
            $errors[] = 'No module name specified.';
        } else {
            // check if it is still in modules table => means not deinit yet
            $stmt = $pdo->prepare("SELECT 1 FROM modules WHERE module_name = :mn LIMIT 1");
            $stmt->execute([':mn' => $modName]);
            $exists = $stmt->fetchColumn();
            if ($exists) {
                $errors[] = "Cannot remove $modName. You must Deactivate it first (so deinit removes DB entries).";
            } else {
                $modFolder = $modulesDir . '/' . $modName;
                if (is_dir($modFolder)) {
                    rrmdir($modFolder);
                    $success = "Module folder [$modName] deleted successfully.";
                } else {
                    $errors[] = "Module folder [$modName] not found or already removed.";
                }
            }
        }
    }
}

// Prepare data for listing modules
$stmt = $pdo->query("SELECT module_name FROM modules");
$activeNames = $stmt->fetchAll(PDO::FETCH_COLUMN, 0);
$activeSet   = array_flip($activeNames);

$installedModules = [];
if (is_dir($modulesDir)) {
    $dirs = scandir($modulesDir);
    foreach ($dirs as $d) {
        if ($d === '.' || $d === '..') continue;
        $fullPath = $modulesDir . '/' . $d;
        if (is_dir($fullPath)) {
            $installedModules[] = $d;
        }
    }
}
sort($installedModules);
?>
<h1>Module Manager</h1>

<?php if (!empty($errors)): ?>
<div class="alert">
  <?php foreach ($errors as $err): ?>
    <p><?php echo nl2br(htmlspecialchars($err)); ?></p>
  <?php endforeach; ?>
  <p><a href="module_manager.php" class="button">Refresh page to see changes</a></p>
</div>
<?php endif; ?>

<?php if ($success): ?>
<div class="success">
  <p><?php echo nl2br(htmlspecialchars($success)); ?></p>
  <p><a href="module_manager.php" class="button">Refresh page to see changes</a></p>
</div>
<?php endif; ?>

<?php if (canInstall()): ?>
<div class="button-container">
  <form method="POST" enctype="multipart/form-data" style="display:inline;">
    <input type="file" name="module_file" accept=".dmod" required>
    <button type="submit" name="action" value="install" class="button">Install Module</button>
  </form>
</div>
<?php endif; ?>

<table>
  <thead>
    <tr>
      <th>Module Name</th>
      <th>Status</th>
      <th>Actions</th>
    </tr>
  </thead>
  <tbody>
  <?php if (empty($installedModules)): ?>
    <tr><td colspan="3">No modules found.</td></tr>
  <?php else: ?>
    <?php foreach ($installedModules as $modName): ?>
      <?php
        $isActive   = isset($activeSet[$modName]);
        $statusText = $isActive ? 'Activated' : 'Installed';
      ?>
      <tr>
        <td><?php echo htmlspecialchars($modName); ?></td>
        <td><?php echo htmlspecialchars($statusText); ?></td>
        <td>
          <?php if (!$isActive && canActivate()): ?>
            <form action="module_manager.php" method="POST" style="display:inline;">
              <input type="hidden" name="module_name" value="<?php echo htmlspecialchars($modName); ?>">
              <button type="submit" name="action" value="activate" class="button">Activate</button>
            </form>
          <?php endif; ?>

          <?php if ($isActive && canDeactivate()): ?>
            <form action="module_manager.php" method="POST" style="display:inline;">
              <input type="hidden" name="module_name" value="<?php echo htmlspecialchars($modName); ?>">
              <button type="submit" name="action" value="deactivate" class="button delete-button"
                      onclick="return confirm('Are you sure you want to deactivate this module?');">
                Deactivate
              </button>
            </form>
          <?php endif; ?>

          <?php if (!$isActive && canDeleteModule()): ?>
            <form action="module_manager.php" method="POST" style="display:inline;">
              <input type="hidden" name="module_name" value="<?php echo htmlspecialchars($modName); ?>">
              <button type="submit" name="action" value="delete" class="button delete-button"
                      onclick="return confirm('Are you sure you want to DELETE this module folder?');">
                Remove
              </button>
            </form>
          <?php endif; ?>
        </td>
      </tr>
    <?php endforeach; ?>
  <?php endif; ?>
  </tbody>
</table>

<?php require_once('components/footer.php'); ?>
