<?php
// Removed session_start(); as it's already handled in auth.php

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('../config/config.php');
require_once('../core/transactions.php');
require_once('../core/accounts.php');
require_once('../core/financial_periods.php');
require_once('../core/vat_rates.php'); // Ensure we have this so we can load $vatRates
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

$errors = [];
$success = [];

// We need VAT Rates for the "Add Transaction With VAT" modal:
$vatRates = listVATRates($pdo);
$allowedDateRanges = getAllowedTransactionDateRanges($pdo);
/**
 * Build an account hierarchy (tree).
 */
function buildAccountTree(array $accounts, string $parentCode = null): array {
    $branch = [];
    foreach ($accounts as $account) {
        if ($account['parent'] === $parentCode) {
            $children = buildAccountTree($accounts, $account['code']);
            if ($children) {
                $account['children'] = $children;
            }
            $branch[] = $account;
        }
    }
    return $branch;
}
    function isTransactionDateAllowed(string $transactionDate, array $allowedRanges): bool {
        if (empty($transactionDate)) return false;
        $checkTs = strtotime($transactionDate);

        foreach ($allowedRanges as $range) {
            $startTs = strtotime($range['start']);
            $endTs   = strtotime($range['end']);
            if ($checkTs >= $startTs && $checkTs <= $endTs) {
                return true;
            }
        }
        return false;
    }
/**
 * Render account options (recursive)
 */
function renderAccountOptions(array $tree, int $level = 0): void {
    foreach ($tree as $account) {
        $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $level);
        $acctType = strtolower($account['account_type'] ?? '');
        switch ($acctType) {
            case 'asset':     $color = '#1E90FF';  break;
            case 'liability': $color = '#FF4500';  break;
            case 'equity':    $color = '#32CD32';  break;
            case 'revenue':
            case 'income':    $color = '#FFD700';  break;
            case 'expense':   $color = '#FF6347';  break;
            default:          $color = '#000000';  break;
        }
        echo '<option value="' . htmlspecialchars($account['code']) . '" style="color:' . $color . ';">'
             . $indent . htmlspecialchars($account['name']) . ' (' . htmlspecialchars($account['code']) . ')</option>';
        if (isset($account['children'])) {
            renderAccountOptions($account['children'], $level + 1);
        }
    }
}

/**
 * Render with selected value
 */
function renderAccountOptionsWithSelected(array $tree, string $selectedCode = '', int $level = 0): void {
    foreach ($tree as $account) {
        $indent   = str_repeat('&nbsp;&nbsp;&nbsp;', $level);
        $selected = ($account['code'] === $selectedCode) ? 'selected' : '';
        $acctType = strtolower($account['account_type'] ?? '');
        switch ($acctType) {
            case 'asset':     $color = '#1E90FF';  break;
            case 'liability': $color = '#FF4500';  break;
            case 'equity':    $color = '#32CD32';  break;
            case 'revenue':
            case 'income':    $color = '#FFD700';  break;
            case 'expense':   $color = '#FF6347';  break;
            default:          $color = '#000000';  break;
        }
        echo '<option value="' . htmlspecialchars($account['code']) . '" data-account-type="' . $acctType . '" style="color:' . $color . ';" ' . $selected . '>'
             . $indent . htmlspecialchars($account['name']) . ' (' . htmlspecialchars($account['code']) . ')</option>';
        if (isset($account['children'])) {
            renderAccountOptionsWithSelected($account['children'], $selectedCode, $level + 1);
        }
    }
}

/**
 * Render only Asset, Liability, Equity for the "Primary" accounts
 */
function renderPrimaryAccounts(array $tree, string $selectedCode = '', int $level = 0) {
    foreach ($tree as $account) {
        $acctType = strtolower($account['account_type'] ?? '');
        if (in_array($acctType, ['asset','liability','equity'])) {
            $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $level);
            switch ($acctType) {
                case 'asset':     $color = '#1E90FF';  break;
                case 'liability': $color = '#FF4500';  break;
                case 'equity':    $color = '#32CD32';  break;
                default:          $color = '#000000';  break;
            }
            $selected = ($account['code'] === $selectedCode) ? 'selected' : '';
            echo '<option value="' . htmlspecialchars($account['code']) . '" data-account-type="' . $acctType . '" style="color:' . $color . ';" ' . $selected . '>'
                 . $indent . htmlspecialchars($account['name']) . ' (' . htmlspecialchars($account['code']) . ')</option>';
            if (isset($account['children'])) {
                renderPrimaryAccounts($account['children'], $selectedCode, $level + 1);
            }
        } else {
            if (isset($account['children'])) {
                renderPrimaryAccounts($account['children'], $selectedCode, $level + 1);
            }
        }
    }
}

/**
 * Render only Revenue (Income) and Expense for the "Secondary" accounts
 */
function renderSecondaryAccounts(array $tree, string $selectedCode = '', int $level = 0) {
    foreach ($tree as $account) {
        $acctType = strtolower($account['account_type'] ?? '');
        if (in_array($acctType, ['revenue','income','expense'])) {
            $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $level);
            switch ($acctType) {
                case 'revenue':
                case 'income':    $color = '#FFD700'; break;
                case 'expense':   $color = '#FF6347'; break;
                default:          $color = '#000000'; break;
            }
            $selected = ($account['code'] === $selectedCode) ? 'selected' : '';
            echo '<option value="' . htmlspecialchars($account['code']) . '" data-account-type="' . $acctType . '" style="color:' . $color . ';" ' . $selected . '>'
                 . $indent . htmlspecialchars($account['name']) . ' (' . htmlspecialchars($account['code']) . ')</option>';
            if (isset($account['children'])) {
                renderSecondaryAccounts($account['children'], $selectedCode, $level + 1);
            }
        } else {
            if (isset($account['children'])) {
                renderSecondaryAccounts($account['children'], $selectedCode, $level + 1);
            }
        }
    }
}

function validateDate($date, $format = 'Y-m-d'): bool {
    $d = DateTime::createFromFormat($format, $date);
    return $d && $d->format($format) === $date;
}

function restoreTransactionsStateFromSession(): array {
    if (!isset($_SESSION['transactions_state'])) {
        $_SESSION['transactions_state'] = [];
    }
    $merged = $_SESSION['transactions_state'];
    foreach ($_GET as $key => $value) {
        if ($value !== '') {
            $merged[$key] = $value;
        } else {
            unset($merged[$key]);
        }
    }
    $_SESSION['transactions_state'] = $merged;
    return $merged;
}

if (isset($_GET['reset_filters']) && $_GET['reset_filters'] === '1') {
    unset($_SESSION['transactions_state']);
    header('Location: transactions.php');
    exit;
}

$state = restoreTransactionsStateFromSession();
$_GET = $state;

if (!checkRole(['transactions_view'])) {
    $errors[] = 'Access denied. You do not have permission to view transactions.';
    echo '<h1>Transactions</h1>';
    if (!empty($errors)) {
        echo '<div class="alert">';
        foreach ($errors as $error) {
            echo '<p>' . htmlspecialchars($error) . '</p>';
        }
        echo '</div>';
    }
    require_once('components/footer.php');
    exit;
}

// ------------------------------------
// POST Handling
// ------------------------------------
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // 1) Delete Transaction
    if (isset($_POST['delete_transaction_id'])) {
        if (!checkRole(['transactions_delete'])) {
            $errors[] = 'Access denied. You do not have permission to delete transactions.';
        } else {
            $transactionId = (int)$_POST['delete_transaction_id'];
            if ($transactionId > 0) {
                $result = deleteTransaction($pdo, $transactionId, $_SESSION['user_id']);
                if ($result) {
                    $_SESSION['success'] = 'Transaction deleted successfully.';
                    header('Location: transactions.php');
                    exit;
                } else {
                    if (isset($_SESSION['transaction_error']) && !empty($_SESSION['transaction_error'])) {
                        $errors[] = $_SESSION['transaction_error'];
                        unset($_SESSION['transaction_error']);
                    } else {
                        $errors[] = 'Failed to delete transaction.';
                    }
                }
            } else {
                $errors[] = 'Invalid transaction ID.';
            }
        }
    }
    // 2) Add Transaction (No VAT)
    elseif (isset($_POST['add_transaction'])) {
        if (!checkRole(['transactions_add'])) {
            $errors[] = 'Access denied. You do not have permission to add transactions.';
        } else {
            $journalEntryId = 0;
            $debitAccount   = trim($_POST['debit_account']);
            $creditAccount  = trim($_POST['credit_account']);
            $amount         = floatval($_POST['amount']);
            $description    = trim($_POST['transaction_description']);
            $transactionDate= trim($_POST['transaction_date']);
            $userId         = $_SESSION['user_id'] ?? 0;

            if (empty($debitAccount) || empty($creditAccount) || $amount <= 0) {
                $errors[] = 'Debit Account, Credit Account, and Amount are required. Amount must be > 0.';
            }
            if (empty($transactionDate)) {
                $errors[] = 'Transaction Date is required.';
            } elseif (!validateDate($transactionDate)) {
                $errors[] = 'Invalid Transaction Date format.';
            } else {
                $periodId = getFinancialPeriodIdByDate($pdo, $transactionDate);
                if (!$periodId) {
                    $errors[] = 'No financial period found for the selected transaction date.';
                }
            }
            if (empty($errors)) {
                $transactionId = createTransaction(
                    $pdo,
                    $journalEntryId,
                    $debitAccount,
                    $creditAccount,
                    $amount,
                    $description,
                    $transactionDate,
                    $periodId,
                    $userId
                );

                if ($transactionId) {
                    $_SESSION['success'] = 'Transaction added successfully.';
                    header('Location: transactions.php');
                    exit;
                } else {
                    if (isset($_SESSION['transaction_error']) && !empty($_SESSION['transaction_error'])) {
                        $errors[] = $_SESSION['transaction_error'];
                        unset($_SESSION['transaction_error']);
                    } else {
                        $errors[] = 'Failed to add transaction.';
                    }
                }
            }
        }
    }
    // 3) Add Transaction WITH VAT
    elseif (isset($_POST['add_transaction_with_vat'])) {
        if (!checkRole(['transactions_add'])) {
            $errors[] = 'Access denied. You do not have permission to add transactions.';
        } else {
            $userId   = $_SESSION['user_id'] ?? 0;
            $journId  = 0;
            $tDate    = trim($_POST['vat_transaction_date'] ?? '');
            $pAcc     = trim($_POST['vat_primary_account'] ?? '');
            $pPos     = trim($_POST['vat_primary_position'] ?? '');  // 'debit' or 'credit'
            $amtGross = floatval($_POST['TotalAmount'] ?? 0);
            $desc     = trim($_POST['vat_description'] ?? '');
            $sAcc     = trim($_POST['vat_secondary_account'] ?? '');
            $vatRateId= (int)($_POST['vat_rate_id'] ?? 0);

            // New: handle "Reversal" checkbox
            $reversal = isset($_POST['vat_reversal']) ? 1 : 0;

            // Validate
            if (empty($tDate)) {
                $errors[] = 'Transaction date is required.';
            } elseif (!validateDate($tDate)) {
                $errors[] = 'Invalid transaction date format.';
            } else {
                $periodId = getFinancialPeriodIdByDate($pdo, $tDate);
                if (!$periodId) {
                    $errors[] = 'No financial period found for the selected date.';
                }
            }
            if (empty($pAcc) || empty($sAcc)) {
                $errors[] = 'Primary and secondary accounts are required.';
            }
            if (!in_array($pPos, ['debit','credit'])) {
                $errors[] = 'You must specify whether the primary account is Debit or Credit.';
            }
            if ($amtGross <= 0) {
                $errors[] = 'Amount must be greater than zero (total including VAT).';
            }
            if ($vatRateId <= 0) {
                $errors[] = 'Please select a valid VAT Rate.';
            } else {
                $vatRateInfo = getVATRate($pdo, $vatRateId);
                if (!$vatRateInfo) {
                    $errors[] = 'Invalid VAT Rate selected.';
                }
            }

            if (empty($errors)) {
                $vatValue = (float)$vatRateInfo['rate_value'];
                // The total is $amtGross; line 1 is the total
                // The portion for VAT is: $vat = $amtGross * (vatValue/(100+vatValue)) if you want net/vat
                // But we only care about the second line's portion (VAT).
                // We'll just do the second line = "amtGross * (vatValue/(100+vatValue))" or simpler approach:
                // Let's do the simpler approach: vat = amtGross - (net). But net is optional for the lines.
                // We'll define vat as amtGross * vatValue/(100 + vatValue).
                // For clarity, though, line1 is "pAcc vs. sAcc" with the full $amtGross
                // line2 is "VAT" portion

                $vat = 0.0;
                if ($vatValue > 0) {
                    // e.g. if vatValue=15 => total=115, vat=15 => 115*(15/115) = 15
                    $vat = $amtGross * ($vatValue / (100.0 + $vatValue));
                }

                // LINE 1 => always primary vs secondary for the total
                if ($pPos === 'debit') {
                    // DEBIT primary => $amtGross, CREDIT secondary => $amtGross
                    $tidTotal = createTransaction(
                        $pdo,
                        $journId,
                        $pAcc,
                        $sAcc,
                        $amtGross,
                        $desc,
                        $tDate,
                        $periodId,
                        $userId,
                        0,
                        $vatRateId
                    );
                } else {
                    // pPos === 'credit'
                    // DEBIT secondary => $amtGross, CREDIT primary => $amtGross
                    $tidTotal = createTransaction(
                        $pdo,
                        $journId,
                        $sAcc,
                        $pAcc,
                        $amtGross,
                        $desc,
                        $tDate,
                        $periodId,
                        $userId,
                        0,
                        $vatRateId
                    );
                }

                // Next, line2 depends on expense/income + reversal
                $secondaryAccountData = getAccount($pdo, $sAcc);
                $secondaryType        = strtolower($secondaryAccountData['account_type'] ?? '');

                $tidVat = false; // Default
                if ($secondaryType === 'expense') {
                    if ($reversal) {
                        // expense + reversal => VAT is credit => VAT_PAYABLE, secondary is debit
                        $tidVat = createTransaction(
                            $pdo,
                            $journId,
                            $sAcc,                // DEBIT secondary
                            VAT_PAYABLE_ACC_CODE, // CREDIT VAT
                            $vat,
                            $desc . ' (VAT)',
                            $tDate,
                            $periodId,
                            $userId,
                            0,
                            $vatRateId
                        );
                    } else {
                        // expense normal => VAT is debit => VAT_RECEIVABLE, secondary is credit
                        $tidVat = createTransaction(
                            $pdo,
                            $journId,
                            VAT_RECEIVABLE_ACC_CODE, // DEBIT VAT
                            $sAcc,                   // CREDIT secondary
                            $vat,
                            $desc . ' (VAT)',
                            $tDate,
                            $periodId,
                            $userId,
                            0,
                            $vatRateId
                        );
                    }
                } elseif (in_array($secondaryType, ['revenue','income'])) {
                    if ($reversal) {
                        // income + reversal => VAT is debit => VAT_RECEIVABLE, secondary is credit
                        $tidVat = createTransaction(
                            $pdo,
                            $journId,
                            VAT_RECEIVABLE_ACC_CODE,
                            $sAcc,
                            $vat,
                            $desc . ' (VAT)',
                            $tDate,
                            $periodId,
                            $userId,
                            0,
                            $vatRateId
                        );
                    } else {
                        // income normal => VAT is credit => VAT_PAYABLE, secondary is debit
                        $tidVat = createTransaction(
                            $pdo,
                            $journId,
                            $sAcc,                // DEBIT secondary
                            VAT_PAYABLE_ACC_CODE, // CREDIT VAT
                            $vat,
                            $desc . ' (VAT)',
                            $tDate,
                            $periodId,
                            $userId,
                            0,
                            $vatRateId
                        );
                    }
                } else {
                    // If secondary is not expense/income, skip or error
                    $errors[] = 'Secondary account is not expense or income; cannot allocate VAT.';
                }

                if (!$tidTotal || !$tidVat || !empty($errors)) {
                    if (!isset($errors[0])) {
                        $errors[] = $_SESSION['transaction_error'];
                    }
                } else {
                    $_SESSION['success'] = 'Transaction with VAT added successfully.';
                    header('Location: transactions.php');
                    exit;
                }
            }
        }
    }
}

// ------------------------------------
// Pagination, Sorting
// ------------------------------------
$allowedPageSizes = [5, 10, 20, 50, 100];
$pageSize = isset($state['page_size']) && in_array((int)$state['page_size'], $allowedPageSizes)
    ? (int)$state['page_size'] : 20;

if (!in_array($pageSize, $allowedPageSizes)) {
    $pageSize = 20;
}

$page = isset($state['page']) && is_numeric($state['page']) && $state['page'] > 0 ? (int)$state['page'] : 1;
$sortBy    = $state['sort_by']    ?? 'transaction_date';
$sortOrder = (isset($state['sort_order']) && strtolower($state['sort_order']) === 'asc') ? 'ASC' : 'DESC';

$allowedSortColumns = ['id','debit_account','credit_account','amount','description','transaction_date'];
if (!in_array($sortBy, $allowedSortColumns)) {
    $sortBy = 'transaction_date';
}

$filters = [
    'page_size'  => $pageSize,
    'sort_by'    => $sortBy,
    'sort_order' => $sortOrder,
];

$appliedFilters = [];

// Build filter query
$accounts = listAccounts($pdo);
$accountMap = [];
foreach ($accounts as $acc) {
    $accountMap[$acc['code']] = $acc['name'];
}

$accountTree = buildAccountTree($accounts);

function buildQuery($params, $exclude = []) {
    foreach ($exclude as $key) {
        unset($params[$key]);
    }
    return http_build_query($params);
}

// Filter fields
if (isset($state['journal_entry_id']) && $state['journal_entry_id'] !== '') {
    $filters['journal_entry_id'] = $state['journal_entry_id'];
    $appliedFilters[] = 'Journal Entry ID: ' . htmlspecialchars($state['journal_entry_id']);
}
if (isset($state['debit_account']) && $state['debit_account'] !== '') {
    $filters['debit_account'] = $state['debit_account'];
    $appliedFilters[] = 'Debit Account: ' . htmlspecialchars($state['debit_account']);
}
if (isset($state['credit_account']) && $state['credit_account'] !== '') {
    $filters['credit_account'] = $state['credit_account'];
    $appliedFilters[] = 'Credit Account: ' . htmlspecialchars($state['credit_account']);
}
if (isset($state['transaction_date']) && $state['transaction_date'] !== '') {
    $filters['transaction_date'] = $state['transaction_date'];
    $appliedFilters[] = 'Transaction Date: ' . htmlspecialchars($state['transaction_date']);
}
if (isset($state['start_date']) && isset($state['end_date']) && $state['start_date'] !== '' && $state['end_date'] !== '') {
    $filters['start_date'] = $state['start_date'];
    $filters['end_date']   = $state['end_date'];
    $appliedFilters[] = 'Date Range: ' . htmlspecialchars($state['start_date']) . ' to ' . htmlspecialchars($state['end_date']);
}
if (isset($state['description']) && $state['description'] !== '') {
    $filters['description'] = $state['description'];
    $appliedFilters[] = 'Description: ' . htmlspecialchars($state['description']);
}
if (isset($state['amount_min']) && isset($state['amount_max']) && $state['amount_min'] !== '' && $state['amount_max'] !== '') {
    $filters['amount_min'] = $state['amount_min'];
    $filters['amount_max'] = $state['amount_max'];
    $appliedFilters[] = 'Amount: ' . htmlspecialchars($state['amount_min']) . ' to ' . htmlspecialchars($state['amount_max']);
}

// Build WHERE
$sqlWhere   = [];
$sqlParams  = [];
$sqlWhere[] = 'is_deleted = 0';

if (isset($filters['journal_entry_id'])) {
    $sqlWhere[] = 'journal_entry_id = :journal_entry_id';
    $sqlParams[':journal_entry_id'] = $filters['journal_entry_id'];
}
if (isset($filters['debit_account'])) {
    $sqlWhere[] = 'debit_account = :debit_account';
    $sqlParams[':debit_account'] = $filters['debit_account'];
}
if (isset($filters['credit_account'])) {
    $sqlWhere[] = 'credit_account = :credit_account';
    $sqlParams[':credit_account'] = $filters['credit_account'];
}
if (isset($filters['transaction_date'])) {
    $sqlWhere[] = 'transaction_date = :transaction_date';
    $sqlParams[':transaction_date'] = $filters['transaction_date'];
}
if (isset($filters['start_date']) && isset($filters['end_date'])) {
    $sqlWhere[] = 'transaction_date BETWEEN :start_date AND :end_date';
    $sqlParams[':start_date'] = $filters['start_date'];
    $sqlParams[':end_date']   = $filters['end_date'];
}
if (isset($filters['description'])) {
    $sqlWhere[] = 'description LIKE :description';
    $sqlParams[':description'] = '%' . $filters['description'] . '%';
}
if (isset($filters['amount_min']) && isset($filters['amount_max'])) {
    $sqlWhere[] = 'amount BETWEEN :amount_min AND :amount_max';
    $sqlParams[':amount_min'] = $filters['amount_min'];
    $sqlParams[':amount_max'] = $filters['amount_max'];
}

$whereSQL = '';
if (!empty($sqlWhere)) {
    $whereSQL = 'WHERE ' . implode(' AND ', $sqlWhere);
}

$orderSQL = 'ORDER BY ' . $sortBy . ' ' . $sortOrder;
$offset   = ($page - 1) * $pageSize;
$limitSQL = 'LIMIT :limit OFFSET :offset';

// Counting
$countSQL = "SELECT COUNT(*) FROM transactions $whereSQL";
$countStmt= $pdo->prepare($countSQL);
foreach ($sqlParams as $param => $value) {
    $countStmt->bindValue($param, $value);
}
$countStmt->execute();
$totalRecords = $countStmt->fetchColumn();
$totalPages   = ceil($totalRecords / $pageSize);

// Fetch data
$dataSQL = "SELECT * FROM transactions $whereSQL $orderSQL $limitSQL";
$dataStmt= $pdo->prepare($dataSQL);
foreach ($sqlParams as $param => $value) {
    $type = PDO::PARAM_STR;
    if (is_int($value))  $type = PDO::PARAM_INT;
    if (is_bool($value)) $type = PDO::PARAM_BOOL;
    $dataStmt->bindValue($param, $value, $type);
}
$dataStmt->bindValue(':limit', (int)$pageSize, PDO::PARAM_INT);
$dataStmt->bindValue(':offset', (int)$offset, PDO::PARAM_INT);
$dataStmt->execute();
$transactions = $dataStmt->fetchAll(PDO::FETCH_ASSOC);

// Build module info
$moduleIds = [];
foreach ($transactions as $tran) {
    if (!empty($tran['module_id']) && $tran['module_id'] > 0) {
        $moduleIds[] = (int)$tran['module_id'];
    }
}
$moduleIds = array_unique($moduleIds);

$moduleInfo = [];
if (!empty($moduleIds)) {
    $inPlaceholders = implode(',', array_fill(0, count($moduleIds), '?'));
    $stmt = $pdo->prepare("SELECT rowid as id, module_name, redirect_transaction FROM modules WHERE rowid IN ($inPlaceholders)");
    $stmt->execute($moduleIds);
    while ($m = $stmt->fetch(PDO::FETCH_ASSOC)) {
        $moduleInfo[(int)$m['id']] = [
            'module_name'         => $m['module_name'],
            'redirect_transaction'=> $m['redirect_transaction']
        ];
    }
}
?>
<h1>Transactions</h1>
<?php
if (!empty($errors)) {
    echo '<div id="message-container" class="alert sticky-message">';
    foreach ($errors as $error) {
        echo '<p>' . htmlspecialchars($error) . '</p>';
    }
    echo '</div>';
}
if (!empty($_SESSION['success'])) {
    echo '<div id="message-container" class="success sticky-message" >';
    echo '<p>' . htmlspecialchars($_SESSION['success']) . '</p>';
    echo '</div>';
    unset($_SESSION['success']);
}
?>

<script>
var accountMap = <?php
    $jsMap = [];
    foreach ($accountMap as $code => $name) {
        $jsMap[$code] = $name . " ($code)";
    }
    echo json_encode($jsMap, JSON_HEX_TAG|JSON_HEX_AMP|JSON_HEX_APOS|JSON_HEX_QUOT);
?>;
</script>

<div class="button-container">
    <?php if (checkRole(['transactions_add'])): ?>
        <button type="button" class="button button-left" id="add-transaction-button">Add Transaction</button>
        <button type="button" class="button button-left" id="add-transaction-with-vat-button">Add Transaction With VAT</button>
    <?php endif; ?>
    <button type="button" class="button button-right" id="toggle-filters-button">Toggle Filters</button>
</div>

<form class="filter-form" method="GET" action="transactions.php" id="filter-form">
    <div>
        <label for="journal_entry_id">Journal Entry ID:</label>
        <input type="number" id="journal_entry_id" name="journal_entry_id"
               value="<?php echo htmlspecialchars($state['journal_entry_id'] ?? ''); ?>">
    </div>
    <div>
        <label for="debit_account">Debit Account:</label>
        <select id="debit_account" name="debit_account">
            <option value="">-- All --</option>
            <?php
            $selectedDebit = $state['debit_account'] ?? '';
            renderAccountOptionsWithSelected($accountTree, $selectedDebit);
            ?>
        </select>
    </div>
    <div>
        <label for="credit_account">Credit Account:</label>
        <select id="credit_account" name="credit_account">
            <option value="">-- All --</option>
            <?php
            $selectedCredit = $state['credit_account'] ?? '';
            renderAccountOptionsWithSelected($accountTree, $selectedCredit);
            ?>
        </select>
    </div>
    <div>
        <label for="transaction_date">Transaction Date:</label>
        <input type="date" id="transaction_date" name="transaction_date"
               value="<?php echo htmlspecialchars($state['transaction_date'] ?? ''); ?>">
    </div>
    <div>
        <label for="start_date">Start Date:</label>
        <input type="date" id="start_date" name="start_date"
               value="<?php echo htmlspecialchars($state['start_date'] ?? ''); ?>">
    </div>
    <div>
        <label for="end_date">End Date:</label>
        <input type="date" id="end_date" name="end_date"
               value="<?php echo htmlspecialchars($state['end_date'] ?? ''); ?>">
    </div>
    <div>
        <label for="description">Description:</label>
        <input type="text" id="description" name="description"
               value="<?php echo htmlspecialchars($state['description'] ?? ''); ?>">
    </div>
    <div>
        <label for="amount_min">Amount Min:</label>
        <input type="number" step="0.01" id="amount_min" name="amount_min"
               value="<?php echo htmlspecialchars($state['amount_min'] ?? ''); ?>">
    </div>
    <div>
        <label for="amount_max">Amount Max:</label>
        <input type="number" step="0.01" id="amount_max" name="amount_max"
               value="<?php echo htmlspecialchars($state['amount_max'] ?? ''); ?>">
    </div>
    <div style="margin-top: 10px;">
        <button type="submit" class="button">Filter</button>
        <a href="transactions.php?reset_filters=1" class="button" id="reset-filters-button">Reset</a>
    </div>
</form>

<div class="applied-filters">
    <?php
    if (!empty($appliedFilters)) {
        echo 'Applied Filters: ' . implode(', ', $appliedFilters);
    } else {
        echo 'No filters applied.';
    }
    ?>
</div>

<table>
    <thead>
    <tr>
        <?php
        $headers = [
            'id'              => 'ID',
            'debit_account'   => 'Debit Account',
            'credit_account'  => 'Credit Account',
            'amount'          => 'Amount',
            'description'     => 'Description',
            'transaction_date'=> 'Transaction Date'
        ];
        foreach ($headers as $key => $label) {
            $currentSort = ($sortBy === $key) ? 'sort-' . strtolower($sortOrder) : '';
            echo '<th class="' . $currentSort . '">';
            echo '<a href="?' . buildQuery(array_merge($state, [
                'sort_by' => $key,
                'sort_order' => ($sortBy === $key && $sortOrder === 'ASC') ? 'desc' : 'asc',
                'page' => 1
            ])) . '">';
            echo htmlspecialchars($label);
            echo '</a>';
            echo '</th>';
        }
        ?>
        <th>Actions</th>
    </tr>
    </thead>
    <tbody>
    <?php if (!empty($transactions)): ?>
        <?php foreach ($transactions as $transaction): ?>
            <tr>
                <td><?php echo htmlspecialchars($transaction['id'] ?? ''); ?></td>
                <td>
                    <?php
                    $debitCode = $transaction['debit_account'] ?? '';
                    $debitName = $accountMap[$debitCode] ?? $debitCode;
                    echo htmlspecialchars($debitName);
                    ?>
                </td>
                <td>
                    <?php
                    $creditCode = $transaction['credit_account'] ?? '';
                    $creditName = $accountMap[$creditCode] ?? $creditCode;
                    echo htmlspecialchars($creditName);
                    ?>
                </td>
                <td><?php echo htmlspecialchars(number_format($transaction['amount'] ?? 0, 2)); ?></td>
                <td><?php echo htmlspecialchars($transaction['description'] ?? ''); ?></td>
                <td><?php echo htmlspecialchars($transaction['transaction_date'] ?? ''); ?></td>
                <td>
                    <?php
                    // If there's a module, show that button...
                    if (!empty($transaction['module_id']) && isset($moduleInfo[$transaction['module_id']])) {
                        $mName     = htmlspecialchars($moduleInfo[$transaction['module_id']]['module_name']);
                        $mRedirect = htmlspecialchars($moduleInfo[$transaction['module_id']]['redirect_transaction']);
                        echo '<a href="' . $mRedirect .'?id=' . htmlspecialchars($transaction['journal_entry_id'])
                             . '&transaction_id=' . htmlspecialchars($transaction['id']) . '" class="button">' . $mName . '</a>';
                        // Optional: link to Journal Entry if allowed
                        if (checkRole(['journal_entry_view']) && ($transaction['journal_entry_id'] ?? 0) != 0) {
                            echo '<a href="edit_journal_entry.php?id=' . htmlspecialchars($transaction['journal_entry_id'])
                                 . '&transaction_id=' . htmlspecialchars($transaction['id'])
                                 . '" class="button" target="_blank" rel="noopener noreferrer">Journal Entry</a>';
                        }
                    } else {
                        // If no module, standard delete is possible if allowed
                        if (
                            checkRole(['transactions_delete']) &&
                            ($transaction['journal_entry_id'] ?? 0) == 0 &&
                            isTransactionDateAllowed($transaction['transaction_date'], $allowedDateRanges)
                        ) {
                            echo '<form action="transactions.php" method="POST" style="display:inline;">';
                            echo '<input type="hidden" name="delete_transaction_id" value="' . htmlspecialchars($transaction['id']) . '">';
                            echo '<button type="submit" class="button delete-button" onclick="return confirm(\'Are you sure?\');">Delete</button>';
                            echo '</form>';
                        }
                        // If there's a journal_entry_id != 0, link to that if permitted
                        if (checkRole(['journal_entry_view']) && ($transaction['journal_entry_id'] ?? 0) != 0) {
                            echo '<a href="edit_journal_entry.php?id=' . htmlspecialchars($transaction['journal_entry_id'])
                                 . '&transaction_id=' . htmlspecialchars($transaction['id'])
                                 . '" class="button" target="_blank" rel="noopener noreferrer">Journal Entry</a>';
                        }
                    }
                    ?>
                </td>
            </tr>
        <?php endforeach; ?>
    <?php else: ?>
        <tr><td colspan="7">No transactions found.</td></tr>
    <?php endif; ?>
    </tbody>
</table>

<div class="pagination-controls">
    <div class="pagination">
        <?php
        $range = 2;
        $start = max(1, $page - $range);
        $end   = min($totalPages, $page + $range);

        if ($page > 1) {
            echo '<a href="?' . buildQuery(array_merge($state, ['page' => $page - 1])) . '">&laquo; Prev</a>';
        }
        if ($start > 1) {
            echo '<a href="?' . buildQuery(array_merge($state, ['page' => 1])) . '">1</a>';
            if ($start > 2) {
                echo '<span>...</span>';
            }
        }
        for ($i = $start; $i <= $end; $i++) {
            if ($i == $page) {
                echo '<a href="?' . buildQuery(array_merge($state, ['page' => $i])) . '" class="active">' . $i . '</a>';
            } else {
                echo '<a href="?' . buildQuery(array_merge($state, ['page' => $i])) . '">' . $i . '</a>';
            }
        }
        if ($end < $totalPages) {
            if ($end < $totalPages - 1) {
                echo '<span>...</span>';
            }
            echo '<a href="?' . buildQuery(array_merge($state, ['page' => $totalPages])) . '">' . $totalPages . '</a>';
        }
        if ($page < $totalPages) {
            echo '<a href="?' . buildQuery(array_merge($state, ['page' => $page + 1])) . '">Next &raquo;</a>';
        }
        ?>
    </div>
    <form method="GET" action="transactions.php" id="page_size_form">
        <?php
        // Keep all existing parameters except page_size, page
        $currentParams = $state;
        unset($currentParams['page_size'], $currentParams['page']);
        foreach ($currentParams as $key => $val) {
            echo '<input type="hidden" name="' . htmlspecialchars($key) . '" value="' . htmlspecialchars($val) . '">';
        }
        ?>
        <input type="hidden" name="page" value="1">
        <label for="page_size">Show
            <select id="page_size" name="page_size" onchange="document.getElementById('page_size_form').submit()" style="padding: 5px;">
                <?php
                foreach ($allowedPageSizes as $sz) {
                    $selected = ($sz === $pageSize) ? 'selected' : '';
                    echo '<option value="' . $sz . '" ' . $selected . '>' . $sz . '</option>';
                }
                ?>
            </select>
            results per page
        </label>
    </form>
</div>

<?php if (checkRole(['transactions_add'])): ?>
<!-- Existing Add Transaction modal (no VAT) -->
<div id="add-transaction-modal" class="transaction-modal">
    <div class="transaction-modal-content form-container">
        <span class="transaction-close-modal">&times;</span>
        <h2>Add Transaction</h2>

        <label for="no_vat_template" style="width: 100%">Template</label>
        <select id="no_vat_template" name="no_vat_template" style="width: 100%;">
            <option value="">Custom Transaction</option>
            <!-- dynamic JS injection from /templates/transactions.php -->
        </select>

        <form action="transactions.php" method="POST">
            <label for="modal_transaction_date">Transaction Date</label>
            <input type="date" id="modal_transaction_date" name="transaction_date" required
                   value="<?php echo !empty($errors) ? htmlspecialchars($_POST['transaction_date'] ?? '') : ''; ?>">

            <label for="modal_debit_account">Debit Account</label>
            <select id="modal_debit_account" name="debit_account" required>
                <option value="">-- Select Debit Account --</option>
                <?php
                $modalSelectedDebit = (!empty($errors)) ? ($_POST['debit_account'] ?? '') : '';
                renderAccountOptionsWithSelected($accountTree, $modalSelectedDebit);
                ?>
            </select>

            <label for="modal_credit_account">Credit Account</label>
            <select id="modal_credit_account" name="credit_account" required>
                <option value="">-- Select Credit Account --</option>
                <?php
                $modalSelectedCredit = (!empty($errors)) ? ($_POST['credit_account'] ?? '') : '';
                renderAccountOptionsWithSelected($accountTree, $modalSelectedCredit);
                ?>
            </select>

            <?php
            $amountValue = !empty($errors) ? htmlspecialchars($_POST['amount'] ?? '') : '';
            renderInput('number', 'amount', 'Amount', $amountValue, true, ['step' => '0.01', 'min' => '0']);
            $descriptionValue = !empty($errors) ? htmlspecialchars($_POST['transaction_description'] ?? '') : '';
            renderTextarea('transaction_description', 'Description', $descriptionValue);
            ?>

            <div class="button-group">
                <button type="submit" name="add_transaction" class="button">Add Transaction</button>
                <button type="button" class="button" id="cancel-button">Cancel</button>
            </div>
        </form>
    </div>
</div>

<!-- Add Transaction With VAT modal -->
<div id="add-transaction-with-vat-modal" class="transaction-modal">
    <div class="transaction-modal-content form-container">
        <span class="transaction-close-modal vat-modal-close">&times;</span>
        <h2>Add Transaction With VAT</h2>

        <label for="vat_template" style="width: 100%">Template</label>
        <select id="vat_template" name="vat_template" style="width: 100%">
            <option value="">Custom Transaction</option>
            <!-- dynamic JS injection from /templates/transactions.php -->
        </select>

        <form action="transactions.php" method="POST" id="vat-transaction-form">
            <label for="vat_transaction_date">Transaction Date</label>
            <input type="date" id="vat_transaction_date" name="vat_transaction_date" required
                   value="<?php echo !empty($errors) ? htmlspecialchars($_POST['vat_transaction_date'] ?? '') : ''; ?>">

            <label for="vat_primary_account">Primary Account</label>
            <select id="vat_primary_account" name="vat_primary_account" required>
                <option value="">-- Select Primary Account --</option>
                <?php
                $postedPrimary = (!empty($errors)) ? ($_POST['vat_primary_account'] ?? '') : '';
                renderPrimaryAccounts($accountTree, $postedPrimary);
                ?>
            </select>

            <label for="vat_primary_position">Primary Account Position</label>
            <select id="vat_primary_position" name="vat_primary_position" required>
                <option value="">-- Select Debit or Credit --</option>
                <option value="debit"  <?php echo (!empty($_POST['vat_primary_position']) && $_POST['vat_primary_position'] === 'debit') ? 'selected' : ''; ?>>Debit</option>
                <option value="credit" <?php echo (!empty($_POST['vat_primary_position']) && $_POST['vat_primary_position'] === 'credit') ? 'selected' : ''; ?>>Credit</option>
            </select>

            <?php
            $vatAmountValue = !empty($errors) ? htmlspecialchars($_POST['TotalAmount'] ?? '') : '';
            renderInput('number', 'TotalAmount', 'Amount (Total)', $vatAmountValue, true, ['step' => '0.01', 'min' => '0']);

            $vatDescValue = !empty($errors) ? htmlspecialchars($_POST['vat_description'] ?? '') : '';
            renderTextarea('vat_description', 'Description', $vatDescValue);
            ?>

            <!-- ADD Reversal checkbox here -->
            <label for="vat_reversal" style="display:inline;">
                <input type="checkbox" id="vat_reversal" name="vat_reversal" value="1"
                <?php
                // If there were validation errors, repopulate
                echo (!empty($_POST['vat_reversal'])) ? 'checked' : '';
                ?>
                />
                Reversal
            </label>

            <br><br>

            <label for="vat_secondary_account">Secondary Account</label>
            <select id="vat_secondary_account" name="vat_secondary_account" required>
                <option value="">-- Select Secondary Account --</option>
                <?php
                $postedSecondary = (!empty($errors)) ? ($_POST['vat_secondary_account'] ?? '') : '';
                renderSecondaryAccounts($accountTree, $postedSecondary);
                ?>
            </select>

            <label for="vat_rate_id">VAT Rate</label>
            <select id="vat_rate_id" name="vat_rate_id" required>
                <option value="">-- Select VAT Rate --</option>
                <?php
                if (!empty($vatRates)) {
                    foreach ($vatRates as $vr) {
                        $rId   = (int)$vr['id'];
                        $rName = htmlspecialchars($vr['rate_name']);
                        $rVal  = htmlspecialchars($vr['rate_value']);
                        $sel   = (!empty($_POST['vat_rate_id']) && (int)$_POST['vat_rate_id'] === $rId) ? 'selected' : '';
                        echo "<option value=\"$rId\" data-rate-value=\"$rVal\" $sel>$rName ({$rVal}%)</option>";
                    }
                }
                ?>
            </select>

            <!-- The preview area -->
            <div id="vat-example-preview" style="background-color:#fafafa;padding:10px;margin-top:10px;border:1px solid #ccc;">
                <strong>Example of the two transactions to be added:</strong>
                <ul style="list-style-type:disc;padding-left:20px;">
                    <li id="vat-net-line">Line 1: TBD</li>
                    <li id="vat-vat-line">Line 2: TBD</li>
                </ul>
            </div>

            <div class="button-group">
                <button type="submit" name="add_transaction_with_vat" class="button">Add Transaction With VAT</button>
                <button type="button" class="button" id="cancel-vat-button">Cancel</button>
            </div>
        </form>
    </div>
</div>
<?php endif; ?>

<script>
document.addEventListener('DOMContentLoaded', function() {
    var modal = document.getElementById('add-transaction-modal');
    var addBtn = document.getElementById('add-transaction-button');
    var toggleFiltersBtn = document.getElementById('toggle-filters-button');
    var filterForm = document.getElementById('filter-form');
    var span = document.getElementsByClassName('transaction-close-modal')[0];
    var cancelBtn = document.getElementById('cancel-button');

    var allowedRanges = <?php echo json_encode($allowedDateRanges); ?>;
    function isDateAllowed(dateStr) {
        var test = new Date(dateStr + 'T00:00:00');
        for(var i=0; i<allowedRanges.length; i++){
            var rStart = new Date(allowedRanges[i].start + 'T00:00:00');
            var rEnd   = new Date(allowedRanges[i].end   + 'T00:00:00');
            if(test >= rStart && test <= rEnd){
                return true;
            }
        }
        return false;
    }
    var dateInputs = [
        document.getElementById('modal_transaction_date'),
        document.getElementById('vat_transaction_date')
    ].filter(Boolean); // remove null if an element doesn't exist

    dateInputs.forEach(function(inp){
        inp.addEventListener('change', function(){
            var val = inp.value;
            if(val && !isDateAllowed(val)){
                alert('That date is not allowed because the financial period or VAT period is closed.');
                inp.value = ''; // or revert to old value
            }
        });
    });

    const messageContainer = document.getElementById('message-container');
    if (messageContainer) {
        setTimeout(() => {
            messageContainer.style.display = 'none';
        }, 7000);
        messageContainer.addEventListener('click', () => {
            messageContainer.style.display = 'none';
        });
    }

    function setFilterFormDisplay(isVisible) {
        filterForm.style.display = isVisible ? 'block' : 'none';
        sessionStorage.setItem('filterFormVisible', isVisible ? 'true' : 'false');
    }
    function areFiltersApplied() {
        var fields = [
            'journal_entry_id','debit_account','credit_account','transaction_date',
            'start_date','end_date','description','amount_min','amount_max'
        ];
        for (var i=0; i<fields.length; i++){
            var f = document.getElementById(fields[i]);
            if (f) {
                if ((f.type==='select-one'||f.type==='select-multiple') && f.value!=='') return true;
                else if ((f.value||'').trim()!=='') return true;
            }
        }
        return false;
    }
    var savedState = sessionStorage.getItem('filterFormVisible');
    if (savedState===null) {
        if (areFiltersApplied()) {
            setFilterFormDisplay(true);
        } else {
            setFilterFormDisplay(false);
        }
    } else {
        setFilterFormDisplay(savedState==='true');
    }
    if (toggleFiltersBtn && filterForm) {
        toggleFiltersBtn.addEventListener('click', function() {
            var currentlyVisible = (filterForm.style.display !== 'none' && filterForm.style.display !== '');
            setFilterFormDisplay(!currentlyVisible);
        });
    }

    if(addBtn){
        addBtn.addEventListener('click', function() {
            modal.style.display = 'block';
        });
    }
    if(span){
        span.addEventListener('click', function() {
            modal.style.display = 'none';
        });
    }
    if(cancelBtn){
        cancelBtn.addEventListener('click', function() {
            modal.style.display = 'none';
        });
    }
    window.addEventListener('click', function(e) {
        if (e.target === modal) {
            modal.style.display = 'none';
        }
    });

    if (sessionStorage.getItem('transactionsScrollPosition')) {
        window.scrollTo(0, parseInt(sessionStorage.getItem('transactionsScrollPosition'), 10));
    }
    window.addEventListener('scroll', function() {
        sessionStorage.setItem('transactionsScrollPosition', window.scrollY);
    });

    // For the VAT modal
    var vatModal          = document.getElementById('add-transaction-with-vat-modal');
    var addTransactionVat = document.getElementById('add-transaction-with-vat-button');
    var vatModalCloseSpan = document.querySelector('.vat-modal-close');
    var cancelVatBtn      = document.getElementById('cancel-vat-button');

    if(addTransactionVat && vatModal) {
        addTransactionVat.addEventListener('click', function() {
            vatModal.style.display = 'block';
        });
    }
    if(vatModalCloseSpan) {
        vatModalCloseSpan.addEventListener('click', function() {
            vatModal.style.display = 'none';
        });
    }
    if(cancelVatBtn) {
        cancelVatBtn.addEventListener('click', function() {
            vatModal.style.display = 'none';
        });
    }
    window.addEventListener('click', function(e) {
        if(e.target === vatModal) {
            vatModal.style.display = 'none';
        }
    });

    // The preview lines for WITH VAT
    var totalAmountInput       = document.getElementById('TotalAmount');
    var vatRateSelect          = document.getElementById('vat_rate_id');
    var primaryPositionSelect  = document.getElementById('vat_primary_position');
    var primaryAccountSelect   = document.getElementById('vat_primary_account');
    var secondaryAccountSelect = document.getElementById('vat_secondary_account');
    var reversalCheckbox       = document.getElementById('vat_reversal');

    var netLineElem            = document.getElementById('vat-net-line');
    var vatLineElem            = document.getElementById('vat-vat-line');

    function updateVatPreview() {
        if (!totalAmountInput || !vatRateSelect) return;
        var total = parseFloat(totalAmountInput.value)||0;
        var rateId= vatRateSelect.value;
        var isReversal = reversalCheckbox && reversalCheckbox.checked;
        if(!rateId){
            netLineElem.innerHTML= "Line 1: (Select a VAT rate.)";
            vatLineElem.innerHTML= "Line 2: (Select a VAT rate.)";
            return;
        }
        var selectedOption= vatRateSelect.options[vatRateSelect.selectedIndex];
        var rateString = selectedOption.getAttribute('data-rate-value') || '0';
        var vatVal= parseFloat(rateString);

        var pCode= primaryAccountSelect.value||'';
        var sCode= secondaryAccountSelect.value||'';
        var pName= (accountMap[pCode] || "Primary");
        var sName= (accountMap[sCode] || "Secondary");

        var positionStr= primaryPositionSelect ? primaryPositionSelect.value : 'debit';
        // We'll pick 'VAT_PAYABLE' if the line2 is a credit, or 'VAT_RECEIVABLE' if line2 is a debit
        // But the direction is decided by expense/income + reversal. We'll just show "VAT_PAYABLE" / "VAT_RECEIVABLE" in preview:
        // For demonstration let's check if it's expense or income:
        var sElem = secondaryAccountSelect.querySelector('option[value="'+sCode+'"]');
        var sAcctType = sElem ? sElem.getAttribute('data-account-type')||'' : '';
        // We won't get super fancy here; we'll show the correct line 1 => total, line 2 => vat portion

        // line1 => always total
        if(positionStr==='debit'){
            netLineElem.innerHTML =
              "Line 1: DEBIT " + pName + " " + total.toFixed(2) +
              " | CREDIT " + sName + " " + total.toFixed(2) + " (TOTAL)";
        } else {
            netLineElem.innerHTML =
              "Line 1: DEBIT " + sName + " " + total.toFixed(2) +
              " | CREDIT " + pName + " " + total.toFixed(2) + " (TOTAL)";
        }

        // let's compute vat portion
        var vat = 0.0;
        if(vatVal>0){
            vat = total * (vatVal / (100.0 + vatVal));
        }
        // Now we see if it's reversed or not, plus expense or income
        // We'll pick 'VAT_PAYABLE_ACC_CODE' or 'VAT_RECEIVABLE_ACC_CODE' for display
        var vatAccountName = (isReversal ? "VAT_PAYABLE" : "VAT_RECEIVABLE"); // default for "expense" scenario
        if (sAcctType==='income' || sAcctType==='revenue') {
            // normal => vat is credit => "VAT_PAYABLE"
            // reversal => vat is debit => "VAT_RECEIVABLE"
            if(!isReversal){
                vatAccountName = "VAT_PAYABLE";
            } else {
                vatAccountName = "VAT_RECEIVABLE";
            }
        } else if(sAcctType==='expense'){
            // normal => vat is debit => "VAT_RECEIVABLE"
            // reversal => vat is credit => "VAT_PAYABLE"
            if(!isReversal){
                vatAccountName = "VAT_RECEIVABLE";
            } else {
                vatAccountName = "VAT_PAYABLE";
            }
        }

        // If isReversal => line2 is DEBIT secondary, CREDIT vat
        // else => line2 is DEBIT vat, CREDIT secondary
        if (isReversal) {
            vatLineElem.innerHTML =
              "Line 2: DEBIT " + sName + " " + vat.toFixed(2) +
              " | CREDIT " + vatAccountName + " " + vat.toFixed(2) + " (VAT)";
        } else {
            vatLineElem.innerHTML =
              "Line 2: DEBIT " + vatAccountName + " " + vat.toFixed(2) +
              " | CREDIT " + sName + " " + vat.toFixed(2) + " (VAT)";
        }
    }

    const noVatSelect = document.getElementById('no_vat_template');
    if (noVatSelect && window.TransactionsTemplates && window.TransactionsTemplates.noVAT) {
        while (noVatSelect.options.length > 1) {
            noVatSelect.remove(1);
        }
        (window.TransactionsTemplates.noVAT.templates || []).forEach(t => {
            const opt = document.createElement('option');
            opt.value = t.value;
            opt.textContent = t.label;
            noVatSelect.appendChild(opt);
        });
    }

    const vatSelect = document.getElementById('vat_template');
    if (vatSelect && window.TransactionsTemplates && window.TransactionsTemplates.withVAT) {
        while (vatSelect.options.length > 1) {
            vatSelect.remove(1);
        }
        (window.TransactionsTemplates.withVAT.templates || []).forEach(t => {
            const opt = document.createElement('option');
            opt.value = t.value;
            opt.textContent = t.label;
            vatSelect.appendChild(opt);
        });
    }

    if (noVatSelect) {
        noVatSelect.addEventListener('change', function() {
            if (typeof TransactionsTemplates !== 'undefined'
                && typeof TransactionsTemplates.noVAT !== 'undefined') {
                TransactionsTemplates.noVAT.applyTemplate(
                    noVatSelect.value,
                    document.getElementById('modal_debit_account'),
                    document.getElementById('modal_credit_account')
                );
            }
        });
    }

    if (vatSelect) {
        vatSelect.addEventListener('change', function(){
            if (typeof TransactionsTemplates !== 'undefined'
                && typeof TransactionsTemplates.withVAT !== 'undefined') {
                TransactionsTemplates.withVAT.applyTemplate(
                    vatSelect.value,
                    document.getElementById('vat_primary_account'),
                    document.getElementById('vat_primary_position'),
                    document.getElementById('vat_secondary_account')
                );
            }
            updateVatPreview();
        });
    }

    totalAmountInput?.addEventListener('input', updateVatPreview);
    vatRateSelect?.addEventListener('change', updateVatPreview);
    primaryPositionSelect?.addEventListener('change', updateVatPreview);
    primaryAccountSelect?.addEventListener('change', updateVatPreview);
    secondaryAccountSelect?.addEventListener('change', updateVatPreview);
    reversalCheckbox?.addEventListener('change', updateVatPreview);

    updateVatPreview();
});
</script>

<style>
#message-container.sticky-message {
    position: fixed;
    top: 0;
    left:0;
    z-index: 999;
    cursor: pointer;
    width:100%;
}
th, td {
    border: 1px solid #ddd;
    padding: 8px;
    vertical-align: middle;
}
th {
    cursor: pointer;
    background-color: #f2f2f2;
    position: relative;
    color: #000;
}
th a {
    text-decoration: none;
    color: inherit;
    display: block;
}
th.sort-asc::after {
    content: " ▲";
    position: absolute;
    top: 10px;
    right: 8px;
}
th.sort-desc::after {
    content: " ▼";
    position: absolute;
    top: 10px;
    right: 8px;
}
.alert {
    position: fixed;
    background-color: #f8d7da;
    color: #721c24;
    padding: 10px;
    margin-bottom: 15px;
    border-radius: 4px;
}
.success {
    background-color: #d4edda;
    color: #155724;
    padding: 10px;
    margin-bottom: 15px;
    border-radius: 4px;
}
.filter-form {
    margin-bottom: 20px;
    display: none;
    padding: 15px;
    border: 1px solid #ccc;
    border-radius: 4px;
    background-color: #fafafa;
}
.filter-form div {
    margin-bottom: 10px;
}
.filter-form label {
    margin-right: 10px;
    display: inline-block;
    width: 150px;
    font-weight: bold;
}
.filter-form input, .filter-form select {
    margin-bottom: 10px;
    padding: 5px;
    width: 200px;
    display: inline-block;
}
.pagination-controls {
    display: flex;
    justify-content: space-between;
    align-items: center;
    margin-top: 20px;
}
.pagination a, .pagination span {
    margin: 0 5px;
    text-decoration: none;
    padding: 5px 10px;
    border: 1px solid #ddd;
    border-radius: 4px;
    color: #007bff;
}
.pagination a.active {
    background-color: #007bff;
    color: white;
    border-color: #007bff;
}
.pagination span {
    padding: 5px 10px;
    color: #555;
}
.button-left {
    margin-right: 10px;
}
.button-right {
    margin-left: 10px;
    float: right !important;
}
.button:hover {
    background-color: #0056b3;
}
.transaction-modal {
    display: none;
    position: fixed;
    z-index: 1000;
    left: 0;
    top: 0;
    width: 100%;
    height: 100%;
    overflow: auto;
    background-color: rgba(0,0,0,0.4);
}
.transaction-modal-content {
    background-color: #fefefe;
    margin: 2% auto;
    padding: 20px;
    border: 1px solid #888;
    width: 50%;
    border-radius: 4px;
    position: relative;
}
.transaction-close-modal:hover,
.transaction-close-modal:focus {
    color: black;
    text-decoration: none;
    cursor: pointer;
}
.applied-filters {
    margin-top: 10px;
    padding: 10px;
    background-color: #e2e3e5;
    border: 1px solid #d6d8db;
    border-radius: 4px;
    color: #383d41;
}
.delete-button {
    background-color: #c0392b;
}
.delete-button:hover {
    background-color: #a93226;
}
</style>
<script src="/templates/transactions.php"></script>
<?php require_once('components/footer.php'); ?>
