<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('../core/financial_reports.php');
require_once('../core/financial_periods.php');
require_once('components/auth.php');


// 1) Place your "list_tb_layouts" route at the top, with exit:
if (isset($_GET['action']) && $_GET['action'] === 'list_tb_layouts') {
    header('Content-Type: application/json; charset=utf-8');

    $layoutsDir = __DIR__ . '/documents';
    // Make sure findTrialBalanceLayouts is defined above or in another included file:
    $tbLayouts = findTrialBalanceLayouts($layoutsDir);

    if (count($tbLayouts) === 0) {
        echo json_encode(['success' => false, 'message' => 'No TB layouts']);
        exit;
    }
    echo json_encode(['success' => true, 'layouts' => $tbLayouts]);
    exit;
}

$errors = [];

require_once('components/header.php');
require_once('components/form_component.php');

// **Role Check: Ensure the user has 'trial_balance' role**
if (!checkRole(['trial_balance'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to view this page.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Trial Balance</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

/**
 * Builds a tree structure from a flat list of accounts.
 *
 * @param array $accounts The flat list of accounts.
 * @return array The hierarchical tree of accounts.
 */
function buildAccountTree(array $accounts): array
{
    $tree = [];
    $references = [];

    // Initialize references
    foreach ($accounts as &$account) {
        $account['children'] = [];
        $references[$account['code']] = &$account;
    }
    unset($account);

    // Build the tree
    foreach ($accounts as &$account) {
        if ($account['parent'] && isset($references[$account['parent']])) {
            $references[$account['parent']]['children'][] = &$account;
        } else {
            $tree[] = &$account;
        }
    }
    unset($account);

    return $tree;
}

/**
 * Renders the account tree into table rows with indentation.
 *
 * @param array $tree The hierarchical tree of accounts.
 * @param int $depth The current depth level.
 */
function renderAccountRows(array $tree, $depth = 0)
{
    foreach ($tree as $account) {
        // Determine row class based on account_type for color coding
        $rowClass = '';
        switch (strtoupper($account['account_type'])) {
            case 'ASSET':
                $rowClass = 'account-asset';
                break;
            case 'LIABILITY':
                $rowClass = 'account-liability';
                break;
            case 'EQUITY':
                $rowClass = 'account-equity';
                break;
            case 'REVENUE':
            case 'INCOME':
                $rowClass = 'account-revenue';
                break;
            case 'EXPENSE':
                $rowClass = 'account-expense';
                break;
            default:
                $rowClass = '';
        }

        echo '<tr class="' . htmlspecialchars($rowClass) . '">';

        // Display Account Type
        echo '<td>' . htmlspecialchars(ucfirst(strtolower($account['account_type']))) . '</td>';

        // Display Code
        echo '<td>' . htmlspecialchars($account['code']) . '</td>';

        // Display Name with Hierarchy (Indentation)
        $indent = str_repeat('&nbsp;&nbsp;&nbsp;', $depth);
        echo '<td>' . $indent . htmlspecialchars($account['name']) . '</td>';

        // Display Total Debit
        echo '<td>' . number_format($account['total_debit'], 2) . '</td>';

        // Display Total Credit
        echo '<td>' . number_format($account['total_credit'], 2) . '</td>';

        echo '</tr>';

        // Recursively render child accounts
        if (!empty($account['children'])) {
            renderAccountRows($account['children'], $depth + 1);
        }
    }
}

// NEW CODE: function to find all .layout with "type":"trialbalance"
function findTrialBalanceLayouts($dir) {
    $results = [];
    if (!is_dir($dir)) {
        return $results;
    }
    $files = scandir($dir);
    foreach ($files as $f) {
        if (strtolower(pathinfo($f, PATHINFO_EXTENSION)) === 'layout') {
            $fullPath = $dir . '/' . $f;
            $json = @json_decode(file_get_contents($fullPath), true);
            if (is_array($json) && isset($json['type']) && $json['type'] === 'trialbalance') {
                $results[] = $f;
            }
        }
    }
    return $results;
}

// Existing code to process form submission for normal "Generate" (HTML) Trial Balance
$financialPeriods = listFinancialPeriods($pdo);
$selectedPeriodId = '';
$startDate = '';
$endDate = '';
$trialBalance = [];

if ($_SERVER['REQUEST_METHOD'] === 'POST' && !isset($_GET['action'])) {
    // Determine if a Financial Period is selected
    if (isset($_POST['financial_period_id']) && !empty($_POST['financial_period_id'])) {
        $selectedPeriodId = (int)$_POST['financial_period_id'];
        $period = getFinancialPeriod($pdo, $selectedPeriodId);
        if ($period) {
            $startDate = $period['start_date'];
            $endDate = $period['end_date'];
        } else {
            $errors[] = 'Selected financial period not found.';
        }
    } else {
        // Handle custom date range
        $startDate = trim($_POST['start_date']);
        $endDate = trim($_POST['end_date']);
        if (empty($startDate) || empty($endDate)) {
            $errors[] = 'Please provide both start date and end date.';
        } elseif ($startDate > $endDate) {
            $errors[] = 'Start date cannot be after end date.';
        }
    }

    // If no errors, calculate the Trial Balance for HTML display
    if (empty($errors)) {
        $trialBalance = calculateTrialBalanceForDateRange($pdo, $startDate, $endDate);
        if (empty($trialBalance)) {
            $errors[] = 'No transactions found for the selected period.';
        }
    }
}

?>
<!DOCTYPE html>
<html>
<head>
    <title>Trial Balance</title>
    <link rel="stylesheet" href="css/style.css">
</head>
<body>
<h1>Trial Balance</h1>

<!-- NEW CODE: Hidden error div for no layout found -->
<div id="tb-no-layouts-error" class="alert" style="display:none;">
  <p>No saved layout data found for trialbalance.
     <a href="document_editor.php">Create layout here</a>.
  </p>
</div>
<!-- /NEW CODE -->

<div class="form-container">
    <form action="trialbalance.php" method="POST">
        <h2>Select Financial Period or Date Range</h2>
        <label for="financial_period_id">Financial Period:</label>
        <select id="financial_period_id" name="financial_period_id">
            <option value="">-- Custom Date Range --</option>
            <?php foreach ($financialPeriods as $period): ?>
                <option value="<?php echo htmlspecialchars($period['id']); ?>" <?php echo ($selectedPeriodId == $period['id']) ? 'selected' : ''; ?>>
                    <?php echo htmlspecialchars($period['name'] . ' (' . $period['start_date'] . ' to ' . $period['end_date'] . ')'); ?>
                </option>
            <?php endforeach; ?>
        </select>

        <p>Or enter a custom date range:</p>
        <label for="start_date">Start Date:</label>
        <input type="date" id="start_date" name="start_date" value="<?php echo htmlspecialchars($startDate); ?>">

        <label for="end_date">End Date:</label>
        <input type="date" id="end_date" name="end_date" value="<?php echo htmlspecialchars($endDate); ?>">

        <div class="button-group" style="margin-top:10px;">
            <button type="submit">Generate Trial Balance</button>

            <!-- NEW CODE: the PDF button is now type="button" -->
            <button type="button"
                    name="generate_pdf"
                    id="generate_pdf_button"
                    style="background-color: orange; color: #fff; border: none; padding: 8px 12px; cursor: pointer;">
                Generate Trial Balance PDF
            </button>
        </div>
    </form>
</div>

<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>

<?php if (!empty($trialBalance)): ?>
    <h2>Trial Balance Report from <?php echo htmlspecialchars($startDate); ?> to <?php echo htmlspecialchars($endDate); ?></h2>
    <?php
    $columns = [
        ['header' => 'Account Type', 'field' => 'account_type'],
        ['header' => 'Code', 'field' => 'code'],
        ['header' => 'Name', 'field' => 'name'],
        ['header' => 'Total Debit', 'field' => 'total_debit'],
        ['header' => 'Total Credit', 'field' => 'total_credit']
    ];
    $accountTree = buildAccountTree($trialBalance);
    $totalDebit = 0;
    $totalCredit = 0;
    foreach ($trialBalance as $acc) {
        $totalDebit  += $acc['total_debit'];
        $totalCredit += $acc['total_credit'];
    }
    ?>
    <table>
        <thead>
            <tr>
            <?php foreach ($columns as $col): ?>
                <th><?php echo htmlspecialchars($col['header']); ?></th>
            <?php endforeach; ?>
            </tr>
        </thead>
        <tbody>
            <?php renderAccountRows($accountTree); ?>
        </tbody>
        <tfoot>
            <tr class="subtotal">
                <td colspan="3">Total</td>
                <td><?php echo number_format($totalDebit, 2); ?></td>
                <td><?php echo number_format($totalCredit, 2); ?></td>
            </tr>
        </tfoot>
    </table>
<?php endif; ?>

<!-- NEW CODE: The hidden modal for multiple layouts -->
<div id="tb-modal-overlay" style="
    position: fixed;
    top:0; left:0;
    width:100%; height:100%;
    background: rgba(0,0,0,0.5);
    display: none;
    align-items: center;
    justify-content: center;
    z-index: 9999;">
  <div id="tb-modal-content" style="
      background: #fff;
      padding: 20px;
      border-radius: 4px;
      min-width: 300px;">
    <h2>Select a Trial Balance Layout</h2>
    <div id="tb-modal-error" style="color: red; font-weight: bold; display:none;"></div>
    <label for="tb-modal-select">Layouts:</label>
    <select id="tb-modal-select" style="width:100%; margin-bottom:15px;"></select>
    <div style="text-align:right;">
      <button id="tb-modal-cancel" style="margin-right:10px;">Cancel</button>
      <button id="tb-modal-ok" style="background-color: orange; color:#fff; border:none; padding:5px 12px; cursor:pointer;">OK</button>
    </div>
  </div>
</div>
<!-- /NEW CODE -->

<script>
document.addEventListener('DOMContentLoaded', function(){

  const pdfBtn = document.getElementById('generate_pdf_button');
  if(pdfBtn){
    pdfBtn.addEventListener('click', function(e){
      e.preventDefault();

      // 1) Validate date fields (like the normal HTML form)
      const periodSel = document.getElementById('financial_period_id');
      const sd = document.getElementById('start_date').value.trim();
      const ed = document.getElementById('end_date').value.trim();
      if(!periodSel.value && (!sd || !ed)){
        alert("Please provide both start date and end date, or select a financial period.");
        return;
      }
      if(sd && ed && sd > ed){
        alert("Start date cannot be after end date.");
        return;
      }

      // 2) Ajax fetch layout info
      fetch('trialbalance.php?action=list_tb_layouts')
        .then(r=>r.json())
        .then(resp=>{
          if(!resp.success){
            // Show the same style error
            const noLayoutDiv = document.getElementById('tb-no-layouts-error');
            if(noLayoutDiv){
              noLayoutDiv.style.display = 'block';
            }
            return;
          }
          const layouts = resp.layouts || [];
          if(layouts.length===0){
            const noLayoutDiv = document.getElementById('tb-no-layouts-error');
            if(noLayoutDiv){
              noLayoutDiv.style.display = 'block';
            }
            return;
          }
          if(layouts.length===1){
            openPdfInNewTab(layouts[0]);
          } else {
            showLayoutModal(layouts);
          }
        })
        .catch(err=>{
          console.error("Error listing TB layouts:", err);
          alert("Failed to list TB layouts. See console for details.");
        });
    });
  }

  // The updated function that fetches the layout from disk:
  function openPdfInNewTab(layoutName){
    // gather date/period from the page's form
    const pSel = document.getElementById('financial_period_id').value.trim();
    const sd = document.getElementById('start_date').value.trim();
    const ed = document.getElementById('end_date').value.trim();

    // 1) Fetch the actual layout JSON from disk
    fetch('documents/' + layoutName)
      .then(r => r.json())
      .then(fullLayout => {
        // 2) Add or override dialogData with user date info
        fullLayout.dialogData = {
          financial_period_id: pSel,
          start_date: sd,
          end_date: ed
        };

        // 3) Now post that entire layout to document_display_pdf.php
        const f = document.createElement('form');
        f.method = 'POST';
        f.action = 'document_display_pdf.php';
        f.target = '_blank';

        const inp = document.createElement('input');
        inp.type = 'hidden';
        inp.name = 'layoutJSON';
        inp.value = JSON.stringify(fullLayout);
        f.appendChild(inp);

        document.body.appendChild(f);
        f.submit();

        // Optional: If you don't want a reload, comment out:
        // setTimeout(() => {
        //   window.location.reload();
        // }, 500);
      })
      .catch(err => {
        console.error("Cannot load layout file:", err);
        alert("Error: Failed to load layout JSON for " + layoutName);
      });
  }

  function showLayoutModal(listOfLayouts){
    const overlay = document.getElementById('tb-modal-overlay');
    const select = document.getElementById('tb-modal-select');
    const cancelBtn = document.getElementById('tb-modal-cancel');
    const okBtn = document.getElementById('tb-modal-ok');
    const errDiv = document.getElementById('tb-modal-error');

    // Clear old
    select.innerHTML = '';
    errDiv.style.display='none';
    errDiv.textContent='';

    listOfLayouts.forEach(lay=>{
      const opt=document.createElement('option');
      opt.value=lay;
      opt.textContent=lay;
      select.appendChild(opt);
    });

    overlay.style.display='flex';

    cancelBtn.onclick=function(){
      overlay.style.display='none';
    };
    okBtn.onclick=function(){
      const chosen = select.value;
      if(!chosen){
        errDiv.style.display='block';
        errDiv.textContent="Please select a layout.";
        return;
      }
      overlay.style.display='none';
      openPdfInNewTab(chosen);
    };
  }
});
</script>

<?php require_once('components/footer.php'); ?>
</body>
</html>
