<?php
require_once('../core/vat_rates.php');
require_once('components/auth.php');
require_once('components/header.php');

$errors = [];
$success = '';

// Check if the user has the required role
if (!checkRole(['vat_rates_update'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to update VAT rates.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Update VAT Rate</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Retrieve the user ID from the session
$userId = $_SESSION['user_id'] ?? 0;

// Validate user ID
if (!is_numeric($userId) || (int)$userId <= 0) {
    $errors[] = 'Invalid user session. Please log in again.';
    ?>
    <h1>Update VAT Rate</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Validate and retrieve VAT Rate ID from GET parameters
if (!isset($_GET['id']) || empty(trim($_GET['id']))) {
    $_SESSION['vat_rate_error'] = "Invalid VAT Rate ID.";
    header('Location: vat_rates.php');
    exit;
}

$vatRateId = trim($_GET['id']);

// Additional validation to ensure $vatRateId is a positive integer
if (!ctype_digit($vatRateId) || (int)$vatRateId <= 0) {
    $_SESSION['vat_rate_error'] = "Invalid VAT Rate ID.";
    header('Location: vat_rates.php');
    exit;
}

$vatRateId = (int)$vatRateId;

$vatRate = getVATRate($pdo, $vatRateId);
if (!$vatRate) {
    $_SESSION['vat_rate_error'] = "VAT Rate not found.";
    header('Location: vat_rates.php');
    exit;
}

// Initialize form variables with existing VAT rate data
$rateName = $vatRate['rate_name'];
$rateValue = $vatRate['rate_value'];
$isDefault = $vatRate['is_default'];

// Handle form submission
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // Retrieve and sanitize POST data
    $rateName = trim($_POST['rate_name']);
    $rateValue = trim($_POST['rate_value']);
    $isDefaultInput = isset($_POST['is_default']) ? 1 : 0;

    // Validate form inputs
    if (empty($rateName)) {
        $errors[] = 'Rate Name is required.';
    }

    if ($rateValue === '') {
        $errors[] = 'Rate Value is required.';
    } elseif (!is_numeric($rateValue)) {
        $errors[] = 'Rate Value must be a numeric value.';
    } elseif ($rateValue < 0) {
        $errors[] = 'Rate Value cannot be negative.';
    }

    if (empty($errors)) {
        // Call the updateVATRate function with the required parameters
        $result = updateVATRate($pdo, $vatRateId, $rateName, $rateValue, (int)$userId, $isDefaultInput);
        if ($result) {
            $_SESSION['success_message'] = 'VAT Rate updated successfully.';
            header('Location: vat_rates.php');
            exit;
        } else {
            // Retrieve and display any error messages from the session
            if (isset($_SESSION['vat_rate_error']) && !empty($_SESSION['vat_rate_error'])) {
                $errors[] = $_SESSION['vat_rate_error'];
                unset($_SESSION['vat_rate_error']);
            } else {
                $errors[] = 'Failed to update VAT Rate. Please try again.';
            }
        }
    }
}
?>
<h1>Update VAT Rate</h1>
<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>
<?php if ($success): ?>
    <div class="success">
        <p><?php echo htmlspecialchars($success); ?></p>
        <a href="vat_rates.php" class="button">Back to VAT Rates</a>
    </div>
<?php endif; ?>
<div class="form-container">
    <form action="update_vat_rates.php?id=<?php echo urlencode($vatRateId); ?>" method="POST">
        <label for="rate_name">Rate Name</label>
        <input type="text" id="rate_name" name="rate_name" value="<?php echo htmlspecialchars($rateName); ?>" required>

        <label for="rate_value">Rate Value (%)</label>
        <input type="text" id="rate_value" name="rate_value" value="<?php echo htmlspecialchars($rateValue); ?>" required>

        <label for="is_default">Set as Default</label>
        <input type="checkbox" id="is_default" name="is_default" <?php echo ($isDefault == 1 || isset($_POST['is_default'])) ? 'checked' : ''; ?>>

        <div class="button-group">
            <button type="submit">Update VAT Rate</button>
            <a href="vat_rates.php" class="button">Back</a>
        </div>
    </form>
</div>
<?php require_once('components/footer.php'); ?>
