<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// 1) Provide a route to list .layout files with "type":"vatreport"
if (isset($_GET['action']) && $_GET['action'] === 'list_vr_layouts') {
    header('Content-Type: application/json; charset=utf-8');

    $layoutsDir = __DIR__ . '/documents';

    function findVatReportLayouts($dir) {
        $results = [];
        if (!is_dir($dir)) {
            return $results;
        }
        $files = scandir($dir);
        foreach ($files as $f) {
            if (strtolower(pathinfo($f, PATHINFO_EXTENSION)) === 'layout') {
                $fullPath = $dir . '/' . $f;
                $jsonData = @json_decode(file_get_contents($fullPath), true);
                if (is_array($jsonData) && isset($jsonData['type']) && $jsonData['type'] === 'vatreport') {
                    $results[] = $f;
                }
            }
        }
        return $results;
    }

    $vrLayouts = findVatReportLayouts($layoutsDir);
    if (count($vrLayouts) === 0) {
        echo json_encode(['success'=>false,'message'=>'No layout for vatreport']);
        exit;
    }
    echo json_encode(['success'=>true,'layouts'=>$vrLayouts]);
    exit;
}

// Include necessary core files
require_once('../core/financial_reports.php');
require_once('../core/vat_periods.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

$errors = [];
$vatReport = [];

// **Role Check: Ensure the user has 'vat_report' role**
if (!checkRole(['vat_report'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to view this page.';
    unset($_SESSION['error_message']);
    ?>
    <h1>VAT Report</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Fetch all VAT periods for the dropdown
$vatPeriods = listVATPeriods($pdo);
$selectedVatPeriodId = '';
$startDate = '';
$endDate = '';

function fetchVATTransactionsByRate(PDO $pdo, string $start, string $end): array
{
    $sql = "SELECT t.*, vr.rate_name, vr.rate_value,
                   accD.name AS debit_name,  accC.name AS credit_name
            FROM transactions t
            JOIN vat_rates vr ON t.vat_rates_id = vr.id
            LEFT JOIN accounts accD ON t.debit_account = accD.code
            LEFT JOIN accounts accC ON t.credit_account = accC.code
            WHERE t.vat_rates_id != 0
              AND t.is_deleted = 0
              AND t.transaction_date BETWEEN :start AND :end
            ORDER BY t.transaction_date ASC";
    $stmt = $pdo->prepare($sql);
    $stmt->execute([
        ':start' => $start,
        ':end'   => $end
    ]);

    $results = [];
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
        $rateId = $row['vat_rates_id'];
        if (!isset($results[$rateId])) {
            $results[$rateId] = [
                'rate_name' => $row['rate_name'],
                'rate_value'=> $row['rate_value'],
                'transactions' => []
            ];
        }
        $results[$rateId]['transactions'][] = $row;
    }
    return $results;
}

// Handle form submission
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (!empty($_POST['vat_period_id'])) {
        $selectedVatPeriodId = (int)$_POST['vat_period_id'];
        $vatPeriod = getVATPeriod($pdo, $selectedVatPeriodId);
        if ($vatPeriod) {
            $startDate = $vatPeriod['start_date'];
            $endDate   = $vatPeriod['end_date'];
        } else {
            $errors[] = 'Selected VAT period not found.';
        }
    } else {
        // Custom date range
        $startDate = trim($_POST['start_date']);
        $endDate   = trim($_POST['end_date']);
        if (empty($startDate) || empty($endDate)) {
            $errors[] = 'Please provide both start date and end date.';
        } elseif ($startDate > $endDate) {
            $errors[] = 'Start date cannot be after end date.';
        }
    }

    // If no errors, generate
    if (empty($errors)) {
        if ($selectedVatPeriodId) {
            $vatReport = generateVATReport($pdo, $selectedVatPeriodId);
        } else {
            $vatReport = generateVATReport($pdo, null, $startDate, $endDate);
        }
        if (empty($vatReport)) {
            $errors[] = 'No VAT transactions found for the selected criteria.';
        }
    }
}

// Table columns
$columns = [
    ['header' => 'Account Code', 'field' => 'account_code'],
    ['header' => 'Account Name', 'field' => 'account_name'],
    ['header' => 'Account Type', 'field' => 'account_type'],
    ['header' => 'Total Debit', 'field' => 'total_debit'],
    ['header' => 'Total Credit', 'field' => 'total_credit'],
    ['header' => 'Net Change', 'field' => 'net_change']
];

$totalNetChange = 0;
if (!empty($vatReport) && empty($errors)) {
    foreach ($vatReport as $row) {
        $totalNetChange += (float)$row['net_change'];
    }
}

// remark based on totalNetChange
$remark = '';
if ($totalNetChange > 0) {
    $remark = "Pay VAT of " . number_format($totalNetChange, 2);
} elseif ($totalNetChange < 0) {
    $remark = "Refundable VAT of " . number_format(abs($totalNetChange), 2);
} else {
    $remark = "No VAT payable or credit at this time.";
}
?>
<h1>VAT Report</h1>

<div class="form-container">
    <form action="vatreport.php" method="POST">
        <h2>Select VAT Period or Date Range</h2>
        <label for="vat_period_id">VAT Period:</label>
        <select id="vat_period_id" name="vat_period_id">
            <option value="">-- Custom Date Range --</option>
            <?php foreach ($vatPeriods as $period): ?>
                <option value="<?php echo htmlspecialchars($period['id']); ?>"
                    <?php echo ($selectedVatPeriodId == $period['id'])?'selected':''; ?>>
                    <?php echo htmlspecialchars($period['name'].' ('.$period['start_date'].' to '.$period['end_date'].')'); ?>
                </option>
            <?php endforeach; ?>
        </select>
        <p>Or enter a custom date range (if no VAT period selected):</p>
        <label for="start_date">Start Date:</label>
        <input type="date" id="start_date" name="start_date" value="<?php echo htmlspecialchars($startDate); ?>">
        <label for="end_date">End Date:</label>
        <input type="date" id="end_date" name="end_date" value="<?php echo htmlspecialchars($endDate); ?>">
        <div class="button-group">
            <button type="submit">Generate VAT Report</button>
            <!-- NEW ORANGE BUTTON -->
            <button type="button"
                    id="generate_vr_pdf_button"
                    style="background-color: orange; color: #fff; border: none; padding: 8px 12px; cursor: pointer;">
                Generate Income Statement PDF
            </button>
        </div>
    </form>
</div>

<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>

<!-- Show transactions grouped by VAT rate if valid date range & no errors -->
<?php if (!empty($startDate) && !empty($endDate) && empty($errors)): ?>
    <?php
    $vatRateTables = fetchVATTransactionsByRate($pdo, $startDate, $endDate);
    if (!empty($vatRateTables)):
    ?>
        <h2>Transaction Listing by VAT Rate</h2>
        <?php foreach ($vatRateTables as $rateId => $vatGroup): ?>
            <h3>
                <?php
                echo htmlspecialchars($vatGroup['rate_name']);
                echo ' ('.number_format((float)$vatGroup['rate_value'],2).'%)';
                ?>
            </h3>
            <table>
                <thead>
                    <tr>
                        <th>Transaction Date</th>
                        <th>Debit Account</th>
                        <th>Credit Account</th>
                        <th>Amount</th>
                        <th>Description</th>
                    </tr>
                </thead>
                <tbody>
                    <?php foreach ($vatGroup['transactions'] as $tx): ?>
                        <tr>
                            <td><?php echo htmlspecialchars($tx['transaction_date']); ?></td>
                            <td><?php echo htmlspecialchars($tx['debit_name'] ?? $tx['debit_account']); ?></td>
                            <td><?php echo htmlspecialchars($tx['credit_name'] ?? $tx['credit_account']); ?></td>
                            <td><?php echo number_format($tx['amount'],2); ?></td>
                            <td><?php echo htmlspecialchars($tx['description']); ?></td>
                        </tr>
                    <?php endforeach; ?>
                </tbody>
            </table>
            <br>
        <?php endforeach; ?>
    <?php endif; ?>
<?php endif; ?>

<?php if (!empty($vatReport) && empty($errors)): ?>
    <h2>VAT Report Results</h2>
    <table>
        <thead>
            <tr>
                <?php foreach ($columns as $col): ?>
                  <th><?php echo htmlspecialchars($col['header']); ?></th>
                <?php endforeach; ?>
            </tr>
        </thead>
        <tbody>
            <?php foreach ($vatReport as $r): ?>
              <tr>
                <?php foreach ($columns as $col): ?>
                  <td><?php echo htmlspecialchars($r[$col['field']] ?? ''); ?></td>
                <?php endforeach; ?>
              </tr>
            <?php endforeach; ?>
            <tr style="font-weight:bold;">
                <td colspan="5" style="text-align:right;">Total Net Change:</td>
                <td><?php echo number_format($totalNetChange,2); ?></td>
            </tr>
        </tbody>
    </table>
    <p style="text-align:center; font-weight:bold;"><?php echo htmlspecialchars($remark); ?></p>
<?php endif; ?>

<!-- Hidden alert if no vatreport layouts -->
<div id="vr-no-layout-error" class="alert" style="display:none;">
    <p>No saved layout found for <strong>vatreport</strong>.
       <a href="document_editor.php">Create a layout here</a>.
    </p>
</div>

<!-- Modal for multiple vatreport layouts -->
<div id="vr-modal-overlay" style="
    display: none;
    position: fixed;
    top: 0; left: 0;
    width:100%; height:100%;
    background: rgba(0,0,0,0.4);
    align-items:center;
    justify-content:center;
    z-index:9999;">
  <div id="vr-modal-content" style="
      background:#fff;
      padding:20px;
      border-radius:4px;
      min-width:300px;">
    <h2>Select a VAT Report Layout</h2>
    <div id="vr-modal-error" style="color: red; font-weight: bold; display:none;"></div>
    <label for="vr-modal-select">Layouts:</label>
    <select id="vr-modal-select" style="width:100%; margin-bottom:15px;"></select>
    <div style="text-align:right;">
      <button id="vr-modal-cancel" style="margin-right:10px;">Cancel</button>
      <button id="vr-modal-ok"
              style="background-color: orange; color: #fff; border:none; padding:5px 12px; cursor:pointer;">
        OK
      </button>
    </div>
  </div>
</div>

<script>
document.addEventListener('DOMContentLoaded', function() {
    const pdfBtn = document.getElementById('generate_vr_pdf_button');
    if (!pdfBtn) return;

    pdfBtn.addEventListener('click', function(e) {
        e.preventDefault();
        // 1) Validate form
        const periodSelect  = document.getElementById('vat_period_id');
        const startDateElem = document.getElementById('start_date');
        const endDateElem   = document.getElementById('end_date');

        if ((!periodSelect.value || periodSelect.value==='') &&
            (!startDateElem.value || !endDateElem.value)) {
            alert("Please select a VAT Period OR provide a Start and End Date.");
            return;
        }

        // 2) Fetch .layout files with type="vatreport"
        fetch('vatreport.php?action=list_vr_layouts')
        .then(r=>r.json())
        .then(resp=>{
            if (!resp.success) {
                document.getElementById('vr-no-layout-error').style.display = 'block';
                return;
            }
            const layouts = resp.layouts||[];
            if (layouts.length===0) {
                document.getElementById('vr-no-layout-error').style.display = 'block';
                return;
            }
            if (layouts.length===1) {
                openVrPdfInNewTab(layouts[0]);
            } else {
                showVrModal(layouts);
            }
        })
        .catch(err=>{
            console.error("Error fetching vatreport layouts:", err);
            alert("Failed to list vatreport layouts. See console.");
        });
    });

    function showVrModal(layoutList) {
        const overlay   = document.getElementById('vr-modal-overlay');
        const modalSel  = document.getElementById('vr-modal-select');
        const cancelBtn = document.getElementById('vr-modal-cancel');
        const okBtn     = document.getElementById('vr-modal-ok');
        const errDiv    = document.getElementById('vr-modal-error');

        modalSel.innerHTML='';
        errDiv.textContent='';
        errDiv.style.display='none';

        layoutList.forEach(lay=>{
            const opt=document.createElement('option');
            opt.value=lay;
            opt.textContent=lay;
            modalSel.appendChild(opt);
        });

        overlay.style.display='flex';

        cancelBtn.onclick=()=>{ overlay.style.display='none'; };
        okBtn.onclick=()=>{
            const chosen=modalSel.value.trim();
            if(!chosen){
                errDiv.textContent="Please select a layout.";
                errDiv.style.display='block';
                return;
            }
            overlay.style.display='none';
            openVrPdfInNewTab(chosen);
        };
    }

    function openVrPdfInNewTab(layoutName) {
        // gather form data
        const periodVal = document.getElementById('vat_period_id').value;
        const sDateVal  = document.getElementById('start_date').value;
        const eDateVal  = document.getElementById('end_date').value;

        // fetch layout JSON
        fetch('documents/'+layoutName)
        .then(r=>r.json())
        .then(layData=>{
            layData.dialogData = {
                vat_period_id: periodVal,
                start_date   : sDateVal,
                end_date     : eDateVal
            };
            const formElem=document.createElement('form');
            formElem.method='POST';
            formElem.action='document_display_pdf.php';
            formElem.target='_blank';

            const inp=document.createElement('input');
            inp.type='hidden';
            inp.name='layoutJSON';
            inp.value=JSON.stringify(layData);
            formElem.appendChild(inp);

            document.body.appendChild(formElem);
            formElem.submit();
        })
        .catch(err=>{
            console.error("Cannot load layout for vatreport:", err);
            alert("Error loading layout file: "+layoutName);
        });
    }
});
</script>

<?php require_once('components/footer.php'); ?>
